/*******************************************************************************
Automatically generated file
Created by 'Control Macro Library Conversion Tool'
Creation time: 01.12.2020 19:17
Copyright (C) Honeywell GmbH D-71101 Schoenaich 2020
*******************************************************************************/

package cl.hvac.air_conditioning;

import javax.baja.status.*;
import javax.baja.sys.*;
import javax.baja.units.*;
import cl.hvac.*;
import cl.hvac.base.*;

/*==============================================================================
Macro name:    HCA_HeatingCoil
Macro id:      5797211810614476859
Macro version: 130
Last change:   22.03.2018 - 12:32
Access Level:  2
Inputs:
  [1] HeatCtrlSig
  [2] FilteredOATemp
  [3] FrostThermostat
  [4] DischargeAirTmp
  [5] ReturnWaterTemp
  [6] OffAlarm
  [7] FailCmdAlarm
  [8] PlantMode
  [9] HumidityMode
Outputs:
  [1] ValveControl
  [2] PumpCmd
  [3] DamperEnable
  [4] DamperEnableAlm
  [5] HeatDemand
Parameter:
  [1] SYS-01: heating coil type selection
  [2] RWT-01: return water temp SP shift un-occupied period
  [3] RWT-02: return water temperature low setpoint
  [4] RWT-03: return water temperature high setpoint
  [5] RWT-04: return water temperature P-band
  [6] RWT-05: return water temperature control
  [7] DME-01: damper enable or alarm delay
  [8] DAT-01: discharge air temperature control
  [9] DAT-02: discharge air temperature low setpoint
  [10] DAT-03: discharge air temperature high setpoint
  [11] DAT-04: discharge air temperature P-band
  [12] DAT-05: discharge air temperature I-time
  [13] HTG-01: calculated heating setpoint supply temp. min. setpoint
  [14] HTG-02: calculated heating setpoint supply temp. max. setpoint
  [15] HTG-03: calculated heating setpoint supply temp. start point
  [16] HTG-04: heat demand increase/ decrease value
  [17] HTG-05: heating valve pos. diff. to calculate heating setpoint
  [18] OAT-01: outside air temperature minimum limit
  [19] OAT-02: outside air temperature maximum limit
  [20] OAT-03: low limit frost protection
  [21] FEZ-01: frost protection RWT/ DAT setpoint raise
  [22] FEZ-02: raised setpoint decrease during frost recovery
  [23] FEZ-03: valve position during frost protection
  [24] FEZ-04: valve raised position decrease during frost recovery
  [25] HPU-01: startup delay heating pump
  [26] HPU-02: pump switch off delay
  [27] ALM-03: AHU stops at pump off alarm at low outdoor temperature
  [28] DISP-01: calculated heating demand temperature setpoint
  [29] DISP-02: calculated return water temperature setpoint
  [30] DISP-03: calculated discharge air temperature setpoint
Description:
The "HCA_HeatingCoil" macro provides modulating control of the heating coil valve actuator and controls the heating pump. The macro receives its heating control signal from the control strategy. A frost protection thermostat, if connected, will prevent freezing of the air handling unit coils at low outdoor temperatures. Continuous freeze protection can override the strategy signal by the:
 - coil's own coil off air temperature or
 - return water temperature control

The heating coil enables the dampers, only if the coils return water temperature (if configured) is high enough. For energy saving reasons, the heating coil can stop heating at high outdoor temperatures. The macro generates pump-on, fail-to-command and pump-off alarms, if the pump status is connected. Pump and valve exercise prevent the coil's equipment from getting stuck. The calculated heating demand sent to heat generators, depends on the actual heating valve position. Plant- and humidity mode signals can override the normal control signals.
================================================================================*/

public class BCmHCA_HeatingCoil
  extends BControlFunctionSupport
  implements IEnvironment
{
  public void doExecute()
  {
    if (getEnabledIn().getBoolean())
    {
      Execute( );
      if (getCmHCA_HeatingCoil_output_553().getStatus() == BStatus.disabled)
        setOutputStatusOk();
    }
    else
    {
      setOutputStatusDisabled();
    }
  }

  public void setOutputStatusDisabled()
  {
    getCmHCA_HeatingCoil_output_553().setStatus(BStatus.disabled);
    getCmHCA_HeatingCoil_output_554().setStatus(BStatus.disabled);
    getCmHCA_HeatingCoil_output_8().setStatus(BStatus.disabled);
    getCmHCA_HeatingCoil_output_9().setStatus(BStatus.disabled);
    getCmHCA_HeatingCoil_output_12().setStatus(BStatus.disabled);
  }

  public void setOutputStatusOk()
  {
    getCmHCA_HeatingCoil_output_553().setStatus(BStatus.ok);
    getCmHCA_HeatingCoil_output_554().setStatus(BStatus.ok);
    getCmHCA_HeatingCoil_output_8().setStatus(BStatus.ok);
    getCmHCA_HeatingCoil_output_9().setStatus(BStatus.ok);
    getCmHCA_HeatingCoil_output_12().setStatus(BStatus.ok);
  }

  void Execute( )
  {
    // cycle time calculation
    long ticks = ControlFunction.GetSystemTicks();
    lastCycleTime = (float)(ticks - lastSystemTicks) / 1000.0F;
    lastSystemTicks = ticks;

    // [24] Function call CfHysteresis
    func_24.inputX = (float) getCmHCA_HeatingCoil_parameter_21();
    func_24.inputW = (float) getCmHCA_HeatingCoil_input_1().getValue();
    func_24.Execute( this, 0 );

    // [40] Function call CfEqual
    func_40.input = FloatToInt((float) getCmHCA_HeatingCoil_input_6().getValue());
    func_40.inputCompare[1-1] = 1;
    func_40.inputCompare[2-1] = 2;
    func_40.inputCompare[3-1] = 4;
    func_40.inputCompare[4-1] = 5;
    func_40.inputCompare[5-1] = 6;
    func_40.inputCompare[6-1] = 30;
    func_40.Execute(  );

    // [64] Function call CfCompare1
    func_64.inputA = (float) getCmHCA_HeatingCoil_input_5().getValue();
    func_64.inputB = 999F;
    func_64.Execute(  );

    // [381] Function call CfEqual
    func_381.input = FloatToInt((float) getCmHCA_HeatingCoil_input_6().getValue());
    func_381.inputCompare[1-1] = 20;
    func_381.Execute(  );

    // [408] Macro call CmIDT10
    func_408.CmIDT10_input_1 = (float) getCmHCA_HeatingCoil_parameter_88();
    func_408.CmIDT10_input_2 = (float) getCmHCA_HeatingCoil_parameter_132();
    func_408.CmIDT10_input_3 = (float) getCmHCA_HeatingCoil_parameter_133();
    func_408.CmIDT10_input_4 = (float) getCmHCA_HeatingCoil_parameter_62();
    func_408.CmIDT10_input_5 = (float) getCmHCA_HeatingCoil_parameter_106();
    func_408.CmIDT10_input_6 = (float) getCmHCA_HeatingCoil_parameter_107();
    func_408.CmIDT10_input_7 = (float) getCmHCA_HeatingCoil_parameter_555();
    func_408.CmIDT10_input_8 = 0F;
    func_408.CmIDT10_input_9 = 0F;
    func_408.CmIDT10_input_10 = 0F;
    func_408.Execute(  );
    setParameter(23, func_408.CmIDT10_output_21);
    setParameter(24, func_408.CmIDT10_output_22);
    setParameter(25, func_408.CmIDT10_output_23);
    setParameter(26, func_408.CmIDT10_output_24);
    setParameter(27, func_408.CmIDT10_output_25);
    setParameter(28, func_408.CmIDT10_output_26);
    setParameter(29, func_408.CmIDT10_output_27);

    // [421] Function call CfIdt
    func_421.input = (float) getCmHCA_HeatingCoil_parameter_87();
    func_421.Execute(  );
    setParameter(1, func_421.output[0]);

    // [423] Macro call CmIDT05
    func_423.CmIDT05_input_1 = (float) getCmHCA_HeatingCoil_parameter_200();
    func_423.CmIDT05_input_2 = (float) getCmHCA_HeatingCoil_parameter_201();
    func_423.CmIDT05_input_3 = (float) getCmHCA_HeatingCoil_parameter_202();
    func_423.CmIDT05_input_4 = (float) getCmHCA_HeatingCoil_parameter_203();
    func_423.CmIDT05_input_5 = (float) getCmHCA_HeatingCoil_parameter_204();
    func_423.Execute(  );
    setParameter(36, func_423.CmIDT05_output_21);
    setParameter(37, func_423.CmIDT05_output_22);
    setParameter(38, func_423.CmIDT05_output_23);
    setParameter(39, func_423.CmIDT05_output_24);
    setParameter(40, func_423.CmIDT05_output_25);

    // [438] Macro call CmIDT05
    func_438.CmIDT05_input_1 = (float) getCmHCA_HeatingCoil_parameter_46();
    func_438.CmIDT05_input_2 = (float) getCmHCA_HeatingCoil_parameter_47();
    func_438.CmIDT05_input_3 = (float) getCmHCA_HeatingCoil_parameter_15();
    func_438.CmIDT05_input_4 = (float) getCmHCA_HeatingCoil_parameter_16();
    func_438.CmIDT05_input_5 = 0F;
    func_438.Execute(  );
    setParameter(6, func_438.CmIDT05_output_21);
    setParameter(7, func_438.CmIDT05_output_22);
    setParameter(8, func_438.CmIDT05_output_23);
    setParameter(9, func_438.CmIDT05_output_24);

    // [451] Macro call CmIDT10
    func_451.CmIDT10_input_1 = (float) getCmHCA_HeatingCoil_parameter_87();
    func_451.CmIDT10_input_2 = (float) getCmHCA_HeatingCoil_parameter_183();
    func_451.CmIDT10_input_3 = (float) getCmHCA_HeatingCoil_parameter_150();
    func_451.CmIDT10_input_4 = (float) getCmHCA_HeatingCoil_parameter_151();
    func_451.CmIDT10_input_5 = (float) getCmHCA_HeatingCoil_parameter_160();
    func_451.CmIDT10_input_6 = (float) getCmHCA_HeatingCoil_parameter_161();
    func_451.CmIDT10_input_7 = (float) getCmHCA_HeatingCoil_parameter_106();
    func_451.CmIDT10_input_8 = (float) getCmHCA_HeatingCoil_parameter_107();
    func_451.CmIDT10_input_9 = 0F;
    func_451.CmIDT10_input_10 = 0F;
    func_451.Execute(  );
    setParameter(10, func_451.CmIDT10_output_21);
    setParameter(11, func_451.CmIDT10_output_22);
    setParameter(12, func_451.CmIDT10_output_23);
    setParameter(13, func_451.CmIDT10_output_24);
    setParameter(14, func_451.CmIDT10_output_25);
    setParameter(15, func_451.CmIDT10_output_26);
    setParameter(16, func_451.CmIDT10_output_27);
    setParameter(17, func_451.CmIDT10_output_28);

    // [461] Function call CfIdt
    func_461.input = (float) getCmHCA_HeatingCoil_parameter_15();
    func_461.Execute(  );
    setParameter(35, func_461.output[0]);

    // [489] Function call CfIdt
    func_489.input = (float) getCmHCA_HeatingCoil_parameter_14();
    func_489.Execute(  );
    setParameter(2, func_489.output[0]);

    // [490] Function call CfIdt
    func_490.input = (float) getCmHCA_HeatingCoil_parameter_19();
    func_490.Execute(  );
    setParameter(3, func_490.output[0]);

    // [511] Function call CfOr
    func_511.input[0] = getCmHCA_HeatingCoil_input_549().getValue();
    func_511.input[1] = getCmHCA_HeatingCoil_input_548().getValue();
    func_511.Execute(  );

    // [547] Macro call CmIDT05
    func_547.CmIDT05_input_1 = (float) getCmHCA_HeatingCoil_parameter_237();
    func_547.CmIDT05_input_2 = (float) getCmHCA_HeatingCoil_parameter_13();
    func_547.CmIDT05_input_3 = (float) getCmHCA_HeatingCoil_parameter_21();
    func_547.CmIDT05_input_4 = 0F;
    func_547.CmIDT05_input_5 = 0F;
    func_547.Execute(  );
    setParameter(45, func_547.CmIDT05_output_21);
    setParameter(46, func_547.CmIDT05_output_22);
    setParameter(44, func_547.CmIDT05_output_23);

    // [351] Macro call CmHumidity_Control_HC
    func_351.CmHumidity_Control_HC_input_1 = (float) getCmHCA_HeatingCoil_input_2().getValue();
    func_351.CmHumidity_Control_HC_input_3 = (float) getCmHCA_HeatingCoil_input_334().getValue();
    func_351.CmHumidity_Control_HC_input_4 = BooleanToFloat(func_40.output);
    func_351.Execute( this, 1 );

    // [486] Macro call CmDamper_Enable
    func_486.CmDamper_Enable_input_8 = func_24.output1;
    func_486.CmDamper_Enable_input_6 = (float) getCmHCA_HeatingCoil_input_6().getValue();
    func_486.CmDamper_Enable_input_10 = func_511.outputOR;
    func_486.CmDamper_Enable_input_12 = func_64.outputEqual;
    func_486.CmDamper_Enable_input_11 = func_314.CmReturn_Water_Setpoint_Control_output_6;
    func_486.Execute( this, 2 );
    setCmHCA_HeatingCoil_output_8(func_486.CmDamper_Enable_output_4);
    setCmHCA_HeatingCoil_output_9(func_486.CmDamper_Enable_output_5);

    // [205] Macro call CmHeating_Signal_Setup
    func_205.CmHeating_Signal_Setup_input_25 = func_64.outputEqual;
    func_205.CmHeating_Signal_Setup_input_29 = func_24.output1;
    func_205.CmHeating_Signal_Setup_input_31 = getCmHCA_HeatingCoil_input_3().getValue();
    func_205.CmHeating_Signal_Setup_input_32 = (float) getCmHCA_HeatingCoil_input_6().getValue();
    func_205.CmHeating_Signal_Setup_input_24 = (float) getCmHCA_HeatingCoil_input_4().getValue();
    func_205.CmHeating_Signal_Setup_input_66 = getCmHCA_HeatingCoil_output_8().getValue();
    func_205.Execute( this, 6 );

    // [313] Macro call CmDischarge_Air_Temperature_Control
    func_313.CmDischarge_Air_Temperature_Control_input_244 = (float) getCmHCA_HeatingCoil_input_4().getValue();
    func_313.CmDischarge_Air_Temperature_Control_input_245 = (float) getCmHCA_HeatingCoil_input_1().getValue();
    func_313.CmDischarge_Air_Temperature_Control_input_246 = BooleanToFloat(func_40.output);
    func_313.CmDischarge_Air_Temperature_Control_input_247 = func_205.CmHeating_Signal_Setup_output_68;
    func_313.CmDischarge_Air_Temperature_Control_input_259 = func_24.output2;
    func_313.Execute( this, 10 );
    setCmHCA_HeatingCoil_parameter_341(func_313.CmDischarge_Air_Temperature_Control_output_271);

    // [314] Macro call CmReturn_Water_Setpoint_Control
    func_314.CmReturn_Water_Setpoint_Control_input_2 = (float) getCmHCA_HeatingCoil_input_5().getValue();
    func_314.CmReturn_Water_Setpoint_Control_input_1 = (float) getCmHCA_HeatingCoil_input_1().getValue();
    func_314.CmReturn_Water_Setpoint_Control_input_90 = func_24.output2;
    func_314.CmReturn_Water_Setpoint_Control_input_4 = func_205.CmHeating_Signal_Setup_output_68;
    func_314.CmReturn_Water_Setpoint_Control_input_139 = (float) getCmHCA_HeatingCoil_input_6().getValue();
    func_314.CmReturn_Water_Setpoint_Control_input_151 = func_40.output;
    func_314.CmReturn_Water_Setpoint_Control_input_201 = getCmHCA_HeatingCoil_output_8().getValue();
    func_314.Execute( this, 23 );
    setCmHCA_HeatingCoil_parameter_340(func_314.CmReturn_Water_Setpoint_Control_output_126);

    // [27] Macro call CmHC_Valve
    func_27.CmHC_Valve_input_29 = func_381.output;
    func_27.CmHC_Valve_input_1 = getCmHCA_HeatingCoil_input_3().getValue();
    func_27.CmHC_Valve_input_47 = BooleanToFloat(func_40.output);
    func_27.CmHC_Valve_input_14 = func_314.CmReturn_Water_Setpoint_Control_output_5;
    func_27.CmHC_Valve_input_15 = func_313.CmDischarge_Air_Temperature_Control_output_243;
    func_27.CmHC_Valve_input_16 = func_205.CmHeating_Signal_Setup_output_1;
    func_27.CmHC_Valve_input_27 = func_351.CmHumidity_Control_HC_output_5;
    func_27.Execute( this, 35 );
    setCmHCA_HeatingCoil_output_553(func_27.CmHC_Valve_output_6);

    // [29] Macro call CmDemand_As_A_Function_Of_The_Actual_Valve_Position
    func_29.CmDemand_As_A_Function_Of_The_Actual_Valve_Position_input_6 = (float) getCmHCA_HeatingCoil_output_553().getValue();
    func_29.Execute( this, 36 );
    setCmHCA_HeatingCoil_output_12(func_29.CmDemand_As_A_Function_Of_The_Actual_Valve_Position_output_7);

    // [546] Macro call CmPumpEnable
    func_546.CmPumpEnable_input_4 = (float) getCmHCA_HeatingCoil_output_553().getValue();
    func_546.CmPumpEnable_input_9 = (float) getCmHCA_HeatingCoil_input_1().getValue();
    func_546.CmPumpEnable_input_22 = (float) getCmHCA_HeatingCoil_input_6().getValue();
    func_546.CmPumpEnable_input_57 = getCmHCA_HeatingCoil_input_3().getValue();
    func_546.Execute( this, 44 );
    setCmHCA_HeatingCoil_output_554(func_546.CmPumpEnable_output_21);

    // [473] Function call CfIdt
    func_473.input = (float) getCmHCA_HeatingCoil_output_12().getValue();
    func_473.Execute(  );
    setCmHCA_HeatingCoil_parameter_342(func_473.output[0]);

  }

  // parameter for functions and sub macros
  protected float [] parameterList = new float []
  {
    1F, 0F, 0F, 0F, 10F, 10F, 15F, 1F,
    100F, 10F, 0F, 0F, 5F, 15F, 10F, 0F,
    -10F, 10F, 10F, 0F, 0F, 100F, 0F, 0F,
    0F, 0F, 1F, 0F, 0F, 2F, 10F, 0F,
    0F, 100F, 0F, 0F, 20F, 85F, 50F, 1F,
    25F, 2F, 60F, 60F, 6F, 5F, 300F, 2F,
    1F, 10F, 10F
  };

  // implementation of IEnvironment
  public float getParameter(int index)
  {
    return parameterList[index];
  };

  public void setParameter(int index, float value)
  {
    parameterList[index] = value;
  };

  // macro register which are written multiple times
  // macro outputs
// ValveControl
  public static final Property CmHCA_HeatingCoil_output_553 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmHCA_HeatingCoil_output_553() { return (BStatusNumeric) get(CmHCA_HeatingCoil_output_553); }
  public void setCmHCA_HeatingCoil_output_553(BStatusNumeric v) { set(CmHCA_HeatingCoil_output_553,v,null); }
  public void setCmHCA_HeatingCoil_output_553(float v) { getCmHCA_HeatingCoil_output_553().setValue(v); }
// PumpCmd
  public static final Property CmHCA_HeatingCoil_output_554 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeatingCoil_output_554() { return (BStatusBoolean) get(CmHCA_HeatingCoil_output_554); }
  public void setCmHCA_HeatingCoil_output_554(BStatusBoolean v) { set(CmHCA_HeatingCoil_output_554,v,null); }
  public void setCmHCA_HeatingCoil_output_554(boolean v) { getCmHCA_HeatingCoil_output_554().setValue(v); }
// DamperEnable
  public static final Property CmHCA_HeatingCoil_output_8 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeatingCoil_output_8() { return (BStatusBoolean) get(CmHCA_HeatingCoil_output_8); }
  public void setCmHCA_HeatingCoil_output_8(BStatusBoolean v) { set(CmHCA_HeatingCoil_output_8,v,null); }
  public void setCmHCA_HeatingCoil_output_8(boolean v) { getCmHCA_HeatingCoil_output_8().setValue(v); }
// DamperEnableAlm
  public static final Property CmHCA_HeatingCoil_output_9 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeatingCoil_output_9() { return (BStatusBoolean) get(CmHCA_HeatingCoil_output_9); }
  public void setCmHCA_HeatingCoil_output_9(BStatusBoolean v) { set(CmHCA_HeatingCoil_output_9,v,null); }
  public void setCmHCA_HeatingCoil_output_9(boolean v) { getCmHCA_HeatingCoil_output_9().setValue(v); }
// HeatDemand
  public static final Property CmHCA_HeatingCoil_output_12 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmHCA_HeatingCoil_output_12() { return (BStatusNumeric) get(CmHCA_HeatingCoil_output_12); }
  public void setCmHCA_HeatingCoil_output_12(BStatusNumeric v) { set(CmHCA_HeatingCoil_output_12,v,null); }
  public void setCmHCA_HeatingCoil_output_12(float v) { getCmHCA_HeatingCoil_output_12().setValue(v); }

  // macro parameter
// SYS-01: heating coil type selection
  public static final Property CmHCA_HeatingCoil_parameter_87 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(2)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmHCA_HeatingCoil_parameter_87() { return getDouble(CmHCA_HeatingCoil_parameter_87); }
  public void setCmHCA_HeatingCoil_parameter_87(double v) { setDouble(CmHCA_HeatingCoil_parameter_87,v,null); }
// RWT-01: return water temp SP shift un-occupied period
  public static final Property CmHCA_HeatingCoil_parameter_88 = newProperty( 0, BDouble.make( 5 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-50)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmHCA_HeatingCoil_parameter_88() { return getDouble(CmHCA_HeatingCoil_parameter_88); }
  public void setCmHCA_HeatingCoil_parameter_88(double v) { setDouble(CmHCA_HeatingCoil_parameter_88,v,null); }
// RWT-02: return water temperature low setpoint
  public static final Property CmHCA_HeatingCoil_parameter_132 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_132() { return getDouble(CmHCA_HeatingCoil_parameter_132); }
  public void setCmHCA_HeatingCoil_parameter_132(double v) { setDouble(CmHCA_HeatingCoil_parameter_132,v,null); }
// RWT-03: return water temperature high setpoint
  public static final Property CmHCA_HeatingCoil_parameter_133 = newProperty( 0, BDouble.make( 25 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_133() { return getDouble(CmHCA_HeatingCoil_parameter_133); }
  public void setCmHCA_HeatingCoil_parameter_133(double v) { setDouble(CmHCA_HeatingCoil_parameter_133,v,null); }
// RWT-04: return water temperature P-band
  public static final Property CmHCA_HeatingCoil_parameter_62 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(5000)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmHCA_HeatingCoil_parameter_62() { return getDouble(CmHCA_HeatingCoil_parameter_62); }
  public void setCmHCA_HeatingCoil_parameter_62(double v) { setDouble(CmHCA_HeatingCoil_parameter_62,v,null); }
// RWT-05: return water temperature control
  public static final Property CmHCA_HeatingCoil_parameter_555 = newProperty( 0, BDouble.make( 2 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(2)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmHCA_HeatingCoil_parameter_555() { return getDouble(CmHCA_HeatingCoil_parameter_555); }
  public void setCmHCA_HeatingCoil_parameter_555(double v) { setDouble(CmHCA_HeatingCoil_parameter_555,v,null); }
// DME-01: damper enable or alarm delay
  public static final Property CmHCA_HeatingCoil_parameter_14 = newProperty( 0, BDouble.make( 300 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeatingCoil_parameter_14() { return getDouble(CmHCA_HeatingCoil_parameter_14); }
  public void setCmHCA_HeatingCoil_parameter_14(double v) { setDouble(CmHCA_HeatingCoil_parameter_14,v,null); }
// DAT-01: discharge air temperature control
  public static final Property CmHCA_HeatingCoil_parameter_183 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(2)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmHCA_HeatingCoil_parameter_183() { return getDouble(CmHCA_HeatingCoil_parameter_183); }
  public void setCmHCA_HeatingCoil_parameter_183(double v) { setDouble(CmHCA_HeatingCoil_parameter_183,v,null); }
// DAT-02: discharge air temperature low setpoint
  public static final Property CmHCA_HeatingCoil_parameter_150 = newProperty( 0, BDouble.make( 5 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_150() { return getDouble(CmHCA_HeatingCoil_parameter_150); }
  public void setCmHCA_HeatingCoil_parameter_150(double v) { setDouble(CmHCA_HeatingCoil_parameter_150,v,null); }
// DAT-03: discharge air temperature high setpoint
  public static final Property CmHCA_HeatingCoil_parameter_151 = newProperty( 0, BDouble.make( 15 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_151() { return getDouble(CmHCA_HeatingCoil_parameter_151); }
  public void setCmHCA_HeatingCoil_parameter_151(double v) { setDouble(CmHCA_HeatingCoil_parameter_151,v,null); }
// DAT-04: discharge air temperature P-band
  public static final Property CmHCA_HeatingCoil_parameter_160 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmHCA_HeatingCoil_parameter_160() { return getDouble(CmHCA_HeatingCoil_parameter_160); }
  public void setCmHCA_HeatingCoil_parameter_160(double v) { setDouble(CmHCA_HeatingCoil_parameter_160,v,null); }
// DAT-05: discharge air temperature I-time
  public static final Property CmHCA_HeatingCoil_parameter_161 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeatingCoil_parameter_161() { return getDouble(CmHCA_HeatingCoil_parameter_161); }
  public void setCmHCA_HeatingCoil_parameter_161(double v) { setDouble(CmHCA_HeatingCoil_parameter_161,v,null); }
// HTG-01: calculated heating setpoint supply temp. min. setpoint
  public static final Property CmHCA_HeatingCoil_parameter_200 = newProperty( 0, BDouble.make( 20 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(150)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_200() { return getDouble(CmHCA_HeatingCoil_parameter_200); }
  public void setCmHCA_HeatingCoil_parameter_200(double v) { setDouble(CmHCA_HeatingCoil_parameter_200,v,null); }
// HTG-02: calculated heating setpoint supply temp. max. setpoint
  public static final Property CmHCA_HeatingCoil_parameter_201 = newProperty( 0, BDouble.make( 85 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(150)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_201() { return getDouble(CmHCA_HeatingCoil_parameter_201); }
  public void setCmHCA_HeatingCoil_parameter_201(double v) { setDouble(CmHCA_HeatingCoil_parameter_201,v,null); }
// HTG-03: calculated heating setpoint supply temp. start point
  public static final Property CmHCA_HeatingCoil_parameter_202 = newProperty( 0, BDouble.make( 50 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(150)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_202() { return getDouble(CmHCA_HeatingCoil_parameter_202); }
  public void setCmHCA_HeatingCoil_parameter_202(double v) { setDouble(CmHCA_HeatingCoil_parameter_202,v,null); }
// HTG-04: heat demand increase/ decrease value
  public static final Property CmHCA_HeatingCoil_parameter_203 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(10)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("degrees kelvin per minute"))));
  public double getCmHCA_HeatingCoil_parameter_203() { return getDouble(CmHCA_HeatingCoil_parameter_203); }
  public void setCmHCA_HeatingCoil_parameter_203(double v) { setDouble(CmHCA_HeatingCoil_parameter_203,v,null); }
// HTG-05: heating valve pos. diff. to calculate heating setpoint
  public static final Property CmHCA_HeatingCoil_parameter_204 = newProperty( 0, BDouble.make( 25 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(5)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmHCA_HeatingCoil_parameter_204() { return getDouble(CmHCA_HeatingCoil_parameter_204); }
  public void setCmHCA_HeatingCoil_parameter_204(double v) { setDouble(CmHCA_HeatingCoil_parameter_204,v,null); }
// OAT-01: outside air temperature minimum limit
  public static final Property CmHCA_HeatingCoil_parameter_106 = newProperty( 0, BDouble.make( -10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-50)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_106() { return getDouble(CmHCA_HeatingCoil_parameter_106); }
  public void setCmHCA_HeatingCoil_parameter_106(double v) { setDouble(CmHCA_HeatingCoil_parameter_106,v,null); }
// OAT-02: outside air temperature maximum limit
  public static final Property CmHCA_HeatingCoil_parameter_107 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-50)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_107() { return getDouble(CmHCA_HeatingCoil_parameter_107); }
  public void setCmHCA_HeatingCoil_parameter_107(double v) { setDouble(CmHCA_HeatingCoil_parameter_107,v,null); }
// OAT-03: low limit frost protection
  public static final Property CmHCA_HeatingCoil_parameter_21 = newProperty( 0, BDouble.make( 6 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-50)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_21() { return getDouble(CmHCA_HeatingCoil_parameter_21); }
  public void setCmHCA_HeatingCoil_parameter_21(double v) { setDouble(CmHCA_HeatingCoil_parameter_21,v,null); }
// FEZ-01: frost protection RWT/ DAT setpoint raise
  public static final Property CmHCA_HeatingCoil_parameter_46 = newProperty( 0, BDouble.make( 15 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmHCA_HeatingCoil_parameter_46() { return getDouble(CmHCA_HeatingCoil_parameter_46); }
  public void setCmHCA_HeatingCoil_parameter_46(double v) { setDouble(CmHCA_HeatingCoil_parameter_46,v,null); }
// FEZ-02: raised setpoint decrease during frost recovery
  public static final Property CmHCA_HeatingCoil_parameter_47 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(20)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("degrees kelvin per minute"))));
  public double getCmHCA_HeatingCoil_parameter_47() { return getDouble(CmHCA_HeatingCoil_parameter_47); }
  public void setCmHCA_HeatingCoil_parameter_47(double v) { setDouble(CmHCA_HeatingCoil_parameter_47,v,null); }
// FEZ-03: valve position during frost protection
  public static final Property CmHCA_HeatingCoil_parameter_15 = newProperty( 0, BDouble.make( 100 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmHCA_HeatingCoil_parameter_15() { return getDouble(CmHCA_HeatingCoil_parameter_15); }
  public void setCmHCA_HeatingCoil_parameter_15(double v) { setDouble(CmHCA_HeatingCoil_parameter_15,v,null); }
// FEZ-04: valve raised position decrease during frost recovery
  public static final Property CmHCA_HeatingCoil_parameter_16 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("per minute"))));
  public double getCmHCA_HeatingCoil_parameter_16() { return getDouble(CmHCA_HeatingCoil_parameter_16); }
  public void setCmHCA_HeatingCoil_parameter_16(double v) { setDouble(CmHCA_HeatingCoil_parameter_16,v,null); }
// HPU-01: startup delay heating pump
  public static final Property CmHCA_HeatingCoil_parameter_237 = newProperty( 0, BDouble.make( 5 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(300)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeatingCoil_parameter_237() { return getDouble(CmHCA_HeatingCoil_parameter_237); }
  public void setCmHCA_HeatingCoil_parameter_237(double v) { setDouble(CmHCA_HeatingCoil_parameter_237,v,null); }
// HPU-02: pump switch off delay
  public static final Property CmHCA_HeatingCoil_parameter_13 = newProperty( 0, BDouble.make( 300 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeatingCoil_parameter_13() { return getDouble(CmHCA_HeatingCoil_parameter_13); }
  public void setCmHCA_HeatingCoil_parameter_13(double v) { setDouble(CmHCA_HeatingCoil_parameter_13,v,null); }
// ALM-03: AHU stops at pump off alarm at low outdoor temperature
  public static final Property CmHCA_HeatingCoil_parameter_19 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmHCA_HeatingCoil_parameter_19() { return getDouble(CmHCA_HeatingCoil_parameter_19); }
  public void setCmHCA_HeatingCoil_parameter_19(double v) { setDouble(CmHCA_HeatingCoil_parameter_19,v,null); }
// DISP-01: calculated heating demand temperature setpoint
  public static final Property CmHCA_HeatingCoil_parameter_342 = newProperty( Flags.READONLY|Flags.TRANSIENT, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(0)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_342() { return getDouble(CmHCA_HeatingCoil_parameter_342); }
  public void setCmHCA_HeatingCoil_parameter_342(double v) { setDouble(CmHCA_HeatingCoil_parameter_342,v,null); }
// DISP-02: calculated return water temperature setpoint
  public static final Property CmHCA_HeatingCoil_parameter_340 = newProperty( Flags.READONLY|Flags.TRANSIENT, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(0)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_340() { return getDouble(CmHCA_HeatingCoil_parameter_340); }
  public void setCmHCA_HeatingCoil_parameter_340(double v) { setDouble(CmHCA_HeatingCoil_parameter_340,v,null); }
// DISP-03: calculated discharge air temperature setpoint
  public static final Property CmHCA_HeatingCoil_parameter_341 = newProperty( Flags.READONLY|Flags.TRANSIENT, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(0)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeatingCoil_parameter_341() { return getDouble(CmHCA_HeatingCoil_parameter_341); }
  public void setCmHCA_HeatingCoil_parameter_341(double v) { setDouble(CmHCA_HeatingCoil_parameter_341,v,null); }

  // macro inputs
// HeatCtrlSig
  public static final Property CmHCA_HeatingCoil_input_2 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), null);
  public BStatusNumeric getCmHCA_HeatingCoil_input_2() { return (BStatusNumeric) get(CmHCA_HeatingCoil_input_2); }
  public void setCmHCA_HeatingCoil_input_2(BStatusNumeric v) { set(CmHCA_HeatingCoil_input_2,v,null); }
  public void setCmHCA_HeatingCoil_input_2(float v) { getCmHCA_HeatingCoil_input_2().setValue(v); }
// FilteredOATemp
  public static final Property CmHCA_HeatingCoil_input_1 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmHCA_HeatingCoil_input_1() { return (BStatusNumeric) get(CmHCA_HeatingCoil_input_1); }
  public void setCmHCA_HeatingCoil_input_1(BStatusNumeric v) { set(CmHCA_HeatingCoil_input_1,v,null); }
  public void setCmHCA_HeatingCoil_input_1(float v) { getCmHCA_HeatingCoil_input_1().setValue(v); }
// FrostThermostat
  public static final Property CmHCA_HeatingCoil_input_3 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmHCA_HeatingCoil_input_3() { return (BStatusBoolean) get(CmHCA_HeatingCoil_input_3); }
  public void setCmHCA_HeatingCoil_input_3(BStatusBoolean v) { set(CmHCA_HeatingCoil_input_3,v,null); }
  public void setCmHCA_HeatingCoil_input_3(boolean v) { getCmHCA_HeatingCoil_input_3().setValue(v); }
// DischargeAirTmp
  public static final Property CmHCA_HeatingCoil_input_4 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmHCA_HeatingCoil_input_4() { return (BStatusNumeric) get(CmHCA_HeatingCoil_input_4); }
  public void setCmHCA_HeatingCoil_input_4(BStatusNumeric v) { set(CmHCA_HeatingCoil_input_4,v,null); }
  public void setCmHCA_HeatingCoil_input_4(float v) { getCmHCA_HeatingCoil_input_4().setValue(v); }
// ReturnWaterTemp
  public static final Property CmHCA_HeatingCoil_input_5 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmHCA_HeatingCoil_input_5() { return (BStatusNumeric) get(CmHCA_HeatingCoil_input_5); }
  public void setCmHCA_HeatingCoil_input_5(BStatusNumeric v) { set(CmHCA_HeatingCoil_input_5,v,null); }
  public void setCmHCA_HeatingCoil_input_5(float v) { getCmHCA_HeatingCoil_input_5().setValue(v); }
// OffAlarm
  public static final Property CmHCA_HeatingCoil_input_549 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmHCA_HeatingCoil_input_549() { return (BStatusBoolean) get(CmHCA_HeatingCoil_input_549); }
  public void setCmHCA_HeatingCoil_input_549(BStatusBoolean v) { set(CmHCA_HeatingCoil_input_549,v,null); }
  public void setCmHCA_HeatingCoil_input_549(boolean v) { getCmHCA_HeatingCoil_input_549().setValue(v); }
// FailCmdAlarm
  public static final Property CmHCA_HeatingCoil_input_548 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmHCA_HeatingCoil_input_548() { return (BStatusBoolean) get(CmHCA_HeatingCoil_input_548); }
  public void setCmHCA_HeatingCoil_input_548(BStatusBoolean v) { set(CmHCA_HeatingCoil_input_548,v,null); }
  public void setCmHCA_HeatingCoil_input_548(boolean v) { getCmHCA_HeatingCoil_input_548().setValue(v); }
// PlantMode
  public static final Property CmHCA_HeatingCoil_input_6 = newProperty( Flags.SUMMARY, new BStatusNumeric( 4 ), null);
  public BStatusNumeric getCmHCA_HeatingCoil_input_6() { return (BStatusNumeric) get(CmHCA_HeatingCoil_input_6); }
  public void setCmHCA_HeatingCoil_input_6(BStatusNumeric v) { set(CmHCA_HeatingCoil_input_6,v,null); }
  public void setCmHCA_HeatingCoil_input_6(float v) { getCmHCA_HeatingCoil_input_6().setValue(v); }
// HumidityMode
  public static final Property CmHCA_HeatingCoil_input_334 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), null);
  public BStatusNumeric getCmHCA_HeatingCoil_input_334() { return (BStatusNumeric) get(CmHCA_HeatingCoil_input_334); }
  public void setCmHCA_HeatingCoil_input_334(BStatusNumeric v) { set(CmHCA_HeatingCoil_input_334,v,null); }
  public void setCmHCA_HeatingCoil_input_334(float v) { getCmHCA_HeatingCoil_input_334().setValue(v); }

  public static final Property enabledIn = newProperty( 0, new BStatusBoolean( true ), null);
  public BStatusBoolean getEnabledIn() { return (BStatusBoolean) get(enabledIn); }
  public void setEnabledIn(BStatusBoolean v) { set(enabledIn,v,null); }
  public void setEnabledIn(boolean v) { getEnabledIn().setValue(v); }


  public static final Property versionInfo = newProperty( Flags.READONLY, BString.make( "51.08 [130] 22.03.2018-12:32").getString(), null);
  public String getVersionInfo() { return getString(versionInfo); }
  public void setVersionInfo(String v) { setString(versionInfo,v,null); }


  public static final Property macroId = newProperty( Flags.READONLY|Flags.HIDDEN, BString.make( "5797211810614476859").getString(), null);
  public String getMacroId() { return getString(macroId); }
  public void setMacroId(String v) { setString(macroId,v,null); }

  // TYPE declaration
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCmHCA_HeatingCoil.class);


  // cycle time
  protected long lastSystemTicks = 0;
  protected float lastCycleTime = 0;

  // implementation of IEnvironment interface
  public float getCycleTime()
  { 
    return lastCycleTime;
  }

  public void started() throws Exception
  {
    super.started();
    Start();
    lastSystemTicks = ControlFunction.GetSystemTicks();

  }

  public int getHighestPriority(int inputId)
  {
    return 0;
  }

  public float getPriorityValue(int inputId, int priorityIndex)
  {
    return Float.NaN;
  }

  public void setPriorityValue(int outputId, int priorityIndex, float value, boolean setNull)
  {
  }

  // helper functions
  private float   BooleanToFloat(boolean value) { return value ? 1 : 0; }
  private int     FloatToInt    (float   value) { return (int) value;   }

  // class variables
  protected CfHysteresis func_24 = new CfHysteresis();
  protected CfEqual func_40 = new CfEqual();
  protected CfCompare1 func_64 = new CfCompare1();
  protected CfEqual func_381 = new CfEqual();
  protected CmIDT10 func_408 = new CmIDT10();
  protected CfIdt func_421 = new CfIdt();
  protected CmIDT05 func_423 = new CmIDT05();
  protected CmIDT05 func_438 = new CmIDT05();
  protected CmIDT10 func_451 = new CmIDT10();
  protected CfIdt func_461 = new CfIdt();
  protected CfIdt func_489 = new CfIdt();
  protected CfIdt func_490 = new CfIdt();
  protected CfOr func_511 = new CfOr();
  protected CmIDT05 func_547 = new CmIDT05();
  protected CmHumidity_Control_HC func_351 = new CmHumidity_Control_HC();
  protected CmDamper_Enable func_486 = new CmDamper_Enable();
  protected CmHeating_Signal_Setup func_205 = new CmHeating_Signal_Setup();
  protected CmDischarge_Air_Temperature_Control func_313 = new CmDischarge_Air_Temperature_Control();
  protected CmReturn_Water_Setpoint_Control func_314 = new CmReturn_Water_Setpoint_Control();
  protected CmHC_Valve func_27 = new CmHC_Valve();
  protected CmDemand_As_A_Function_Of_The_Actual_Valve_Position func_29 = new CmDemand_As_A_Function_Of_The_Actual_Valve_Position();
  protected CmPumpEnable func_546 = new CmPumpEnable();
  protected CfIdt func_473 = new CfIdt();

  // default constructor
  public BCmHCA_HeatingCoil()
  {
    func_40.configure(7);
    func_381.configure(2);
  }

  public void Start()
  {
    func_24.Start();
    func_408.Start();
    func_423.Start();
    func_438.Start();
    func_451.Start();
    func_547.Start();
    func_351.Start();
    func_486.Start();
    func_205.Start();
    func_313.Start();
    func_314.Start();
    func_27.Start();
    func_29.Start();
    func_546.Start();
  }
}
