/*******************************************************************************
Automatically generated file
Created by 'Control Macro Library Conversion Tool'
Creation time: 01.12.2020 17:24
Copyright (C) Honeywell GmbH D-71101 Schoenaich 2020
*******************************************************************************/

package cl.hvac.energy_management;

import javax.baja.status.*;
import javax.baja.sys.*;
import javax.baja.units.*;
import cl.hvac.*;
import cl.hvac.base.*;

/*==============================================================================
Macro name:    TTB_SolarEnergy
Macro id:      6171992678971998582
Macro version: 37
Last change:   18.12.2015 - 07:48
Access Level:  2
Inputs:
  [1] SolPanelTemp
  [2] SolarRetTemp
  [3] SolPumpCmd
  [4] SolPumpCtrl
  [5] SolarFlow
  [6] SolarPower
Outputs:
  [1] SolarPower
  [2] CO2_Savings
  [3] ElectrEnergyPmp
Parameter:
  [1] CFG-01: flow meter engineering unit
  [2] PMP-06: pump nominal electrical power
  [3] PMP-07: collector pump flow
  [4] POW-01: collector glycol concentration
  [5] POW-02: collector fluid heat capacity - cF
  [6] POW-03:  glycol heat capacity
  [7] CO2-01: kg of CO2 equivalent of a kWh of fuel
Description:
The macro TTB_SolarEnergy calculates:
-	solar power
-	CO2 savings 
-	Calculated electric energy consumed by the solar pump

To calculate the solar power, we have the following options:
-	Calculation using a flow signal and the temperature difference
                     The flow signal can be connected form any:
-	external flow meter no matter if it comes from physical connection, a pulse meter                               using TTA_PulseCounter,  M-Bus, Modbus or other meters connected to the                                      supported buses. 
-	the pump flow at nominal capacity (PMP-07) ignoring the differential pressure                                    influence
-	Calculation using external heat power or energy meters
-	These meters can be connected to M-Bus, Modbus or external pulse meters – using                       the macro TTA_PulseCounter.
Any heat energy meter has a heat power signal. When using a pulse meter with the macro “TTA_PulseCounter” connect the heat power to the output “MomentaryUse”,

The external heat meters can be of two types:
-	meters suitable for solar installations and considering the low heat capacity of the                            water-glycol solar fluid
-	In this case, you do not need this macro and can connect the values directly to the                           solar panel macro.
•	meters for water, which do not consider the lower heat power due to the lower heat                       capacity of the solar fluid.
-	In this case you need the macro to apply correction factors to the power and energy                       values.

Table 14.   Parameter List
CFG-01: flow meter engineering unit 0; 1 = l/sec    2 = m3/h
PMP-06: pump nominal electrical power	W
PMP-07: collector pump flow	l/ sec or m3/h
POW-01: collector glycol concentration	%
POW-02: collector fluid heat capacity - cF	kJ/(kg·K)
POW-03:  glycol heat capacity	kJ/(kg·K)
POW-04:  thermal coefficient of the glycol heat capacity	kJ/(kg·K °C)
CO2-01: kg of CO2 equivalent of a kWh of fuel	Kg/ kWh


================================================================================*/

public class BCmTTB_SolarEnergy
  extends BControlFunctionSupport
  implements IEnvironment
{
  public void doExecute()
  {
    if (getEnabledIn().getBoolean())
    {
      Execute( );
      if (getCmTTB_SolarEnergy_output_8().getStatus() == BStatus.disabled)
        setOutputStatusOk();
    }
    else
    {
      setOutputStatusDisabled();
    }
  }

  public void setOutputStatusDisabled()
  {
    getCmTTB_SolarEnergy_output_8().setStatus(BStatus.disabled);
    getCmTTB_SolarEnergy_output_10().setStatus(BStatus.disabled);
    getCmTTB_SolarEnergy_output_11().setStatus(BStatus.disabled);
  }

  public void setOutputStatusOk()
  {
    getCmTTB_SolarEnergy_output_8().setStatus(BStatus.ok);
    getCmTTB_SolarEnergy_output_10().setStatus(BStatus.ok);
    getCmTTB_SolarEnergy_output_11().setStatus(BStatus.ok);
  }

  void Execute( )
  {
    // cycle time calculation
    long ticks = ControlFunction.GetSystemTicks();
    lastCycleTime = (float)(ticks - lastSystemTicks) / 1000.0F;
    lastSystemTicks = ticks;

    // [70] Macro call CmFlow
    func_70.CmFlow_input_29 = (float) getCmTTB_SolarEnergy_input_5().getValue();
    func_70.CmFlow_input_30 = getCmTTB_SolarEnergy_input_3().getValue();
    func_70.CmFlow_input_31 = (float) getCmTTB_SolarEnergy_input_4().getValue();
    func_70.CmFlow_input_32 = 999F;
    func_70.CmFlow_input_39 = 999F;
    func_70.CmFlow_input_33 = (float) getCmTTB_SolarEnergy_parameter_14();
    func_70.CmFlow_input_65 = (float) getCmTTB_SolarEnergy_parameter_15();
    func_70.CmFlow_input_34 = (float) getCmTTB_SolarEnergy_parameter_16();
    func_70.Execute( this, 0 );
    setCmTTB_SolarEnergy_output_11(func_70.CmFlow_output_66);

    // [72] Function call CfSubtract
    func_72.input[0] = 100F;
    func_72.input[1] = (float) getCmTTB_SolarEnergy_parameter_17();
    func_72.Execute(  );

    // [74] Function call CfMultiply
    func_74.input[0] = (float) getCmTTB_SolarEnergy_parameter_19();
    func_74.input[1] = (float) getCmTTB_SolarEnergy_parameter_17();
    func_74.Execute(  );

    // [76] Function call CfCompare1
    func_76.inputA = (float) getCmTTB_SolarEnergy_parameter_18();
    func_76.inputB = 0F;
    func_76.Execute(  );

    // [131] Function call CfCompare1
    func_131.inputA = (float) getCmTTB_SolarEnergy_input_5().getValue();
    func_131.inputB = 999F;
    func_131.Execute(  );

    // [153] Function call CfWindow
    func_153.inputValue = (float) getCmTTB_SolarEnergy_input_6().getValue();
    func_153.inputMinimum = 998.8F;
    func_153.inputMaximum = 999.2F;
    func_153.Execute(  );

    // [156] Function call CfNot
    func_156.input = ControlFunction.GetStartupFlag();
    func_156.Execute(  );

    // [167] Function call CfCompare1
    func_167.inputA = (float) getCmTTB_SolarEnergy_input_6().getValue();
    func_167.inputB = 999F;
    func_167.Execute(  );

    // [182] Function call CfDelayOnOff
    func_182.input = ControlFunction.GetStartupFlag();
    func_182.Execute( this, 2 );

    // [41] Function call CfMultiply
    func_41.input[0] = getParameter(16);
    func_41.input[1] = func_72.output;
    func_41.Execute(  );

    // [152] Function call CfAnd
    func_152.input[0] = func_182.output;
    func_152.input[1] = func_153.output;
    func_152.Execute(  );

    // [168] Function call CfOr
    func_168.input[0] = func_167.outputGreater;
    func_168.input[1] = func_167.outputLess;
    func_168.Execute(  );

    // [73] Function call CfAddition
    func_73.input[0] = func_41.output;
    func_73.input[1] = func_74.output;
    func_73.Execute(  );

    // [157] Function call CfAnd
    func_157.input[0] = func_156.output;
    func_157.input[1] = func_168.outputOR;
    func_157.Execute(  );

    // [86] Function call CfDivide
    func_86.inputA = func_73.output;
    func_86.inputB = 100F;
    func_86.Execute(  );

    // [170] Function call CfDelayOnOff
    func_170.input = func_157.outputAND;
    func_170.Execute( this, 4 );

    // [75] Function call CfInputMultiplexer
    func_75.inputSelect = BooleanToInt(func_76.outputGreater);
    func_75.input[1-1] = func_86.output;
    func_75.input[2-1] = (float) getCmTTB_SolarEnergy_parameter_18();
    func_75.Execute(  );

    // [154] Function call CfRsFlipFlop
    func_154.inputSet = func_152.outputAND;
    func_154.inputReset = func_170.output;
    func_154.Execute(  );

    // [184] Function call CfOr
    func_184.input[0] = func_154.output;
    func_184.input[1] = func_131.outputLess;
    func_184.Execute(  );

    // [100] Function call CfNot
    func_100.input = func_184.outputOR;
    func_100.Execute(  );

    // [99] Function call CfCycle
    func_99.input = func_100.output;
    func_99.Execute( this, 6 );

    // [113] Function call CfOr
    func_113.input[0] = func_184.outputOR;
    func_113.input[1] = func_99.output;
    func_113.Execute(  );

    // [20] Macro call CmHeat_Output
    if ( func_113.outputOR)
    {
      func_20.CmHeat_Output_input_1 = func_70.CmFlow_output_37;
      func_20.CmHeat_Output_input_2 = (float) getCmTTB_SolarEnergy_input_1().getValue();
      func_20.CmHeat_Output_input_3 = (float) getCmTTB_SolarEnergy_input_2().getValue();
      func_20.Execute( this, 8 );
    }

    // [129] Function call CfMultiply
    func_129.input[0] = func_20.CmHeat_Output_output_4;
    func_129.input[1] = 1000F;
    func_129.Execute(  );

    // [104] Function call CfInputMultiplexer
    func_104.inputSelect = BooleanToInt(func_184.outputOR);
    func_104.input[1-1] = (float) getCmTTB_SolarEnergy_input_6().getValue();
    func_104.input[2-1] = func_129.output;
    func_104.Execute(  );

    // [77] Function call CfMultiply
    func_77.input[0] = func_104.output;
    func_77.input[1] = func_75.output;
    func_77.Execute(  );

    // [79] Function call CfDivide
    func_79.inputA = func_77.output;
    func_79.inputB = getParameter(16);
    func_79.Execute(  );
    setCmTTB_SolarEnergy_output_8(func_79.output);

    // [109] Function call CfMultiply
    func_109.input[0] = (float) getCmTTB_SolarEnergy_output_8().getValue();
    func_109.input[1] = (float) getCmTTB_SolarEnergy_parameter_108();
    func_109.Execute(  );
    setCmTTB_SolarEnergy_output_10(func_109.output);

  }

  // parameter for functions and sub macros
  protected float [] parameterList = new float []
  {
    0F, 0F, 0F, 90F, 120F, 0F, 1F, 60F,
    0F, 0F, 0F, 0F, 0F, 0F, 0F, 0F,
    0F, 0F, 0F, 0F, 0F, 0F, 0F, 45F,
    4F, 5F, 60F, 200F, 60F, 4F, 100F, 100F,
    4F, 200F, 4F, 4F, 5F, 0F, 60F, 0F,
    4F, 20F, 20F, 4F, 60F, 4F, 800F, 1000F,
    60F, 200F, 60F, 983F, 100F, 100F, 958F, 200F,
    865F, 800F, 1000F, 0F, 60F, 0F, 999F, 20F,
    20F, 999F, 60F, 983F
  };

  // implementation of IEnvironment
  public float getParameter(int index)
  {
    return parameterList[index];
  };

  public void setParameter(int index, float value)
  {
    parameterList[index] = value;
  };

  // macro register which are written multiple times
  // macro outputs
// SolarPower
  public static final Property CmTTB_SolarEnergy_output_8 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), BFacets.make(BFacets.UNITS, BUnit.getUnit("kilowatt")));
  public BStatusNumeric getCmTTB_SolarEnergy_output_8() { return (BStatusNumeric) get(CmTTB_SolarEnergy_output_8); }
  public void setCmTTB_SolarEnergy_output_8(BStatusNumeric v) { set(CmTTB_SolarEnergy_output_8,v,null); }
  public void setCmTTB_SolarEnergy_output_8(float v) { getCmTTB_SolarEnergy_output_8().setValue(v); }
// CO2_Savings
  public static final Property CmTTB_SolarEnergy_output_10 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmTTB_SolarEnergy_output_10() { return (BStatusNumeric) get(CmTTB_SolarEnergy_output_10); }
  public void setCmTTB_SolarEnergy_output_10(BStatusNumeric v) { set(CmTTB_SolarEnergy_output_10,v,null); }
  public void setCmTTB_SolarEnergy_output_10(float v) { getCmTTB_SolarEnergy_output_10().setValue(v); }
// ElectrEnergyPmp
  public static final Property CmTTB_SolarEnergy_output_11 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmTTB_SolarEnergy_output_11() { return (BStatusNumeric) get(CmTTB_SolarEnergy_output_11); }
  public void setCmTTB_SolarEnergy_output_11(BStatusNumeric v) { set(CmTTB_SolarEnergy_output_11,v,null); }
  public void setCmTTB_SolarEnergy_output_11(float v) { getCmTTB_SolarEnergy_output_11().setValue(v); }

  // macro parameter
// CFG-01: flow meter engineering unit
  public static final Property CmTTB_SolarEnergy_parameter_14 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(2)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmTTB_SolarEnergy_parameter_14() { return getDouble(CmTTB_SolarEnergy_parameter_14); }
  public void setCmTTB_SolarEnergy_parameter_14(double v) { setDouble(CmTTB_SolarEnergy_parameter_14,v,null); }
// PMP-06: pump nominal electrical power
  public static final Property CmTTB_SolarEnergy_parameter_15 = newProperty( 0, BDouble.make( 50 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(50000)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmTTB_SolarEnergy_parameter_15() { return getDouble(CmTTB_SolarEnergy_parameter_15); }
  public void setCmTTB_SolarEnergy_parameter_15(double v) { setDouble(CmTTB_SolarEnergy_parameter_15,v,null); }
// PMP-07: collector pump flow
  public static final Property CmTTB_SolarEnergy_parameter_16 = newProperty( 0, BDouble.make( 0.01 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(99999)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmTTB_SolarEnergy_parameter_16() { return getDouble(CmTTB_SolarEnergy_parameter_16); }
  public void setCmTTB_SolarEnergy_parameter_16(double v) { setDouble(CmTTB_SolarEnergy_parameter_16,v,null); }
// POW-01: collector glycol concentration
  public static final Property CmTTB_SolarEnergy_parameter_17 = newProperty( 0, BDouble.make( 50 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmTTB_SolarEnergy_parameter_17() { return getDouble(CmTTB_SolarEnergy_parameter_17); }
  public void setCmTTB_SolarEnergy_parameter_17(double v) { setDouble(CmTTB_SolarEnergy_parameter_17,v,null); }
// POW-02: collector fluid heat capacity - cF
  public static final Property CmTTB_SolarEnergy_parameter_18 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(120)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmTTB_SolarEnergy_parameter_18() { return getDouble(CmTTB_SolarEnergy_parameter_18); }
  public void setCmTTB_SolarEnergy_parameter_18(double v) { setDouble(CmTTB_SolarEnergy_parameter_18,v,null); }
// POW-03:  glycol heat capacity
  public static final Property CmTTB_SolarEnergy_parameter_19 = newProperty( 0, BDouble.make( 2.277 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(120)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmTTB_SolarEnergy_parameter_19() { return getDouble(CmTTB_SolarEnergy_parameter_19); }
  public void setCmTTB_SolarEnergy_parameter_19(double v) { setDouble(CmTTB_SolarEnergy_parameter_19,v,null); }
// CO2-01: kg of CO2 equivalent of a kWh of fuel
  public static final Property CmTTB_SolarEnergy_parameter_108 = newProperty( 0, BDouble.make( 0.31 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(10)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmTTB_SolarEnergy_parameter_108() { return getDouble(CmTTB_SolarEnergy_parameter_108); }
  public void setCmTTB_SolarEnergy_parameter_108(double v) { setDouble(CmTTB_SolarEnergy_parameter_108,v,null); }

  // macro inputs
// SolPanelTemp
  public static final Property CmTTB_SolarEnergy_input_1 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius")));
  public BStatusNumeric getCmTTB_SolarEnergy_input_1() { return (BStatusNumeric) get(CmTTB_SolarEnergy_input_1); }
  public void setCmTTB_SolarEnergy_input_1(BStatusNumeric v) { set(CmTTB_SolarEnergy_input_1,v,null); }
  public void setCmTTB_SolarEnergy_input_1(float v) { getCmTTB_SolarEnergy_input_1().setValue(v); }
// SolarRetTemp
  public static final Property CmTTB_SolarEnergy_input_2 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius")));
  public BStatusNumeric getCmTTB_SolarEnergy_input_2() { return (BStatusNumeric) get(CmTTB_SolarEnergy_input_2); }
  public void setCmTTB_SolarEnergy_input_2(BStatusNumeric v) { set(CmTTB_SolarEnergy_input_2,v,null); }
  public void setCmTTB_SolarEnergy_input_2(float v) { getCmTTB_SolarEnergy_input_2().setValue(v); }
// SolPumpCmd
  public static final Property CmTTB_SolarEnergy_input_3 = newProperty( Flags.SUMMARY, new BStatusBoolean( true ), null);
  public BStatusBoolean getCmTTB_SolarEnergy_input_3() { return (BStatusBoolean) get(CmTTB_SolarEnergy_input_3); }
  public void setCmTTB_SolarEnergy_input_3(BStatusBoolean v) { set(CmTTB_SolarEnergy_input_3,v,null); }
  public void setCmTTB_SolarEnergy_input_3(boolean v) { getCmTTB_SolarEnergy_input_3().setValue(v); }
// SolPumpCtrl
  public static final Property CmTTB_SolarEnergy_input_4 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmTTB_SolarEnergy_input_4() { return (BStatusNumeric) get(CmTTB_SolarEnergy_input_4); }
  public void setCmTTB_SolarEnergy_input_4(BStatusNumeric v) { set(CmTTB_SolarEnergy_input_4,v,null); }
  public void setCmTTB_SolarEnergy_input_4(float v) { getCmTTB_SolarEnergy_input_4().setValue(v); }
// SolarFlow
  public static final Property CmTTB_SolarEnergy_input_5 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmTTB_SolarEnergy_input_5() { return (BStatusNumeric) get(CmTTB_SolarEnergy_input_5); }
  public void setCmTTB_SolarEnergy_input_5(BStatusNumeric v) { set(CmTTB_SolarEnergy_input_5,v,null); }
  public void setCmTTB_SolarEnergy_input_5(float v) { getCmTTB_SolarEnergy_input_5().setValue(v); }
// SolarPower
  public static final Property CmTTB_SolarEnergy_input_6 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), null);
  public BStatusNumeric getCmTTB_SolarEnergy_input_6() { return (BStatusNumeric) get(CmTTB_SolarEnergy_input_6); }
  public void setCmTTB_SolarEnergy_input_6(BStatusNumeric v) { set(CmTTB_SolarEnergy_input_6,v,null); }
  public void setCmTTB_SolarEnergy_input_6(float v) { getCmTTB_SolarEnergy_input_6().setValue(v); }

  public static final Property enabledIn = newProperty( 0, new BStatusBoolean( true ), null);
  public BStatusBoolean getEnabledIn() { return (BStatusBoolean) get(enabledIn); }
  public void setEnabledIn(BStatusBoolean v) { set(enabledIn,v,null); }
  public void setEnabledIn(boolean v) { getEnabledIn().setValue(v); }


  public static final Property versionInfo = newProperty( Flags.READONLY, BString.make( "51.01 [37] 18.12.2015-07:48").getString(), null);
  public String getVersionInfo() { return getString(versionInfo); }
  public void setVersionInfo(String v) { setString(versionInfo,v,null); }


  public static final Property macroId = newProperty( Flags.READONLY|Flags.HIDDEN, BString.make( "6171992678971998582").getString(), null);
  public String getMacroId() { return getString(macroId); }
  public void setMacroId(String v) { setString(macroId,v,null); }

  // TYPE declaration
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCmTTB_SolarEnergy.class);


  // cycle time
  protected long lastSystemTicks = 0;
  protected float lastCycleTime = 0;

  // implementation of IEnvironment interface
  public float getCycleTime()
  { 
    return lastCycleTime;
  }

  public void started() throws Exception
  {
    super.started();
    Start();
    lastSystemTicks = ControlFunction.GetSystemTicks();

  }

  public int getHighestPriority(int inputId)
  {
    return 0;
  }

  public float getPriorityValue(int inputId, int priorityIndex)
  {
    return Float.NaN;
  }

  public void setPriorityValue(int outputId, int priorityIndex, float value, boolean setNull)
  {
  }

  // helper functions
  private int     BooleanToInt  (boolean value) { return value ? 1 : 0; }

  // class variables
  protected CmFlow func_70 = new CmFlow();
  protected CfSubtract func_72 = new CfSubtract();
  protected CfMultiply func_74 = new CfMultiply();
  protected CfCompare1 func_76 = new CfCompare1();
  protected CfCompare1 func_131 = new CfCompare1();
  protected CfWindow func_153 = new CfWindow();
  protected CfNot func_156 = new CfNot();
  protected CfCompare1 func_167 = new CfCompare1();
  protected CfDelayOnOff func_182 = new CfDelayOnOff();
  protected CfMultiply func_41 = new CfMultiply();
  protected CfAnd func_152 = new CfAnd();
  protected CfOr func_168 = new CfOr();
  protected CfAddition func_73 = new CfAddition();
  protected CfAnd func_157 = new CfAnd();
  protected CfDivide func_86 = new CfDivide();
  protected CfDelayOnOff func_170 = new CfDelayOnOff();
  protected CfInputMultiplexer func_75 = new CfInputMultiplexer();
  protected CfRsFlipFlop func_154 = new CfRsFlipFlop();
  protected CfOr func_184 = new CfOr();
  protected CfNot func_100 = new CfNot();
  protected CfCycle func_99 = new CfCycle();
  protected CfOr func_113 = new CfOr();
  protected CmHeat_Output func_20 = new CmHeat_Output();
  protected CfMultiply func_129 = new CfMultiply();
  protected CfInputMultiplexer func_104 = new CfInputMultiplexer();
  protected CfMultiply func_77 = new CfMultiply();
  protected CfDivide func_79 = new CfDivide();
  protected CfMultiply func_109 = new CfMultiply();

  // default constructor
  public BCmTTB_SolarEnergy()
  {
  }

  public void Start()
  {
    func_70.Start();
    func_182.Start();
    func_170.Start();
    func_154.Start();
    func_99.Start();
    func_20.Start();
  }
}
