/*******************************************************************************
Automatically generated file
Created by 'Control Macro Library Conversion Tool'
Creation time: 03.12.2020 12:02
Copyright (C) Honeywell GmbH D-71101 Schoenaich 2020
*******************************************************************************/

package cl.hvac.general;

import javax.baja.status.*;
import javax.baja.sys.*;
import javax.baja.units.*;
import cl.hvac.*;
import cl.hvac.base.*;

/*==============================================================================
Macro name:    EPID
Macro id:      6533553453234389458
Macro version: 12
Last change:   16.03.2018 - 16:13
Access Level:  2
Inputs:
  [1] TempInput
  [2] Setpoint
  [3] Enable
  [4] Start ramp time
  [5] Output start value
  [6] Auxiliary
Outputs:
  [1] Ctrl_Out
  [2] Ramp
Parameter:
  [1] P01 Rev_Acting_Enable: 0=x-w, 1=w-x
  [2] P02 Proportional_Band
  [3] P03 Integration_Time
  [4] P04 Derivative_Time
  [5] P05 Output_Minimum
  [6] P06 Output_Maximum
  [7] P08 Aux_Input_Mode
  [8] P10 Prop_Limit
Description:
Enhanced PID: PID control with start-up ramp, direct or 
reverse action selection, and integral value reset.

- Inputs: Inp = input value
              SPt = setpoint
              Ena = enable
              SRT = start ramp time
              OSV = output start value
              Aux = auxiliary input
- Outputs: Out = output
                 Rmp = start ramp value (0-100 pct)
- Parameter P1 controls action of output, 0 = direct action
  1 = reverse action (default=direct acting)
- The output start value input controls the initial value of the
  output at the beginning of the start ramp period. This value
  will also be present at the output when the enable input is
  off.
- The ramp output goes from 0 to 100 during the start-up
  ramp period. (Provided for diagnostic use, if desired)
- The start ramp time input sets the duration of the start
  ramp.
- The auxiliary input value may be MIN'd or MAX'd with the
  internal PID value depending on the value of parameter
  P7: -1 = MIN the two values, +1 = MAX the two values,
          0 = disable Aux input
  The Aux input is primarily intended for applications where
  EPID is used for high or low limit applications.

translated from rev 1.0 - 04/24/93

================================================================================*/

public class BCmEPID
  extends BControlFunctionSupport
  implements IEnvironment
{
  public void doExecute()
  {
    if (getEnabledIn().getBoolean())
    {
      Execute( );
      if (getCmEPID_output_7().getStatus() == BStatus.disabled)
        setOutputStatusOk();
    }
    else
    {
      setOutputStatusDisabled();
    }
  }

  public void setOutputStatusDisabled()
  {
    getCmEPID_output_7().setStatus(BStatus.disabled);
    getCmEPID_output_8().setStatus(BStatus.disabled);
  }

  public void setOutputStatusOk()
  {
    getCmEPID_output_7().setStatus(BStatus.ok);
    getCmEPID_output_8().setStatus(BStatus.ok);
  }

  void Execute( )
  {
    // cycle time calculation
    long ticks = ControlFunction.GetSystemTicks();
    lastCycleTime = (float)(ticks - lastSystemTicks) / 1000.0F;
    lastSystemTicks = ticks;

    // [19] Function call CfSubtract
    func_19.input[0] = (float) getCmEPID_input_1().getValue();
    func_19.input[1] = (float) getCmEPID_input_2().getValue();
    func_19.Execute(  );

    // [22] Function call CfIdt
    func_22.input = func_97.output[0];
    func_22.Execute(  );

    // [26] Function call CfIdt
    func_26.input = (float) getCmEPID_parameter_12();
    func_26.Execute(  );
    setParameter(0, func_26.output[0]);

    // [27] Function call CfDifferential
    func_27.input = func_19.output;
    func_27.Execute( this, 0 );

    // [28] Function call CfDivide
    func_28.inputA = 100F;
    func_28.inputB = (float) getCmEPID_parameter_10();
    func_28.Execute(  );

    // [30] Function call CfSubtract
    func_30.input[0] = 100F;
    func_30.input[1] = func_22.output[0];
    func_30.Execute(  );

    // [34] Function call CfMultiply
    func_34.input[0] = func_27.output;
    func_34.input[1] = func_28.output;
    func_34.Execute(  );

    // [35] Function call CfMultiply
    func_35.input[0] = func_19.output;
    func_35.input[1] = func_28.output;
    func_35.Execute(  );

    // [36] Function call CfMultiply
    func_36.input[0] = func_95.output[0];
    func_36.input[1] = (float) getCmEPID_parameter_11();
    func_36.Execute(  );

    // [37] Function call CfInputMultiplexer
    func_37.inputSelect = FloatToInt((float) getCmEPID_parameter_9());
    func_37.input[1-1] = func_22.output[0];
    func_37.input[2-1] = func_30.output;
    func_37.Execute(  );

    // [38] Function call CfAddition
    func_38.input[0] = register_34;
    func_38.input[1] = func_40.output[0];
    func_38.input[2] = 50F;
    func_38.Execute(  );

    // [40] Function call CfIdt
    func_40.input = func_34.output;
    func_40.Execute(  );

    // [41] Function call CfIdt
    func_41.input = func_35.output;
    func_41.Execute(  );
    register_34 = func_41.output[0];

    // [43] Function call CfCompare1
    func_43.inputA = func_36.output;
    func_43.inputB = 0F;
    func_43.Execute(  );

    // [45] Function call CfDivide
    func_45.inputA = func_35.output;
    func_45.inputB = func_36.output;
    func_45.Execute(  );

    // [46] Function call CfSubtract
    func_46.input[0] = func_37.output;
    func_46.input[1] = func_38.output;
    func_46.Execute(  );

    // [49] Function call CfAddition
    func_49.input[0] = func_45.output;
    func_49.input[1] = func_46.output;
    func_49.Execute(  );

    // [51] Function call CfMinimum
    func_51.input[0] = register_34;
    func_51.input[1] = (float) getCmEPID_parameter_18();
    func_51.Execute(  );

    // [52] Function call CfNegate
    func_52.input = (float) getCmEPID_parameter_18();
    func_52.Execute(  );

    // [55] Function call CfMaximum
    func_55.input[0] = func_51.output;
    func_55.input[1] = func_52.output;
    func_55.Execute(  );

    // [181] Function call CfInputMultiplexer
    func_181.inputSelect = BooleanToInt(func_43.outputEqual);
    func_181.input[1-1] = func_55.output;
    func_181.input[2-1] = register_34;
    func_181.Execute(  );

    // [186] Function call CfIdt
    func_186.input = func_181.output;
    func_186.Execute(  );
    register_34 = func_186.output[0];

    // [178] Function call CfInputMultiplexer
    func_178.inputSelect = BooleanToInt(func_43.outputEqual);
    func_178.input[1-1] = func_49.output;
    func_178.input[2-1] = 0F;
    func_178.Execute(  );

    // [65] Function call CfAddition
    func_65.input[0] = func_181.output;
    func_65.input[1] = func_178.output;
    func_65.input[2] = func_40.output[0];
    func_65.input[3] = 50F;
    func_65.Execute(  );

    // [66] Function call CfIdt
    func_66.input = (float) getCmEPID_input_4().getValue();
    func_66.Execute(  );
    setParameter(1, func_66.output[0]);

    // [67] Function call CfIntegral
    func_67.inputX = 1F;
    func_67.inputMax = BooleanToFloat(getCmEPID_input_3().getValue());
    func_67.Execute( this, 1 );

    // [68] Function call CfIdt
    func_68.input = func_82.output[0];
    func_68.Execute(  );

    // [70] Function call CfSubtract
    func_70.input[0] = 100F;
    func_70.input[1] = func_65.output;
    func_70.Execute(  );

    // [71] Function call CfIdt
    func_71.input = (float) getCmEPID_input_6().getValue();
    func_71.Execute(  );

    // [72] Function call CfIdt
    func_72.input = (float) getCmEPID_input_5().getValue();
    func_72.Execute(  );

    // [73] Function call CfSubtract
    func_73.input[0] = 1F;
    func_73.input[1] = func_67.output;
    func_73.Execute(  );

    // [74] Function call CfMultiply
    func_74.input[0] = 100F;
    func_74.input[1] = func_67.output;
    func_74.Execute(  );

    // [75] Function call CfSubtract
    func_75.input[0] = 1F;
    func_75.input[1] = func_68.output[0];
    func_75.Execute(  );

    // [77] Function call CfInputMultiplexer
    func_77.inputSelect = FloatToInt((float) getCmEPID_parameter_9());
    func_77.input[1-1] = func_65.output;
    func_77.input[2-1] = func_70.output;
    func_77.Execute(  );

    // [78] Function call CfInputMultiplexer
    func_78.inputSelect = FloatToInt((float) getCmEPID_parameter_16());
    func_78.input[1-1] = (float) getCmEPID_parameter_13();
    func_78.input[2-1] = func_71.output[0];
    func_78.Execute(  );

    // [79] Function call CfMaximum
    func_79.input[0] = func_71.output[0];
    func_79.input[1] = (float) getCmEPID_parameter_13();
    func_79.Execute(  );

    // [80] Function call CfNegate
    func_80.input = (float) getCmEPID_parameter_16();
    func_80.Execute(  );

    // [81] Function call CfMultiply
    func_81.input[0] = func_72.output[0];
    func_81.input[1] = func_73.output;
    func_81.Execute(  );

    // [82] Function call CfIdt
    func_82.input = func_75.output;
    func_82.Execute(  );

    // [83] Function call CfIdt
    func_83.input = 1F;
    func_83.Execute(  );
    setParameter(2, func_83.output[0]);

    // [84] Function call CfDifferential
    func_84.input = func_75.output;
    func_84.Execute( this, 2 );

    // [87] Function call CfMaximum
    func_87.input[0] = func_77.output;
    func_87.input[1] = func_78.output;
    func_87.input[2] = (float) getCmEPID_parameter_13();
    func_87.input[3] = func_81.output;
    func_87.Execute(  );

    // [88] Function call CfInputMultiplexer
    func_88.inputSelect = FloatToInt(func_80.output);
    func_88.input[1-1] = (float) getCmEPID_parameter_14();
    func_88.input[2-1] = func_79.output;
    func_88.Execute(  );

    // [89] Function call CfAddition
    func_89.input[0] = func_81.output;
    func_89.input[1] = func_74.output;
    func_89.Execute(  );

    // [90] Function call CfIdt
    func_90.input = func_74.output;
    func_90.Execute(  );
    setCmEPID_output_8(func_90.output[0]);

    // [91] Function call CfSubtract
    func_91.input[0] = func_84.output;
    func_91.input[1] = 0F;
    func_91.Execute(  );

    // [92] Function call CfAbsolute
    func_92.input = func_91.output;
    func_92.Execute(  );

    // [94] Function call CfMinimum
    func_94.input[0] = func_87.output;
    func_94.input[1] = func_88.output;
    func_94.input[2] = (float) getCmEPID_parameter_14();
    func_94.input[3] = func_89.output;
    func_94.Execute(  );

    // [95] Function call CfIdt
    func_95.input = func_92.outputAbs;
    func_95.Execute(  );

    // [97] Function call CfIdt
    func_97.input = func_94.output;
    func_97.Execute(  );

    // [99] Function call CfIdt
    func_99.input = func_94.output;
    func_99.Execute(  );
    setCmEPID_output_7(func_99.output[0]);

  }

  // parameter for functions and sub macros
  protected float [] parameterList = new float []
  {
    0F, 300F, 1F
  };

  // implementation of IEnvironment
  public float getParameter(int index)
  {
    return parameterList[index];
  };

  public void setParameter(int index, float value)
  {
    parameterList[index] = value;
  };

  // macro register which are written multiple times
  protected float register_34 = 0;
  // macro outputs
// Ctrl_Out
  public static final Property CmEPID_output_7 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmEPID_output_7() { return (BStatusNumeric) get(CmEPID_output_7); }
  public void setCmEPID_output_7(BStatusNumeric v) { set(CmEPID_output_7,v,null); }
  public void setCmEPID_output_7(float v) { getCmEPID_output_7().setValue(v); }
// Ramp
  public static final Property CmEPID_output_8 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmEPID_output_8() { return (BStatusNumeric) get(CmEPID_output_8); }
  public void setCmEPID_output_8(BStatusNumeric v) { set(CmEPID_output_8,v,null); }
  public void setCmEPID_output_8(float v) { getCmEPID_output_8().setValue(v); }

  // macro parameter
// P01 Rev_Acting_Enable: 0=x-w, 1=w-x
  public static final Property CmEPID_parameter_9 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmEPID_parameter_9() { return getDouble(CmEPID_parameter_9); }
  public void setCmEPID_parameter_9(double v) { setDouble(CmEPID_parameter_9,v,null); }
// P02 Proportional_Band
  public static final Property CmEPID_parameter_10 = newProperty( 0, BDouble.make( 30 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1000000)), BFacets.make(BFacets.MIN, BDouble.make(1)) ));
  public double getCmEPID_parameter_10() { return getDouble(CmEPID_parameter_10); }
  public void setCmEPID_parameter_10(double v) { setDouble(CmEPID_parameter_10,v,null); }
// P03 Integration_Time
  public static final Property CmEPID_parameter_11 = newProperty( 0, BDouble.make( 300 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(1000000)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmEPID_parameter_11() { return getDouble(CmEPID_parameter_11); }
  public void setCmEPID_parameter_11(double v) { setDouble(CmEPID_parameter_11,v,null); }
// P04 Derivative_Time
  public static final Property CmEPID_parameter_12 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(1000000)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmEPID_parameter_12() { return getDouble(CmEPID_parameter_12); }
  public void setCmEPID_parameter_12(double v) { setDouble(CmEPID_parameter_12,v,null); }
// P05 Output_Minimum
  public static final Property CmEPID_parameter_13 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-25)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmEPID_parameter_13() { return getDouble(CmEPID_parameter_13); }
  public void setCmEPID_parameter_13(double v) { setDouble(CmEPID_parameter_13,v,null); }
// P06 Output_Maximum
  public static final Property CmEPID_parameter_14 = newProperty( 0, BDouble.make( 100 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(125)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmEPID_parameter_14() { return getDouble(CmEPID_parameter_14); }
  public void setCmEPID_parameter_14(double v) { setDouble(CmEPID_parameter_14,v,null); }
// P08 Aux_Input_Mode
  public static final Property CmEPID_parameter_16 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(-1)) ));
  public double getCmEPID_parameter_16() { return getDouble(CmEPID_parameter_16); }
  public void setCmEPID_parameter_16(double v) { setDouble(CmEPID_parameter_16,v,null); }
// P10 Prop_Limit
  public static final Property CmEPID_parameter_18 = newProperty( 0, BDouble.make( 50 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(1000000)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmEPID_parameter_18() { return getDouble(CmEPID_parameter_18); }
  public void setCmEPID_parameter_18(double v) { setDouble(CmEPID_parameter_18,v,null); }

  // macro inputs
// TempInput
  public static final Property CmEPID_input_1 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), null);
  public BStatusNumeric getCmEPID_input_1() { return (BStatusNumeric) get(CmEPID_input_1); }
  public void setCmEPID_input_1(BStatusNumeric v) { set(CmEPID_input_1,v,null); }
  public void setCmEPID_input_1(float v) { getCmEPID_input_1().setValue(v); }
// Setpoint
  public static final Property CmEPID_input_2 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), null);
  public BStatusNumeric getCmEPID_input_2() { return (BStatusNumeric) get(CmEPID_input_2); }
  public void setCmEPID_input_2(BStatusNumeric v) { set(CmEPID_input_2,v,null); }
  public void setCmEPID_input_2(float v) { getCmEPID_input_2().setValue(v); }
// Enable
  public static final Property CmEPID_input_3 = newProperty( Flags.SUMMARY, new BStatusBoolean( true ), null);
  public BStatusBoolean getCmEPID_input_3() { return (BStatusBoolean) get(CmEPID_input_3); }
  public void setCmEPID_input_3(BStatusBoolean v) { set(CmEPID_input_3,v,null); }
  public void setCmEPID_input_3(boolean v) { getCmEPID_input_3().setValue(v); }
// Start ramp time
  public static final Property CmEPID_input_4 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), BFacets.make(BFacets.UNITS, BUnit.getUnit("second")));
  public BStatusNumeric getCmEPID_input_4() { return (BStatusNumeric) get(CmEPID_input_4); }
  public void setCmEPID_input_4(BStatusNumeric v) { set(CmEPID_input_4,v,null); }
  public void setCmEPID_input_4(float v) { getCmEPID_input_4().setValue(v); }
// Output start value
  public static final Property CmEPID_input_5 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmEPID_input_5() { return (BStatusNumeric) get(CmEPID_input_5); }
  public void setCmEPID_input_5(BStatusNumeric v) { set(CmEPID_input_5,v,null); }
  public void setCmEPID_input_5(float v) { getCmEPID_input_5().setValue(v); }
// Auxiliary
  public static final Property CmEPID_input_6 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), null);
  public BStatusNumeric getCmEPID_input_6() { return (BStatusNumeric) get(CmEPID_input_6); }
  public void setCmEPID_input_6(BStatusNumeric v) { set(CmEPID_input_6,v,null); }
  public void setCmEPID_input_6(float v) { getCmEPID_input_6().setValue(v); }

  public static final Property enabledIn = newProperty( 0, new BStatusBoolean( true ), null);
  public BStatusBoolean getEnabledIn() { return (BStatusBoolean) get(enabledIn); }
  public void setEnabledIn(BStatusBoolean v) { set(enabledIn,v,null); }
  public void setEnabledIn(boolean v) { getEnabledIn().setValue(v); }


  public static final Property versionInfo = newProperty( Flags.READONLY, BString.make( "51.01 [12] 16.03.2018-16:13").getString(), null);
  public String getVersionInfo() { return getString(versionInfo); }
  public void setVersionInfo(String v) { setString(versionInfo,v,null); }


  public static final Property macroId = newProperty( Flags.READONLY|Flags.HIDDEN, BString.make( "6533553453234389458").getString(), null);
  public String getMacroId() { return getString(macroId); }
  public void setMacroId(String v) { setString(macroId,v,null); }

  // TYPE declaration
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCmEPID.class);


  // cycle time
  protected long lastSystemTicks = 0;
  protected float lastCycleTime = 0;

  // implementation of IEnvironment interface
  public float getCycleTime()
  { 
    return lastCycleTime;
  }

  public void started() throws Exception
  {
    super.started();
    Start();
    lastSystemTicks = ControlFunction.GetSystemTicks();

  }

  public int getHighestPriority(int inputId)
  {
    return 0;
  }

  public float getPriorityValue(int inputId, int priorityIndex)
  {
    return Float.NaN;
  }

  public void setPriorityValue(int outputId, int priorityIndex, float value, boolean setNull)
  {
  }

  // helper functions
  private float   BooleanToFloat(boolean value) { return value ? 1 : 0; }
  private int     BooleanToInt  (boolean value) { return value ? 1 : 0; }
  private int     FloatToInt    (float   value) { return (int) value;   }

  // class variables
  protected CfSubtract func_19 = new CfSubtract();
  protected CfIdt func_22 = new CfIdt();
  protected CfIdt func_26 = new CfIdt();
  protected CfDifferential func_27 = new CfDifferential();
  protected CfDivide func_28 = new CfDivide();
  protected CfSubtract func_30 = new CfSubtract();
  protected CfMultiply func_34 = new CfMultiply();
  protected CfMultiply func_35 = new CfMultiply();
  protected CfMultiply func_36 = new CfMultiply();
  protected CfInputMultiplexer func_37 = new CfInputMultiplexer();
  protected CfAddition func_38 = new CfAddition();
  protected CfIdt func_40 = new CfIdt();
  protected CfIdt func_41 = new CfIdt();
  protected CfCompare1 func_43 = new CfCompare1();
  protected CfDivide func_45 = new CfDivide();
  protected CfSubtract func_46 = new CfSubtract();
  protected CfAddition func_49 = new CfAddition();
  protected CfMinimum func_51 = new CfMinimum();
  protected CfNegate func_52 = new CfNegate();
  protected CfMaximum func_55 = new CfMaximum();
  protected CfInputMultiplexer func_181 = new CfInputMultiplexer();
  protected CfIdt func_186 = new CfIdt();
  protected CfInputMultiplexer func_178 = new CfInputMultiplexer();
  protected CfAddition func_65 = new CfAddition();
  protected CfIdt func_66 = new CfIdt();
  protected CfIntegral func_67 = new CfIntegral();
  protected CfIdt func_68 = new CfIdt();
  protected CfSubtract func_70 = new CfSubtract();
  protected CfIdt func_71 = new CfIdt();
  protected CfIdt func_72 = new CfIdt();
  protected CfSubtract func_73 = new CfSubtract();
  protected CfMultiply func_74 = new CfMultiply();
  protected CfSubtract func_75 = new CfSubtract();
  protected CfInputMultiplexer func_77 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_78 = new CfInputMultiplexer();
  protected CfMaximum func_79 = new CfMaximum();
  protected CfNegate func_80 = new CfNegate();
  protected CfMultiply func_81 = new CfMultiply();
  protected CfIdt func_82 = new CfIdt();
  protected CfIdt func_83 = new CfIdt();
  protected CfDifferential func_84 = new CfDifferential();
  protected CfMaximum func_87 = new CfMaximum();
  protected CfInputMultiplexer func_88 = new CfInputMultiplexer();
  protected CfAddition func_89 = new CfAddition();
  protected CfIdt func_90 = new CfIdt();
  protected CfSubtract func_91 = new CfSubtract();
  protected CfAbsolute func_92 = new CfAbsolute();
  protected CfMinimum func_94 = new CfMinimum();
  protected CfIdt func_95 = new CfIdt();
  protected CfIdt func_97 = new CfIdt();
  protected CfIdt func_99 = new CfIdt();

  // default constructor
  public BCmEPID()
  {
    func_38.configure(3);
    func_65.configure(4);
    func_87.configure(4);
    func_94.configure(4);
  }

  public void Start()
  {
    func_27.Start();
    func_67.Start();
    func_84.Start();
  }
}
