/*******************************************************************************
Automatically generated file
Created by 'Control Macro Library Conversion Tool'
Creation time: 03.12.2020 12:02
Copyright (C) Honeywell GmbH D-71101 Schoenaich 2020
*******************************************************************************/

package cl.hvac.general;

import javax.baja.status.*;
import javax.baja.sys.*;
import javax.baja.units.*;
import cl.hvac.*;
import cl.hvac.base.*;

/*==============================================================================
Macro name:    FLOAT_OnboardIOs
Macro id:      6344912960254640139
Macro version: 7
Last change:   03.01.2017 - 12:19
Access Level:  2
Inputs:
  [1] Valve
  [2] SyncOpen
  [3] SyncClose
  [4] Enable
  [5] FeedbackPosition
Outputs:
  [1] RelayOpen
  [2] RelayClose
  [3] MCD3
  [4] ActualPosition
  [5] SyncMode
Parameter:
  [1] FLT-01: Valve runtime
  [2] FLT-02: Direction change hysteresis
  [3] FLT-03: EndstopMode 0=no, 1=permanent, 2=cyclic
  [4] FLT-04: Resolution
  [5] FLT-05: PVE enabled
  [6] FLT-06: Valve minimum runtime
  [7] FLT-07: Over-travel Close Position
  [8] FTL-08: Over-travel Open Position
  [9] SNY-01: SyncStartup  0=close 1=open 2=no
  [10] SNY-02: Sync interval enable (0=disabled, 1 = enabled)
  [11] SNY-03: Sync interval
  [12] SNY-04: Sync interval position (0=close 1=open)
  [13] PEX-01: exercise period
  [14] PEX-02: exercise time
  [15] FED-01: Feedback signal at 0%
  [16] FED-02: Feedback signal at 100%
Description:
This macro generates a floating output for on-board BOs or an AO with an MCD3 module. 
It contains a valve exercise macro. 

A floating actuator needs a valve synchronization run. The synchronization drives the valve actuator to closed or open positions for a time of 150% of the respective valve runtime (time to open FLT-01 or time to close FLT-02 ). The synchronization starts, when the internal motor model reaches the end position. The end positon is determined by the parameter SYN-03 and can be increased by higher RACL cycle times . If the desired position leaves the end position, the sync is interrupted.

Because the configured runtime could be lower than the possible valve runtime, the synchronization to the open direction is disabled by default. 

Direct switching from open to close direction and vice versa is not possible to protect the relays. There is always one RACL loop in between.

Movement suppression to minimize valve wear is not required, since the minimum 
movement is determined by the control loop cycle time of default 2000 msec. The 
minimum value can be configured with parameter FLT-04. 

With a cycle time of 2 seconds and runtime of 106 seconds a minimum resolution of 3% 
can be achieved. Fast actuators with smaller runtimes will reduce the resolution. It is 
recommended to use slow actuators. The actual value of the resolution is displayed in the register Reg_Delta. 

If Reg_Delta is bigger than 8% for more than 5 minutes, the output “SyncMode” shows 
5, and both relay outputs will stop.

Override functions:
Sync can be activated by: 
- Macro Input (open & close)
- At close and open position 
- At positions greater 105% (open) and smaller -5% (close)
- Valve exercise interval PEX (close) 
- In a configurable interval, see SYN-04 and SYN-05 (close/open)

Inputs: 
Valve            : Required input for -5 ...105% (default n/a)
Sync Open   : Forces sync to open position.  (default 0)?? 0 - disabled or start anyway?
Sync Close   : Forces sync to close position.  (default 0) ??  0 - disabled or start anyway?
Enable          : An input to stop operation (remains in current position) (default 1)
FeedbackPosition  : Instead of the internal model a physical feedback signal. 

For the NTK10 potentiometer use the IO Configuration:  “AI 0...10V with pull-up”

Outputs:
RelayOpen      : BO for motor open
RelayClose      : BO for motor close
MCD3              : AO as voltage level to drive the Honeywell MCD3 
ActualPosition  : Actual position of the motor model
SyncMode       : Multistate if Sync is active or not 
1 = No Sync
2 = Sync to close position
3 = Sync to open position
4 = Exercise Run
5 = Error

Parameters:
FLT-01: Valve runtime
FLT-02: Direction change hysteresis
FLT-03: End switch mode: 0 = no, 1 = permanent; 2 = cyclic (default 0)
FLT-04: Resolution
FLT-05: PVE enabled: (default 1)
FLT-06: Valve minimum runtime
FLT-07: Over-Travel Close Position
FLT-08: Over-Travel Open Position
SYN-01: Sync Startup: 0 = close; 1 = open; 2 = no
SYN-02: Sync interval enable: 0 = disabled; 1 = enabled
SYN-03: Sync interval
SYN-04: Sync interval to position: 0 = close; 1 = open
PEX-01: exercise period
PEX-02: exercise time
FED-01: Feedback signal at 0%
FED-02: Feedback signal at 100% 

Please use this macro only for on-board IOs. Due to different bus loads, the timing cannot be guaranteed for panel bus or Lon binary output modules. 

Note to MCD-3: 
Connect Open contact to terminal 4
Connect Close contact to terminal 5
================================================================================*/

public class BCmFLOAT_OnboardIOs
  extends BControlFunctionSupport
  implements IEnvironment
{
  public void doExecute()
  {
    if (getEnabledIn().getBoolean())
    {
      Execute( );
      if (getCmFLOAT_OnboardIOs_output_6().getStatus() == BStatus.disabled)
        setOutputStatusOk();
    }
    else
    {
      setOutputStatusDisabled();
    }
  }

  public void setOutputStatusDisabled()
  {
    getCmFLOAT_OnboardIOs_output_6().setStatus(BStatus.disabled);
    getCmFLOAT_OnboardIOs_output_7().setStatus(BStatus.disabled);
    getCmFLOAT_OnboardIOs_output_8().setStatus(BStatus.disabled);
    getCmFLOAT_OnboardIOs_output_13().setStatus(BStatus.disabled);
    getCmFLOAT_OnboardIOs_output_5().setStatus(BStatus.disabled);
  }

  public void setOutputStatusOk()
  {
    getCmFLOAT_OnboardIOs_output_6().setStatus(BStatus.ok);
    getCmFLOAT_OnboardIOs_output_7().setStatus(BStatus.ok);
    getCmFLOAT_OnboardIOs_output_8().setStatus(BStatus.ok);
    getCmFLOAT_OnboardIOs_output_13().setStatus(BStatus.ok);
    getCmFLOAT_OnboardIOs_output_5().setStatus(BStatus.ok);
  }

  void Execute( )
  {
    // cycle time calculation
    long ticks = ControlFunction.GetSystemTicks();
    lastCycleTime = (float)(ticks - lastSystemTicks) / 1000.0F;
    lastSystemTicks = ticks;

    // [463] Function call CfIdt
    func_463.input = BooleanToFloat(getCmFLOAT_OnboardIOs_output_6().getValue());
    func_463.Execute(  );
    register_8 = func_463.output[0];

    // [464] Function call CfIdt
    func_464.input = BooleanToFloat(getCmFLOAT_OnboardIOs_output_7().getValue());
    func_464.Execute(  );
    register_20 = func_464.output[0];

    // [581] Function call CfIdt
    func_581.input = (float) getCmFLOAT_OnboardIOs_input_1().getValue();
    func_581.Execute(  );

    // [285] Function call CfMultiply
    func_285.input[0] = 100F;
    func_285.input[1] = (float) getCmFLOAT_OnboardIOs_parameter_275();
    func_285.Execute(  );

    // [286] Function call CfDivide
    func_286.inputA = func_285.output;
    func_286.inputB = (float) getCmFLOAT_OnboardIOs_parameter_9();
    func_286.Execute(  );

    // [471] Function call CfHysteresis
    func_471.inputX = func_581.output[0];
    func_471.inputW = func_286.output;
    func_471.Execute( this, 0 );

    // [482] Function call CfCompare1 ("Disabled?")
    func_482.inputA = func_286.output;
    func_482.inputB = 0F;
    func_482.Execute(  );

    // [295] Macro call CmRising__Faling
    func_295.CmRising__Faling_input_1 = FloatToBoolean(func_581.output[0]);
    func_295.Execute(  );

    // [468] Function call CfAnd ("Start")
    func_468.input[0] = func_471.output2;
    func_468.input[1] = func_295.CmRising__Faling_output_3;
    func_468.input[2] = func_482.outputGreater;
    func_468.Execute(  );

    // [474] Function call CfAnd ("Selflock")
    func_474.input[0] = func_471.output2;
    func_474.input[1] = func_477.outputOR;
    func_474.input[2] = func_482.outputGreater;
    func_474.Execute(  );

    // [477] Function call CfOr
    func_477.input[0] = func_468.outputAND;
    func_477.input[1] = func_474.outputAND;
    func_477.Execute(  );

    // [309] Function call CfInputMultiplexer
    func_309.inputSelect = BooleanToInt(func_477.outputOR);
    func_309.input[1-1] = func_581.output[0];
    func_309.input[2-1] = func_286.output;
    func_309.Execute(  );

    // [332] Function call CfLimit
    func_332.input = func_309.output;
    func_332.Execute( this, 1 );

    // [30] Macro call CmPositionChange
    func_30.CmPositionChange_input_1 = (float) getCmFLOAT_OnboardIOs_parameter_9();
    func_30.CmPositionChange_input_10 = FloatToBoolean(register_8);
    func_30.Execute( this, 3 );

    // [32] Macro call CmPositionChange
    func_32.CmPositionChange_input_1 = (float) getCmFLOAT_OnboardIOs_parameter_9();
    func_32.CmPositionChange_input_10 = FloatToBoolean(register_20);
    func_32.Execute( this, 8 );

    // [550] Function call CfOutputMultiplexer
    func_550.inputSelect = FloatToInt(register_20);
    func_550.inputValue = 0F;
    func_550.Execute( this, 13 );
    if ( func_550.outputValid[1]) register_140 = func_550.output[1];

    // [553] Function call CfOutputMultiplexer
    func_553.inputSelect = FloatToInt(register_8);
    func_553.inputValue = 1F;
    func_553.Execute( this, 14 );
    if ( func_553.outputValid[1]) register_140 = func_553.output[1];

    // [583] Macro call CmRising__Faling
    func_583.CmRising__Faling_input_1 = FloatToBoolean(register_140);
    func_583.Execute(  );

    // [585] Function call CfInputMultiplexer
    func_585.inputSelect = BooleanToInt(func_583.CmRising__Faling_output_3);
    func_585.input[1-1] = 0F;
    func_585.input[2-1] = -1F;
    func_585.Execute(  );

    // [587] Function call CfInputMultiplexer
    func_587.inputSelect = BooleanToInt(func_583.CmRising__Faling_output_2);
    func_587.input[1-1] = func_585.output;
    func_587.input[2-1] = 1F;
    func_587.Execute(  );

    // [642] Function call CfDivide
    func_642.inputA = (float) getCmFLOAT_OnboardIOs_parameter_10();
    func_642.inputB = func_195.output;
    func_642.Execute(  );

    // [643] Function call CfMultiply
    func_643.input[0] = func_642.output;
    func_643.input[1] = getCycleTime();
    func_643.input[2] = 1.5F;
    func_643.Execute(  );
    setParameter(15, func_643.output);

    // [614] Function call CfOr
    func_614.input[0] = func_583.CmRising__Faling_output_3;
    func_614.input[1] = func_583.CmRising__Faling_output_2;
    func_614.Execute(  );

    // [615] Function call CfMonoflop
    func_615.input = BooleanToFloat(func_614.outputOR);
    func_615.Execute( this, 15 );

    // [635] Function call CfSubtract
    func_635.input[0] = 0F;
    func_635.input[1] = (float) getCmFLOAT_OnboardIOs_parameter_10();
    func_635.Execute(  );

    // [637] Function call CfAddition
    func_637.input[0] = 100F;
    func_637.input[1] = (float) getCmFLOAT_OnboardIOs_parameter_10();
    func_637.Execute(  );

    // [633] Function call CfInputMultiplexer
    func_633.inputSelect = BooleanToInt(func_615.output);
    func_633.input[1-1] = 0F;
    func_633.input[2-1] = func_635.output;
    func_633.Execute(  );
    setParameter(39, func_633.output);

    // [636] Function call CfInputMultiplexer
    func_636.inputSelect = BooleanToInt(func_615.output);
    func_636.input[1-1] = 100F;
    func_636.input[2-1] = func_637.output;
    func_636.Execute(  );
    setParameter(38, func_636.output);

    // [590] Function call CfMultiply
    func_590.input[0] = func_587.output;
    func_590.input[1] = (float) getCmFLOAT_OnboardIOs_parameter_10();
    func_590.Execute(  );

    // [23] Function call CfAddition
    func_23.input[0] = func_561.output;
    func_23.input[1] = func_30.CmPositionChange_output_2;
    func_23.input[2] = func_32.CmPositionChange_output_2;
    func_23.input[3] = func_590.output;
    func_23.Execute(  );

    // [555] Function call CfRamp
    func_555.input = (float) getCmFLOAT_OnboardIOs_input_554().getValue();
    func_555.Execute( this, 16 );

    // [557] Function call CfRamp
    func_557.input = func_555.output;
    func_557.Execute( this, 27 );

    // [649] Function call CfCompare1
    func_649.inputA = (float) getCmFLOAT_OnboardIOs_parameter_568();
    func_649.inputB = -1F;
    func_649.Execute(  );

    // [653] Function call CfInputMultiplexer
    func_653.inputSelect = BooleanToInt(func_649.outputEqual);
    func_653.input[1-1] = func_557.output;
    func_653.input[2-1] = (float) getCmFLOAT_OnboardIOs_input_554().getValue();
    func_653.Execute(  );

    // [570] Macro call CmIDT05
    func_570.CmIDT05_input_1 = (float) getCmFLOAT_OnboardIOs_parameter_568();
    func_570.CmIDT05_input_2 = (float) getCmFLOAT_OnboardIOs_parameter_569();
    func_570.CmIDT05_input_3 = 0F;
    func_570.CmIDT05_input_4 = 0F;
    func_570.CmIDT05_input_5 = 0F;
    func_570.Execute(  );
    setParameter(31, func_570.CmIDT05_output_21);
    setParameter(33, func_570.CmIDT05_output_22);

    // [218] Macro call CmIDT05
    func_218.CmIDT05_input_1 = (float) getCmFLOAT_OnboardIOs_parameter_224();
    func_218.CmIDT05_input_2 = (float) getCmFLOAT_OnboardIOs_parameter_225();
    func_218.CmIDT05_input_3 = (float) getCmFLOAT_OnboardIOs_parameter_230();
    func_218.CmIDT05_input_4 = 0F;
    func_218.CmIDT05_input_5 = 0F;
    func_218.Execute(  );
    setParameter(61, func_218.CmIDT05_output_21);
    setParameter(62, func_218.CmIDT05_output_22);
    setParameter(50, func_218.CmIDT05_output_23);

    // [133] Function call CfLimit
    func_133.input = func_23.output;
    func_133.Execute( this, 38 );

    // [673] Function call CfCompare1
    func_673.inputA = (float) getCmFLOAT_OnboardIOs_parameter_107();
    func_673.inputB = 2F;
    func_673.Execute(  );

    // [670] Function call CfAnd
    func_670.input[0] = func_673.outputEqual;
    func_670.input[1] = ControlFunction.GetStartupFlag();
    func_670.Execute(  );

    // [666] Function call CfInputMultiplexer
    func_666.inputSelect = BooleanToInt(func_670.outputAND);
    func_666.input[1-1] = func_133.output;
    func_666.input[2-1] = func_581.output[0];
    func_666.Execute(  );

    // [563] Function call CfCompare1
    func_563.inputA = (float) getCmFLOAT_OnboardIOs_input_554().getValue();
    func_563.inputB = 999F;
    func_563.Execute(  );

    // [561] Function call CfInputMultiplexer
    func_561.inputSelect = BooleanToInt(func_563.outputEqual);
    func_561.input[1-1] = func_653.output;
    func_561.input[2-1] = func_666.output;
    func_561.Execute(  );

    // [180] Function call CfSubtract
    func_180.input[0] = func_332.output;
    func_180.input[1] = func_561.output;
    func_180.Execute(  );

    // [185] Function call CfDivide
    func_185.inputA = (float) getCmFLOAT_OnboardIOs_parameter_27();
    func_185.inputB = 2F;
    func_185.Execute(  );

    // [191] Function call CfNegate
    func_191.input = func_185.output;
    func_191.Execute(  );

    // [195] Function call CfMaximum
    func_195.input[0] = func_185.output;
    func_195.input[1] = func_30.CmPositionChange_output_23;
    func_195.Execute(  );

    // [272] Function call CfMinimum
    func_272.input[0] = func_191.output;
    func_272.input[1] = func_32.CmPositionChange_output_23;
    func_272.Execute(  );

    // [184] Function call CfCompare1
    func_184.inputA = func_180.output;
    func_184.inputB = func_195.output;
    func_184.Execute(  );

    // [454] Function call CfNot
    func_454.input = FloatToBoolean(register_20);
    func_454.Execute(  );

    // [451] Function call CfAnd
    func_451.input[0] = func_184.outputGreater;
    func_451.input[1] = func_454.output;
    func_451.Execute(  );

    // [186] Function call CfCompare1
    func_186.inputA = func_180.output;
    func_186.inputB = func_272.output;
    func_186.Execute(  );

    // [456] Function call CfNot
    func_456.input = FloatToBoolean(register_8);
    func_456.Execute(  );

    // [457] Function call CfAnd
    func_457.input[0] = func_186.outputLess;
    func_457.input[1] = func_456.output;
    func_457.Execute(  );

    // [344] Function call CfNegate
    func_344.input = func_272.output;
    func_344.Execute(  );

    // [346] Function call CfMaximum
    func_346.input[0] = func_344.output;
    func_346.input[1] = func_195.output;
    func_346.Execute(  );

    // [352] Function call CfAddition
    func_352.input[0] = func_346.output;
    func_352.input[1] = 1F;
    func_352.Execute(  );

    // [353] Function call CfMaximum
    func_353.input[0] = func_352.output;
    func_353.input[1] = 2F;
    func_353.Execute(  );

    // [403] Function call CfCompare1
    func_403.inputA = func_353.output;
    func_403.inputB = 8F;
    func_403.Execute(  );

    // [408] Function call CfDelayOnOff
    func_408.input = func_403.outputGreater;
    func_408.Execute( this, 40 );

    // [142] Function call CfHysteresis
    func_142.inputX = func_561.output;
    func_142.inputW = func_353.output;
    func_142.Execute( this, 42 );

    // [336] Function call CfCompare2
    func_336.inputA = func_581.output[0];
    func_336.inputB = -5F;
    func_336.Execute(  );

    // [522] Function call CfMonoflop
    func_522.input = BooleanToFloat(func_336.outputEqualOrLess);
    func_522.Execute( this, 43 );

    // [516] Function call CfMonoflop
    func_516.input = BooleanToFloat(getCmFLOAT_OnboardIOs_input_78().getValue());
    func_516.Execute( this, 44 );

    // [413] Function call CfHysteresis
    func_413.inputX = func_581.output[0];
    func_413.inputW = func_353.output;
    func_413.Execute( this, 45 );

    // [77] Function call CfAnd
    func_77.input[0] = func_142.output2;
    func_77.input[1] = FloatToBoolean((float) getCmFLOAT_OnboardIOs_parameter_12());
    func_77.input[2] = func_411.output;
    func_77.input[3] = func_413.output2;
    func_77.Execute(  );

    // [390] Macro call CmRising__Faling
    func_390.CmRising__Faling_input_1 = ControlFunction.GetStartupFlag();
    func_390.Execute(  );

    // [662] Function call CfCompare1
    func_662.inputA = (float) getCmFLOAT_OnboardIOs_parameter_107();
    func_662.inputB = 0F;
    func_662.Execute(  );

    // [113] Function call CfAnd
    func_113.input[0] = func_662.outputEqual;
    func_113.input[1] = func_390.CmRising__Faling_output_2;
    func_113.Execute(  );

    // [366] Function call CfLastValue ("Sync after PVE")
    func_366.input = func_644.output;
    func_366.Execute(  );

    // [372] Function call CfCompare1
    func_372.inputA = func_366.outputLast;
    func_372.inputB = 4F;
    func_372.Execute(  );

    // [368] Function call CfCompare1
    func_368.inputA = func_366.outputCurrent;
    func_368.inputB = 1F;
    func_368.Execute(  );

    // [379] Function call CfAnd
    func_379.input[0] = func_368.outputEqual;
    func_379.input[1] = func_372.outputEqual;
    func_379.Execute(  );

    // [519] Function call CfMonoflop
    func_519.input = BooleanToFloat(getCmFLOAT_OnboardIOs_input_2().getValue());
    func_519.Execute( this, 46 );

    // [63] Function call CfSubtract
    func_63.input[0] = 100F;
    func_63.input[1] = func_353.output;
    func_63.Execute(  );

    // [139] Function call CfHysteresis
    func_139.inputX = func_561.output;
    func_139.inputW = func_63.output;
    func_139.Execute( this, 47 );

    // [529] Function call CfHysteresis
    func_529.inputX = func_581.output[0];
    func_529.inputW = func_63.output;
    func_529.Execute( this, 48 );

    // [528] Function call CfAnd
    func_528.input[0] = func_139.output1;
    func_528.input[1] = FloatToBoolean((float) getCmFLOAT_OnboardIOs_parameter_11());
    func_528.input[2] = func_529.output1;
    func_528.input[3] = func_411.output;
    func_528.Execute(  );

    // [335] Function call CfCompare2
    func_335.inputA = func_581.output[0];
    func_335.inputB = 105F;
    func_335.Execute(  );

    // [524] Function call CfMonoflop
    func_524.input = BooleanToFloat(func_335.outputEqualOrGreater);
    func_524.Execute( this, 49 );

    // [658] Function call CfCompare1
    func_658.inputA = (float) getCmFLOAT_OnboardIOs_parameter_107();
    func_658.inputB = 1F;
    func_658.Execute(  );

    // [116] Function call CfAnd
    func_116.input[0] = func_658.outputEqual;
    func_116.input[1] = func_390.CmRising__Faling_output_2;
    func_116.Execute(  );

    // [251] Function call CfOr
    func_251.input[0] = func_93.output;
    func_251.input[1] = func_92.output;
    func_251.Execute(  );

    // [445] Macro call CmPVE_Exercise
    func_445.CmPVE_Exercise_input_207 = BooleanToFloat(func_251.outputOR);
    func_445.Execute( this, 50 );

    // [447] Function call CfCompare1
    func_447.inputA = func_445.CmPVE_Exercise_output_205;
    func_447.inputB = 100F;
    func_447.Execute(  );

    // [446] Function call CfAnd
    func_446.input[0] = func_447.outputEqual;
    func_446.input[1] = FloatToBoolean((float) getCmFLOAT_OnboardIOs_parameter_228());
    func_446.Execute(  );

    // [324] Function call CfInputMultiplexer
    func_324.inputSelect = FloatToInt((float) getCmFLOAT_OnboardIOs_parameter_321());
    func_324.input[1-1] = BooleanToFloat(func_446.outputAND);
    func_324.input[2-1] = 0F;
    func_324.Execute(  );

    // [322] Function call CfInputMultiplexer
    func_322.inputSelect = FloatToInt((float) getCmFLOAT_OnboardIOs_parameter_321());
    func_322.input[1-1] = 0F;
    func_322.input[2-1] = BooleanToFloat(func_446.outputAND);
    func_322.Execute(  );

    // [82] Function call CfOr
    func_82.input[0] = func_516.output;
    func_82.input[1] = func_522.output;
    func_82.input[2] = func_77.outputAND;
    func_82.input[3] = func_113.outputAND;
    func_82.input[4] = func_379.outputAND;
    func_82.input[5] = FloatToBoolean(func_324.output);
    func_82.Execute(  );

    // [395] Function call CfInputMultiplexer
    func_395.inputSelect = BooleanToInt(ControlFunction.GetStartupFlag());
    func_395.input[1-1] = BooleanToFloat(func_82.outputOR);
    func_395.input[2-1] = 0F;
    func_395.Execute(  );

    // [94] Function call CfMultiply
    func_94.input[0] = (float) getCmFLOAT_OnboardIOs_parameter_9();
    func_94.input[1] = 1.5F;
    func_94.Execute(  );

    // [441] Function call CfLastValue
    func_441.input = BooleanToFloat(func_77.outputAND);
    func_441.Execute(  );

    // [436] Function call CfNot
    func_436.input = func_413.output2;
    func_436.Execute(  );

    // [438] Function call CfAnd
    func_438.input[0] = func_436.output;
    func_438.input[1] = FloatToBoolean(func_441.outputLast);
    func_438.input[2] = func_93.output;
    func_438.Execute(  );

    // [415] Function call CfInputMultiplexer
    func_415.inputSelect = BooleanToInt(func_438.outputAND);
    func_415.input[1-1] = func_94.output;
    func_415.input[2-1] = 0F;
    func_415.Execute(  );
    setParameter(59, func_415.output);

    // [93] Function call CfMonoflop
    func_93.input = func_395.output;
    func_93.Execute( this, 59 );

    // [79] Function call CfOr
    func_79.input[0] = FloatToBoolean(func_322.output);
    func_79.input[1] = func_519.output;
    func_79.input[2] = func_524.output;
    func_79.input[3] = func_528.outputAND;
    func_79.input[4] = func_116.outputAND;
    func_79.Execute(  );

    // [114] Function call CfInputMultiplexer
    func_114.inputSelect = BooleanToInt(func_93.output);
    func_114.input[1-1] = BooleanToFloat(func_79.outputOR);
    func_114.input[2-1] = 0F;
    func_114.Execute(  );

    // [394] Function call CfInputMultiplexer
    func_394.inputSelect = BooleanToInt(ControlFunction.GetStartupFlag());
    func_394.input[1-1] = func_114.output;
    func_394.input[2-1] = 0F;
    func_394.Execute(  );

    // [537] Function call CfNot
    func_537.input = func_529.output1;
    func_537.Execute(  );

    // [536] Function call CfLastValue
    func_536.input = BooleanToFloat(func_528.outputAND);
    func_536.Execute(  );

    // [539] Function call CfAnd
    func_539.input[0] = func_537.output;
    func_539.input[1] = FloatToBoolean(func_536.outputLast);
    func_539.input[2] = func_92.output;
    func_539.Execute(  );

    // [98] Function call CfMultiply
    func_98.input[0] = (float) getCmFLOAT_OnboardIOs_parameter_9();
    func_98.input[1] = 1.5F;
    func_98.Execute(  );

    // [538] Function call CfInputMultiplexer
    func_538.inputSelect = BooleanToInt(func_539.outputAND);
    func_538.input[1-1] = func_98.output;
    func_538.input[2-1] = 0F;
    func_538.Execute(  );
    setParameter(60, func_538.output);

    // [92] Function call CfMonoflop
    func_92.input = func_394.output;
    func_92.Execute( this, 60 );

    // [15] Function call CfIdt
    func_15.input = func_561.output;
    func_15.Execute(  );
    setCmFLOAT_OnboardIOs_output_13(func_15.output[0]);

    // [244] Function call CfInputMultiplexer
    func_244.inputSelect = BooleanToInt(func_92.output);
    func_244.input[1-1] = 1F;
    func_244.input[2-1] = 3F;
    func_244.Execute(  );

    // [247] Function call CfInputMultiplexer
    func_247.inputSelect = BooleanToInt(func_93.output);
    func_247.input[1-1] = func_244.output;
    func_247.input[2-1] = 2F;
    func_247.Execute(  );

    // [361] Function call CfOr
    func_361.input[0] = func_451.outputAND;
    func_361.input[1] = func_92.output;
    func_361.input[2] = FloatToBoolean((float) getCmFLOAT_OnboardIOs_input_1().getValue());
    func_361.Execute(  );

    // [198] Macro call CmPVE_Exercise
    func_198.CmPVE_Exercise_input_207 = BooleanToFloat(func_361.outputOR);
    func_198.Execute( this, 61 );

    // [387] Function call CfCompare1
    func_387.inputA = func_198.CmPVE_Exercise_output_205;
    func_387.inputB = 100F;
    func_387.Execute(  );

    // [340] Function call CfAnd
    func_340.input[0] = func_387.outputEqual;
    func_340.input[1] = FloatToBoolean((float) getCmFLOAT_OnboardIOs_parameter_201());
    func_340.Execute(  );

    // [202] Function call CfInputMultiplexer
    func_202.inputSelect = BooleanToInt(func_340.outputAND);
    func_202.input[1-1] = func_247.output;
    func_202.input[2-1] = 4F;
    func_202.Execute(  );

    // [401] Function call CfInputMultiplexer
    func_401.inputSelect = BooleanToInt(func_408.output);
    func_401.input[1-1] = func_202.output;
    func_401.input[2-1] = 5F;
    func_401.Execute(  );

    // [610] Function call CfInputMultiplexer
    func_610.inputSelect = BooleanToInt(ControlFunction.GetStartupFlag());
    func_610.input[1-1] = func_401.output;
    func_610.input[2-1] = 6F;
    func_610.Execute(  );

    // [646] Function call CfNot
    func_646.input = getCmFLOAT_OnboardIOs_input_4().getValue();
    func_646.Execute(  );

    // [644] Function call CfInputMultiplexer
    func_644.inputSelect = BooleanToInt(func_646.output);
    func_644.input[1-1] = func_610.output;
    func_644.input[2-1] = 7F;
    func_644.Execute(  );

    // [52] Function call CfIdt
    func_52.input = func_644.output;
    func_52.Execute(  );
    setCmFLOAT_OnboardIOs_output_5(func_52.output[0]);

    // [269] Function call CfInputMultiplexer
    func_269.inputSelect = FloatToInt(func_644.output);
    func_269.input[1-1] = BooleanToFloat(func_451.outputAND);
    func_269.input[2-1] = BooleanToFloat(func_451.outputAND);
    func_269.input[3-1] = 0F;
    func_269.input[4-1] = 1F;
    func_269.input[5-1] = 1F;
    func_269.input[6-1] = BooleanToFloat(func_451.outputAND);
    func_269.input[7-1] = 0F;
    func_269.input[8-1] = 0F;
    func_269.Execute(  );
    register_8 = func_269.output;

    // [270] Function call CfInputMultiplexer
    func_270.inputSelect = FloatToInt(func_644.output);
    func_270.input[1-1] = BooleanToFloat(func_457.outputAND);
    func_270.input[2-1] = BooleanToFloat(func_457.outputAND);
    func_270.input[3-1] = 1F;
    func_270.input[4-1] = 0F;
    func_270.input[5-1] = 0F;
    func_270.input[6-1] = BooleanToFloat(func_457.outputAND);
    func_270.input[7-1] = 0F;
    func_270.input[8-1] = 0F;
    func_270.Execute(  );
    register_20 = func_270.output;

    // [426] Function call CfDivide
    func_426.inputA = (float) getCmFLOAT_OnboardIOs_parameter_9();
    func_426.inputB = 3F;
    func_426.Execute(  );

    // [430] Function call CfSubtract
    func_430.input[0] = 1800F;
    func_430.input[1] = func_426.output;
    func_430.Execute(  );
    setParameter(71, func_430.output);

    // [428] Function call CfIdt
    func_428.input = func_426.output;
    func_428.Execute(  );
    setParameter(70, func_428.output[0]);

    // [612] Function call CfInputMultiplexer
    func_612.inputSelect = FloatToInt(func_644.output);
    func_612.input[1-1] = 0F;
    func_612.input[2-1] = 1F;
    func_612.input[3-1] = 0F;
    func_612.input[4-1] = 0F;
    func_612.input[5-1] = 0F;
    func_612.input[6-1] = 1F;
    func_612.input[7-1] = 0F;
    func_612.Execute(  );

    // [500] Function call CfAnd
    func_500.input[0] = FloatToBoolean(func_612.output);
    func_500.input[1] = func_142.output2;
    func_500.input[2] = func_413.output2;
    func_500.Execute(  );

    // [424] Function call CfCycle
    func_424.input = true;
    func_424.Execute( this, 70 );

    // [432] Function call CfInputMultiplexer
    func_432.inputSelect = BooleanToInt(func_500.outputAND);
    func_432.input[1-1] = 0F;
    func_432.input[2-1] = (float) getCmFLOAT_OnboardIOs_parameter_130();
    func_432.Execute(  );

    // [146] Function call CfInputMultiplexer
    func_146.inputSelect = FloatToInt(func_432.output);
    func_146.input[1-1] = register_20;
    func_146.input[2-1] = 1F;
    func_146.input[3-1] = BooleanToFloat(func_424.output);
    func_146.Execute(  );
    setCmFLOAT_OnboardIOs_output_7(FloatToBoolean(func_146.output));

    // [611] Function call CfInputMultiplexer
    func_611.inputSelect = FloatToInt(func_644.output);
    func_611.input[1-1] = 0F;
    func_611.input[2-1] = 1F;
    func_611.input[3-1] = 0F;
    func_611.input[4-1] = 0F;
    func_611.input[5-1] = 0F;
    func_611.input[6-1] = 1F;
    func_611.input[7-1] = 0F;
    func_611.Execute(  );

    // [506] Function call CfAnd
    func_506.input[0] = FloatToBoolean(func_611.output);
    func_506.input[1] = func_139.output1;
    func_506.input[2] = func_529.output1;
    func_506.Execute(  );

    // [431] Function call CfInputMultiplexer
    func_431.inputSelect = BooleanToInt(func_506.outputAND);
    func_431.input[1-1] = 0F;
    func_431.input[2-1] = (float) getCmFLOAT_OnboardIOs_parameter_130();
    func_431.Execute(  );

    // [154] Function call CfInputMultiplexer
    func_154.inputSelect = FloatToInt(func_431.output);
    func_154.input[1-1] = register_8;
    func_154.input[2-1] = 1F;
    func_154.input[3-1] = BooleanToFloat(func_424.output);
    func_154.Execute(  );

    // [466] Function call CfInputMultiplexer
    func_466.inputSelect = BooleanToInt(getCmFLOAT_OnboardIOs_output_7().getValue());
    func_466.input[1-1] = func_154.output;
    func_466.input[2-1] = 0F;
    func_466.Execute(  );
    setCmFLOAT_OnboardIOs_output_6(FloatToBoolean(func_466.output));

    // [156] Function call CfInputMultiplexer ("Open")
    func_156.inputSelect = BooleanToInt(getCmFLOAT_OnboardIOs_output_6().getValue());
    func_156.input[1-1] = 65F;
    func_156.input[2-1] = 100F;
    func_156.Execute(  );

    // [158] Function call CfInputMultiplexer ("Close")
    func_158.inputSelect = BooleanToInt(getCmFLOAT_OnboardIOs_output_7().getValue());
    func_158.input[1-1] = func_156.output;
    func_158.input[2-1] = 35F;
    func_158.Execute(  );
    setCmFLOAT_OnboardIOs_output_8(func_158.output);

    // [411] Function call CfNot
    func_411.input = ControlFunction.GetStartupFlag();
    func_411.Execute(  );

  }

  // parameter for functions and sub macros
  protected float [] parameterList = new float []
  {
    0F, 100F, 0F, 0F, 1F, 1E+07F, 2F, 0F,
    1F, 1F, 1E+07F, 2F, 0F, 0F, 0F, 1F,
    0F, 100F, 0F, 100F, 0F, 0F, 99F, 50F,
    50F, 100F, 100F, 0F, 100F, 0F, 100F, 20F,
    0F, 80F, 99F, 100F, 100F, 100F, 100F, 0F,
    300F, 0F, 1F, 1F, 1F, 1F, 1F, 1F,
    0F, 1F, 168F, 120F, 0F, 0F, 168F, 120F,
    10F, 0F, 1F, 1F, 1F, 168F, 120F, 0F,
    0F, 168F, 120F, 10F, 0F, 1F, 20F, 0F,
  };

  // implementation of IEnvironment
  public float getParameter(int index)
  {
    return parameterList[index];
  };

  public void setParameter(int index, float value)
  {
    parameterList[index] = value;
  };

  // macro register which are written multiple times
  protected float register_8 = 0;
  protected float register_20 = 0;
  protected float register_140 = 0;
  // macro outputs
// RelayOpen
  public static final Property CmFLOAT_OnboardIOs_output_6 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmFLOAT_OnboardIOs_output_6() { return (BStatusBoolean) get(CmFLOAT_OnboardIOs_output_6); }
  public void setCmFLOAT_OnboardIOs_output_6(BStatusBoolean v) { set(CmFLOAT_OnboardIOs_output_6,v,null); }
  public void setCmFLOAT_OnboardIOs_output_6(boolean v) { getCmFLOAT_OnboardIOs_output_6().setValue(v); }
// RelayClose
  public static final Property CmFLOAT_OnboardIOs_output_7 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmFLOAT_OnboardIOs_output_7() { return (BStatusBoolean) get(CmFLOAT_OnboardIOs_output_7); }
  public void setCmFLOAT_OnboardIOs_output_7(BStatusBoolean v) { set(CmFLOAT_OnboardIOs_output_7,v,null); }
  public void setCmFLOAT_OnboardIOs_output_7(boolean v) { getCmFLOAT_OnboardIOs_output_7().setValue(v); }
// MCD3
  public static final Property CmFLOAT_OnboardIOs_output_8 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmFLOAT_OnboardIOs_output_8() { return (BStatusNumeric) get(CmFLOAT_OnboardIOs_output_8); }
  public void setCmFLOAT_OnboardIOs_output_8(BStatusNumeric v) { set(CmFLOAT_OnboardIOs_output_8,v,null); }
  public void setCmFLOAT_OnboardIOs_output_8(float v) { getCmFLOAT_OnboardIOs_output_8().setValue(v); }
// ActualPosition
  public static final Property CmFLOAT_OnboardIOs_output_13 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmFLOAT_OnboardIOs_output_13() { return (BStatusNumeric) get(CmFLOAT_OnboardIOs_output_13); }
  public void setCmFLOAT_OnboardIOs_output_13(BStatusNumeric v) { set(CmFLOAT_OnboardIOs_output_13,v,null); }
  public void setCmFLOAT_OnboardIOs_output_13(float v) { getCmFLOAT_OnboardIOs_output_13().setValue(v); }
// SyncMode
  public static final Property CmFLOAT_OnboardIOs_output_5 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmFLOAT_OnboardIOs_output_5() { return (BStatusNumeric) get(CmFLOAT_OnboardIOs_output_5); }
  public void setCmFLOAT_OnboardIOs_output_5(BStatusNumeric v) { set(CmFLOAT_OnboardIOs_output_5,v,null); }
  public void setCmFLOAT_OnboardIOs_output_5(float v) { getCmFLOAT_OnboardIOs_output_5().setValue(v); }

  // macro parameter
// FLT-01: Valve runtime
  public static final Property CmFLOAT_OnboardIOs_parameter_9 = newProperty( 0, BDouble.make( 106 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(20)), BFacets.make(BFacets.MAX, BDouble.make(600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmFLOAT_OnboardIOs_parameter_9() { return getDouble(CmFLOAT_OnboardIOs_parameter_9); }
  public void setCmFLOAT_OnboardIOs_parameter_9(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_9,v,null); }
// FLT-02: Direction change hysteresis
  public static final Property CmFLOAT_OnboardIOs_parameter_10 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(20)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmFLOAT_OnboardIOs_parameter_10() { return getDouble(CmFLOAT_OnboardIOs_parameter_10); }
  public void setCmFLOAT_OnboardIOs_parameter_10(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_10,v,null); }
// FLT-03: EndstopMode 0=no, 1=permanent, 2=cyclic
  public static final Property CmFLOAT_OnboardIOs_parameter_130 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(2)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_130() { return getDouble(CmFLOAT_OnboardIOs_parameter_130); }
  public void setCmFLOAT_OnboardIOs_parameter_130(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_130,v,null); }
// FLT-04: Resolution
  public static final Property CmFLOAT_OnboardIOs_parameter_27 = newProperty( 0, BDouble.make( 3 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(30)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmFLOAT_OnboardIOs_parameter_27() { return getDouble(CmFLOAT_OnboardIOs_parameter_27); }
  public void setCmFLOAT_OnboardIOs_parameter_27(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_27,v,null); }
// FLT-05: PVE enabled
  public static final Property CmFLOAT_OnboardIOs_parameter_201 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_201() { return getDouble(CmFLOAT_OnboardIOs_parameter_201); }
  public void setCmFLOAT_OnboardIOs_parameter_201(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_201,v,null); }
// FLT-06: Valve minimum runtime
  public static final Property CmFLOAT_OnboardIOs_parameter_275 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmFLOAT_OnboardIOs_parameter_275() { return getDouble(CmFLOAT_OnboardIOs_parameter_275); }
  public void setCmFLOAT_OnboardIOs_parameter_275(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_275,v,null); }
// FLT-07: Over-travel Close Position
  public static final Property CmFLOAT_OnboardIOs_parameter_12 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_12() { return getDouble(CmFLOAT_OnboardIOs_parameter_12); }
  public void setCmFLOAT_OnboardIOs_parameter_12(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_12,v,null); }
// FTL-08: Over-travel Open Position
  public static final Property CmFLOAT_OnboardIOs_parameter_11 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_11() { return getDouble(CmFLOAT_OnboardIOs_parameter_11); }
  public void setCmFLOAT_OnboardIOs_parameter_11(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_11,v,null); }
// SNY-01: SyncStartup  0=close 1=open 2=no
  public static final Property CmFLOAT_OnboardIOs_parameter_107 = newProperty( 0, BDouble.make( 2 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(2)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_107() { return getDouble(CmFLOAT_OnboardIOs_parameter_107); }
  public void setCmFLOAT_OnboardIOs_parameter_107(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_107,v,null); }
// SNY-02: Sync interval enable (0=disabled, 1 = enabled)
  public static final Property CmFLOAT_OnboardIOs_parameter_228 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_228() { return getDouble(CmFLOAT_OnboardIOs_parameter_228); }
  public void setCmFLOAT_OnboardIOs_parameter_228(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_228,v,null); }
// SNY-03: Sync interval
  public static final Property CmFLOAT_OnboardIOs_parameter_230 = newProperty( 0, BDouble.make( 24 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(600)), BFacets.make(BFacets.MIN, BDouble.make(1)) ));
  public double getCmFLOAT_OnboardIOs_parameter_230() { return getDouble(CmFLOAT_OnboardIOs_parameter_230); }
  public void setCmFLOAT_OnboardIOs_parameter_230(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_230,v,null); }
// SNY-04: Sync interval position (0=close 1=open)
  public static final Property CmFLOAT_OnboardIOs_parameter_321 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmFLOAT_OnboardIOs_parameter_321() { return getDouble(CmFLOAT_OnboardIOs_parameter_321); }
  public void setCmFLOAT_OnboardIOs_parameter_321(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_321,v,null); }
// PEX-01: exercise period
  public static final Property CmFLOAT_OnboardIOs_parameter_224 = newProperty( 0, BDouble.make( 168 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("hour"))));
  public double getCmFLOAT_OnboardIOs_parameter_224() { return getDouble(CmFLOAT_OnboardIOs_parameter_224); }
  public void setCmFLOAT_OnboardIOs_parameter_224(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_224,v,null); }
// PEX-02: exercise time
  public static final Property CmFLOAT_OnboardIOs_parameter_225 = newProperty( 0, BDouble.make( 120 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmFLOAT_OnboardIOs_parameter_225() { return getDouble(CmFLOAT_OnboardIOs_parameter_225); }
  public void setCmFLOAT_OnboardIOs_parameter_225(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_225,v,null); }
// FED-01: Feedback signal at 0%
  public static final Property CmFLOAT_OnboardIOs_parameter_568 = newProperty( 0, BDouble.make( 20 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-1)), BFacets.make(BFacets.MAX, BDouble.make(40)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmFLOAT_OnboardIOs_parameter_568() { return getDouble(CmFLOAT_OnboardIOs_parameter_568); }
  public void setCmFLOAT_OnboardIOs_parameter_568(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_568,v,null); }
// FED-02: Feedback signal at 100%
  public static final Property CmFLOAT_OnboardIOs_parameter_569 = newProperty( 0, BDouble.make( 80 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(60)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmFLOAT_OnboardIOs_parameter_569() { return getDouble(CmFLOAT_OnboardIOs_parameter_569); }
  public void setCmFLOAT_OnboardIOs_parameter_569(double v) { setDouble(CmFLOAT_OnboardIOs_parameter_569,v,null); }

  // macro inputs
// Valve
  public static final Property CmFLOAT_OnboardIOs_input_1 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmFLOAT_OnboardIOs_input_1() { return (BStatusNumeric) get(CmFLOAT_OnboardIOs_input_1); }
  public void setCmFLOAT_OnboardIOs_input_1(BStatusNumeric v) { set(CmFLOAT_OnboardIOs_input_1,v,null); }
  public void setCmFLOAT_OnboardIOs_input_1(float v) { getCmFLOAT_OnboardIOs_input_1().setValue(v); }
// SyncOpen
  public static final Property CmFLOAT_OnboardIOs_input_2 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmFLOAT_OnboardIOs_input_2() { return (BStatusBoolean) get(CmFLOAT_OnboardIOs_input_2); }
  public void setCmFLOAT_OnboardIOs_input_2(BStatusBoolean v) { set(CmFLOAT_OnboardIOs_input_2,v,null); }
  public void setCmFLOAT_OnboardIOs_input_2(boolean v) { getCmFLOAT_OnboardIOs_input_2().setValue(v); }
// SyncClose
  public static final Property CmFLOAT_OnboardIOs_input_78 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmFLOAT_OnboardIOs_input_78() { return (BStatusBoolean) get(CmFLOAT_OnboardIOs_input_78); }
  public void setCmFLOAT_OnboardIOs_input_78(BStatusBoolean v) { set(CmFLOAT_OnboardIOs_input_78,v,null); }
  public void setCmFLOAT_OnboardIOs_input_78(boolean v) { getCmFLOAT_OnboardIOs_input_78().setValue(v); }
// Enable
  public static final Property CmFLOAT_OnboardIOs_input_4 = newProperty( Flags.SUMMARY, new BStatusBoolean( true ), null);
  public BStatusBoolean getCmFLOAT_OnboardIOs_input_4() { return (BStatusBoolean) get(CmFLOAT_OnboardIOs_input_4); }
  public void setCmFLOAT_OnboardIOs_input_4(BStatusBoolean v) { set(CmFLOAT_OnboardIOs_input_4,v,null); }
  public void setCmFLOAT_OnboardIOs_input_4(boolean v) { getCmFLOAT_OnboardIOs_input_4().setValue(v); }
// FeedbackPosition
  public static final Property CmFLOAT_OnboardIOs_input_554 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmFLOAT_OnboardIOs_input_554() { return (BStatusNumeric) get(CmFLOAT_OnboardIOs_input_554); }
  public void setCmFLOAT_OnboardIOs_input_554(BStatusNumeric v) { set(CmFLOAT_OnboardIOs_input_554,v,null); }
  public void setCmFLOAT_OnboardIOs_input_554(float v) { getCmFLOAT_OnboardIOs_input_554().setValue(v); }

  public static final Property enabledIn = newProperty( 0, new BStatusBoolean( true ), null);
  public BStatusBoolean getEnabledIn() { return (BStatusBoolean) get(enabledIn); }
  public void setEnabledIn(BStatusBoolean v) { set(enabledIn,v,null); }
  public void setEnabledIn(boolean v) { getEnabledIn().setValue(v); }


  public static final Property versionInfo = newProperty( Flags.READONLY, BString.make( "51.01 [7] 03.01.2017-12:19").getString(), null);
  public String getVersionInfo() { return getString(versionInfo); }
  public void setVersionInfo(String v) { setString(versionInfo,v,null); }


  public static final Property macroId = newProperty( Flags.READONLY|Flags.HIDDEN, BString.make( "6344912960254640139").getString(), null);
  public String getMacroId() { return getString(macroId); }
  public void setMacroId(String v) { setString(macroId,v,null); }

  // TYPE declaration
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCmFLOAT_OnboardIOs.class);


  // cycle time
  protected long lastSystemTicks = 0;
  protected float lastCycleTime = 0;

  // implementation of IEnvironment interface
  public float getCycleTime()
  { 
    return lastCycleTime;
  }

  public void started() throws Exception
  {
    super.started();
    Start();
    lastSystemTicks = ControlFunction.GetSystemTicks();

  }

  public int getHighestPriority(int inputId)
  {
    return 0;
  }

  public float getPriorityValue(int inputId, int priorityIndex)
  {
    return Float.NaN;
  }

  public void setPriorityValue(int outputId, int priorityIndex, float value, boolean setNull)
  {
  }

  // helper functions
  private float   BooleanToFloat(boolean value) { return value ? 1 : 0; }
  private int     BooleanToInt  (boolean value) { return value ? 1 : 0; }
  private int     FloatToInt    (float   value) { return (int) value;   }
  private boolean FloatToBoolean(float   value) { return value >= 0.5F; }

  // class variables
  protected CfIdt func_463 = new CfIdt();
  protected CfIdt func_464 = new CfIdt();
  protected CfIdt func_581 = new CfIdt();
  protected CfMultiply func_285 = new CfMultiply();
  protected CfDivide func_286 = new CfDivide();
  protected CfHysteresis func_471 = new CfHysteresis();
  protected CfCompare1 func_482 = new CfCompare1();
  protected CmRising__Faling func_295 = new CmRising__Faling();
  protected CfAnd func_468 = new CfAnd();
  protected CfAnd func_474 = new CfAnd();
  protected CfOr func_477 = new CfOr();
  protected CfInputMultiplexer func_309 = new CfInputMultiplexer();
  protected CfLimit func_332 = new CfLimit();
  protected CmPositionChange func_30 = new CmPositionChange();
  protected CmPositionChange func_32 = new CmPositionChange();
  protected CfOutputMultiplexer func_550 = new CfOutputMultiplexer();
  protected CfOutputMultiplexer func_553 = new CfOutputMultiplexer();
  protected CmRising__Faling func_583 = new CmRising__Faling();
  protected CfInputMultiplexer func_585 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_587 = new CfInputMultiplexer();
  protected CfDivide func_642 = new CfDivide();
  protected CfMultiply func_643 = new CfMultiply();
  protected CfOr func_614 = new CfOr();
  protected CfMonoflop func_615 = new CfMonoflop();
  protected CfSubtract func_635 = new CfSubtract();
  protected CfAddition func_637 = new CfAddition();
  protected CfInputMultiplexer func_633 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_636 = new CfInputMultiplexer();
  protected CfMultiply func_590 = new CfMultiply();
  protected CfAddition func_23 = new CfAddition();
  protected CfRamp func_555 = new CfRamp();
  protected CfRamp func_557 = new CfRamp();
  protected CfCompare1 func_649 = new CfCompare1();
  protected CfInputMultiplexer func_653 = new CfInputMultiplexer();
  protected CmIDT05 func_570 = new CmIDT05();
  protected CmIDT05 func_218 = new CmIDT05();
  protected CfLimit func_133 = new CfLimit();
  protected CfCompare1 func_673 = new CfCompare1();
  protected CfAnd func_670 = new CfAnd();
  protected CfInputMultiplexer func_666 = new CfInputMultiplexer();
  protected CfCompare1 func_563 = new CfCompare1();
  protected CfInputMultiplexer func_561 = new CfInputMultiplexer();
  protected CfSubtract func_180 = new CfSubtract();
  protected CfDivide func_185 = new CfDivide();
  protected CfNegate func_191 = new CfNegate();
  protected CfMaximum func_195 = new CfMaximum();
  protected CfMinimum func_272 = new CfMinimum();
  protected CfCompare1 func_184 = new CfCompare1();
  protected CfNot func_454 = new CfNot();
  protected CfAnd func_451 = new CfAnd();
  protected CfCompare1 func_186 = new CfCompare1();
  protected CfNot func_456 = new CfNot();
  protected CfAnd func_457 = new CfAnd();
  protected CfNegate func_344 = new CfNegate();
  protected CfMaximum func_346 = new CfMaximum();
  protected CfAddition func_352 = new CfAddition();
  protected CfMaximum func_353 = new CfMaximum();
  protected CfCompare1 func_403 = new CfCompare1();
  protected CfDelayOnOff func_408 = new CfDelayOnOff();
  protected CfHysteresis func_142 = new CfHysteresis();
  protected CfCompare2 func_336 = new CfCompare2();
  protected CfMonoflop func_522 = new CfMonoflop();
  protected CfMonoflop func_516 = new CfMonoflop();
  protected CfHysteresis func_413 = new CfHysteresis();
  protected CfAnd func_77 = new CfAnd();
  protected CmRising__Faling func_390 = new CmRising__Faling();
  protected CfCompare1 func_662 = new CfCompare1();
  protected CfAnd func_113 = new CfAnd();
  protected CfLastValue func_366 = new CfLastValue();
  protected CfCompare1 func_372 = new CfCompare1();
  protected CfCompare1 func_368 = new CfCompare1();
  protected CfAnd func_379 = new CfAnd();
  protected CfMonoflop func_519 = new CfMonoflop();
  protected CfSubtract func_63 = new CfSubtract();
  protected CfHysteresis func_139 = new CfHysteresis();
  protected CfHysteresis func_529 = new CfHysteresis();
  protected CfAnd func_528 = new CfAnd();
  protected CfCompare2 func_335 = new CfCompare2();
  protected CfMonoflop func_524 = new CfMonoflop();
  protected CfCompare1 func_658 = new CfCompare1();
  protected CfAnd func_116 = new CfAnd();
  protected CfOr func_251 = new CfOr();
  protected CmPVE_Exercise func_445 = new CmPVE_Exercise();
  protected CfCompare1 func_447 = new CfCompare1();
  protected CfAnd func_446 = new CfAnd();
  protected CfInputMultiplexer func_324 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_322 = new CfInputMultiplexer();
  protected CfOr func_82 = new CfOr();
  protected CfInputMultiplexer func_395 = new CfInputMultiplexer();
  protected CfMultiply func_94 = new CfMultiply();
  protected CfLastValue func_441 = new CfLastValue();
  protected CfNot func_436 = new CfNot();
  protected CfAnd func_438 = new CfAnd();
  protected CfInputMultiplexer func_415 = new CfInputMultiplexer();
  protected CfMonoflop func_93 = new CfMonoflop();
  protected CfOr func_79 = new CfOr();
  protected CfInputMultiplexer func_114 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_394 = new CfInputMultiplexer();
  protected CfNot func_537 = new CfNot();
  protected CfLastValue func_536 = new CfLastValue();
  protected CfAnd func_539 = new CfAnd();
  protected CfMultiply func_98 = new CfMultiply();
  protected CfInputMultiplexer func_538 = new CfInputMultiplexer();
  protected CfMonoflop func_92 = new CfMonoflop();
  protected CfIdt func_15 = new CfIdt();
  protected CfInputMultiplexer func_244 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_247 = new CfInputMultiplexer();
  protected CfOr func_361 = new CfOr();
  protected CmPVE_Exercise func_198 = new CmPVE_Exercise();
  protected CfCompare1 func_387 = new CfCompare1();
  protected CfAnd func_340 = new CfAnd();
  protected CfInputMultiplexer func_202 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_401 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_610 = new CfInputMultiplexer();
  protected CfNot func_646 = new CfNot();
  protected CfInputMultiplexer func_644 = new CfInputMultiplexer();
  protected CfIdt func_52 = new CfIdt();
  protected CfInputMultiplexer func_269 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_270 = new CfInputMultiplexer();
  protected CfDivide func_426 = new CfDivide();
  protected CfSubtract func_430 = new CfSubtract();
  protected CfIdt func_428 = new CfIdt();
  protected CfInputMultiplexer func_612 = new CfInputMultiplexer();
  protected CfAnd func_500 = new CfAnd();
  protected CfCycle func_424 = new CfCycle();
  protected CfInputMultiplexer func_432 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_146 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_611 = new CfInputMultiplexer();
  protected CfAnd func_506 = new CfAnd();
  protected CfInputMultiplexer func_431 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_154 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_466 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_156 = new CfInputMultiplexer();
  protected CfInputMultiplexer func_158 = new CfInputMultiplexer();
  protected CfNot func_411 = new CfNot();

  // default constructor
  public BCmFLOAT_OnboardIOs()
  {
    func_468.configure(3);
    func_474.configure(3);
    func_643.configure(3);
    func_23.configure(4);
    func_77.configure(4);
    func_528.configure(4);
    func_82.configure(6);
    func_438.configure(3);
    func_79.configure(5);
    func_539.configure(3);
    func_361.configure(3);
    func_269.configure(9);
    func_270.configure(9);
    func_612.configure(8);
    func_500.configure(3);
    func_146.configure(4);
    func_611.configure(8);
    func_506.configure(3);
    func_154.configure(4);
  }

  public void Start()
  {
    func_471.Start();
    func_295.Start();
    func_30.Start();
    func_32.Start();
    func_583.Start();
    func_615.Start();
    func_570.Start();
    func_218.Start();
    func_408.Start();
    func_142.Start();
    func_522.Start();
    func_516.Start();
    func_413.Start();
    func_390.Start();
    func_366.Start();
    func_519.Start();
    func_139.Start();
    func_529.Start();
    func_524.Start();
    func_445.Start();
    func_441.Start();
    func_93.Start();
    func_536.Start();
    func_92.Start();
    func_198.Start();
    func_424.Start();
  }
}
