/*******************************************************************************
Automatically generated file
Created by 'Control Macro Library Conversion Tool'
Creation time: 01.12.2020 17:33
Copyright (C) Honeywell GmbH D-71101 Schoenaich 2020
*******************************************************************************/

package cl.hvac.heating;

import javax.baja.status.*;
import javax.baja.sys.*;
import javax.baja.units.*;
import cl.hvac.*;
import cl.hvac.base.*;

/*==============================================================================
Macro name:    DHA_CircTempControl
Macro id:      6025419754957701737
Macro version: 20
Last change:   05.03.2018 - 11:16
Access Level:  2
Inputs:
  [1] CircTemp
  [2] CircTempSp
  [3] CircPumpCmd
  [4] LegionellaActive
  [5] Reset
Outputs:
  [1] CalcCirTempSp
  [2] CircPumpSpeed
  [3] CircTemp_Alm
Parameter:
  [1] Cir-01: P-band circulation temperature control
  [2] Cir-02: I-time system circulation temperature control
  [3] Cir-03: D-time system circulation temperature control
  [4] LEG-01: setpoint increase during legionella function
  [5] SYS-01: circulation temperature alarm enable after release
Description:
This macro should be used if a minimum circulation temperature is specified due to legal requirements, or in critical cases where this temperature might be lower than the minimum setpoint.

The macro measures the circulation temperature and compares it with the circulation temperature setpoint. When the measured value drops below the setpoint, the circulation pump speed will be increased. This function does not increase the domestic hot water setpoint if the circulation temperature is too low. When the setpoint cannot be reached in a pre-defined time, the macro generates an alarm.
================================================================================*/

public class BCmDHA_CircTempControl
  extends BControlFunctionSupport
  implements IEnvironment
{
  public void doExecute()
  {
    if (getEnabledIn().getBoolean())
    {
      Execute( );
      if (getCmDHA_CircTempControl_output_6().getStatus() == BStatus.disabled)
        setOutputStatusOk();
    }
    else
    {
      setOutputStatusDisabled();
    }
  }

  public void setOutputStatusDisabled()
  {
    getCmDHA_CircTempControl_output_6().setStatus(BStatus.disabled);
    getCmDHA_CircTempControl_output_7().setStatus(BStatus.disabled);
    getCmDHA_CircTempControl_output_8().setStatus(BStatus.disabled);
  }

  public void setOutputStatusOk()
  {
    getCmDHA_CircTempControl_output_6().setStatus(BStatus.ok);
    getCmDHA_CircTempControl_output_7().setStatus(BStatus.ok);
    getCmDHA_CircTempControl_output_8().setStatus(BStatus.ok);
  }

  void Execute( )
  {
    // cycle time calculation
    long ticks = ControlFunction.GetSystemTicks();
    lastCycleTime = (float)(ticks - lastSystemTicks) / 1000.0F;
    lastSystemTicks = ticks;

    // [23] Function call CfInputMultiplexer
    func_23.inputSelect = BooleanToInt(getCmDHA_CircTempControl_input_4().getValue());
    func_23.input[1-1] = 0F;
    func_23.input[2-1] = (float) getCmDHA_CircTempControl_parameter_12();
    func_23.Execute(  );

    // [30] Function call CfMonoflop
    func_30.input = BooleanToFloat(getCmDHA_CircTempControl_input_5().getValue());
    func_30.Execute( this, 0 );

    // [41] Function call CfDivide
    func_41.inputA = (float) getCmDHA_CircTempControl_parameter_9();
    func_41.inputB = 2F;
    func_41.Execute(  );

    // [56] Macro call CmIDT10
    func_56.CmIDT10_input_1 = (float) getCmDHA_CircTempControl_parameter_9();
    func_56.CmIDT10_input_2 = (float) getCmDHA_CircTempControl_parameter_10();
    func_56.CmIDT10_input_3 = (float) getCmDHA_CircTempControl_parameter_11();
    func_56.CmIDT10_input_4 = 0F;
    func_56.CmIDT10_input_5 = 100F;
    func_56.CmIDT10_input_6 = 0F;
    func_56.CmIDT10_input_7 = 600F;
    func_56.CmIDT10_input_8 = 0F;
    func_56.CmIDT10_input_9 = 0F;
    func_56.CmIDT10_input_10 = 5F;
    func_56.Execute(  );
    setParameter(4, func_56.CmIDT10_output_21);
    setParameter(6, func_56.CmIDT10_output_22);
    setParameter(5, func_56.CmIDT10_output_23);
    setParameter(8, func_56.CmIDT10_output_24);
    setParameter(7, func_56.CmIDT10_output_25);
    setParameter(12, func_56.CmIDT10_output_26);
    setParameter(9, func_56.CmIDT10_output_27);
    setParameter(10, func_56.CmIDT10_output_28);
    setParameter(0, func_56.CmIDT10_output_30);

    // [62] Function call CfSubtract
    func_62.input[0] = (float) getCmDHA_CircTempControl_parameter_13();
    func_62.input[1] = 600F;
    func_62.Execute(  );

    // [86] Function call CfCompare1
    func_86.inputA = (float) getCmDHA_CircTempControl_parameter_13();
    func_86.inputB = 0.5F;
    func_86.Execute(  );

    // [118] Function call CfNot
    func_118.input = ControlFunction.GetStartupFlag();
    func_118.Execute(  );

    // [149] Function call CfNot
    func_149.input = getCmDHA_CircTempControl_input_3().getValue();
    func_149.Execute(  );

    // [158] Function call CfWindow
    func_158.inputValue = (float) getCmDHA_CircTempControl_parameter_12();
    func_158.inputMinimum = -0.2F;
    func_158.inputMaximum = 0.2F;
    func_158.Execute(  );

    // [20] Function call CfAddition
    func_20.input[0] = (float) getCmDHA_CircTempControl_input_2().getValue();
    func_20.input[1] = func_23.output;
    func_20.Execute(  );

    // [64] Function call CfMaximum
    func_64.input[0] = func_62.output;
    func_64.input[1] = 0F;
    func_64.Execute(  );
    setParameter(11, func_64.output);

    // [131] Function call CfDelayOnOff
    func_131.input = func_149.output;
    func_131.Execute( this, 1 );

    // [150] Function call CfAnd
    func_150.input[0] = func_158.output;
    func_150.input[1] = getCmDHA_CircTempControl_input_4().getValue();
    func_150.Execute(  );

    // [16] Function call CfInputMultiplexer
    func_16.inputSelect = BooleanToInt(getCmDHA_CircTempControl_input_3().getValue());
    func_16.input[1-1] = -999F;
    func_16.input[2-1] = func_20.output;
    func_16.Execute(  );

    // [26] Function call CfIdt
    func_26.input = func_20.output;
    func_26.Execute(  );
    setCmDHA_CircTempControl_output_6(func_26.output[0]);

    // [141] Function call CfMonoflop
    func_141.input = BooleanToFloat(func_131.output);
    func_141.Execute( this, 3 );

    // [14] Function call CfPidPlus
    func_14.inputX = (float) getCmDHA_CircTempControl_input_1().getValue();
    func_14.inputW = func_16.output;
    func_14.Execute( this, 4 );

    // [40] Function call CfSubtract
    func_40.input[0] = func_16.output;
    func_40.input[1] = func_41.output;
    func_40.Execute(  );

    // [78] Function call CfCompare1
    func_78.inputA = (float) getCmDHA_CircTempControl_input_1().getValue();
    func_78.inputB = func_40.output;
    func_78.Execute(  );

    // [159] Function call CfInputMultiplexer
    func_159.inputSelect = BooleanToInt(func_150.outputAND);
    func_159.input[1-1] = func_14.output;
    func_159.input[2-1] = 100F;
    func_159.Execute(  );
    setCmDHA_CircTempControl_output_7(func_159.output);

    // [95] Function call CfDelayOnOff
    func_95.input = func_82.outputAND;
    func_95.Execute( this, 9 );
    setCmDHA_CircTempControl_output_8(func_95.output);

    // [77] Function call CfAnd
    func_77.input[0] = getCmDHA_CircTempControl_output_8().getValue();
    func_77.input[1] = func_30.output;
    func_77.Execute(  );

    // [111] Function call CfNot
    func_111.input = getCmDHA_CircTempControl_output_8().getValue();
    func_111.Execute(  );

    // [146] Function call CfNot
    func_146.input = getCmDHA_CircTempControl_output_8().getValue();
    func_146.Execute(  );

    // [47] Function call CfAnd
    func_47.input[0] = func_118.output;
    func_47.input[1] = getCmDHA_CircTempControl_input_3().getValue();
    func_47.input[2] = func_111.output;
    func_47.Execute(  );

    // [143] Function call CfAnd
    func_143.input[0] = func_141.output;
    func_143.input[1] = func_146.output;
    func_143.Execute(  );

    // [139] Function call CfOr
    func_139.input[0] = func_77.outputAND;
    func_139.input[1] = func_143.outputAND;
    func_139.Execute(  );

    // [69] Function call CfJkFlipFlop
    func_69.inputSet = func_47.outputAND;
    func_69.inputClock = false;
    func_69.inputReset = func_139.outputOR;
    func_69.Execute(  );

    // [42] Function call CfDelayOnOff
    func_42.input = func_69.output;
    func_42.Execute( this, 11 );

    // [82] Function call CfAnd
    func_82.input[0] = func_78.outputLess;
    func_82.input[1] = func_86.outputGreater;
    func_82.input[2] = func_42.output;
    func_82.Execute(  );

  }

  // parameter for functions and sub macros
  protected float [] parameterList = new float []
  {
    5F, 3F, 0F, 3F, 3F, 0F, 1000F, 100F,
    0F, 10F, 10F, 10F, 10F
  };

  // implementation of IEnvironment
  public float getParameter(int index)
  {
    return parameterList[index];
  };

  public void setParameter(int index, float value)
  {
    parameterList[index] = value;
  };

  // macro register which are written multiple times
  // macro outputs
// CalcCirTempSp
  public static final Property CmDHA_CircTempControl_output_6 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius")));
  public BStatusNumeric getCmDHA_CircTempControl_output_6() { return (BStatusNumeric) get(CmDHA_CircTempControl_output_6); }
  public void setCmDHA_CircTempControl_output_6(BStatusNumeric v) { set(CmDHA_CircTempControl_output_6,v,null); }
  public void setCmDHA_CircTempControl_output_6(float v) { getCmDHA_CircTempControl_output_6().setValue(v); }
// CircPumpSpeed
  public static final Property CmDHA_CircTempControl_output_7 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), BFacets.make(BFacets.UNITS, BUnit.getUnit("percent")));
  public BStatusNumeric getCmDHA_CircTempControl_output_7() { return (BStatusNumeric) get(CmDHA_CircTempControl_output_7); }
  public void setCmDHA_CircTempControl_output_7(BStatusNumeric v) { set(CmDHA_CircTempControl_output_7,v,null); }
  public void setCmDHA_CircTempControl_output_7(float v) { getCmDHA_CircTempControl_output_7().setValue(v); }
// CircTemp_Alm
  public static final Property CmDHA_CircTempControl_output_8 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmDHA_CircTempControl_output_8() { return (BStatusBoolean) get(CmDHA_CircTempControl_output_8); }
  public void setCmDHA_CircTempControl_output_8(BStatusBoolean v) { set(CmDHA_CircTempControl_output_8,v,null); }
  public void setCmDHA_CircTempControl_output_8(boolean v) { getCmDHA_CircTempControl_output_8().setValue(v); }

  // macro parameter
// Cir-01: P-band circulation temperature control
  public static final Property CmDHA_CircTempControl_parameter_9 = newProperty( 0, BDouble.make( 20 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(500)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmDHA_CircTempControl_parameter_9() { return getDouble(CmDHA_CircTempControl_parameter_9); }
  public void setCmDHA_CircTempControl_parameter_9(double v) { setDouble(CmDHA_CircTempControl_parameter_9,v,null); }
// Cir-02: I-time system circulation temperature control
  public static final Property CmDHA_CircTempControl_parameter_10 = newProperty( 0, BDouble.make( 300 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmDHA_CircTempControl_parameter_10() { return getDouble(CmDHA_CircTempControl_parameter_10); }
  public void setCmDHA_CircTempControl_parameter_10(double v) { setDouble(CmDHA_CircTempControl_parameter_10,v,null); }
// Cir-03: D-time system circulation temperature control
  public static final Property CmDHA_CircTempControl_parameter_11 = newProperty( 0, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(1000)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmDHA_CircTempControl_parameter_11() { return getDouble(CmDHA_CircTempControl_parameter_11); }
  public void setCmDHA_CircTempControl_parameter_11(double v) { setDouble(CmDHA_CircTempControl_parameter_11,v,null); }
// LEG-01: setpoint increase during legionella function
  public static final Property CmDHA_CircTempControl_parameter_12 = newProperty( 0, BDouble.make( 5 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(20)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmDHA_CircTempControl_parameter_12() { return getDouble(CmDHA_CircTempControl_parameter_12); }
  public void setCmDHA_CircTempControl_parameter_12(double v) { setDouble(CmDHA_CircTempControl_parameter_12,v,null); }
// SYS-01: circulation temperature alarm enable after release
  public static final Property CmDHA_CircTempControl_parameter_13 = newProperty( 0, BDouble.make( 1800 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmDHA_CircTempControl_parameter_13() { return getDouble(CmDHA_CircTempControl_parameter_13); }
  public void setCmDHA_CircTempControl_parameter_13(double v) { setDouble(CmDHA_CircTempControl_parameter_13,v,null); }

  // macro inputs
// CircTemp
  public static final Property CmDHA_CircTempControl_input_1 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius")));
  public BStatusNumeric getCmDHA_CircTempControl_input_1() { return (BStatusNumeric) get(CmDHA_CircTempControl_input_1); }
  public void setCmDHA_CircTempControl_input_1(BStatusNumeric v) { set(CmDHA_CircTempControl_input_1,v,null); }
  public void setCmDHA_CircTempControl_input_1(float v) { getCmDHA_CircTempControl_input_1().setValue(v); }
// CircTempSp
  public static final Property CmDHA_CircTempControl_input_2 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0, BStatus.nullStatus), BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius")));
  public BStatusNumeric getCmDHA_CircTempControl_input_2() { return (BStatusNumeric) get(CmDHA_CircTempControl_input_2); }
  public void setCmDHA_CircTempControl_input_2(BStatusNumeric v) { set(CmDHA_CircTempControl_input_2,v,null); }
  public void setCmDHA_CircTempControl_input_2(float v) { getCmDHA_CircTempControl_input_2().setValue(v); }
// CircPumpCmd
  public static final Property CmDHA_CircTempControl_input_3 = newProperty( Flags.SUMMARY, new BStatusBoolean( true ), null);
  public BStatusBoolean getCmDHA_CircTempControl_input_3() { return (BStatusBoolean) get(CmDHA_CircTempControl_input_3); }
  public void setCmDHA_CircTempControl_input_3(BStatusBoolean v) { set(CmDHA_CircTempControl_input_3,v,null); }
  public void setCmDHA_CircTempControl_input_3(boolean v) { getCmDHA_CircTempControl_input_3().setValue(v); }
// LegionellaActive
  public static final Property CmDHA_CircTempControl_input_4 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmDHA_CircTempControl_input_4() { return (BStatusBoolean) get(CmDHA_CircTempControl_input_4); }
  public void setCmDHA_CircTempControl_input_4(BStatusBoolean v) { set(CmDHA_CircTempControl_input_4,v,null); }
  public void setCmDHA_CircTempControl_input_4(boolean v) { getCmDHA_CircTempControl_input_4().setValue(v); }
// Reset
  public static final Property CmDHA_CircTempControl_input_5 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmDHA_CircTempControl_input_5() { return (BStatusBoolean) get(CmDHA_CircTempControl_input_5); }
  public void setCmDHA_CircTempControl_input_5(BStatusBoolean v) { set(CmDHA_CircTempControl_input_5,v,null); }
  public void setCmDHA_CircTempControl_input_5(boolean v) { getCmDHA_CircTempControl_input_5().setValue(v); }

  public static final Property enabledIn = newProperty( 0, new BStatusBoolean( true ), null);
  public BStatusBoolean getEnabledIn() { return (BStatusBoolean) get(enabledIn); }
  public void setEnabledIn(BStatusBoolean v) { set(enabledIn,v,null); }
  public void setEnabledIn(boolean v) { getEnabledIn().setValue(v); }


  public static final Property versionInfo = newProperty( Flags.READONLY, BString.make( "51.03 [20] 05.03.2018-11:16").getString(), null);
  public String getVersionInfo() { return getString(versionInfo); }
  public void setVersionInfo(String v) { setString(versionInfo,v,null); }


  public static final Property macroId = newProperty( Flags.READONLY|Flags.HIDDEN, BString.make( "6025419754957701737").getString(), null);
  public String getMacroId() { return getString(macroId); }
  public void setMacroId(String v) { setString(macroId,v,null); }

  // TYPE declaration
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCmDHA_CircTempControl.class);


  // cycle time
  protected long lastSystemTicks = 0;
  protected float lastCycleTime = 0;

  // implementation of IEnvironment interface
  public float getCycleTime()
  { 
    return lastCycleTime;
  }

  public void started() throws Exception
  {
    super.started();
    Start();
    lastSystemTicks = ControlFunction.GetSystemTicks();

  }

  public int getHighestPriority(int inputId)
  {
    return 0;
  }

  public float getPriorityValue(int inputId, int priorityIndex)
  {
    return Float.NaN;
  }

  public void setPriorityValue(int outputId, int priorityIndex, float value, boolean setNull)
  {
  }

  // helper functions
  private float   BooleanToFloat(boolean value) { return value ? 1 : 0; }
  private int     BooleanToInt  (boolean value) { return value ? 1 : 0; }

  // class variables
  protected CfInputMultiplexer func_23 = new CfInputMultiplexer();
  protected CfMonoflop func_30 = new CfMonoflop();
  protected CfDivide func_41 = new CfDivide();
  protected CmIDT10 func_56 = new CmIDT10();
  protected CfSubtract func_62 = new CfSubtract();
  protected CfCompare1 func_86 = new CfCompare1();
  protected CfNot func_118 = new CfNot();
  protected CfNot func_149 = new CfNot();
  protected CfWindow func_158 = new CfWindow();
  protected CfAddition func_20 = new CfAddition();
  protected CfMaximum func_64 = new CfMaximum();
  protected CfDelayOnOff func_131 = new CfDelayOnOff();
  protected CfAnd func_150 = new CfAnd();
  protected CfInputMultiplexer func_16 = new CfInputMultiplexer();
  protected CfIdt func_26 = new CfIdt();
  protected CfMonoflop func_141 = new CfMonoflop();
  protected CfPidPlus func_14 = new CfPidPlus();
  protected CfSubtract func_40 = new CfSubtract();
  protected CfCompare1 func_78 = new CfCompare1();
  protected CfInputMultiplexer func_159 = new CfInputMultiplexer();
  protected CfDelayOnOff func_95 = new CfDelayOnOff();
  protected CfAnd func_77 = new CfAnd();
  protected CfNot func_111 = new CfNot();
  protected CfNot func_146 = new CfNot();
  protected CfAnd func_47 = new CfAnd();
  protected CfAnd func_143 = new CfAnd();
  protected CfOr func_139 = new CfOr();
  protected CfJkFlipFlop func_69 = new CfJkFlipFlop();
  protected CfDelayOnOff func_42 = new CfDelayOnOff();
  protected CfAnd func_82 = new CfAnd();

  // default constructor
  public BCmDHA_CircTempControl()
  {
    func_47.configure(3);
    func_82.configure(3);
  }

  public void Start()
  {
    func_30.Start();
    func_56.Start();
    func_131.Start();
    func_141.Start();
    func_14.Start();
    func_95.Start();
    func_69.Start();
    func_42.Start();
  }
}
