/*******************************************************************************
Automatically generated file
Created by 'Control Macro Library Conversion Tool'
Creation time: 01.12.2020 17:37
Copyright (C) Honeywell GmbH D-71101 Schoenaich 2020
*******************************************************************************/

package cl.hvac.nordicaircondition;

import javax.baja.status.*;
import javax.baja.sys.*;
import javax.baja.units.*;
import cl.hvac.*;
import cl.hvac.base.*;

/*==============================================================================
Macro name:    HCA_HeaterNordic
Macro id:      5973979754190026861
Macro version: 21
Last change:   09.11.2015 - 14:15
Access Level:  2
Inputs:
  [1] HeatCtrlSig
  [2] FilteredOATemp
  [3] FrostThermostat
  [4] ReturnWaterTemp
  [5] RetTmpSpOcc
  [6] RetTmpSpUnOcc
  [7] RetSetpointSelect
  [8] SupplyAirTemp
  [9] FailCmdAlarm
  [10] PlantMode
Outputs:
  [1] ValveControl
  [2] PumpCmd
  [3] DamperEnable
  [4] FrostAlarm
  [5] DamperEnableAlm
  [6] HeatDemand
Parameter:
  [1] RWT-03: return water temperature maximum setpoint
  [2] RWT-04: return water temperature P-band
  [3] RWT-05: return water temperature setpoint ramp
  [4] RWT-06: return water temperature frost protection limit
  [5] SAT-01: supply air frost protection limit
  [6] SAT-02: supply air frost protection limit delay
  [7] DME-01: damper enable or alarm delay
  [8] HTG-01: calculated heating setpoint supply temp. min. setpoint
  [9] HTG-02: calculated heating setpoint supply temp. max. setpoint
  [10] HTG-03: calculated heating setpoint supply temp. start point
  [11] HTG-04: heat demand increase/ decrease value
  [12] HTG-05: heating valve pos. diff. to calculate heating setpoint
  [13] OAT-03: low limit frost protection
  [14] FEZ-01: frost protection RWT/ DAT setpoint raise
  [15] FEZ-02: raised setpoint decrease during frost recovery
  [16] FEZ-03: valve position during frost protection
  [17] FEZ-04: valve raised position decrease during frost recovery
  [18] HPU-01: startup delay heating pump
  [19] HPU-02: pump switch off delay
  [20] ALM-03: AHU stops at pump off alarm at low outdoor temperature
  [21] DISP-01: calculated heating demand temperature setpoint
  [22] DISP-02: calculated return water temperature setpoint
Description:
The "HCA_HeatingCoil" macro provides modulating control of the heating coil valve actuator and controls the heating pump. The macro receives its heating control signal from the control strategy. A frost protection thermostat, if connected, will prevent freezing of the air handling unit coils at low outdoor temperatures. Continuous freeze protection can override the strategy signal by the:
 - coil's own coil off air temperature or 
 - return water temperature control

The heating coil enables the dampers, only if the coils return water temperature (if configured) is high enough. For energy saving reasons, the heating coil can stop heating at high outdoor temperatures. The macro generates pump-on, fail-to-command and pump-off alarms, if the pump status is connected. Pump and valve exercise prevent the coil's equipment from getting stuck. The calculated heating demand sent to heat generators, depends on the actual heating valve position. Plant- and humidity mode signals can override the normal control signals.
================================================================================*/

public class BCmHCA_HeaterNordic
  extends BControlFunctionSupport
  implements IEnvironment
{
  public void doExecute()
  {
    if (getEnabledIn().getBoolean())
    {
      Execute( );
      if (getCmHCA_HeaterNordic_output_593().getStatus() == BStatus.disabled)
        setOutputStatusOk();
    }
    else
    {
      setOutputStatusDisabled();
    }
  }

  public void setOutputStatusDisabled()
  {
    getCmHCA_HeaterNordic_output_593().setStatus(BStatus.disabled);
    getCmHCA_HeaterNordic_output_594().setStatus(BStatus.disabled);
    getCmHCA_HeaterNordic_output_8().setStatus(BStatus.disabled);
    getCmHCA_HeaterNordic_output_564().setStatus(BStatus.disabled);
    getCmHCA_HeaterNordic_output_9().setStatus(BStatus.disabled);
    getCmHCA_HeaterNordic_output_12().setStatus(BStatus.disabled);
  }

  public void setOutputStatusOk()
  {
    getCmHCA_HeaterNordic_output_593().setStatus(BStatus.ok);
    getCmHCA_HeaterNordic_output_594().setStatus(BStatus.ok);
    getCmHCA_HeaterNordic_output_8().setStatus(BStatus.ok);
    getCmHCA_HeaterNordic_output_564().setStatus(BStatus.ok);
    getCmHCA_HeaterNordic_output_9().setStatus(BStatus.ok);
    getCmHCA_HeaterNordic_output_12().setStatus(BStatus.ok);
  }

  void Execute( )
  {
    // cycle time calculation
    long ticks = ControlFunction.GetSystemTicks();
    lastCycleTime = (float)(ticks - lastSystemTicks) / 1000.0F;
    lastSystemTicks = ticks;

    // [24] Function call CfHysteresis
    func_24.inputX = (float) getCmHCA_HeaterNordic_parameter_21();
    func_24.inputW = (float) getCmHCA_HeaterNordic_input_1().getValue();
    func_24.Execute( this, 0 );

    // [40] Function call CfEqual
    func_40.input = FloatToInt((float) getCmHCA_HeaterNordic_input_6().getValue());
    func_40.inputCompare[1-1] = 1;
    func_40.inputCompare[2-1] = 2;
    func_40.inputCompare[3-1] = 4;
    func_40.inputCompare[4-1] = 5;
    func_40.inputCompare[5-1] = 6;
    func_40.inputCompare[6-1] = 30;
    func_40.inputCompare[7-1] = 22;
    func_40.Execute(  );

    // [64] Function call CfCompare1
    func_64.inputA = (float) getCmHCA_HeaterNordic_input_5().getValue();
    func_64.inputB = 999F;
    func_64.Execute(  );

    // [381] Function call CfEqual
    func_381.input = FloatToInt((float) getCmHCA_HeaterNordic_input_6().getValue());
    func_381.inputCompare[1-1] = 20;
    func_381.Execute(  );

    // [423] Macro call CmIDT05
    func_423.CmIDT05_input_1 = (float) getCmHCA_HeaterNordic_parameter_200();
    func_423.CmIDT05_input_2 = (float) getCmHCA_HeaterNordic_parameter_201();
    func_423.CmIDT05_input_3 = (float) getCmHCA_HeaterNordic_parameter_202();
    func_423.CmIDT05_input_4 = (float) getCmHCA_HeaterNordic_parameter_203();
    func_423.CmIDT05_input_5 = (float) getCmHCA_HeaterNordic_parameter_204();
    func_423.Execute(  );
    setParameter(24, func_423.CmIDT05_output_21);
    setParameter(25, func_423.CmIDT05_output_22);
    setParameter(26, func_423.CmIDT05_output_23);
    setParameter(27, func_423.CmIDT05_output_24);
    setParameter(28, func_423.CmIDT05_output_25);

    // [438] Macro call CmIDT05
    func_438.CmIDT05_input_1 = (float) getCmHCA_HeaterNordic_parameter_46();
    func_438.CmIDT05_input_2 = (float) getCmHCA_HeaterNordic_parameter_47();
    func_438.CmIDT05_input_3 = (float) getCmHCA_HeaterNordic_parameter_15();
    func_438.CmIDT05_input_4 = (float) getCmHCA_HeaterNordic_parameter_16();
    func_438.CmIDT05_input_5 = (float) getCmHCA_HeaterNordic_parameter_567();
    func_438.Execute(  );
    setParameter(7, func_438.CmIDT05_output_21);
    setParameter(8, func_438.CmIDT05_output_22);
    setParameter(9, func_438.CmIDT05_output_23);
    setParameter(10, func_438.CmIDT05_output_24);
    setParameter(1, func_438.CmIDT05_output_25);

    // [461] Function call CfIdt
    func_461.input = (float) getCmHCA_HeaterNordic_parameter_15();
    func_461.Execute(  );
    setParameter(19, func_461.output[0]);

    // [489] Function call CfIdt
    func_489.input = (float) getCmHCA_HeaterNordic_parameter_14();
    func_489.Execute(  );
    setParameter(20, func_489.output[0]);

    // [490] Function call CfIdt
    func_490.input = (float) getCmHCA_HeaterNordic_parameter_19();
    func_490.Execute(  );
    setParameter(21, func_490.output[0]);

    // [556] Macro call CmIDT05
    func_556.CmIDT05_input_1 = (float) getCmHCA_HeaterNordic_parameter_237();
    func_556.CmIDT05_input_2 = (float) getCmHCA_HeaterNordic_parameter_13();
    func_556.CmIDT05_input_3 = (float) getCmHCA_HeaterNordic_parameter_21();
    func_556.CmIDT05_input_4 = 0F;
    func_556.CmIDT05_input_5 = 0F;
    func_556.Execute(  );
    setParameter(33, func_556.CmIDT05_output_21);
    setParameter(34, func_556.CmIDT05_output_22);
    setParameter(32, func_556.CmIDT05_output_23);

    // [579] Macro call CmIDT05
    func_579.CmIDT05_input_1 = (float) getCmHCA_HeaterNordic_parameter_566();
    func_579.CmIDT05_input_2 = (float) getCmHCA_HeaterNordic_parameter_62();
    func_579.CmIDT05_input_3 = (float) getCmHCA_HeaterNordic_parameter_565();
    func_579.CmIDT05_input_4 = (float) getCmHCA_HeaterNordic_parameter_568();
    func_579.CmIDT05_input_5 = (float) getCmHCA_HeaterNordic_parameter_569();
    func_579.Execute(  );
    setParameter(11, func_579.CmIDT05_output_21);
    setParameter(12, func_579.CmIDT05_output_22);
    setParameter(13, func_579.CmIDT05_output_23);
    setParameter(2, func_579.CmIDT05_output_24);
    setParameter(3, func_579.CmIDT05_output_25);

    // [585] Macro call CmFrostProtectionNordic
    func_585.CmFrostProtectionNordic_input_1 = getCmHCA_HeaterNordic_input_3().getValue();
    func_585.CmFrostProtectionNordic_input_2 = (float) getCmHCA_HeaterNordic_input_5().getValue();
    func_585.CmFrostProtectionNordic_input_3 = (float) getCmHCA_HeaterNordic_input_563().getValue();
    func_585.Execute( this, 1 );

    // [586] Function call CfIdt
    func_586.input = BooleanToFloat(func_585.CmFrostProtectionNordic_output_4);
    func_586.Execute(  );
    setCmHCA_HeaterNordic_output_564(FloatToBoolean(func_586.output[0]));

    // [205] Macro call CmHeating_Signal_Setup
    func_205.CmHeating_Signal_Setup_input_25 = func_64.outputEqual;
    func_205.CmHeating_Signal_Setup_input_29 = func_24.output1;
    func_205.CmHeating_Signal_Setup_input_31 = func_585.CmFrostProtectionNordic_output_4;
    func_205.CmHeating_Signal_Setup_input_32 = (float) getCmHCA_HeaterNordic_input_6().getValue();
    func_205.CmHeating_Signal_Setup_input_24 = 999F;
    func_205.CmHeating_Signal_Setup_input_66 = getCmHCA_HeaterNordic_output_8().getValue();
    func_205.Execute( this, 7 );

    // [574] Macro call CmReturn_Water_Setpoint_ControlNordic
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_2 = (float) getCmHCA_HeaterNordic_input_5().getValue();
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_90 = func_24.output2;
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_4 = func_205.CmHeating_Signal_Setup_output_68;
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_139 = (float) getCmHCA_HeaterNordic_input_6().getValue();
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_155 = (float) getCmHCA_HeaterNordic_input_560().getValue();
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_156 = (float) getCmHCA_HeaterNordic_input_561().getValue();
    func_574.CmReturn_Water_Setpoint_ControlNordic_input_157 = (float) getCmHCA_HeaterNordic_input_562().getValue();
    func_574.Execute( this, 11 );
    setCmHCA_HeaterNordic_parameter_340(func_574.CmReturn_Water_Setpoint_ControlNordic_output_126);

    // [27] Macro call CmHC_Valve
    func_27.CmHC_Valve_input_29 = func_381.output;
    func_27.CmHC_Valve_input_1 = func_585.CmFrostProtectionNordic_output_4;
    func_27.CmHC_Valve_input_47 = BooleanToFloat(func_40.output);
    func_27.CmHC_Valve_input_14 = func_574.CmReturn_Water_Setpoint_ControlNordic_output_5;
    func_27.CmHC_Valve_input_15 = 0F;
    func_27.CmHC_Valve_input_16 = func_205.CmHeating_Signal_Setup_output_1;
    func_27.CmHC_Valve_input_27 = (float) getCmHCA_HeaterNordic_input_2().getValue();
    func_27.Execute( this, 19 );
    setCmHCA_HeaterNordic_output_593(func_27.CmHC_Valve_output_6);

    // [486] Macro call CmDamper_Enable
    func_486.CmDamper_Enable_input_8 = func_24.output1;
    func_486.CmDamper_Enable_input_6 = (float) getCmHCA_HeaterNordic_input_6().getValue();
    func_486.CmDamper_Enable_input_10 = getCmHCA_HeaterNordic_input_552().getValue();
    func_486.CmDamper_Enable_input_12 = func_64.outputEqual;
    func_486.CmDamper_Enable_input_11 = func_574.CmReturn_Water_Setpoint_ControlNordic_output_6;
    func_486.Execute( this, 20 );
    setCmHCA_HeaterNordic_output_8(func_486.CmDamper_Enable_output_4);
    setCmHCA_HeaterNordic_output_9(func_486.CmDamper_Enable_output_5);

    // [29] Macro call CmDemand_As_A_Function_Of_The_Actual_Valve_Position
    func_29.CmDemand_As_A_Function_Of_The_Actual_Valve_Position_input_6 = (float) getCmHCA_HeaterNordic_output_593().getValue();
    func_29.Execute( this, 24 );
    setCmHCA_HeaterNordic_output_12(func_29.CmDemand_As_A_Function_Of_The_Actual_Valve_Position_output_7);

    // [554] Macro call CmPumpEnable
    func_554.CmPumpEnable_input_4 = (float) getCmHCA_HeaterNordic_output_593().getValue();
    func_554.CmPumpEnable_input_9 = (float) getCmHCA_HeaterNordic_input_1().getValue();
    func_554.CmPumpEnable_input_22 = (float) getCmHCA_HeaterNordic_input_6().getValue();
    func_554.CmPumpEnable_input_57 = getCmHCA_HeaterNordic_input_3().getValue();
    func_554.Execute( this, 32 );
    setCmHCA_HeaterNordic_output_594(func_554.CmPumpEnable_output_21);

    // [473] Function call CfIdt
    func_473.input = (float) getCmHCA_HeaterNordic_output_12().getValue();
    func_473.Execute(  );
    setCmHCA_HeaterNordic_parameter_342(func_473.output[0]);

  }

  // parameter for functions and sub macros
  protected float [] parameterList = new float []
  {
    1F, 6F, 10F, 600F, 0F, 0F, 10F, 15F,
    1F, 100F, 10F, 85F, 10F, 10F, 3F, 0F,
    0F, 100F, 0F, 0F, 20F, 1F, 10F, 10F,
    20F, 85F, 50F, 1F, 25F, 2F, 60F, 60F,
    6F, 5F, 300F, 2F, 1F, 10F, 10F
  };

  // implementation of IEnvironment
  public float getParameter(int index)
  {
    return parameterList[index];
  };

  public void setParameter(int index, float value)
  {
    parameterList[index] = value;
  };

  // macro register which are written multiple times
  // macro outputs
// ValveControl
  public static final Property CmHCA_HeaterNordic_output_593 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmHCA_HeaterNordic_output_593() { return (BStatusNumeric) get(CmHCA_HeaterNordic_output_593); }
  public void setCmHCA_HeaterNordic_output_593(BStatusNumeric v) { set(CmHCA_HeaterNordic_output_593,v,null); }
  public void setCmHCA_HeaterNordic_output_593(float v) { getCmHCA_HeaterNordic_output_593().setValue(v); }
// PumpCmd
  public static final Property CmHCA_HeaterNordic_output_594 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeaterNordic_output_594() { return (BStatusBoolean) get(CmHCA_HeaterNordic_output_594); }
  public void setCmHCA_HeaterNordic_output_594(BStatusBoolean v) { set(CmHCA_HeaterNordic_output_594,v,null); }
  public void setCmHCA_HeaterNordic_output_594(boolean v) { getCmHCA_HeaterNordic_output_594().setValue(v); }
// DamperEnable
  public static final Property CmHCA_HeaterNordic_output_8 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeaterNordic_output_8() { return (BStatusBoolean) get(CmHCA_HeaterNordic_output_8); }
  public void setCmHCA_HeaterNordic_output_8(BStatusBoolean v) { set(CmHCA_HeaterNordic_output_8,v,null); }
  public void setCmHCA_HeaterNordic_output_8(boolean v) { getCmHCA_HeaterNordic_output_8().setValue(v); }
// FrostAlarm
  public static final Property CmHCA_HeaterNordic_output_564 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeaterNordic_output_564() { return (BStatusBoolean) get(CmHCA_HeaterNordic_output_564); }
  public void setCmHCA_HeaterNordic_output_564(BStatusBoolean v) { set(CmHCA_HeaterNordic_output_564,v,null); }
  public void setCmHCA_HeaterNordic_output_564(boolean v) { getCmHCA_HeaterNordic_output_564().setValue(v); }
// DamperEnableAlm
  public static final Property CmHCA_HeaterNordic_output_9 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusBoolean( false, BStatus.disabled), null);
  public BStatusBoolean getCmHCA_HeaterNordic_output_9() { return (BStatusBoolean) get(CmHCA_HeaterNordic_output_9); }
  public void setCmHCA_HeaterNordic_output_9(BStatusBoolean v) { set(CmHCA_HeaterNordic_output_9,v,null); }
  public void setCmHCA_HeaterNordic_output_9(boolean v) { getCmHCA_HeaterNordic_output_9().setValue(v); }
// HeatDemand
  public static final Property CmHCA_HeaterNordic_output_12 = newProperty( Flags.SUMMARY|Flags.READONLY|Flags.OPERATOR|Flags.TRANSIENT, new BStatusNumeric( 0, BStatus.disabled), null);
  public BStatusNumeric getCmHCA_HeaterNordic_output_12() { return (BStatusNumeric) get(CmHCA_HeaterNordic_output_12); }
  public void setCmHCA_HeaterNordic_output_12(BStatusNumeric v) { set(CmHCA_HeaterNordic_output_12,v,null); }
  public void setCmHCA_HeaterNordic_output_12(float v) { getCmHCA_HeaterNordic_output_12().setValue(v); }

  // macro parameter
// RWT-03: return water temperature maximum setpoint
  public static final Property CmHCA_HeaterNordic_parameter_566 = newProperty( 0, BDouble.make( 85 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_566() { return getDouble(CmHCA_HeaterNordic_parameter_566); }
  public void setCmHCA_HeaterNordic_parameter_566(double v) { setDouble(CmHCA_HeaterNordic_parameter_566,v,null); }
// RWT-04: return water temperature P-band
  public static final Property CmHCA_HeaterNordic_parameter_62 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(25)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmHCA_HeaterNordic_parameter_62() { return getDouble(CmHCA_HeaterNordic_parameter_62); }
  public void setCmHCA_HeaterNordic_parameter_62(double v) { setDouble(CmHCA_HeaterNordic_parameter_62,v,null); }
// RWT-05: return water temperature setpoint ramp
  public static final Property CmHCA_HeaterNordic_parameter_565 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("degrees kelvin per minute"))));
  public double getCmHCA_HeaterNordic_parameter_565() { return getDouble(CmHCA_HeaterNordic_parameter_565); }
  public void setCmHCA_HeaterNordic_parameter_565(double v) { setDouble(CmHCA_HeaterNordic_parameter_565,v,null); }
// RWT-06: return water temperature frost protection limit
  public static final Property CmHCA_HeaterNordic_parameter_567 = newProperty( 0, BDouble.make( 6 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_567() { return getDouble(CmHCA_HeaterNordic_parameter_567); }
  public void setCmHCA_HeaterNordic_parameter_567(double v) { setDouble(CmHCA_HeaterNordic_parameter_567,v,null); }
// SAT-01: supply air frost protection limit
  public static final Property CmHCA_HeaterNordic_parameter_568 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_568() { return getDouble(CmHCA_HeaterNordic_parameter_568); }
  public void setCmHCA_HeaterNordic_parameter_568(double v) { setDouble(CmHCA_HeaterNordic_parameter_568,v,null); }
// SAT-02: supply air frost protection limit delay
  public static final Property CmHCA_HeaterNordic_parameter_569 = newProperty( 0, BDouble.make( 600 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeaterNordic_parameter_569() { return getDouble(CmHCA_HeaterNordic_parameter_569); }
  public void setCmHCA_HeaterNordic_parameter_569(double v) { setDouble(CmHCA_HeaterNordic_parameter_569,v,null); }
// DME-01: damper enable or alarm delay
  public static final Property CmHCA_HeaterNordic_parameter_14 = newProperty( 0, BDouble.make( 300 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeaterNordic_parameter_14() { return getDouble(CmHCA_HeaterNordic_parameter_14); }
  public void setCmHCA_HeaterNordic_parameter_14(double v) { setDouble(CmHCA_HeaterNordic_parameter_14,v,null); }
// HTG-01: calculated heating setpoint supply temp. min. setpoint
  public static final Property CmHCA_HeaterNordic_parameter_200 = newProperty( 0, BDouble.make( 20 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(150)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_200() { return getDouble(CmHCA_HeaterNordic_parameter_200); }
  public void setCmHCA_HeaterNordic_parameter_200(double v) { setDouble(CmHCA_HeaterNordic_parameter_200,v,null); }
// HTG-02: calculated heating setpoint supply temp. max. setpoint
  public static final Property CmHCA_HeaterNordic_parameter_201 = newProperty( 0, BDouble.make( 85 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(150)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_201() { return getDouble(CmHCA_HeaterNordic_parameter_201); }
  public void setCmHCA_HeaterNordic_parameter_201(double v) { setDouble(CmHCA_HeaterNordic_parameter_201,v,null); }
// HTG-03: calculated heating setpoint supply temp. start point
  public static final Property CmHCA_HeaterNordic_parameter_202 = newProperty( 0, BDouble.make( 50 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(150)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_202() { return getDouble(CmHCA_HeaterNordic_parameter_202); }
  public void setCmHCA_HeaterNordic_parameter_202(double v) { setDouble(CmHCA_HeaterNordic_parameter_202,v,null); }
// HTG-04: heat demand increase/ decrease value
  public static final Property CmHCA_HeaterNordic_parameter_203 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(10)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("degrees kelvin per minute"))));
  public double getCmHCA_HeaterNordic_parameter_203() { return getDouble(CmHCA_HeaterNordic_parameter_203); }
  public void setCmHCA_HeaterNordic_parameter_203(double v) { setDouble(CmHCA_HeaterNordic_parameter_203,v,null); }
// HTG-05: heating valve pos. diff. to calculate heating setpoint
  public static final Property CmHCA_HeaterNordic_parameter_204 = newProperty( 0, BDouble.make( 25 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(5)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmHCA_HeaterNordic_parameter_204() { return getDouble(CmHCA_HeaterNordic_parameter_204); }
  public void setCmHCA_HeaterNordic_parameter_204(double v) { setDouble(CmHCA_HeaterNordic_parameter_204,v,null); }
// OAT-03: low limit frost protection
  public static final Property CmHCA_HeaterNordic_parameter_21 = newProperty( 0, BDouble.make( 6 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(-50)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_21() { return getDouble(CmHCA_HeaterNordic_parameter_21); }
  public void setCmHCA_HeaterNordic_parameter_21(double v) { setDouble(CmHCA_HeaterNordic_parameter_21,v,null); }
// FEZ-01: frost protection RWT/ DAT setpoint raise
  public static final Property CmHCA_HeaterNordic_parameter_46 = newProperty( 0, BDouble.make( 15 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(50)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("kelvin"))));
  public double getCmHCA_HeaterNordic_parameter_46() { return getDouble(CmHCA_HeaterNordic_parameter_46); }
  public void setCmHCA_HeaterNordic_parameter_46(double v) { setDouble(CmHCA_HeaterNordic_parameter_46,v,null); }
// FEZ-02: raised setpoint decrease during frost recovery
  public static final Property CmHCA_HeaterNordic_parameter_47 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(20)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("degrees kelvin per minute"))));
  public double getCmHCA_HeaterNordic_parameter_47() { return getDouble(CmHCA_HeaterNordic_parameter_47); }
  public void setCmHCA_HeaterNordic_parameter_47(double v) { setDouble(CmHCA_HeaterNordic_parameter_47,v,null); }
// FEZ-03: valve position during frost protection
  public static final Property CmHCA_HeaterNordic_parameter_15 = newProperty( 0, BDouble.make( 100 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmHCA_HeaterNordic_parameter_15() { return getDouble(CmHCA_HeaterNordic_parameter_15); }
  public void setCmHCA_HeaterNordic_parameter_15(double v) { setDouble(CmHCA_HeaterNordic_parameter_15,v,null); }
// FEZ-04: valve raised position decrease during frost recovery
  public static final Property CmHCA_HeaterNordic_parameter_16 = newProperty( 0, BDouble.make( 10 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(1)), BFacets.make(BFacets.MAX, BDouble.make(100)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("percent"))));
  public double getCmHCA_HeaterNordic_parameter_16() { return getDouble(CmHCA_HeaterNordic_parameter_16); }
  public void setCmHCA_HeaterNordic_parameter_16(double v) { setDouble(CmHCA_HeaterNordic_parameter_16,v,null); }
// HPU-01: startup delay heating pump
  public static final Property CmHCA_HeaterNordic_parameter_237 = newProperty( 0, BDouble.make( 5 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(300)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeaterNordic_parameter_237() { return getDouble(CmHCA_HeaterNordic_parameter_237); }
  public void setCmHCA_HeaterNordic_parameter_237(double v) { setDouble(CmHCA_HeaterNordic_parameter_237,v,null); }
// HPU-02: pump switch off delay
  public static final Property CmHCA_HeaterNordic_parameter_13 = newProperty( 0, BDouble.make( 300 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(3600)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("second"))));
  public double getCmHCA_HeaterNordic_parameter_13() { return getDouble(CmHCA_HeaterNordic_parameter_13); }
  public void setCmHCA_HeaterNordic_parameter_13(double v) { setDouble(CmHCA_HeaterNordic_parameter_13,v,null); }
// ALM-03: AHU stops at pump off alarm at low outdoor temperature
  public static final Property CmHCA_HeaterNordic_parameter_19 = newProperty( 0, BDouble.make( 1 ), BFacets.make( BFacets.make(BFacets.MAX, BDouble.make(1)), BFacets.make(BFacets.MIN, BDouble.make(0)) ));
  public double getCmHCA_HeaterNordic_parameter_19() { return getDouble(CmHCA_HeaterNordic_parameter_19); }
  public void setCmHCA_HeaterNordic_parameter_19(double v) { setDouble(CmHCA_HeaterNordic_parameter_19,v,null); }
// DISP-01: calculated heating demand temperature setpoint
  public static final Property CmHCA_HeaterNordic_parameter_342 = newProperty( Flags.READONLY|Flags.TRANSIENT, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(0)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_342() { return getDouble(CmHCA_HeaterNordic_parameter_342); }
  public void setCmHCA_HeaterNordic_parameter_342(double v) { setDouble(CmHCA_HeaterNordic_parameter_342,v,null); }
// DISP-02: calculated return water temperature setpoint
  public static final Property CmHCA_HeaterNordic_parameter_340 = newProperty( Flags.READONLY|Flags.TRANSIENT, BDouble.make( 0 ), BFacets.make( BFacets.make( BFacets.make(BFacets.MIN, BDouble.make(0)), BFacets.make(BFacets.MAX, BDouble.make(0)) ),  BFacets.make(BFacets.UNITS, BUnit.getUnit("celsius"))));
  public double getCmHCA_HeaterNordic_parameter_340() { return getDouble(CmHCA_HeaterNordic_parameter_340); }
  public void setCmHCA_HeaterNordic_parameter_340(double v) { setDouble(CmHCA_HeaterNordic_parameter_340,v,null); }

  // macro inputs
// HeatCtrlSig
  public static final Property CmHCA_HeaterNordic_input_2 = newProperty( Flags.SUMMARY, new BStatusNumeric( 0 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_2() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_2); }
  public void setCmHCA_HeaterNordic_input_2(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_2,v,null); }
  public void setCmHCA_HeaterNordic_input_2(float v) { getCmHCA_HeaterNordic_input_2().setValue(v); }
// FilteredOATemp
  public static final Property CmHCA_HeaterNordic_input_1 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_1() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_1); }
  public void setCmHCA_HeaterNordic_input_1(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_1,v,null); }
  public void setCmHCA_HeaterNordic_input_1(float v) { getCmHCA_HeaterNordic_input_1().setValue(v); }
// FrostThermostat
  public static final Property CmHCA_HeaterNordic_input_3 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmHCA_HeaterNordic_input_3() { return (BStatusBoolean) get(CmHCA_HeaterNordic_input_3); }
  public void setCmHCA_HeaterNordic_input_3(BStatusBoolean v) { set(CmHCA_HeaterNordic_input_3,v,null); }
  public void setCmHCA_HeaterNordic_input_3(boolean v) { getCmHCA_HeaterNordic_input_3().setValue(v); }
// ReturnWaterTemp
  public static final Property CmHCA_HeaterNordic_input_5 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_5() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_5); }
  public void setCmHCA_HeaterNordic_input_5(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_5,v,null); }
  public void setCmHCA_HeaterNordic_input_5(float v) { getCmHCA_HeaterNordic_input_5().setValue(v); }
// RetTmpSpOcc
  public static final Property CmHCA_HeaterNordic_input_560 = newProperty( Flags.SUMMARY, new BStatusNumeric( 22 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_560() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_560); }
  public void setCmHCA_HeaterNordic_input_560(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_560,v,null); }
  public void setCmHCA_HeaterNordic_input_560(float v) { getCmHCA_HeaterNordic_input_560().setValue(v); }
// RetTmpSpUnOcc
  public static final Property CmHCA_HeaterNordic_input_561 = newProperty( Flags.SUMMARY, new BStatusNumeric( 15 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_561() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_561); }
  public void setCmHCA_HeaterNordic_input_561(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_561,v,null); }
  public void setCmHCA_HeaterNordic_input_561(float v) { getCmHCA_HeaterNordic_input_561().setValue(v); }
// RetSetpointSelect
  public static final Property CmHCA_HeaterNordic_input_562 = newProperty( Flags.SUMMARY, new BStatusNumeric( 3 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_562() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_562); }
  public void setCmHCA_HeaterNordic_input_562(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_562,v,null); }
  public void setCmHCA_HeaterNordic_input_562(float v) { getCmHCA_HeaterNordic_input_562().setValue(v); }
// SupplyAirTemp
  public static final Property CmHCA_HeaterNordic_input_563 = newProperty( Flags.SUMMARY, new BStatusNumeric( 999 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_563() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_563); }
  public void setCmHCA_HeaterNordic_input_563(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_563,v,null); }
  public void setCmHCA_HeaterNordic_input_563(float v) { getCmHCA_HeaterNordic_input_563().setValue(v); }
// FailCmdAlarm
  public static final Property CmHCA_HeaterNordic_input_552 = newProperty( Flags.SUMMARY, new BStatusBoolean( false ), null);
  public BStatusBoolean getCmHCA_HeaterNordic_input_552() { return (BStatusBoolean) get(CmHCA_HeaterNordic_input_552); }
  public void setCmHCA_HeaterNordic_input_552(BStatusBoolean v) { set(CmHCA_HeaterNordic_input_552,v,null); }
  public void setCmHCA_HeaterNordic_input_552(boolean v) { getCmHCA_HeaterNordic_input_552().setValue(v); }
// PlantMode
  public static final Property CmHCA_HeaterNordic_input_6 = newProperty( Flags.SUMMARY, new BStatusNumeric( 4 ), null);
  public BStatusNumeric getCmHCA_HeaterNordic_input_6() { return (BStatusNumeric) get(CmHCA_HeaterNordic_input_6); }
  public void setCmHCA_HeaterNordic_input_6(BStatusNumeric v) { set(CmHCA_HeaterNordic_input_6,v,null); }
  public void setCmHCA_HeaterNordic_input_6(float v) { getCmHCA_HeaterNordic_input_6().setValue(v); }

  public static final Property enabledIn = newProperty( 0, new BStatusBoolean( true ), null);
  public BStatusBoolean getEnabledIn() { return (BStatusBoolean) get(enabledIn); }
  public void setEnabledIn(BStatusBoolean v) { set(enabledIn,v,null); }
  public void setEnabledIn(boolean v) { getEnabledIn().setValue(v); }


  public static final Property versionInfo = newProperty( Flags.READONLY, BString.make( "51.02 [21] 09.11.2015-14:15").getString(), null);
  public String getVersionInfo() { return getString(versionInfo); }
  public void setVersionInfo(String v) { setString(versionInfo,v,null); }


  public static final Property macroId = newProperty( Flags.READONLY|Flags.HIDDEN, BString.make( "5973979754190026861").getString(), null);
  public String getMacroId() { return getString(macroId); }
  public void setMacroId(String v) { setString(macroId,v,null); }

  // TYPE declaration
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCmHCA_HeaterNordic.class);


  // cycle time
  protected long lastSystemTicks = 0;
  protected float lastCycleTime = 0;

  // implementation of IEnvironment interface
  public float getCycleTime()
  { 
    return lastCycleTime;
  }

  public void started() throws Exception
  {
    super.started();
    Start();
    lastSystemTicks = ControlFunction.GetSystemTicks();

  }

  public int getHighestPriority(int inputId)
  {
    return 0;
  }

  public float getPriorityValue(int inputId, int priorityIndex)
  {
    return Float.NaN;
  }

  public void setPriorityValue(int outputId, int priorityIndex, float value, boolean setNull)
  {
  }

  // helper functions
  private float   BooleanToFloat(boolean value) { return value ? 1 : 0; }
  private int     FloatToInt    (float   value) { return (int) value;   }
  private boolean FloatToBoolean(float   value) { return value >= 0.5F; }

  // class variables
  protected CfHysteresis func_24 = new CfHysteresis();
  protected CfEqual func_40 = new CfEqual();
  protected CfCompare1 func_64 = new CfCompare1();
  protected CfEqual func_381 = new CfEqual();
  protected CmIDT05 func_423 = new CmIDT05();
  protected CmIDT05 func_438 = new CmIDT05();
  protected CfIdt func_461 = new CfIdt();
  protected CfIdt func_489 = new CfIdt();
  protected CfIdt func_490 = new CfIdt();
  protected CmIDT05 func_556 = new CmIDT05();
  protected CmIDT05 func_579 = new CmIDT05();
  protected CmFrostProtectionNordic func_585 = new CmFrostProtectionNordic();
  protected CfIdt func_586 = new CfIdt();
  protected CmHeating_Signal_Setup func_205 = new CmHeating_Signal_Setup();
  protected CmReturn_Water_Setpoint_ControlNordic func_574 = new CmReturn_Water_Setpoint_ControlNordic();
  protected CmHC_Valve func_27 = new CmHC_Valve();
  protected CmDamper_Enable func_486 = new CmDamper_Enable();
  protected CmDemand_As_A_Function_Of_The_Actual_Valve_Position func_29 = new CmDemand_As_A_Function_Of_The_Actual_Valve_Position();
  protected CmPumpEnable func_554 = new CmPumpEnable();
  protected CfIdt func_473 = new CfIdt();

  // default constructor
  public BCmHCA_HeaterNordic()
  {
    func_40.configure(8);
    func_381.configure(2);
  }

  public void Start()
  {
    func_24.Start();
    func_423.Start();
    func_438.Start();
    func_556.Start();
    func_579.Start();
    func_585.Start();
    func_205.Start();
    func_574.Start();
    func_27.Start();
    func_486.Start();
    func_29.Start();
    func_554.Start();
  }
}
