/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.alarm;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.logging.Logger;

import javax.baja.alarm.AlarmDbConnection;
import javax.baja.alarm.BAckState;
import javax.baja.alarm.BAlarmRecord;
import javax.baja.alarm.BAlarmService;
import javax.baja.alarm.BAlarmSource;
import javax.baja.alarm.BIAlarmSource;
import javax.baja.bacnet.BBacnetNetwork;
import javax.baja.bacnet.datatypes.BBacnetObjectIdentifier;
import javax.baja.control.BControlPoint;
import javax.baja.naming.BOrd;
import javax.baja.naming.BOrdList;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BComponent;
import javax.baja.sys.BObject;
import javax.baja.sys.Context;
import javax.baja.sys.Cursor;
import javax.baja.sys.IterableCursor;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.honeywell.honplantcontroller.hmi.comm.BIHonPlantControllerHMIInterface;
import com.honeywell.honplantcontroller.hmi.messages.RequestAlarmAckMessage;
import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;
import com.tridium.alarm.BConsoleRecipient;
import com.tridium.alarm.db.file.FileAlarmDbConnection;

@NiagaraType
public class BHMIAlarmConsoleRecipient extends BConsoleRecipient implements IAlarmResolver {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.alarm.BHMIAlarmConsoleRecipient(2979906276)1.0$ @*/
/* Generated Tue Mar 01 19:43:21 IST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHMIAlarmConsoleRecipient.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/
  @Override
  public Optional<Integer> getUnAckAlarmCount() {
    return alarmCount(getUnAckAllAlarms());
  } 
  
  @Override
  public Optional<Integer> getUrgentUnAckAlarmCount() {
    return alarmCount(getUrgentUnAckAlarms());
  }
  
  @Override
  public Optional<Integer> getHighUnAckAlarmCount() {
    return alarmCount(getHighUnAckAlarms());
  }
  
  @Override
  public Optional<Integer> getLowUnAckAlarmCount() {
    return alarmCount(getLowUnAckAlarms());
  }
  
  @Override
  public Optional<Integer> getAlarmCount() {
    return alarmCount(getAlarms());
  }
  
  @Override
  public Optional<Integer> getUrgentAlarmCount() {
    return alarmCount(getUrgentAlarms());
  }

  @Override
  public Optional<Integer> getHighAlarmCount() {
    return alarmCount(getHighAlarms());
  }

  @Override
  public Optional<Integer> getLowAlarmCount() {
    return alarmCount(getLowAlarms());
  }

  protected Optional<Integer> alarmCount(Optional<List<BAlarmRecord>> alarmRecords) {
    Optional<Integer> alarmCount = Optional.empty();
    if(alarmRecords.isPresent()) {
      alarmCount = Optional.of(alarmRecords.get().size());
    }
    return alarmCount;
  }

  @Override
  public Optional<List<BAlarmRecord>> getUrgentUnAckAlarms() {
	  return Optional.of(sortedUnackedAlarms.get(URGENT_UNACK_ALARMS));
  }

  @Override
  public Optional<List<BAlarmRecord>> getHighUnAckAlarms() {
	  return Optional.of(sortedUnackedAlarms.get(HIGH_UNACK_ALARMS));
  }

  @Override
  public Optional<List<BAlarmRecord>> getLowUnAckAlarms() {
	  return Optional.of(sortedUnackedAlarms.get(LOW_UNACK_ALARMS));
  }

  @Override
  public Optional<List<BAlarmRecord>> getUnAckAllAlarms() {
	  return Optional.of(sortedUnackedAlarms.get(ALL_UNACK_ALARMS));
  }

  @Override
  public Optional<List<BAlarmRecord>> getUrgentAlarms() {
	  return Optional.of(sortedActiveAlarms.get(URGENT_ACTIVE_ALARMS));

  }

  @Override
  public Optional<List<BAlarmRecord>> getHighAlarms() {
	  return Optional.of(sortedActiveAlarms.get(HIGH_ACTIVE_ALARMS));
  }

  @Override
  public Optional<List<BAlarmRecord>>getLowAlarms() {
	  return Optional.of(sortedActiveAlarms.get(LOW_ACTIVE_ALARMS));
  }

  @Override
  public Optional<List<BAlarmRecord>> getAlarms() {
	  return Optional.of(sortedActiveAlarms.get(ALL_ACTIVE_ALARMS));
  }
  
  	@Override
	public void collectActiveAlarms(BHonPlantControllerHMI hmiDevice) {
  		if(null == hmiDevice) {
  			return;
  		}
		Optional<AlarmDbConnection> alarmDbOptional = getAlarmDbConnection();
		if (!alarmDbOptional.isPresent()) {
			return;
		}
		AlarmDbConnection alarmDbConnection = alarmDbOptional.get();

		try {
			List<BAlarmRecord> urgentActiveAlarms = new ArrayList<>();
			List<BAlarmRecord> highActiveAlarms = new ArrayList<>();
			List<BAlarmRecord> lowActiveAlarms = new ArrayList<>();
			List<BAlarmRecord> allActiveAlarms = new ArrayList<>();
			setTimeBeforeGettingAlarmSources(MessageFormat.format("The time before fetching Alarm Sources {0}", BAbsTime.now().getMillis()));
			List<BAlarmSource> openAlarms = getSortedAlarmSources();
			setTimeAfterGettingAlarmSources(MessageFormat.format("The time after fetching Alarm Sources {0}", BAbsTime.now().getMillis()));
			updateActiveAlarmRecordsAsPerAlarmPriority(hmiDevice, alarmDbConnection, urgentActiveAlarms, highActiveAlarms, lowActiveAlarms,
					allActiveAlarms, openAlarms);
			
			sortedActiveAlarms.put(URGENT_ACTIVE_ALARMS, urgentActiveAlarms);
			sortedActiveAlarms.put(HIGH_ACTIVE_ALARMS, highActiveAlarms);
			sortedActiveAlarms.put(LOW_ACTIVE_ALARMS, lowActiveAlarms);
			sortedActiveAlarms.put(ALL_ACTIVE_ALARMS, allActiveAlarms);

		} catch (Exception e) {
			LOG.warning("could not retrieve alarms " + e.getMessage());
		}
	}

	private void updateActiveAlarmRecordsAsPerAlarmPriority(BHonPlantControllerHMI hmiDevice, AlarmDbConnection alarmDbConnection,
			List<BAlarmRecord> urgentActiveAlarms, List<BAlarmRecord> highActiveAlarms, List<BAlarmRecord> lowActiveAlarms,
			List<BAlarmRecord> allActiveAlarms, List<BAlarmSource> openAlarms) throws IOException {
		setTimeBeforeStartingRecurssionOfAlarmRecords(MessageFormat.format("The time before starting recursion of records is {0}", BAbsTime.now().getMillis()));
		int recordCount = 0;
		int totalRecordCount = 0;
		for(BAlarmSource source : openAlarms) {
			BOrdList sourceOrdList = source.getSourceOrdList();
			Cursor<BAlarmRecord> bAlarmRecordCursor = alarmDbConnection.getAlarmsForSource(sourceOrdList);
			while (bAlarmRecordCursor.next() && recordCount < hmiDevice.getHmiAlarmDbLimit()) {
				final BAlarmRecord alarmRecord = bAlarmRecordCursor.get();
				if (getEventObjectId(alarmRecord) != null && !alarmRecord.isNormal()) {
					allActiveAlarms.add((BAlarmRecord) alarmRecord.newCopy());
					sortAlarmRecordAsPerAlarmPriorityConfig(urgentActiveAlarms, highActiveAlarms, lowActiveAlarms, alarmRecord);
					recordCount++;
				}
				totalRecordCount++;
			}
		}
		setTimeAfterCompletingRecurssionOfAlarmRecords(
				MessageFormat.format("The time after completing recursion of {0} records to fetch {1} records is {2}", totalRecordCount,
						recordCount, BAbsTime.now().getMillis()));
	}
  	
  	@Override
	public void collectUnackedAlarms(BHonPlantControllerHMI hmiDevice) {
  		if(null == hmiDevice) {
  			return;
  		}
		Optional<AlarmDbConnection> alarmDbOptional = getAlarmDbConnection();
		if (!alarmDbOptional.isPresent()) {
			return;
		}
		AlarmDbConnection alarmDbConnection = alarmDbOptional.get();

		try {
			List<BAlarmRecord> urgentUnAckAlarms = new ArrayList<>();
			List<BAlarmRecord> highUnAckAlarms = new ArrayList<>();
			List<BAlarmRecord> lowUnAckAlarms = new ArrayList<>();
			List<BAlarmRecord> allUnAckAlarms = new ArrayList<>();
			setTimeBeforeGettingAlarmSources(MessageFormat.format("The time before fetching Alarm Sources {0}", BAbsTime.now().getMillis()));
			List<BAlarmSource> openAlarms = getSortedAlarmSources();
			setTimeAfterGettingAlarmSources(MessageFormat.format("The time after fetching Alarm Sources {0}", BAbsTime.now().getMillis()));
			updateUnackedAlarmRecordsAsPerAlarmPriority(hmiDevice, alarmDbConnection, urgentUnAckAlarms, highUnAckAlarms, lowUnAckAlarms,
					allUnAckAlarms, openAlarms);
			
			sortedUnackedAlarms.put(URGENT_UNACK_ALARMS, urgentUnAckAlarms);
			sortedUnackedAlarms.put(HIGH_UNACK_ALARMS, highUnAckAlarms);
			sortedUnackedAlarms.put(LOW_UNACK_ALARMS, lowUnAckAlarms);
			sortedUnackedAlarms.put(ALL_UNACK_ALARMS, allUnAckAlarms);
		} catch (Exception e) {
			LOG.warning("could not retrieve alarms " + e.getMessage());
		}
	}

	private void updateUnackedAlarmRecordsAsPerAlarmPriority(BHonPlantControllerHMI hmiDevice, AlarmDbConnection alarmDbConnection,
			List<BAlarmRecord> urgentUnAckAlarms, List<BAlarmRecord> highUnAckAlarms, List<BAlarmRecord> lowUnAckAlarms,
			List<BAlarmRecord> allUnAckAlarms, List<BAlarmSource> openAlarms) throws IOException {
		setTimeBeforeStartingRecurssionOfAlarmRecords(
				MessageFormat.format("The time before starting recursion of records is {0}", BAbsTime.now().getMillis()));
		int recordCount = 0;
		int totalRecordCount = 0;
		for (BAlarmSource source : openAlarms) {
			BOrdList sourceOrdList = source.getSourceOrdList();
			Cursor<BAlarmRecord> bAlarmRecordCursor = alarmDbConnection.getAlarmsForSource(sourceOrdList);
			while (bAlarmRecordCursor.next() && recordCount < hmiDevice.getHmiAlarmDbLimit()) {
				final BAlarmRecord alarmRecord = bAlarmRecordCursor.get();
				if (getEventObjectId(alarmRecord) != null && !alarmRecord.isAcknowledged()) {
					allUnAckAlarms.add((BAlarmRecord) alarmRecord.newCopy());
					sortAlarmRecordAsPerAlarmPriorityConfig(urgentUnAckAlarms, highUnAckAlarms, lowUnAckAlarms, alarmRecord);
					recordCount++;
				}
				totalRecordCount++;
			}
		}
		setTimeAfterCompletingRecurssionOfAlarmRecords(
				MessageFormat.format("The time after completing recursion of {0} records to fetch {1} records is {2}", totalRecordCount, recordCount,
						BAbsTime.now().getMillis()));
	}

	private void sortAlarmRecordAsPerAlarmPriorityConfig(List<BAlarmRecord> urgentAlarms, List<BAlarmRecord> highAlarms, List<BAlarmRecord> lowAlarms,
			final BAlarmRecord alarmRecord) {
		Optional<BAlarmPriorityConfig> alarmPriorityConfigOptional = getAlarmPriorityConfig();
		if (alarmPriorityConfigOptional.isPresent()) {
			BAlarmPriorityConfig alarmPriorityConfig = alarmPriorityConfigOptional.get();
			int alarmPriority = alarmRecord.getPriority();
			if (alarmPriority >= alarmPriorityConfig.getUrgentAlarmPriority().getLowRange()
					&& alarmPriority <= alarmPriorityConfig.getUrgentAlarmPriority().getHighRange()) {
				urgentAlarms.add((BAlarmRecord) alarmRecord.newCopy());
			} else if (alarmPriority >= alarmPriorityConfig.getHighAlarmPriority().getLowRange()
					&& alarmPriority <= alarmPriorityConfig.getHighAlarmPriority().getHighRange()) {
				highAlarms.add((BAlarmRecord) alarmRecord.newCopy());
			} else if (alarmPriority >= alarmPriorityConfig.getLowAlarmPriority().getLowRange()
					&& alarmPriority <= alarmPriorityConfig.getLowAlarmPriority().getHighRange()) {
				lowAlarms.add((BAlarmRecord) alarmRecord.newCopy());
			}
		}
	}
	
	public List<BAlarmSource> getSortedAlarmSources() {
		ArrayList<BAlarmSource> alarmSources = new ArrayList<>();
		Cursor<BAlarmSource> c = this.getOpenAlarmSources(null);

		while (c.next())
			alarmSources.add(c.get());

		alarmSources.sort(new Comparator<BAlarmSource>() {
			@Override
			public int compare(BAlarmSource as1, BAlarmSource as2) {
				try {
					BAlarmRecord ar1 = as1.getCurrentAlarm();
					BAlarmRecord ar2 = as2.getCurrentAlarm();
					if (ar1 == null || ar2 == null)
						return 0;

					BAbsTime t1 = ar1.getTimestamp();
					BAbsTime t2 = ar2.getTimestamp();

					if (t1.isBefore(t2))
						return 1;
					else
						return -1;
				} catch (Exception ex) {
					return 0;
				}
			}
		});
		return alarmSources;
	}

  /**
   * Function for acknowledging alarms for HMI by given private transfer message {@link RequestAlarmAckMessage}
   */
  @Override
  public boolean acknowledgeAlarm(BIAlarmSource alarmSource, RequestAlarmAckMessage alarmAckMessage) {
    final String prefix = "BHMIAlarmConsoleRecipient.acknowledgeAlarm";
    if (null == alarmSource || null == alarmAckMessage) {
      LOG.warning(() -> MessageFormat.format(
          "{0} Could not acknowledge alarm because give alarm source is: {1} alarmAckMessage is {2}",
          prefix, alarmSource, alarmAckMessage));
      return false;
    }
    
    Optional<BAlarmService> alarmServiceOptional = getAlarmService();
    if (!alarmServiceOptional.isPresent()) {
      LOG.warning(() -> MessageFormat.format("{0} Could not acknowledge alarm because alarm service is missing",
          prefix));
      return false;
    }
    
    Optional<AlarmDbConnection> alarmDbOptional = getAlarmDbConnection();
    if (!alarmDbOptional.isPresent()) {
      LOG.warning(() -> MessageFormat.format("{0} Could not acknowledge alarm because alarmDB is missing",
          prefix));
      return false;
    }


    BAlarmService alarmSrv = alarmServiceOptional.get();
    AlarmDbConnection alarmDbConnection = alarmDbOptional.get();
    //needed for alarms that are routed
    BOrd alarmSourceOrdSlotPath = ((BComponent) alarmSource).getSlotPathOrd();

    //needed for alarms that are generated locally
    BOrd alarmSourceOrdNav = ((BComponent) alarmSource).getNavOrd();

    try {

      IterableCursor<BAlarmSource> cursor = (IterableCursor<BAlarmSource>) this.getOpenAlarmSources(null);
      Optional<BAlarmSource> alarmSourceOptional = cursor.stream()
          .filter(dbAlarmSource -> Arrays.stream(dbAlarmSource.getSourceOrdList().toArray())
              .anyMatch(ord -> ord.equals(alarmSourceOrdNav) || ord.equals(alarmSourceOrdSlotPath)))
          .findAny();
      if (alarmSourceOptional.isPresent()) {
        BOrdList ordListMatched = alarmSourceOptional.get().getSourceOrdList();
        Cursor<BAlarmRecord> alarmRecordsCursor = alarmDbConnection.getAlarmsForSource(ordListMatched);
        boolean success = false;

        //This should not acknowledge all alarms at once!!!
        while (alarmRecordsCursor.next()) {
          final BAlarmRecord alarmRecord = alarmRecordsCursor.get();

          if (alarmRecord.getTimestamp().toNormalizedTime().toString().equals(alarmAckMessage.getTimeStamp().toNormalizedTime().toString())) {
            LOG.finest(() -> MessageFormat.format("acknowledge alarm: {0}; alarmRecord.isAcknowledged(): {1}", alarmRecord.toString(), alarmRecord.isAcknowledged()));
            LOG.finest(() -> MessageFormat.format("alarmAckMessage: {0}; ", alarmAckMessage.toDebugString()));

            if (!alarmRecord.isAcknowledged()) {
              alarmRecord.setUser(alarmAckMessage.getAcknowledgementSource());
              alarmRecord.setAckTime(alarmAckMessage.getTimeOfAcknowledgement());
              alarmRecord.setAckState(BAckState.ackPending);
              alarmDbConnection.update(alarmRecord);
              alarmSrv.doAckAlarm(alarmRecord);
              success = true;
              break;
            }
          }
        }
        return success;
      }

    } catch (Exception e) {
      LOG.warning(() -> MessageFormat.format("could not acknowledge alarm {0}", e));
    }

    return false;
  }

  protected Optional<AlarmDbConnection> getAlarmDbConnection() {

    BAlarmService alarmSer = (BAlarmService) Sys.getService(BAlarmService.TYPE);
    try (AlarmDbConnection alarmDbConnection = alarmSer.getAlarmDb().getDbConnection(Context.NULL);) {
      return Optional.of(alarmDbConnection);
    } catch (Exception e) {
      LOG.warning(" could not retrieve alarmDbConnection "+e.getMessage());
    }
    
    return Optional.empty();
  }

  protected static BComponent getEventObject(BAlarmRecord alarmRecord) {
    BComponent eventObject = null;
    BAlarmService base = (BAlarmService) Sys.getService(BAlarmService.TYPE);
    try {
      BObject alarmSource = alarmRecord.getSource().get(0).get(base);

      if(alarmSource instanceof BControlPoint) {
        return (BControlPoint) alarmSource;
      } else {
        if(alarmSource instanceof BComponent) {
          BComponent parent = alarmSource.asComplex().getParent().asComponent();
          if(parent instanceof BControlPoint) {
            return parent;
          }
        }
      }
    } catch (Exception e) {
      LOG.warning("[getEventObject] can not get event object, because of = " + e);
    }
    return eventObject;
  }

  protected static BBacnetObjectIdentifier getEventObjectId(BAlarmRecord alarmRecord) {
    BBacnetObjectIdentifier eventObjectId = null;

    BComponent eventObject = getEventObject(alarmRecord);
    if (eventObject instanceof BControlPoint) {
      BOrd ord = eventObject.getOrdInSession();
      eventObjectId = BBacnetNetwork.localDevice().lookupBacnetObjectId(ord);
    }

    LOG.finest("[getEventObjectId] eventObjectId = " + eventObjectId);
    return eventObjectId;
  }

  protected Optional<BAlarmPriorityConfig> getAlarmPriorityConfig() {
    Optional<BAlarmPriorityConfig> result = Optional.empty();
    BIHonPlantControllerHMIInterface hmiInterface =  BIHonPlantControllerHMIInterface.getHonPlantControllerHMIInterface();
    if(hmiInterface != null) {
      return Optional.of(hmiInterface.getHMIDevice().getAlarmPriorityConfiguration());
    }

    return result;
  }

  protected Optional<BAlarmService> getAlarmService() {
    if(null == alarmService) {
      alarmService = (BAlarmService)(Sys.getService(BAlarmService.TYPE));
    }
    return Optional.of(alarmService);
  }

  private BAlarmService alarmService;

  public static final String URGENT_ALARM_CLASS_NAME = "Urgent"; 
  public static final String HIGH_ALARM_CLASS_NAME = "High"; 
  public static final String LOW_ALARM_CLASS_NAME = "Low";
  
  public static final int URGENT_ALARM_START_PRIORITY = 0;
  public static final int URGENT_ALARM_END_PRIORITY = 80;
  
  public static final int HIGH_ALARM_START_PRIORITY = 81; 
  public static final int HIGH_ALARM_END_PRIORITY = 160;
  
  public static final int LOW_ALARM_START_PRIORITY = 161;
  public static final int LOW_ALARM_END_PRIORITY = 255;
  
  private Map<String, List<BAlarmRecord>> sortedActiveAlarms = new HashMap<>();
  private Map<String, List<BAlarmRecord>> sortedUnackedAlarms = new HashMap<>();
  private static final String URGENT_UNACK_ALARMS = "URGENT_UNACK_ALARMS";
  private static final String HIGH_UNACK_ALARMS = "HIGH_UNACK_ALARMS";
  private static final String LOW_UNACK_ALARMS = "LOW_UNACK_ALARMS";
  private static final String ALL_UNACK_ALARMS = "ALL_UNACK_ALARMS";
  private static final String URGENT_ACTIVE_ALARMS = "URGENT_ACTIVE_ALARMS";
  private static final String HIGH_ACTIVE_ALARMS = "HIGH_ACTIVE_ALARMS";
  private static final String LOW_ACTIVE_ALARMS = "LOW_ACTIVE_ALARMS";
  private static final String ALL_ACTIVE_ALARMS = "ALL_ACTIVE_ALARMS";  
  
	public String getTimeBeforeGettingAlarmSources() {
		return timeBeforeGettingAlarmSources;
	}

	public void setTimeBeforeGettingAlarmSources(String timeBeforeGettingAlarmSources) {
		this.timeBeforeGettingAlarmSources = timeBeforeGettingAlarmSources;
	}

	public String getTimeAfterGettingAlarmSources() {
		return timeAfterGettingAlarmSources;
	}

	public void setTimeAfterGettingAlarmSources(String timeAfterGettingAlarmSources) {
		this.timeAfterGettingAlarmSources = timeAfterGettingAlarmSources;
	}

	public String getTimeBeforeStartingRecurssionOfAlarmRecords() {
		return timeBeforeStartingRecurssionOfAlarmRecords;
	}

	public void setTimeBeforeStartingRecurssionOfAlarmRecords(String timeBeforeStartingRecurssionOfAlarmRecords) {
		this.timeBeforeStartingRecurssionOfAlarmRecords = timeBeforeStartingRecurssionOfAlarmRecords;
	}

	public String getTimeAfterCompletingRecurssionOfAlarmRecords() {
		return timeAfterCompletingRecurssionOfAlarmRecords;
	}

	public void setTimeAfterCompletingRecurssionOfAlarmRecords(String timeAfterCompletingRecurssionOfAlarmRecords) {
		this.timeAfterCompletingRecurssionOfAlarmRecords = timeAfterCompletingRecurssionOfAlarmRecords;
	}

	private String timeBeforeGettingAlarmSources = "";
	private String timeAfterGettingAlarmSources = "";
	private String timeBeforeStartingRecurssionOfAlarmRecords = "";
	private String timeAfterCompletingRecurssionOfAlarmRecords = "";
  
  private static final Logger LOG = Logger.getLogger(BHMIAlarmConsoleRecipient.class.getSimpleName());
}
