/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.alarm;

import java.util.List;
import java.util.Optional;

import javax.baja.alarm.BAlarmRecord;
import javax.baja.alarm.BIAlarmSource;

import com.honeywell.honplantcontroller.hmi.messages.RequestAlarmAckMessage;
import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;


/**
 * Abstraction Layer to console recipient for unit testing
 */
public interface IAlarmResolver {

    public Optional<Integer> getUnAckAlarmCount();

    public Optional<Integer> getUrgentUnAckAlarmCount();

    public Optional<Integer> getHighUnAckAlarmCount();

    public Optional<Integer> getLowUnAckAlarmCount();

    public Optional<Integer> getAlarmCount();

    public Optional<Integer> getUrgentAlarmCount();

    public Optional<Integer> getHighAlarmCount();

    public Optional<Integer> getLowAlarmCount();

    public Optional<List<BAlarmRecord>> getUrgentUnAckAlarms();

    public Optional<List<BAlarmRecord>> getHighUnAckAlarms();

    public Optional<List<BAlarmRecord>> getLowUnAckAlarms();

    public Optional<List<BAlarmRecord>> getUnAckAllAlarms();

    public Optional<List<BAlarmRecord>> getUrgentAlarms();

    public Optional<List<BAlarmRecord>> getHighAlarms();

    public Optional<List<BAlarmRecord>> getLowAlarms();

    public Optional<List<BAlarmRecord>> getAlarms();
    
    public boolean acknowledgeAlarm(BIAlarmSource alarmSource, RequestAlarmAckMessage alarmAckMessage);
    
    public void collectActiveAlarms(BHonPlantControllerHMI hmiDevice);
    
    public void collectUnackedAlarms(BHonPlantControllerHMI hmiDevice);
    
    public String getTimeBeforeGettingAlarmSources();
    
    public String getTimeAfterGettingAlarmSources();
    
    public String getTimeBeforeStartingRecurssionOfAlarmRecords();
    
    public String getTimeAfterCompletingRecurssionOfAlarmRecords();
    
    

}
