/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/
package com.honeywell.honplantcontroller.hmi.comm;


import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.logging.Level;

import javax.baja.bacnet.BBacnetDevice;
import javax.baja.bacnet.BBacnetNetwork;
import javax.baja.bacnet.BacnetException;
import javax.baja.bacnet.datatypes.BBacnetObjectIdentifier;
import javax.baja.bacnet.enums.BBacnetObjectType;
import javax.baja.bacnet.enums.BBacnetPropertyIdentifier;
import javax.baja.bacnet.io.BBacnetComm;

import javax.baja.naming.BOrd;
import javax.baja.nre.annotations.Facet;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BComponent;
import javax.baja.sys.BObject;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.IPropertyValidator;
import javax.baja.sys.LocalizableRuntimeException;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.sys.Validatable;
import javax.baja.util.BFormat;

import com.honeywell.honplantcontroller.hmi.service.BHMIDevice;
import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;
import com.honeywell.honplantcontroller.hmi.utils.BHonPlantControllerHMIInitializationHandler;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
import com.tridium.bacnet.asn.AsnOutputStream;
import com.tridium.bacnet.enums.BBacnetMstpBaudRate;
import com.tridium.bacnet.stack.BBacnetStack;
import com.tridium.bacnet.stack.network.BBacnetNetworkLayer;
import com.tridium.bacnet.stack.network.BNetworkPort;

@NiagaraProperty(
  name = "networkNumber",
  type = "int",
  defaultValue = "2",
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)
@NiagaraProperty(
  name = "objectId",
  type = "BBacnetObjectIdentifier",
  defaultValue = "BBacnetObjectIdentifier.make(BBacnetObjectType.DEVICE, 100)",
  facets = {
    @Facet("BBacnetObjectType.getObjectIdFacets(BBacnetObjectType.DEVICE)")
  },
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE | Flags.HIDDEN
)
@NiagaraProperty(
	name = "HMIInstanceId",
	type = "BBacnetObjectIdentifier",
	defaultValue = "BBacnetObjectIdentifier.make(BBacnetObjectType.DEVICE, 99)",
	facets = {
	@Facet("BBacnetObjectType.getObjectIdFacets(BBacnetObjectType.DEVICE)")
	},
	flags = Flags.DEFAULT_ON_CLONE
)
@NiagaraProperty(
  name = "portName",
  type = "String",
  defaultValue = "BHonPlantControllerBacnetProperties.PORT_NAME",
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)
@NiagaraProperty(
  name = "baudRate",
  type = "BBacnetMstpBaudRate",
  defaultValue = "BBacnetMstpBaudRate.baud_115200",
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)
@NiagaraProperty(
  name = "mstpAddresse",
  type = "int",
  defaultValue = "0",
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)
@NiagaraProperty(
        name = "bacnetNetworkOrd",
        type = "BOrd",
        defaultValue = "BOrd.make(\"station:|slot:/Drivers/BacnetNetwork\")",
        flags = Flags.DEFAULT_ON_CLONE | Flags.READONLY
)
@NiagaraType
public class BHonPlantControllerBacnetProperties extends BComponent {
  
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.comm.BHonPlantControllerBacnetProperties(3855898803)1.0$ @*/
/* Generated Thu Apr 20 10:58:20 IST 2023 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "networkNumber"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code networkNumber} property.
   * @see #getNetworkNumber
   * @see #setNetworkNumber
   */
  public static final Property networkNumber = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, 2, null);
  
  /**
   * Get the {@code networkNumber} property.
   * @see #networkNumber
   */
  public int getNetworkNumber() { return getInt(networkNumber); }
  
  /**
   * Set the {@code networkNumber} property.
   * @see #networkNumber
   */
  public void setNetworkNumber(int v) { setInt(networkNumber, v, null); }

////////////////////////////////////////////////////////////////
// Property "objectId"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code objectId} property.
   * @see #getObjectId
   * @see #setObjectId
   */
  public static final Property objectId = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE | Flags.HIDDEN, BBacnetObjectIdentifier.make(BBacnetObjectType.DEVICE, 100), BBacnetObjectType.getObjectIdFacets(BBacnetObjectType.DEVICE));
  
  /**
   * Get the {@code objectId} property.
   * @see #objectId
   */
  public BBacnetObjectIdentifier getObjectId() { return (BBacnetObjectIdentifier)get(objectId); }
  
  /**
   * Set the {@code objectId} property.
   * @see #objectId
   */
  public void setObjectId(BBacnetObjectIdentifier v) { set(objectId, v, null); }

////////////////////////////////////////////////////////////////
// Property "HMIInstanceId"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code HMIInstanceId} property.
   * @see #getHMIInstanceId
   * @see #setHMIInstanceId
   */
  public static final Property HMIInstanceId = newProperty(Flags.DEFAULT_ON_CLONE, BBacnetObjectIdentifier.make(BBacnetObjectType.DEVICE, 99), BBacnetObjectType.getObjectIdFacets(BBacnetObjectType.DEVICE));
  
  /**
   * Get the {@code HMIInstanceId} property.
   * @see #HMIInstanceId
   */
  public BBacnetObjectIdentifier getHMIInstanceId() { return (BBacnetObjectIdentifier)get(HMIInstanceId); }
  
  /**
   * Set the {@code HMIInstanceId} property.
   * @see #HMIInstanceId
   */
  public void setHMIInstanceId(BBacnetObjectIdentifier v) { set(HMIInstanceId, v, null); }

////////////////////////////////////////////////////////////////
// Property "portName"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code portName} property.
   * @see #getPortName
   * @see #setPortName
   */
  public static final Property portName = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, BHonPlantControllerBacnetProperties.PORT_NAME, null);
  
  /**
   * Get the {@code portName} property.
   * @see #portName
   */
  public String getPortName() { return getString(portName); }
  
  /**
   * Set the {@code portName} property.
   * @see #portName
   */
  public void setPortName(String v) { setString(portName, v, null); }

////////////////////////////////////////////////////////////////
// Property "baudRate"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code baudRate} property.
   * @see #getBaudRate
   * @see #setBaudRate
   */
  public static final Property baudRate = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, BBacnetMstpBaudRate.baud_115200, null);
  
  /**
   * Get the {@code baudRate} property.
   * @see #baudRate
   */
  public BBacnetMstpBaudRate getBaudRate() { return (BBacnetMstpBaudRate)get(baudRate); }
  
  /**
   * Set the {@code baudRate} property.
   * @see #baudRate
   */
  public void setBaudRate(BBacnetMstpBaudRate v) { set(baudRate, v, null); }

////////////////////////////////////////////////////////////////
// Property "mstpAddresse"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code mstpAddresse} property.
   * @see #getMstpAddresse
   * @see #setMstpAddresse
   */
  public static final Property mstpAddresse = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, 0, null);
  
  /**
   * Get the {@code mstpAddresse} property.
   * @see #mstpAddresse
   */
  public int getMstpAddresse() { return getInt(mstpAddresse); }
  
  /**
   * Set the {@code mstpAddresse} property.
   * @see #mstpAddresse
   */
  public void setMstpAddresse(int v) { setInt(mstpAddresse, v, null); }

////////////////////////////////////////////////////////////////
// Property "bacnetNetworkOrd"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code bacnetNetworkOrd} property.
   * @see #getBacnetNetworkOrd
   * @see #setBacnetNetworkOrd
   */
  public static final Property bacnetNetworkOrd = newProperty(Flags.DEFAULT_ON_CLONE | Flags.READONLY, BOrd.make("station:|slot:/Drivers/BacnetNetwork"), null);
  
  /**
   * Get the {@code bacnetNetworkOrd} property.
   * @see #bacnetNetworkOrd
   */
  public BOrd getBacnetNetworkOrd() { return (BOrd)get(bacnetNetworkOrd); }
  
  /**
   * Set the {@code bacnetNetworkOrd} property.
   * @see #bacnetNetworkOrd
   */
  public void setBacnetNetworkOrd(BOrd v) { set(bacnetNetworkOrd, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPlantControllerBacnetProperties.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/
	@Override
	public void changed(Property property, Context context) {
		if (!isRunning() || !Sys.isStation()) {
			return;
		}
		super.changed(property, context);
		if (property.equals(HMIInstanceId)) {
			setHmiInstanceChanged(true);
			BHonPlantControllerHMI hmiDevice = getHMIService();
			if (null == hmiDevice) {
				return;
			}
			hmiDevice.addHMIEntryInBacnetNetwork();
			if(null != hmiDevice.getHMICommhandler()) {
				hmiDevice.getHMICommhandler().sendInstanceidToHMI(getHMIInstanceId(), hmiDevice, false);
			}
		}
	}

  public BHonPlantControllerBacnetProperties() {
  }

  public static BHonPlantControllerBacnetProperties make(/*BOrd ord*/) {
    BHonPlantControllerBacnetProperties honPlantControllerBacnetProperties = new BHonPlantControllerBacnetProperties();
    honPlantControllerBacnetProperties.setFlags(networkNumber, honPlantControllerBacnetProperties.getFlags(networkNumber) & ~Flags.READONLY);
    honPlantControllerBacnetProperties.setFlags(objectId, honPlantControllerBacnetProperties.getFlags(objectId) & ~Flags.READONLY);
    honPlantControllerBacnetProperties.setFlags(mstpAddresse, honPlantControllerBacnetProperties.getFlags(mstpAddresse) & ~Flags.READONLY);
    honPlantControllerBacnetProperties.setFlags(baudRate, honPlantControllerBacnetProperties.getFlags(baudRate));
    honPlantControllerBacnetProperties.setFlags(portName, honPlantControllerBacnetProperties.getFlags(portName));

    return honPlantControllerBacnetProperties;
  }

  @Override
  public void started() throws Exception {
    super.started();
    if (Sys.atSteadyState()) {
      atSteadyState();
    }
  }

  @Override
  public void atSteadyState() throws Exception {

    super.atSteadyState();
  }

  public void setBacnetSettings(BBacnetNetwork bacnetNetwork) {

    if (bacnetNetwork == null) {
      HonPlantControllerHMILogger.checkAndLog(Level.WARNING, "can not set BacnetSettings on a null BacnetNetwork");
      return;
    }

    bacnetNetwork.getLocalDevice().setObjectId(getObjectId());
  }

  public boolean createHMIPort(BBacnetNetwork bacnetNetwork) {

    if (bacnetNetwork == null) {
      HonPlantControllerHMILogger.checkAndLog(Level.WARNING, "can not set HMI-Port on a null BacnetNetwork");
      return false;
    }

    BNetworkPort networkPort = createMstpPort();
    BBacnetNetworkLayer bacnetNetworkLayer = ((BBacnetStack) bacnetNetwork.getBacnetComm()).getNetwork();
    Property mstNetworkPortProperty = bacnetNetworkLayer.add(MSTP_PORT_NAME, networkPort);
    if (HonPlantControllerBacnetUtils.isRunningOnController())
      networkPort.setEnabled(true);
    bacnetNetworkLayer.setDisplayName(mstNetworkPortProperty,
        BFormat.make("%lexicon(honPlantControllerHMI:HMIMstpPort)%"), Context.NULL);

    return true;
  }

  public BNetworkPort createMstpPort() {
    BNetworkPort mstpPort = new BNetworkPort();

    BHMIBacnetMstpLinkLayer mstpLinkLayer = new BHMIBacnetMstpLinkLayer();
    mstpPort.setLink(mstpLinkLayer);

    mstpLinkLayer.setPortName(getPortName());
    mstpLinkLayer.setBaudRate(getBaudRate());
    mstpLinkLayer.setMstpAddress(getMstpAddresse());
    mstpLinkLayer.setMaxMaster(1);
    mstpPort.setNetworkNumber(getNetworkNumber());

    return mstpPort;
  }

  public BBacnetNetwork getBacnetNetwork() {

    try {
      BObject bObject = getBacnetNetworkOrd().resolve(Sys.getStation()).get();
      if (bObject instanceof BBacnetNetwork) {
        return (BBacnetNetwork) bObject;
      }
    } catch (Exception e) {
      return null;
    }
    return null;
  }

	public static boolean compareBacnetProperties(BHonPlantControllerBacnetProperties first, BHonPlantControllerBacnetProperties second) {

		if (first == null || second == null) {
			return false;
		}

		return !(!first.getObjectId().equals(second.getObjectId()) || (first.getNetworkNumber() != second.getNetworkNumber())
				|| !(first.getHMIInstanceId().equals(second.getHMIInstanceId())) || (first.getMstpAddresse() != second.getMstpAddresse())
				|| !first.getBacnetNetworkOrd().equals(second.getBacnetNetworkOrd()));
	}

  public static BHonPlantControllerBacnetProperties exportBacnetProperties(BBacnetNetwork bacnetNetwork, BNetworkPort hmiPort) {
    BHonPlantControllerBacnetProperties bacnetProperties = new BHonPlantControllerBacnetProperties();
    if (null == bacnetNetwork || null == hmiPort)
      return bacnetProperties;

    bacnetProperties.setObjectId(bacnetNetwork.getObjectId());
    bacnetProperties.setNetworkNumber(hmiPort.getNetworkNumber());
    bacnetProperties.setBacnetNetworkOrd(bacnetNetwork.getSlotPathOrd());
    bacnetProperties.setMstpAddresse(((BHMIBacnetMstpLinkLayer) hmiPort.getLink()).getMstpAddress());
    return bacnetProperties;
  }
  
	@Override
	public IPropertyValidator getPropertyValidator(Property property, Context context) {
		if (context != Context.decoding && property.equals(HMIInstanceId)) {
			return this::validatePropertyChanged;
		}
		return null;
	}

	protected void validatePropertyChanged(Validatable validatable, Context context) {
		BBacnetObjectIdentifier proposedHMIInst = (BBacnetObjectIdentifier) validatable.getProposedValue(HMIInstanceId);
		int proposedHMIInstNumber = proposedHMIInst.getInstanceNumber();
		validateAndThrowExceptionIfinstanceIdIsDuplicate(proposedHMIInstNumber);
	}

	public void validateAndThrowExceptionIfinstanceIdIsDuplicate(int proposedHMIInstNumber) {
		if (null == getBacnetNetwork()) {
			HonPlantControllerHMILogger.checkAndLog(Level.WARNING, "can not set BacnetSettings on a null BacnetNetwork");
			return;
		}

		BHonPlantControllerHMI hmiDevice = getHMIService();
		if (null == hmiDevice) {
			return;
		}

		if (!hmiDevice.getEnabled()) {
			return;
		}

		BBacnetNetwork network = getBacnetNetwork();
		if (null == network) {
			return;
		}

		BBacnetDevice[] devices = network.getDeviceList();
		for (int i = 0; i < devices.length; i++) {
			if (devices[i].getObjectId().getInstanceNumber() == proposedHMIInstNumber && !devices[i].getType().equals(BHMIDevice.TYPE)) {
				HonPlantControllerHMILogger.checkAndLog(Level.WARNING, "BacnetDevice " + devices[i] + " with instance id "
						+ devices[i].getObjectId().getInstanceNumber() + " is already present in the network.");
				throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.commission.duplicate.bacnet.device.instance.id",
						new Object[] { devices[i], devices[i].getObjectId().getInstanceNumber() });

			}
		}
	}
	
	public boolean isHmiInstanceChanged() {
		return isHmiInstanceChanged;
	}

	public void setHmiInstanceChanged(boolean isHmiInstanceChanged) {
		this.isHmiInstanceChanged = isHmiInstanceChanged;
	}
	
	private BHonPlantControllerHMI getHMIService() {
		BHonPlantControllerHMI hmiDevice = null;
		if (null != this.getParent()) {
			hmiDevice = (BHonPlantControllerHMI) this.getParent();
		}
		return hmiDevice;
	}
	 
  
	
  //Attributes

	
	private static final int LEFT_SHIFT_BACNET_DEVICE_ID_INT = 22;
	public static final String PORT_NAME = "HMI";
	public static final String MSTP_PORT_NAME = "HMIMstpPort";
	private boolean isHmiInstanceChanged =false;
}
