/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 * This file contains trade secrets of Honeywell International, Inc. No part
 * may be reproduced or transmitted in any form by any means or for any
 * purpose without the express written permission of Honeywell.
 */
package com.honeywell.honplantcontroller.hmi.enums;

public enum KeyExchangeState {
    Idle_State {
        @Override
        public KeyExchangeState nextState() {
            return CertExchange;
        }
        public KeyExchangeState fail() {
            return CertExchange;
        }
    },
    CertExchange{
        @Override
        public KeyExchangeState nextState() {
            return Start_ECDH;
        }
        public KeyExchangeState fail() {
            return this;
        }
    },
    Start_ECDH{
        @Override
        public KeyExchangeState nextState() {
            return Verify_MstSecret;
        }
        public KeyExchangeState fail() {
            return CertExchange;
        }
    },
    Verify_MstSecret{
        @Override
        public KeyExchangeState nextState() {
            return Change_Cipher_MstSecret;
        }
        public KeyExchangeState fail() {
            return this;
        }
    },
    Change_Cipher_MstSecret{
        @Override
        public KeyExchangeState nextState() {
            return Derive_Shared_Key;
        }
        public KeyExchangeState fail() {
            return this;
        }
    },
    Derive_Shared_Key{
        @Override
        public KeyExchangeState nextState() {
            return Verify_Shared_Key;
        }
        public KeyExchangeState fail() {
            return CertExchange;
        }
    },
    Verify_Shared_Key{
        @Override
        public KeyExchangeState nextState() {
            return Exchange_Cipher_Shared_Key;
        }
        public KeyExchangeState fail() {
            return this;
        }
    },
    Exchange_Cipher_Shared_Key{
        @Override
        public KeyExchangeState nextState() {
            return Success;
        }
        public KeyExchangeState fail() {
            return this;
        }
    },
    Success{
        @Override
        public KeyExchangeState nextState() {
            return this;
        }
        public KeyExchangeState fail() {
            return Idle_State;
        }
    };

    public KeyExchangeState nextState() {
        {
            return Idle_State;
        }
    }
    public KeyExchangeState fail() {
        return this;
    }
}
