/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.job;


import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CancellationException;
import java.util.logging.Level;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.baja.alarm.BAlarmClass;
import javax.baja.alarm.BAlarmPriorities;
import javax.baja.alarm.BAlarmRecipient;
import javax.baja.alarm.BAlarmService;
import javax.baja.bacnet.BBacnetNetwork;
import javax.baja.bacnet.enums.BBacnetCommControl;
import javax.baja.bacnet.io.BBacnetComm;
import javax.baja.driver.BDriverContainer;
import javax.baja.job.BJobState;
import javax.baja.job.JobCancelException;
import javax.baja.job.JobLogItem;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.security.BPassword;
import javax.baja.status.BStatus;
import javax.baja.sys.BAbstractService;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BLink;
import javax.baja.sys.BStation;
import javax.baja.sys.BString;
import javax.baja.sys.BValue;
import javax.baja.sys.Context;
import javax.baja.sys.LocalizableRuntimeException;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.user.BUser;
import javax.baja.util.BFormat;
import javax.baja.util.BServiceContainer;

import com.honeywell.honbacnethelper.export.BHonDeviceExtDescriptor;
import com.honeywell.honbacnethelper.export.BHonFastAccessList;
import com.honeywell.honbacnethelper.export.BHonFastAccessLists;
import com.honeywell.honbacnethelper.export.datatypes.BHonBacnetAlarmPriority;
import com.honeywell.honplantcontroller.hmi.alarm.BHMIAlarmConsoleRecipient;
import com.honeywell.honplantcontroller.hmi.comm.BHonPlantControllerBacnetProperties;
import com.honeywell.honplantcontroller.hmi.enums.BHMIAlarmLevelEnum;
import com.honeywell.honplantcontroller.hmi.helper.BHMINotification;
import com.honeywell.honplantcontroller.hmi.limitations.HonHMILimitations;
import com.honeywell.honplantcontroller.hmi.limitations.HonLimitationResult;
import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;
import com.honeywell.honplantcontroller.hmi.user.authenticator.BHonPlantControllerHMIAuthenticator;
import com.honeywell.honplantcontroller.hmi.util.HonPlantControllerFastAccessListExporter;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetPointExportUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetScheduleExportUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILexicon;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerUserUtils;
import com.honeywell.honplantcontroller.hmi.view.BFastAccessList;
import com.honeywell.honplantcontroller.hmi.view.BHonPlantControllerFALHome;
import com.honeywell.honplantcontroller.hmi.view.BHonPlantControllerHome;
import com.tridium.bacnet.stack.BBacnetStack;
import com.tridium.bacnet.stack.network.BBacnetNetworkLayer;
import com.tridium.bacnet.stack.network.BNetworkPort;

@NiagaraType
public class BCommissionHMIJob extends BBaseHMIJob {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.job.BCommissionHMIJob(2979906276)1.0$ @*/
/* Generated Tue Mar 01 15:57:27 IST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BCommissionHMIJob.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public BCommissionHMIJob() {
    super();
  }

  public BCommissionHMIJob(BHonPlantControllerHMI service) {
    super(service);
  }

  @Override
  public void run(Context context) throws Exception {
	notifyHMI = false;
	isCommissionSuccess = false;
    progress(0);
		if(null == service) {
      commissioningFailed("hmi.service.invalid");
			return;
		}

    if(!service.getEnabled()) {
      commissioningFailed("hmi.service.disabled");
      return;
    }

    if(!(service.getParent() instanceof BAbstractService)) {
      commissioningFailed("hmi.service.invalid");
      return;
    }

    if(!((BAbstractService)service.getParent()).getEnabled()) {
      commissioningFailed("hmi.service.disabled");
      return;
    }

    if(!(service.getParent().getParent() instanceof BServiceContainer)) {
      commissioningFailed("hmi.service.invalid");
      return;
    }

	if(!HonPlantControllerUserUtils.getAllNiagaraUsers().findAny().isPresent()) {
		commissioningFailed("hmi.commission.user.no.hmi.user");
		return;
	}
	
	Stream<BUser> usersStream = HonPlantControllerUserUtils.getAllNiagaraUsersWithInvalidPin();
	List<BUser> usersList = usersStream.collect(Collectors.toList());
	StringBuilder invalidUsers = new StringBuilder();
	for (BUser user : usersList) {
		if (user.getChildren(BHonPlantControllerHMIAuthenticator.class)[0].getPin().equals(BPassword.DEFAULT)) {
			invalidUsers.append(user.getName());
			invalidUsers.append(", ");
		}
	}
	if (!invalidUsers.toString().isEmpty()) {
		invalidUsers.delete(invalidUsers.length() - 2, invalidUsers.length());
		commissioningFailed("hmi.commission.pin.empty", invalidUsers.toString());
		return;
	}
	
    isOperationCancelled();
		try{
			service.setIsHmiJobActive(true);
			log("hmi.commission.start.message");
			cleanUpSetup();

			log("hmi.commission.bacnet.network.initialization.start");
			initializeNetwork();
			
			log("hmi.commission.bacnet.network.initialization.completed");

			if(!checkHMINeededStatusInBacnetNetwork()) {
				commissioningFailed(HMI_COMMISSION_FAILED);
				return;
			}

			log("hmi.commission.alarm.service.initialization.start");
			setupAlarmService();
			setUpHMIAlarmPriorities();
			log("hmi.commission.alarm.service.initialization.completed");

			isOperationCancelled();

			progress(40);
			isOperationCancelled();

			log("hmi.commission.alarm.class.start");
			exportAlarmClasses();
			log("hmi.commission.alarm.class.completed");

			log("hmi.commission.bacnet.export.points.start");
			try {
				exportPointsResult = exportPoints();
			}
			catch (Exception e) {
				commissioningFailed("hmi.commission.bacnet.export.points.failed");
				return;
			}
			log("hmi.commission.bacnet.export.points.completed");

			isOperationCancelled();
			progress(60);

			log("hmi.commission.bacnet.export.fal.start");
			exportFalsResult = exportFAL();
			log("hmi.commission.bacnet.export.fal.completed");

			progress(80);

			log("hmi.commission.bacnet.export.schedules.start");
			exportSchedulesAndCalendars();
			log("hmi.commission.bacnet.export.schedules.completed");

			progress(90);
						
			Optional<Integer> result = service.shouldUpdateHMI();
			if (notifyHMI || result.isPresent()) {
				log("hmi.commission.syncstatus.notify.data.changed");
				service.getHMICommhandler().sendDynamicDataSyncRequest(0x01);
			}
			else {
				log("hmi.commission.syncstatus.notify.nothing.changed");
			}
			service.resetFALSyncStatus();
			service.restAlarmPrioritySyncStatus();
			service.initializeSubscriber();
			progress(100);
			log("hmi.commission.completed.message");
			isCommissionSuccess = true;
		}catch (CancellationException e) {
			log().message(e.getMessage());
			HonPlantControllerHMILogger.log(Level.SEVERE, e.getMessage(), e);
			super.canceled();
		}catch (LocalizableRuntimeException e) {
			log().failed(e.getMessage());
			failed(new LocalizableRuntimeException(TYPE.getModule().getModuleName(), HMI_COMMISSION_FAILED));
		}catch (Exception e) {
			HonPlantControllerHMILogger.log(Level.SEVERE, "run: something went wrong", e);
			log().failed(e.getMessage());
			failed(new LocalizableRuntimeException(TYPE.getModule().getModuleName(), HMI_COMMISSION_FAILED));
		} finally {
			service.setIsHmiJobActive(false);
		}
  }

	public void initializeNetwork() {
	  BBacnetNetwork bacnetNetwork = null;
	    if (HonPlantControllerBacnetUtils.findBacnetNetwork(Sys.getStation()) != null) {
	      if (findBacnetNetworkWithOkStatus(Sys.getStation()) != null) {
	        bacnetNetwork = findBacnetNetworkWithOkStatus(Sys.getStation());

			if(bacnetNetwork.getLocalDevice().getObjectId().getInstanceNumber() < 0) {
			 throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.commission.bacnet.device.invalid.id");
			}

	        BHonPlantControllerBacnetProperties bacnetProperties = new BHonPlantControllerBacnetProperties();

			HonPlantControllerBacnetUtils.removeDuplicatedNetworkPort(bacnetNetwork, bacnetProperties);

		    if(!HonPlantControllerBacnetUtils.containsHMIPort(bacnetNetwork, bacnetProperties)) {
			  bacnetProperties.createHMIPort(bacnetNetwork);
			  bacnetProperties.setBacnetSettings(bacnetNetwork);
		    }

		    BNetworkPort hmiPort = HonPlantControllerBacnetUtils.getHMIPort(bacnetNetwork, bacnetProperties);
		    if(hmiPort.getNetworkNumber() < 0) {
				hmiPort.setNetworkNumber(bacnetProperties.getNetworkNumber());
		    }
			if(!hmiPort.getEnabled()){
				if (HonPlantControllerBacnetUtils.isRunningOnController()) {
					hmiPort.setEnabled(true);
				}
			}
			
		    bacnetProperties = BHonPlantControllerBacnetProperties.exportBacnetProperties(bacnetNetwork, hmiPort);

		    BBacnetNetworkLayer bacnetNetworkLayer = ((BBacnetStack) bacnetNetwork.getBacnetComm()).getNetwork();
		    bacnetNetworkLayer.setDisplayName(bacnetNetworkLayer.getProperty(hmiPort.getName()), BFormat.make("%lexicon(honPlantControllerHMI:HMIMstpPort)%"), Context.NULL);
		    if(!BHonPlantControllerBacnetProperties.compareBacnetProperties(bacnetProperties, service.getBacnetProperties())) {
		    	notifyHMI = true;
		    }
		    bacnetProperties.setHMIInstanceId(service.getBacnetProperties().getHMIInstanceId());
		    service.setBacnetProperties(bacnetProperties);
		    HonPlantControllerHMILogger.finest("Registering HMI Device");
		    service.registerHMIDevice();
		    HonPlantControllerHMILogger.finest("Registered HMI Device");
	      }
	      else {
	        throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.commission.bacnet.network.status.not.ok");
	      }
	    }else {
	        BHonPlantControllerBacnetProperties bacnetProperties = new BHonPlantControllerBacnetProperties();
	        bacnetNetwork = HonPlantControllerBacnetUtils.createHMIBacnetNetwork(Sys.getStation(), bacnetProperties);
	        BNetworkPort hmiPort = HonPlantControllerBacnetUtils.getHMIPort(bacnetNetwork, bacnetProperties);
	        bacnetProperties = BHonPlantControllerBacnetProperties.exportBacnetProperties(bacnetNetwork, hmiPort);
	        bacnetProperties.setHMIInstanceId(service.getBacnetProperties().getHMIInstanceId());
	        service.setBacnetProperties(bacnetProperties);
	        HonPlantControllerHMILogger.finest("Registering HMI Device");
	        service.registerHMIDevice();
	        HonPlantControllerHMILogger.finest("Registered HMI Device");
	        notifyHMI = true;
	    }
		HonPlantControllerBacnetUtils.setBrandInformation(bacnetNetwork);
		HonPlantControllerBacnetUtils.exportDeviceExtObject(bacnetNetwork);
		HonPlantControllerBacnetUtils.exportEventLogObject(bacnetNetwork);
		service.validateAndSetHMIInst();
	  }

  private BBacnetNetwork findBacnetNetworkWithOkStatus(BStation station) {

	    if(station == null) {
	      return null;
	    }

	    BDriverContainer[] driverContainer = station.getChildren(BDriverContainer.class);
	    if(null == driverContainer || driverContainer.length <= 0)
	    	return null;
	    return Arrays.stream(driverContainer[0].getChildComponents())
	    .filter(BBacnetNetwork.class::isInstance)
	    .map(BBacnetNetwork.class::cast)
      .filter(network -> network.getStatus().equals(BStatus.ok))
	    .findFirst().orElse(null);
  }


  private void cleanUpSetup() {
		BBacnetNetwork bacnetNetwork = service.getBacnetNetwork();
		if (null == bacnetNetwork)
			return;
		BComponent exportTable = bacnetNetwork.getLocalDevice().getExportTable();
		BHonDeviceExtDescriptor<?,?>[] deviceExtDescriptors = exportTable.getChildren(BHonDeviceExtDescriptor.class);

		if (null == deviceExtDescriptors || deviceExtDescriptors.length < 1)
			return;

		if (deviceExtDescriptors.length > 1) {
			int i = -1;
			for (BHonDeviceExtDescriptor<?,?> honDeviceExtDescriptor : deviceExtDescriptors) {
				++i;
				if (i == 0) {
					continue;
				}
				exportTable.remove(honDeviceExtDescriptor);
			}
		}
		deviceExtDescriptors[0].setFastAccessLists(new BHonFastAccessLists(BHonFastAccessList.TYPE));
  }

  private void exportAlarmClasses() {
		HonPlantControllerBacnetPointExportUtils.exportAlarmClasses(service.getBacnetNetwork());
  }

private void setupAlarmService() {
	  if(null == service)
		  return;
	  BComplex parent = service.getParent();
	  if(null == parent)
		  return;
	  BComplex serviceContainer = parent.getParent();
	  if(!(serviceContainer instanceof BServiceContainer))
		  return;
	  BAlarmService[] alarmServices = ((BServiceContainer)serviceContainer).getChildren(BAlarmService.class);
	  if(null == alarmServices || alarmServices.length < 1)
		  return;
	  BAlarmService alarmService = alarmServices[0];
	  BHMIAlarmConsoleRecipient[] alarmConsoleRecipients = alarmService.getChildren(BHMIAlarmConsoleRecipient.class);
	  
	  if(null == alarmConsoleRecipients)
		  return;
	  if(alarmConsoleRecipients.length > 0)
		  return;

	  BHMIAlarmConsoleRecipient hmiAlarmConsoleRecipient = new BHMIAlarmConsoleRecipient();
	  alarmService.add("HMIAlarmConsoleRecipient", hmiAlarmConsoleRecipient);

		int urgentPriorityLowRange = service.getAlarmPriorityConfiguration().getUrgentAlarmPriority().getLowRange();
	  BValue urgentAlarm = alarmService.get("Urgent");
	  if(null == urgentAlarm) {
		  urgentAlarm = new BAlarmClass();
		  alarmService.add("Urgent", urgentAlarm);
		  ((BAlarmClass)urgentAlarm).setPriority(BAlarmPriorities.make(urgentPriorityLowRange, urgentPriorityLowRange, urgentPriorityLowRange, urgentPriorityLowRange));
	  }

		int highPriorityLowRange = service.getAlarmPriorityConfiguration().getHighAlarmPriority().getLowRange();
	  BValue highAlarm = alarmService.get("High");
	  if(null == highAlarm) {
		  highAlarm = new BAlarmClass();
		  ((BAlarmClass)highAlarm).setPriority(BAlarmPriorities.make(highPriorityLowRange, highPriorityLowRange, highPriorityLowRange, highPriorityLowRange));
		  alarmService.add("High", highAlarm);
	  }

		int lowPriorityLowRange = service.getAlarmPriorityConfiguration().getLowAlarmPriority().getLowRange();
	  BValue lowAlarm = alarmService.get("Low");
	  if(null == lowAlarm) {
		  lowAlarm = new BAlarmClass();
		  ((BAlarmClass)lowAlarm).setPriority(BAlarmPriorities.make(lowPriorityLowRange, lowPriorityLowRange, lowPriorityLowRange, lowPriorityLowRange));
		  alarmService.add("Low", lowAlarm);
	  }
	  
	  if(urgentAlarm instanceof BAlarmClass) {
		  BLink link1 = hmiAlarmConsoleRecipient.makeLink((BAlarmClass)urgentAlarm, BAlarmClass.alarm, BAlarmRecipient.routeAlarm, null);
		  hmiAlarmConsoleRecipient.add(LINK, link1);
	  }
	  
	  if(highAlarm instanceof BAlarmClass) {
		  BLink link2 = hmiAlarmConsoleRecipient.makeLink((BAlarmClass)highAlarm, BAlarmClass.alarm, BAlarmRecipient.routeAlarm, null);
		  hmiAlarmConsoleRecipient.add(LINK, link2);
	  }
	  
	  if(lowAlarm instanceof BAlarmClass) {
		  BLink link3 = hmiAlarmConsoleRecipient.makeLink((BAlarmClass)lowAlarm, BAlarmClass.alarm, BAlarmRecipient.routeAlarm, null);
		  hmiAlarmConsoleRecipient.add(LINK, link3);
	  }
	  
	  BValue defaultAlarmClass = alarmService.get("defaultAlarmClass");
	  if(defaultAlarmClass instanceof BAlarmClass) {
		  BLink link4 = hmiAlarmConsoleRecipient.makeLink((BAlarmClass)defaultAlarmClass, BAlarmClass.alarm, BAlarmRecipient.routeAlarm, null);
		  hmiAlarmConsoleRecipient.add(LINK, link4);
	  }

  }



	private void setUpHMIAlarmPriorities() {
		Optional<BHonDeviceExtDescriptor<?,?>>  deviceExtDescr = getDeviceExtDescriptor();

		BHonBacnetAlarmPriority urgentPriority =  new BHonBacnetAlarmPriority(BHMIAlarmLevelEnum.URGENT_ALARM, service.getAlarmPriorityConfiguration().getUrgentAlarmPriority().getLowRange(), service.getAlarmPriorityConfiguration().getUrgentAlarmPriority().getHighRange());
		BHonBacnetAlarmPriority highPriority =  new BHonBacnetAlarmPriority(BHMIAlarmLevelEnum.HIGH_ALARM, service.getAlarmPriorityConfiguration().getHighAlarmPriority().getLowRange(), service.getAlarmPriorityConfiguration().getHighAlarmPriority().getHighRange());
		BHonBacnetAlarmPriority lowPriority =  new BHonBacnetAlarmPriority(BHMIAlarmLevelEnum.LOW_ALARM, service.getAlarmPriorityConfiguration().getLowAlarmPriority().getLowRange(), service.getAlarmPriorityConfiguration().getLowAlarmPriority().getHighRange());
		//TODO: refactor the HonBacnetHelper.BHonDeviceExtDescriptor all the occurences of HMI
		deviceExtDescr.ifPresent(deviceExt -> deviceExt.setupHmiAlarmPriorities(urgentPriority, highPriority, lowPriority));
	}

	protected Optional<BHonDeviceExtDescriptor<?,?>> getDeviceExtDescriptor() {
		Optional<BHonDeviceExtDescriptor<?,?>> deviceExtDescriptor = Optional.empty();

		BBacnetNetwork bacnetNetwork = service.getBacnetNetwork();
		if (null == bacnetNetwork) {
			return deviceExtDescriptor;
		}
		BComponent exportTable = bacnetNetwork.getLocalDevice().getExportTable();
		BHonDeviceExtDescriptor<?,?>[] deviceExtDescriptors = exportTable.getChildren(BHonDeviceExtDescriptor.class);

		if (null == deviceExtDescriptors || deviceExtDescriptors.length < 1){
			return deviceExtDescriptor;
		}
		
		return Optional.of(deviceExtDescriptors[0]);

	}

  protected Optional<Map<String, Map<String, List<String>>>> exportFAL() {
		Optional<Map<String, Map<String, List<String>>>> result = Optional.empty();

    BHonPlantControllerHome home = service.getHome();
    BHonPlantControllerFALHome fastAccessLists = home.getFastAccessLists();
    BFastAccessList[] children = fastAccessLists.getChildren(BFastAccessList.class);
    boolean isFALEmpty = false;
    for (BFastAccessList fastAccessList : children) {
		  //synchronize the objectList before exporting FAL
		  fastAccessList.doSyncObjectList();
    }

    for (BFastAccessList fastAccessList : children) {
    		BComponent[] childComponents = fastAccessList.getObjectsList().getChildComponents();
    		if(null == childComponents || childComponents.length <= 0) {
    			isFALEmpty = true;
    			warn(
    					"hmi.fal.empty",
    					fastAccessList.getDisplayName(null)
    				);
    		}
	}
    if(isFALEmpty) {
    	throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.fal.empty.error");
    }
    
		//intialize Optional error/warning lists
		falsTypesAndErrors = new HashMap<>();
		
    for (BFastAccessList fastAccessList : children) {
			HonPlantControllerFastAccessListExporter falExporter = new HonPlantControllerFastAccessListExporter();
			
			ArrayList exp = falExporter.exportFalTolocalDevice(fastAccessList);
			dpFalNotExportable = (Map<String, List<String>>)exp.get(0);
			dpFalObjectsNotExportable = (Map<String, List<String>>)exp.get(1);
    }

		if (!dpFalNotExportable.isEmpty()) {
			falsTypesAndErrors.put(FAL_NOT_EXPORTABLE_ERROR, dpFalNotExportable);
  }

		if (!dpFalObjectsNotExportable.isEmpty()) {
			falsTypesAndErrors.put(FAL_OBJECT_NOT_EXPORTABLE_ERROR, dpFalObjectsNotExportable);
		}

		if (!falsTypesAndErrors.isEmpty()) {
			result = Optional.of(falsTypesAndErrors);
		}
		return result;
	}

	protected boolean checkHMINeededStatusInBacnetNetwork() {
    	BBacnetNetwork bacnetNetwork = service.getBacnetNetwork();
    	if(bacnetNetwork.isDisabled()) {
    		commissioningFailed("hmi.commission.bacnet.network.disabled");
    		return false;
    	}

    BBacnetComm bacnetComm = bacnetNetwork.getBacnetComm();
    if(bacnetComm.getCommControl().getOrdinal() == BBacnetCommControl.DISABLE) {
      commissioningFailed("hmi.commission.bacnet.commcontrol.disabled");
      return false;
    }
    BNetworkPort hmiPort = HonPlantControllerBacnetUtils.getHMIPort(bacnetNetwork, service.getBacnetProperties());
		if(hmiPort == null) {
			commissioningFailed("hmi.commission.bacnet.hmiport.not.found");
			return false;
		}
		if(!hmiPort.getEnabled()) {
			commissioningFailed("hmi.commission.bacnet.hmiport.disabled");
			return false;
		}
    return true;
  }

  private String getText(String lexiconMessage, Object ...obj) {
    return HonPlantControllerHMILexicon.getLexicon().getText(lexiconMessage, obj);
  }

	public Optional<Map<String, Map<String, List<String>>>> exportPoints() throws Exception {
		Optional<Map<String, Map<String, List<String>>>> result = Optional.empty();
		try {
			result = HonPlantControllerBacnetPointExportUtils.exportPoints(service.getBacnetNetwork(), service.getPointExportOption());
		} catch (LocalizableRuntimeException exc) {
			String message = exc.getLocalizedMessage();
			log().failed(message);
			HonPlantControllerHMILogger.log(Level.WARNING, message);
		}

		return result;
	}
  
	public void logTheAlarmError(Map<String, List<String>> errorMap) {
		errorMap.forEach((dpSlotPath, dpTypeList) -> {
			dpTypeList.forEach(dpType -> {
				switch (dpType) {
				case "BooleanWritable":
					putLogItemFailed("hmi.commission.bacnet.export.points.nonCompatibleAlarmExt",
							"hmi.commission.bacnet.export.points.alarmBOExtHelp", dpSlotPath);
					break;

				case "BooleanPoint":
					putLogItemFailed("hmi.commission.bacnet.export.points.nonCompatibleAlarmExt",
							"hmi.commission.bacnet.export.points.alarmBIExtHelp", dpSlotPath);
					break;

				case "EnumWritable":
					putLogItemFailed("hmi.commission.bacnet.export.points.nonCompatibleAlarmExt",
							"hmi.commission.bacnet.export.points.alarmMSOExtHelp", dpSlotPath);
					break;

				case "EnumPoint":
					putLogItemFailed("hmi.commission.bacnet.export.points.nonCompatibleAlarmExt",
							"hmi.commission.bacnet.export.points.alarmMSIExtHelp", dpSlotPath);
					break;

				default:
					break;
				}
			});
		});
	}

	public void logThePrecisionError(Map<String, List<String>> errorMap) {
		errorMap.forEach((dpSlotPath, dpTypeList) -> {
			dpTypeList.forEach(dpType -> {
				switch (dpType) {
				case "NumericWritable":
				case "NumericPoint":
					putLogItemFailed("hmi.commission.bacnet.export.points.invalidPrecision",
							"hmi.commission.bacnet.export.points.defaultPrecision", dpSlotPath);

					break;

				default:
					break;
				}

			});
		});
	}

	public void logEnumRangeError(Map<String, List<String>> errorMap) {
		errorMap.forEach((dpSlotPath, dpTypeList) -> {
			dpTypeList.forEach(dpType -> {
				switch (dpType) {
					case "EnumWritable":
					case "EnumPoint":
						putLogItemFailed("hmi.commission.bacnet.export.points.invalidEnumRange",
								"hmi.commission.bacnet.export.points.enumRangeHelp", dpSlotPath);

						break;

					default:
						break;
				}

			});
		});
	}

	public void logTheObjectNameMismatch(Map<String, List<String>> errorMap) {
		errorMap.forEach((sourceExportComponent, exportedObject) -> {
			String message = getText("hmi.commission.finished.name.mismatch.msg");
			String details = String.format(getText("hmi.commission.finished.name.mismatch.details"), sourceExportComponent, exportedObject);
			log().add(new JobLogItem(JobLogItem.FAILED,  message, details));

			HonPlantControllerHMILogger.log(Level.WARNING, MessageFormat.format("{0}  {1}", message, details));
		});
	}

	public void logFalNotExportable(Map<String, List<String>> errorMap) {
		errorMap.forEach((sourceExportComponent, exportedObject) -> {
			String message = String.format(getText("hmi.commission.finished.fal.not.exportable.msg"), sourceExportComponent);
			String details = String.format(getText("hmi.commission.finished.fal.not.exportable.details"), sourceExportComponent, exportedObject);
			log().add(new JobLogItem(JobLogItem.FAILED,  message, details));

			HonPlantControllerHMILogger.log(Level.WARNING, MessageFormat.format("{0}  {1}", message, details));
		});
	}

	public void logFalObjectNotExportable(Map<String, List<String>> errorMap) {
		errorMap.forEach((sourceExportComponent, exportedObject) -> {
			String message = String.format(getText("hmi.commission.finished.fal.object.not.exportable.msg"), sourceExportComponent);
			String details = String.format(getText("hmi.commission.finished.fal.object.not.exportable.details"), sourceExportComponent, exportedObject);
			log().add(new JobLogItem(JobLogItem.FAILED,  message, details));

			HonPlantControllerHMILogger.log(Level.WARNING, MessageFormat.format("{0}  {1}", message, details));
		});
	}

	public void exportSchedulesAndCalendars() throws Exception {
		HonLimitationResult result = HonPlantControllerBacnetScheduleExportUtils.exportSchedulesAndCalenders(
				service.getBacnetNetwork());

		if (result == HonLimitationResult.EXCEEDED) {
			warn(
					"hmi.limitations.exceeded.tp.max.numberOfSchedules",
					HonHMILimitations.getMaxNumberOfSchedules());

			throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(),
					"hmi.limitations.exceeded.commission.fail");
		}
		if (result == HonLimitationResult.EXCEEDED_CALENDARS) {
			warn("hmi.limitations.exceeded.tp.max.numberOfCalendars", HonHMILimitations.getMaxNumberOfCalendars());
			throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(),
					"hmi.limitations.exceeded.commission.fail");
		}
	}

  @Override
  public void doCancel(Context cx) {
    setJobState(BJobState.canceled);
  }

  @Override
  public void success() {
	  if (getJobState() == BJobState.canceling || getJobState() == BJobState.canceled) {
		  complete(BJobState.canceled);
		  return;
	  }

	  //handle errors and warnings!
	  StringBuilder msg = new StringBuilder("");
	  boolean warn = false;
	  if (exportPointsResult.isPresent() ) {
		  Map<String, Map<String, List<String>>> errorMessages = exportPointsResult.get();
		  errorMessages.forEach((errorType, errorMap) -> {
			  switch (errorType) {
				  case HonPlantControllerBacnetPointExportUtils.ALARM_ERROR:
					  logTheAlarmError(errorMap);
					  msg.append(String.format(getText("hmi.commission.finished.alarm.error"), errorMap.size()))
							  .append("\n");
					  break;

				  case HonPlantControllerBacnetPointExportUtils.PRECISION_ERROR:
					  logThePrecisionError(errorMap);
					  msg.append(String.format(getText("hmi.commission.finished.precision.error"), errorMap.size()))
							  .append("\n");
					  break;

				  case HonPlantControllerBacnetPointExportUtils.ENUM_RANGE_ERROR:
					  logEnumRangeError(errorMap);
					  msg.append(String.format(getText("hmi.commission.finished.enum.error"), errorMap.size()))
							  .append("\n");
					  break;

				  case HonPlantControllerBacnetPointExportUtils.OBJECT_NAME_WARNING:
					  logTheObjectNameMismatch(errorMap);
					  msg.append(String.format(getText("hmi.commission.finished.name.mismatch"), errorMap.size()))
							  .append("\n");

				  default:
					  break;
			  }
		  });

		  //A Notification should be shown to the user
		  warn = true;
	  }

	  if (exportFalsResult.isPresent() ) {
		  Map<String, Map<String, List<String>>> errorMessages = exportFalsResult.get();
		  errorMessages.forEach((errorType, errorMap) -> {
			  switch (errorType) {
				  case FAL_NOT_EXPORTABLE_ERROR:
					  logFalNotExportable(errorMap);
					  msg.append(String.format(getText("hmi.commission.finished.fal.not.exportable"), errorMap.size()))
							  .append("\n");
					  break;

				  case FAL_OBJECT_NOT_EXPORTABLE_ERROR:
					  logFalObjectNotExportable(errorMap);
					  msg.append(String.format(getText("hmi.commission.finished.fal.object.not.exportable"), errorMap.size()))
							  .append("\n");
					  break;

				  default:
					  break;
			  }
		  });

		  //A Notification should be shown to the user
		  warn = true;
	  }

	  //Show Notification only if Commissioning succeeded with warnings!
	  if ( (isCommissionSuccess) && (warn) ) {
		  BHMINotification notify = new BHMINotification();
		  String title = new LocalizableRuntimeException(TYPE.getModule().getModuleName(), HMI_COMMISSION_WARN).getLocalizedMessage();
		  notify.add("title", BString.make(title));
		  String message = getText("hmi.commission.finished.warn") + "\n";
		  message += msg.toString();
		  message += getText("hmi.commission.joblog.review");
		  notify.add("message", BString.make(message));
		  notify.raise(true);

		  //Log the warning
		  HonPlantControllerHMILogger.warning(message);
	  }


	  if (!isCommissionSuccess) {
		  complete(BJobState.failed);
		  return;
	  }

	  super.success();
  }

	protected void isOperationCancelled() {
		if(isCommissionCancelled()) {
			throw new CancellationException(getText("hmi.commission.cancelled"));
		}
	}

  public boolean isCommissionCancelled() {
    return getJobState().equals(BJobState.canceled) || getJobState().equals(BJobState.canceling);
  }

  private void commissioningFailed(String lexiconMessage) {
		String message = getText(lexiconMessage);
		HonPlantControllerHMILogger.error("commissioningFailed: " + message);
		log().failed(message);
		failed(new LocalizableRuntimeException(TYPE.getModule().getModuleName(), HMI_COMMISSION_FAILED));
	}
  
	private void commissioningFailed(String lexiconMessage, Object... obj) {
		String message = getText(lexiconMessage, obj);
		HonPlantControllerHMILogger.error("commissioningFailed: " + message);
		log().failed(message);
		failed(new LocalizableRuntimeException(TYPE.getModule().getModuleName(), HMI_COMMISSION_FAILED));
	}

  private void log(String lexiconMessage) {
    String message = getText(lexiconMessage);
    log().message(message);
    HonPlantControllerHMILogger.log(Level.INFO, message);
  }
	
  private void warn(String lexiconMessage, Object ...obj) {
    String message = getText(lexiconMessage, obj);
    log().failed(message);
    HonPlantControllerHMILogger.log(Level.WARNING, message);
  }

	private void putLogItemFailed(String lexiconMessage, String detailsLexicon, Object ...obj) {
		String message = getText(lexiconMessage, obj);
		String details = getText(detailsLexicon);
		log().add(new JobLogItem(JobLogItem.FAILED,  message, details));
		mirrorToConsoleLog(lexiconMessage, detailsLexicon, obj);
	}

	/**
	 * Helper-function to mirror custom job-log-item to console log
	 * @param lexiconMessage
	 * @param detailsLexicon
	 * @param obj
	 */
	private void mirrorToConsoleLog(String lexiconMessage, String detailsLexicon, Object ...obj) {
		String message = getText(lexiconMessage, obj);
		String details = getText(detailsLexicon);

		HonPlantControllerHMILogger.log(Level.WARNING, MessageFormat.format("{0} \n {1}", message, details));
	}

	@Override
	public void failed(Throwable cause) {
		if (this.getJobState() != BJobState.canceling && !(cause instanceof InterruptedException) && !(cause instanceof JobCancelException)) {
			this.log().failed("Job Failed", cause);
			this.complete(BJobState.failed);
		} else {
			this.canceled();
		}
	}

	private static Optional<Map<String, Map<String, List<String>>>> exportPointsResult = Optional.empty();
	private static Optional<Map<String, Map<String, List<String>>>> exportFalsResult = Optional.empty();
  private static final String HMI_COMMISSION_FAILED = "hmi.commission.failed";
	private static final String HMI_COMMISSION_WARN = "hmi.commission.warn";
	private static final String LINK = "link?";
  protected boolean isCommissionSuccess = false;
  private boolean notifyHMI = false;

	private static Map<String, Map<String, List<String>>> falsTypesAndErrors = new HashMap<>();
	private static Map<String, List<String>> dpFalObjectsNotExportable = new HashMap<>();
	private static Map<String, List<String>> dpFalNotExportable = new HashMap<>();

	public static final String FAL_NOT_EXPORTABLE_ERROR = "FAL Not Exportable Error";
	public static final String FAL_OBJECT_NOT_EXPORTABLE_ERROR = "FAL Object Not Exportable Error";
}
