/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 *
 * This file contains trade secrets of Honeywell International, Inc. No part may be reproduced or
 * transmitted in any form by any means or for any purpose without the express written permission of
 * Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.job;

import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILexicon;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;

import javax.baja.job.BJobState;
import javax.baja.job.BSimpleJob;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BAbstractService;
import javax.baja.sys.BString;
import javax.baja.sys.Context;
import javax.baja.sys.LocalizableRuntimeException;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import java.text.MessageFormat;
import java.util.concurrent.CancellationException;
import java.util.logging.Level;


// need this property because the workbench (progressbar) gets the name of the job through toString
@NiagaraProperty(
  name = "nameOfJob",
  type = "String",
  defaultValue = ""
)
@NiagaraType
public class BHMIFileDownloadJob
  extends BSimpleJob
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.job.BHMIFileDownloadJob(742466967)1.0$ @*/
/* Generated Wed May 18 18:10:14 IST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "nameOfJob"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code nameOfJob} property.
   * @see #getNameOfJob
   * @see #setNameOfJob
   */
  public static final Property nameOfJob = newProperty(0, "", null);
  
  /**
   * Get the {@code nameOfJob} property.
   * @see #nameOfJob
   */
  public String getNameOfJob() { return getString(nameOfJob); }
  
  /**
   * Set the {@code nameOfJob} property.
   * @see #nameOfJob
   */
  public void setNameOfJob(String v) { setString(nameOfJob, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHMIFileDownloadJob.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  protected BHonPlantControllerHMI hmiService;
  protected BString filename;
  protected boolean isFwUpdate;


  // Empty constructor for JAVA introspection.
  public BHMIFileDownloadJob()
  {
  }

  public BHMIFileDownloadJob(BHonPlantControllerHMI hmiService, BString filename)
  {
    super();
    this.hmiService = hmiService;
    this.filename = filename;
    this.isFwUpdate = BHonPlantControllerHMI.isFwUpdate(this.filename);

    String lexKeyJobName = (this.isFwUpdate) ? "hmi.downloadFirmware.jobname" : "hmi.downloadLanguage.jobname";
    String jobName = HonPlantControllerHMILexicon.getLexicon().get(lexKeyJobName);
    if (jobName == null)
    {
      if (this.isFwUpdate)
      {
        jobName = "HMI firmware update";
      }
      else
      {
        jobName = "HMI language file download";
      }
    }
    setNameOfJob(jobName);
  }

  @Override
  public void run(Context cx)
    throws Exception
  {
    final String LOG_USER_MSG_PREFIX = "HMI " + ((isFwUpdate) ? "Firmware" : "Language") + " Update Job:";

    progress(0);
    HonPlantControllerHMILogger.info(MessageFormat.format("{0} start", LOG_USER_MSG_PREFIX));
    log().message(getText((isFwUpdate) ? "hmi.downloadFirmware.start" : "hmi.downloadLanguage.start"));

    if ((null == hmiService) || !(hmiService.getParent() instanceof BAbstractService))
    {
      HonPlantControllerHMILogger.error(MessageFormat.format("{0} HMI Service is not valid", LOG_USER_MSG_PREFIX));
      throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.service.invalid");
    }
    if (!hmiService.getEnabled() || !((BAbstractService) hmiService.getParent()).getEnabled())
    {
      HonPlantControllerHMILogger.error(MessageFormat.format("{0} HMI service is disabled", LOG_USER_MSG_PREFIX));
      throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.service.disabled");
    }

    try
    {
      boolean success = hmiService.runFileDownloadHMI(filename, this);
      if(!success) {
    	  isFileDownloadSuccess = true;
    	  return;
      }
      String leyKeyCompleted;
      if (isFwUpdate)
      {
    	hmiService.setFirmwareDownloadCancelled(false);
        leyKeyCompleted = "hmi.downloadFirmware.completed.message";
      }
      else
      {
        leyKeyCompleted = "hmi.downloadLanguage.completed.message";
      }
      log().message(getText(leyKeyCompleted));
      isFileDownloadSuccess = true;
    }
    catch (CancellationException exc)
    {
      log().message(exc.getMessage());
      HonPlantControllerHMILogger.log(Level.SEVERE, exc.getMessage(), exc);
      super.canceled();
    }catch (Exception e) {
      isFileDownloadSuccess = false;
	}
    finally
    {
      HonPlantControllerHMILogger.info(MessageFormat.format("{0} end", LOG_USER_MSG_PREFIX));
    }
  }

  @Override
  public void doCancel(Context context)
  {
    setJobState(BJobState.canceling);
  }

  @Override
  public void success()
  {
    if ((getJobState() == BJobState.canceling) || (getJobState() == BJobState.canceled))
    {
      complete(BJobState.canceled);
      return;
    }
	if(!isFileDownloadSuccess){
		if(isFwUpdate) {
			hmiService.notifyChangesToHMI();
		}
		complete(BJobState.failed);
		return;
	}

    super.success();
  }

  @Override
  public String toString(Context cx)
  {
    String jobName = getNameOfJob();
    if (jobName.isEmpty())
    {
      return super.toString(cx);
    }
    else
    {
      return jobName;
    }
  }

  protected String getText(String lexiconMessage)
  {
    return HonPlantControllerHMILexicon.getLexicon().getText(lexiconMessage);
  }
  
  private boolean isFileDownloadSuccess = false;
}
