/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 * This file contains trade secrets of Honeywell International, Inc. No part
 * may be reproduced or transmitted in any form by any means or for any
 * purpose without the express written permission of Honeywell.
 */


package com.honeywell.honplantcontroller.hmi.limitations;

/**
 * Holds all the limitations for the HMI firmware in one place.
 */
public class HonHMILimitations {

  /*
   * Limitations for Fast Access List related information.
   */
  private static final int FAL_MAX_LIST_COUNT = 100;

  private static final int FAL_MAX_POINTS_PER_LIST = 50;

  private static final int FAL_MAX_LENGTH_OF_LIST_NAME = 32;


  /*
   * Limitations for Alarm related information.
   */
  private static final int ALARM_MAX_POINTS_IN_ALARM = 4100;

  private static final int ALARM_MAX_UNACK_ALARMS = 65535;

  private static final int ALARM_MAX_ENTRIES_IN_HISTROY = 65535;


  /*
   * Limitations for Points related information.
   */
  private static final int POINT_MAX_COUNT = 4100;

  private static final int POINT_MAX_COUNT_IN_MANUAL = 4100;


  /*
   * Limitations for Time Programs related information.
   */
  private static final int TP_MAX_NUMBER_OF_SCHEDULES = 100;

  // TODO: currently not specified.
  private static final int TP_MAX_EXCEPTIONS_PER_SCHEDULE = -1;

  private static final int TP_MAX_TIME_VALUES_PER_DAY_FOR_WEEKLY_SCHEDULES = 30;

  private static final int TP_MAX_TIME_VALUES_PER_DAY_FOR_EXCEPTION_SCHEDULES = 30;

  private static final int TP_MAX_NUMBER_OF_POINTS_ASSIGNED_TO_A_SCHEDULE = 70;

  private static final int TP_MAX_NUMBER_OF_CALENDAR = 100;

  private static final int TP_MAX_ENTRIES_PER_CALENDAR = 50;


  /*
   * Limitations for Users related information.
   */

  private static final int USER_MAX_NUMBER = 20;


  /*
   * Limitations for Text related information.
   */

  private static final int TEXT_MAX_LENGTH_OF_POINT_NAME  = 504;

  private static final int TEXT_MAX_LENGTH_OF_DESCRIPTION = 504;

  private static final int TEXT_MAX_LENGTH_OF_STATE_TEXT  = 504;

  private static final int TEXT_MAX_LENGTH_OF_PLANT_NAME  = 504;

  /**
   * Explicit private Constructor because this class need no initialization.
   */
  /// CLOVER:OFF
  private HonHMILimitations() {}
  /// CLOVER:ON


  /**
   * The max. number of Fast Access Lists the HMI currently supports.
   * @return FAL_MAX_LIST_COUNT
   */
  public static final int getMaxNumberOfFastAccessLists() {
    return FAL_MAX_LIST_COUNT;
  }

  /**
   * The max. number of points a Fast Access List is allowed to hold
   * according to the limitations of the HMI
   * @return FAL_MAX_POINTS_PER_LIST
   */
  public static final int getMaxNumberOfPointsPerFastAccessList() {
    return FAL_MAX_POINTS_PER_LIST;
  }

  /**
   * The max. length the name a Fast Access List can be named.
   * @return FAL_MAX_LENGTH_OF_LIST_NAME
   */
  public static final int getMaxLengthOfFastAccessListName() {
    return FAL_MAX_LENGTH_OF_LIST_NAME;
  }


  /**
   * The max. number of point in alarm that the HMI can show.
   * @return ALARM_MAX_POINTS_IN_ALARM
   */
  public static final int getMaxNumberOfPointsInAlarm(){
    return ALARM_MAX_POINTS_IN_ALARM;
  }

  /**
   * The max. number of unacknowledged alarms the HMI can show
   * @return ALARM_MAX_UNACK_ALARMS
   */
  public static final int getMaxNumberOfUnacknowlagedAlarms() {
    return ALARM_MAX_UNACK_ALARMS;
  }

  /**
   * The max. number of entries in the history the HMI can show.
   * @return ALARM_MAX_ENTRIES_IN_HISTROY
   */
  public static final int getMaxNumberOfEntriesInHistory() {
    return ALARM_MAX_ENTRIES_IN_HISTROY;
  }

  /**
   * The max. number of points the HMI can show.
   * @return POINT_MAX_COUNT
   */
  public static final int getMaxNumberOfExportedPoints() {
    return POINT_MAX_COUNT;
  }

  /**
   * The max. number of points that are in manual the HMI can show.
   * @return POINT_MAX_COUNT_IN_MANUAL
   */
  public static final int getMaxNumberOfPointsInManual() {
    return POINT_MAX_COUNT_IN_MANUAL;
  }

  /**
   * The max. number of schedules the HMI can show.
   * @return TP_MAX_NUMBER_OF_SCHEDULES
   */
  public static final int getMaxNumberOfSchedules() {
    return TP_MAX_NUMBER_OF_SCHEDULES;
  }

  /**
   * The max. number of exceptions the HMI can handle.
   * @return TP_MAX_EXCEPTIONS_PER_SCHEDULE
   */
  public static final int getMaxExceptionsPerSchedule() {
    return TP_MAX_EXCEPTIONS_PER_SCHEDULE;
  }

  /**
   * The max. number of time values per day for a Weekly Schedule the HMI can handle.
   * @return TP_MAX_TIME_VALUES_PER_DAY_FOR_WEEKLY_SCHEDULES
   */
  public static final int getMaxTimeValuesPerDayForWeelkySchedules() {
    return TP_MAX_TIME_VALUES_PER_DAY_FOR_WEEKLY_SCHEDULES;
  }

  /**
   * The max. number of time values per day for a Exception Schedule the HMI can handle.
   * @return TP_MAX_TIME_VALUES_PER_DAY_FOR_EXCEPTION_SCHEDULES
   */
  public static final int getMaxTimeValuesPerDayForExceptionSchedules() {
    return TP_MAX_TIME_VALUES_PER_DAY_FOR_EXCEPTION_SCHEDULES;
  }

  /**
   * The max. number of points that can be assigned to a schedules the HMI can handle.
   * @return TP_MAX_NUMBER_OF_POINTS_ASSIGNED_TO_A_SCHEDULE
   */
  public static final int getMaxNumberOfPointsAssignedToASchedule() {
    return TP_MAX_NUMBER_OF_POINTS_ASSIGNED_TO_A_SCHEDULE;
  }

  /**
   * The max. number of calendars the HMI can handle.
   * @return TP_MAX_NUMBER_OF_CALENDAR
   */
  public static final int getMaxNumberOfCalendars() {
    return TP_MAX_NUMBER_OF_CALENDAR;
  }

  /**
   * The max number of entries in a calendar the HMI can handle.
   * @return TP_MAX_ENTRIES_PER_CALENDAR
   */
  public static final int getMaxNumberOfEntriesPerCalendar() {
    return TP_MAX_ENTRIES_PER_CALENDAR;
  }

  /**
   * The max. number of users the hMI can handle.
   * @return USER_MAX_NUMBER
   */
  public static final int getMaxNumberOfUsers() {
    return USER_MAX_NUMBER;
  }


  /**
   * The max. length of a point's name the hMI can handle.
   * @return TEXT_MAX_LENGTH_OF_POINT_NAME
   */
  public static final int getMaxLengthOfPointName() {
    return TEXT_MAX_LENGTH_OF_POINT_NAME;
  }

  /**
   * The max. length of a description the hMI can handle.
   * @return TEXT_MAX_LENGTH_OF_POINT_NAME
   */
  public static final int getMaxLengthOfDescription() {
    return TEXT_MAX_LENGTH_OF_DESCRIPTION;
  }

  /**
   * The max. length of a state text the HMI can handle.
   * @return
   */
  public static final int getMaxLengthOfStateText() {
    return TEXT_MAX_LENGTH_OF_STATE_TEXT;
  }

  /**
   * The max. length of a plant name the HMI can handle.
   * @return
   */
  public static final int getMaxLengthOfPlantName() {
    return TEXT_MAX_LENGTH_OF_PLANT_NAME;
  }

}
