/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 * This file contains trade secrets of Honeywell International, Inc. No part
 * may be reproduced or transmitted in any form by any means or for any
 * purpose without the express written permission of Honeywell.
 */
package com.honeywell.honplantcontroller.hmi.messages;

import javax.baja.bacnet.datatypes.BBacnetUnsigned;

import com.honeywell.honplantcontroller.hmi.enums.BHMIAlarmLevelEnum;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
import com.tridium.bacnet.asn.AsnInputStream;

public class QueryAlarmPointsMessage {
    public QueryAlarmPointsMessage (byte[] rawMsg) {

        if (rawMsg == null || rawMsg.length <= 2) {
            return;
        }
        //The log entry call is moved here otherwise this call will throw an exception when the rawMsg is null

        AsnInputStream asnInputStream  = new AsnInputStream(rawMsg);

        try {
            BBacnetUnsigned hmiAlarmLevel = asnInputStream.readUnsigned(0);
            BBacnetUnsigned hmiStartIndex = asnInputStream.readUnsigned(1);
            BBacnetUnsigned hmiStopIndex = asnInputStream.readUnsigned(2);

            hmiAlarmLevelEnum = BHMIAlarmLevelEnum.make(hmiAlarmLevel.getInt());
            startIndex = hmiStartIndex.getInt();
            stopIndex = hmiStopIndex.getInt();
            
        } catch (Exception e) {
            HonPlantControllerHMILogger.finest(e.getMessage());
        }


    }



    public BHMIAlarmLevelEnum getHmiAlarmLevel(){return hmiAlarmLevelEnum;}
    public int getStartIndex(){return startIndex;}
    public int getStopIndex(){return stopIndex;}

    private BHMIAlarmLevelEnum hmiAlarmLevelEnum;
    private int startIndex;
    private int stopIndex;
}
