/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.messages;

import java.util.ArrayList;
import java.util.List;

import javax.baja.bacnet.datatypes.BBacnetUnsigned;

import com.honeywell.honplantcontroller.hmi.alarm.BacnetAlarmSummary;
import com.tridium.bacnet.asn.AsnOutputStream;

public class QueryUnackAlarmPointsResultMessage {

    private BBacnetUnsigned unAckAlarmPointsCount;
    private BBacnetUnsigned unAckUrgentAlarmCount;
    private BBacnetUnsigned unAckHighAlarmCount;
    private BBacnetUnsigned unAckLowAlarmCount;
    private List<BacnetAlarmSummary> alarmSummaries = new ArrayList<>();

    private QueryUnackAlarmPointsResultMessage() {

    }

    public static QueryUnackAlarmPointsResultMessage make(long alarmPointsCount, long urgentAlarmCount, long highAlarmCount, long lowAlarmCount) {

        QueryUnackAlarmPointsResultMessage hmiAlarmResult = new QueryUnackAlarmPointsResultMessage();

        hmiAlarmResult.unAckAlarmPointsCount = new BBacnetUnsigned(alarmPointsCount);
        hmiAlarmResult.unAckUrgentAlarmCount = new BBacnetUnsigned(urgentAlarmCount);
        hmiAlarmResult.unAckHighAlarmCount = new BBacnetUnsigned(highAlarmCount);
        hmiAlarmResult.unAckLowAlarmCount = new BBacnetUnsigned(lowAlarmCount);


        return hmiAlarmResult;
    }

    public static QueryUnackAlarmPointsResultMessage make(long alarmPointsCount, long urgentAlarmCount, long highAlarmCount, long lowAlarmCount, List<BacnetAlarmSummary> alarmSummaries) {
        
        QueryUnackAlarmPointsResultMessage hmiAlarmResult = QueryUnackAlarmPointsResultMessage.make(alarmPointsCount, urgentAlarmCount, highAlarmCount, lowAlarmCount);
        hmiAlarmResult.alarmSummaries = alarmSummaries;

        return hmiAlarmResult;
    }
    
    public static QueryUnackAlarmPointsResultMessage make(QueryUnackAlarmPointsResultMessage hmiAlarmResult, List<BacnetAlarmSummary> alarmSummaries) {

        hmiAlarmResult.alarmSummaries = alarmSummaries;
        return hmiAlarmResult;
    }

    public byte[] getBytes() {

        AsnOutputStream asnOutputStream = new AsnOutputStream();

        asnOutputStream.writeUnsignedInteger(0, unAckAlarmPointsCount.getInt());
        asnOutputStream.writeUnsignedInteger(1, unAckUrgentAlarmCount.getInt());
        asnOutputStream.writeUnsignedInteger(2, unAckHighAlarmCount.getInt());
        asnOutputStream.writeUnsignedInteger(3, unAckLowAlarmCount.getInt());

        if (!alarmSummaries.isEmpty()) {
            asnOutputStream.writeOpeningTag(4);
            alarmSummaries.stream().forEach(alarmSummary -> alarmSummaryBytes(alarmSummary, asnOutputStream));
            asnOutputStream.writeClosingTag(4);
        }

        return asnOutputStream.toByteArray();
    }

    public static void alarmSummaryBytes(BacnetAlarmSummary alarmSummary, AsnOutputStream asnOutputStream) {

        asnOutputStream.writeObjectIdentifier(alarmSummary.getObjectIdentifier());

        asnOutputStream.writeEnumerated(alarmSummary.getEventState());

        asnOutputStream.writeOpeningTag(2);
        asnOutputStream.writeDate(alarmSummary.getEventTimeStamp().getDateTime().getDate());
        asnOutputStream.writeTime(alarmSummary.getEventTimeStamp().getDateTime().getTime());
        asnOutputStream.writeClosingTag(2);

        asnOutputStream.writeUnsigned(alarmSummary.getPriority());
    }

    public BBacnetUnsigned getUnAckAlarmPointsCount() {
        return unAckAlarmPointsCount;
    }
    public BBacnetUnsigned getUnAckUrgentAlarmCount() {
        return unAckUrgentAlarmCount;
    }
    public BBacnetUnsigned getUnAckHighAlarmCount() {
        return unAckHighAlarmCount;
    }
    public BBacnetUnsigned getUnAckLowAlarmCount() {
        return unAckLowAlarmCount;
    }

    
    @Override
    public String toString() {
        //{----------------------only for testing, remove for production-------------------------

        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("Alarm Points Count: "+unAckAlarmPointsCount);
        stringBuilder.append("\n");
        stringBuilder.append("Urgent Alarm Count: "+unAckUrgentAlarmCount);
        stringBuilder.append("\n");
        stringBuilder.append("High Alarm Count: "+unAckHighAlarmCount);
        stringBuilder.append("\n");
        stringBuilder.append("Low Alarm Count: "+unAckLowAlarmCount);
        stringBuilder.append("\n");
        
        stringBuilder.append(" List of alarm summaries {");
        stringBuilder.append("\n");

        for (BacnetAlarmSummary alarmSummary : alarmSummaries) {
            stringBuilder.append("  Object identifier: "+alarmSummary.getObjectIdentifier());
            stringBuilder.append("\n");
            stringBuilder.append("  Event State: "+alarmSummary.getEventState());
            stringBuilder.append("\n");
            stringBuilder.append("  Event Time Stamp: "+alarmSummary.getEventTimeStamp());
            stringBuilder.append("\n");
            stringBuilder.append("  Priority: "+alarmSummary.getPriority());
            stringBuilder.append("\n");
            stringBuilder.append("  Present Value: "+alarmSummary.getAlarmPresentValue());
            stringBuilder.append("\n");
            stringBuilder.append("  Unit: "+alarmSummary.getUnit());
            stringBuilder.append("\n");
            stringBuilder.append("----");
        }
        stringBuilder.append("\n");
        stringBuilder.append("}");


        return stringBuilder.toString();
        //}----------------------only for testing, remove for production-------------------------
    }
}
