/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 * This file contains trade secrets of Honeywell International, Inc. No part
 * may be reproduced or transmitted in any form by any means or for any
 * purpose without the express written permission of Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.messages;

import java.util.logging.Logger;

import javax.baja.bacnet.io.AsnException;

import com.tridium.bacnet.asn.AsnInputStream;

public class QueryUserLoginMessage {

    private String userName = null;
    private byte[] userDk = null;
    private String rand = null;

    private static final Logger LOG = Logger.getLogger("HonPlantControllerHMI");


    public static QueryUserLoginMessage make(byte[] data) {

        if (null == data) {
            return null;
        }

        AsnInputStream serviceParameter = AsnInputStream.make(data);

        String name = null;
        byte[] userDk = null;

        try {
            name = serviceParameter.readCharacterString();
        }
        catch (AsnException asne) {
            LOG.warning(() -> "QueryUserLoginMessage.make() CharacterString: Could not read Username from ServiceParameter");
            return null;
        }


        try {
            userDk = serviceParameter.readOctetString();
        } catch (AsnException e) {
            LOG.warning(() -> "QueryUserLoginMessage.make() CharacterString: Could not read User DK from ServiceParameter");
            return null;
        }

        return new QueryUserLoginMessage(name, userDk, "");
    }

    private QueryUserLoginMessage(String userName, byte[] userDk, String rand) {
        this.userName = userName;
        this.userDk = userDk;
        this.rand = rand;
    }

    /**
     * @return the userName
     */
    public String getUserName() {
        return userName;
    }

    /**
     * @return the userDk
     */
    public byte[] getUserDk() {
        return userDk;
    }


    /**
     * @return the rand
     */
    public String getRand() {
        return rand;
    }


    ///CLOVER:OFF
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("UserLogin { ");
        builder.append("userName: ").append(userName).append(" }");
        return builder.toString();
    }
    ///CLOVER:ON

}
