/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 * This file contains trade secrets of Honeywell International, Inc. No part
 * may be reproduced or transmitted in any form by any means or for any
 * purpose without the express written permission of Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.messages;

import java.text.MessageFormat;
import java.util.logging.Logger;

import javax.baja.bacnet.io.AsnException;

import com.tridium.bacnet.asn.AsnInputStream;

public class QueryUserLogoutMessage {

    private String userName = null;

    private static final Logger LOG = Logger.getLogger("HonPlantControllerHMI");


    public static QueryUserLogoutMessage make(byte[] data) {

        if (null == data) {
            return null;
        }

        AsnInputStream serviceParameter = AsnInputStream.make(data);

        String name = null;

        LOG.info(() -> MessageFormat.format("QueryUserLogoutMessage.constructor() Parameter: {0}", serviceParameter));
        try {
            name = serviceParameter.readCharacterString();
        }
        catch (AsnException asne) {
            LOG.severe(() -> "QueryUserLogoutMessage.constructor() CharacterString: Could not read Username from ServiceParameter");
            return null;
        }

        return new QueryUserLogoutMessage(name);
    }

    private QueryUserLogoutMessage(String userName) {
        this.userName = userName;
    }

    /**
     * @return the userName
     */
    public String getUserName() {
        return userName;
    }

    ///CLOVER:OFF
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("UserLogout { ");
        builder.append("userName: ").append(userName).append(" }");
        return builder.toString();
    }
    ///CLOVER:ON

}
