/*
 *  Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 *
 *  This file contains trade secrets of Honeywell International, Inc.  No part 
 *  may be reproduced or transmitted in any form by any means or for any 
 *  purpose without the express written permission of Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.messages;

import java.util.logging.Logger;

import javax.baja.bacnet.io.AsnException;

import com.tridium.bacnet.asn.AsnInputStream;

public class QueryUserMessage {
  private String userName;
  private byte[] data;

  private static final Logger LOG = Logger.getLogger("HonPlantControllerHMI");


  private QueryUserMessage(String userName, byte[] data) {
    this.userName = userName;
    this.data = data;
  }

  public static QueryUserMessage make(byte[] bytes) {
    if (null == bytes) {
      return null;
    }

    AsnInputStream serviceParameter = AsnInputStream.make(bytes);

    String name = null;
    byte[] data = null;
    try {
      name = serviceParameter.readCharacterString();
    }
    catch (AsnException asne) {
      LOG.finest(() -> "QueryUserMessage.make() CharacterString: Could not read Username from ServiceParameter");
      return null;
    }

    try {
      data = serviceParameter.readOctetString();
    }
    catch (AsnException asne) {
      LOG.finest(() -> "QueryUserMessage.make() CharacterString: Could not read data from ServiceParameter");
      return null;
    }

    return new QueryUserMessage(name, data);
  }

  /**
   * @return the data (byte[])
   */
  public byte[] getData() {
    return data;
  }

  /**
   * @return the userName
   */
  public String getUserName() {
    return userName;
  }

  ///CLOVER:OFF
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("UserChangePassword { ");
    builder.append("userName: ").append(userName).append(" }");
    return builder.toString();
  }
  ///CLOVER:ON
}
