/*
 *  Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 *
 *  This file contains trade secrets of Honeywell International, Inc.  No part 
 *  may be reproduced or transmitted in any form by any means or for any 
 *  purpose without the express written permission of Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.messages;

import com.tridium.bacnet.asn.AsnOutputStream;

import javax.baja.nre.util.ByteArrayUtil;

public class RequestCertAckMessage {

    private int index = 0;
    private int length = 0;
    private boolean isLastFrame = true;
    private byte[] certBytes = null;


    public RequestCertAckMessage (byte[] rawData) throws Exception {

        if (rawData.length >= OFFSET_APPLICATION_TAG + OFFSET_PAYLOAD) {

            rawData[0+ OFFSET_APPLICATION_TAG] &= ~(1 << 7); //set highest bit to 0
            index = rawData[0+ OFFSET_APPLICATION_TAG];

            if (((rawData [1+ OFFSET_APPLICATION_TAG] >> 7 ) & 1) == 1) {
                isLastFrame = false;
                rawData[1+ OFFSET_APPLICATION_TAG] &= ~(1 << 7); //set highest bit to 0
            }
            length = ByteArrayUtil.readShort(rawData,1+ OFFSET_APPLICATION_TAG);
            if (length == (rawData.length - (OFFSET_APPLICATION_TAG + OFFSET_PAYLOAD))) {
                certBytes = new byte[length];
                System.arraycopy(rawData, OFFSET_APPLICATION_TAG + OFFSET_PAYLOAD, certBytes, 0, length);
            } else {
                throw new Exception("Expected length of certificate does not match actual length");
            }
        }
    }
    public RequestCertAckMessage(int index, boolean isLastFrame, byte[] data) {
        this.index = index;
        this.isLastFrame = isLastFrame;
        this.certBytes = data;

    }
    public byte[] getBytes () {
        AsnOutputStream asnOutputStream = new AsnOutputStream();

        byte[] data = new byte[certBytes.length + OFFSET_PAYLOAD];

        data[0]=(byte)index;
        ByteArrayUtil.writeShort(data, 1, certBytes.length);
        if (!isLastFrame) {
            data[1] |= 1 << 7;  //set highest bit to 1
        } else {
            data[1] &= ~(1 << 7);//set highest bit to 0
        }

        System.arraycopy(certBytes, 0, data, OFFSET_PAYLOAD, certBytes.length);

        asnOutputStream.writeOctetString(data);

        return asnOutputStream.toByteArray();
    }
    public byte[] getCertBytes () {return certBytes;}
    public int getIndex() {return index;}
    public boolean getIsLastFrame() {return isLastFrame;}
    private static final int OFFSET_APPLICATION_TAG = 4;
    private static final int OFFSET_PAYLOAD = 3;

}
