/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.messages;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import javax.baja.bacnet.datatypes.BBacnetObjectIdentifier;

import com.honeywell.honbacnethelper.export.enums.BHonPointMode;
import com.tridium.bacnet.asn.AsnUtil;

/**
 * Proprietary Messages for Dynamic Manual Mode Points Synchronization 
 */
public class WritePointModeMessage {
    
    public WritePointModeMessage() {
        //
    }

    /**
     * 
     * @param objectIdentifier bacnet ObjectId
     * @param pointMode current point mode
     */
    public void addPointMode(BBacnetObjectIdentifier objectIdentifier, BHonPointMode pointMode) {

        byte[] objectIDBytes = AsnUtil.toAsnObjectId(objectIdentifier);
        byte[] pointModeBytes = AsnUtil.toAsnUnsigned(pointMode.getOrdinal());

        byte[] bytes = ByteBuffer.allocate(objectIDBytes.length+pointModeBytes.length)
                            .put(objectIDBytes)
                            .put(pointModeBytes)
                            .array();
        pointModeBuffer.add(bytes);
        bytesTotal += bytes.length;
    }

    /**
     * Retrieve message as raw byte array to be send over bacnet 
     * @return byte array
     */
    public byte[] getBytes() {
        ByteBuffer byteBuffer = ByteBuffer.allocate(bytesTotal);
        
        pointModeBuffer.forEach(byteBuffer::put);

        rawMsg = byteBuffer.array();
        return rawMsg;
    }

    private byte[] rawMsg = new byte[0];
    private List<byte[]> pointModeBuffer = new ArrayList<>();
    private int bytesTotal = 0;


}
