/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.service;

import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.baja.bacnet.BBacnetNetwork;
import javax.baja.bacnet.datatypes.BBacnetArray;
import javax.baja.bacnet.export.BBacnetPointDescriptor;
import javax.baja.bacnet.export.BIBacnetExportObject;
import javax.baja.collection.BITable;
import javax.baja.driver.BDriverContainer;
import javax.baja.naming.BOrd;
import javax.baja.naming.SlotPath;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BComponentEvent;
import javax.baja.sys.BObject;
import javax.baja.sys.BValue;
import javax.baja.sys.Cursor;
import javax.baja.sys.Subscriber;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.honeywell.honbacnethelper.export.BHonFastAccessList;
import com.honeywell.honbacnethelper.export.BHonFastAccessLists;
import com.honeywell.honbacnethelper.export.BIHonBacnetCustomDescriptor;
import com.honeywell.honbacnethelper.util.BHonBacnetHelperUtil;
import com.honeywell.honbacnethelper.util.BIHonBacnetHelperInterface;
import com.honeywell.honplantcontroller.hmi.comm.BHMIBacnetMstpLinkLayer;
import com.honeywell.honplantcontroller.hmi.comm.UpdateFromHMIDeviceContext;
import com.honeywell.honplantcontroller.hmi.comm.BHonPlantControllerBacnetProperties;
import com.honeywell.honplantcontroller.hmi.comm.BIHonPlantControllerHMIInterface;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
import com.honeywell.honplantcontroller.hmi.view.BFastAccessList;
import com.honeywell.honplantcontroller.hmi.view.BHonPlantControllerFALHome;
import com.tridium.bacnet.datatypes.BSvoSubordinate;
import com.tridium.bacnet.stack.BBacnetStack;
import com.tridium.bacnet.stack.link.BBacnetLinkLayer;
import com.tridium.bacnet.stack.network.BBacnetNetworkLayer;
import com.tridium.bacnet.stack.network.BNetworkPort;
import com.tridium.bacnet.stack.server.BBacnetExportTable;

@NiagaraType
public class BHonPlantControllerHMISubscriber extends BComponent {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honPlantController.hmi.service.BHonPlantControllerHMISubscriber(2979906276)1.0$ @*/
/* Generated Tue Feb 22 10:00:06 CET 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPlantControllerHMISubscriber.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public BHonPlantControllerHMISubscriber(BHonPlantControllerHMI service) {
    this.service = service;
    //initializeSubscriber();
  }

  /**
   * Initializes the internal subscriber
   */
  public void initializeSubscriber() {
    final BHonPlantControllerHMISubscriber thisSubscriber = this;

    subscriber = new Subscriber() {
      private final Object syncObject = new Object();

      @Override
      public void event(BComponentEvent event) {

        String propName = event.getSlot().getName();
        BComponent srcComponent = event.getSourceComponent();
        BValue propertyInSrcComponent = srcComponent.get(propName);
        BValue actualValue = event.getValue();

        switch (event.getId()) {
          case BComponentEvent.PROPERTY_ADDED: {
            handlePropertyAddedEvent(srcComponent, propertyInSrcComponent, actualValue);
          }
          break;

          case BComponentEvent.PROPERTY_CHANGED: {
            handlePropertyChangedEvent(srcComponent, propName, propertyInSrcComponent, actualValue);
          }
          break;

          case BComponentEvent.PROPERTY_REMOVED: {
            handlePropertyRemovedEvent(srcComponent, propertyInSrcComponent, actualValue);
          }
          break;
        
          default:
            break;
        }

      }

      /**
       * 
       * @param bacnetNetwork
       */
      private void trySubscribeBacnetNetworkAndChildren(BBacnetNetwork bacnetNetwork) {
        if(HonPlantControllerBacnetUtils.isHMIBacnetNetwork(bacnetNetwork, service.getBacnetProperties())) {
          BBacnetNetworkLayer bacnetNetworkLayer = ((BBacnetStack) bacnetNetwork.getBacnetComm()).getNetwork();
          BNetworkPort[] networkPorts = bacnetNetworkLayer.getChildren(BNetworkPort.class);
          for (BNetworkPort networkPort : networkPorts) {
            BBacnetLinkLayer bacnetLinkLayer = networkPort.getLink();
            if(bacnetLinkLayer instanceof BHMIBacnetMstpLinkLayer) {
              subscribe(bacnetNetwork);
              subscribe(bacnetNetworkLayer);
              subscribe(bacnetLinkLayer);
              subscribe(networkPort);
  
              HonPlantControllerHMILogger.checkAndLog(Level.FINE,
                "ObjectSubscriber: event: BComponentEvent.PROPERTY_ADDED: subscribed HMIBacnetNetwork and its children");
              break;
            }
          }
        }
      }

      /**
       * 
       * @param srcComponent
       * @param propertyInSrcComponent
       * @param actualValue
       */
      private void handlePropertyAddedEvent(BComponent srcComponent, BValue propertyInSrcComponent, BValue actualValue) {
        HonPlantControllerHMILogger.checkAndLog(Level.FINE,
            "ObjectSubscriber: event: BComponentEvent.PROPERTY_ADDED: property = " + propertyInSrcComponent.toString() + " type " + propertyInSrcComponent.getType());
        HonPlantControllerHMILogger.checkAndLog(Level.FINE,
            "ObjectSubscriber: event: BComponentEvent.PROPERTY_ADDED: srcComponent = " + srcComponent.toString() + " type " + srcComponent.getType());

        if (propertyInSrcComponent instanceof BBacnetNetwork) {
          trySubscribeBacnetNetworkAndChildren((BBacnetNetwork) propertyInSrcComponent);
        } 
        else if (srcComponent instanceof BHonFastAccessLists) {
          if (propertyInSrcComponent instanceof BHonFastAccessList) {
            BHonFastAccessList element = (BHonFastAccessList) propertyInSrcComponent;
            handleAddFastAccessList(element);
            BComponent subordinates = element.getSubordinates();
            subscriber.subscribe(subordinates);
          }
        } 
        else if (srcComponent instanceof BBacnetArray) {
          BBacnetArray comp = (BBacnetArray) srcComponent;

          if(propertyInSrcComponent instanceof BSvoSubordinate) {
            BSvoSubordinate falEntry = (BSvoSubordinate) propertyInSrcComponent;
            BComplex parent = comp.getParent();

            if(parent instanceof BHonFastAccessList) {
              BHonFastAccessList exportedFAl = (BHonFastAccessList)parent;
              String objectName = (!exportedFAl.getObjectName().isEmpty()) ? exportedFAl.getObjectName() : comp.getName();
              objectName = SlotPath.escape(objectName);
              BHonPlantControllerFALHome falHome = thisSubscriber.service.getHome().getFastAccessLists();
              BFastAccessList fastAccessList = (BFastAccessList) falHome.get(objectName);
              if (fastAccessList == null) {
                //Check if objectName is the displayName of a FastAccessList
                BFastAccessList[] fastAccessLists = falHome.getChildren(BFastAccessList.class);
                for (BFastAccessList fal : fastAccessLists) {
                  if (fal.getDisplayName(null).equals(objectName)) {
                    //Entry found
                    fastAccessList = fal;
                    break;
                  }
                }
              }

              if (fastAccessList == null) {
            	falHome.add(objectName, new BFastAccessList(), new UpdateFromHMIDeviceContext());
                fastAccessList = (BFastAccessList) falHome.get(objectName);
              }
              handleAddFastAccessListEntry(falEntry, fastAccessList);
            }
          }
        }
      }

      /**
       * 
       * @param srcComponent
       * @param propertyInSrcComponent
       * @param actualValue
       */
      private void handlePropertyChangedEvent(BComponent srcComponent, String slotNameInSourceComponent, BValue propertyInSrcComponent, BValue actualValue) {
        HonPlantControllerHMILogger.checkAndLog(Level.FINE,
        "ObjectSubscriber: event: BComponentEvent.PROPERTY_CHANGED: property = " + propertyInSrcComponent.toString() + " type " + propertyInSrcComponent.getType() + "actualValue: " + actualValue);
        HonPlantControllerHMILogger.checkAndLog(Level.FINE,
        "ObjectSubscriber: event: BComponentEvent.PROPERTY_CHANGED: srcComponent = " + srcComponent.toString() + " type " + srcComponent.getType());

        if(srcComponent instanceof BHonFastAccessList && slotNameInSourceComponent.equals(BHonFastAccessList.objectName.getName())) {
        	String objectName =  propertyInSrcComponent.toString();
          	objectName = SlotPath.escape(objectName);

        	BOrd serviceFALOrd = ((BHonFastAccessList) srcComponent).getServiceFALOrd();
        	BIHonBacnetHelperInterface honBacnetHelperInterface = BHonBacnetHelperUtil.getHonBacnetHelperInterface();
        	if(null != honBacnetHelperInterface) {
        		try {
        			BObject object = serviceFALOrd.resolve((BComponent)honBacnetHelperInterface).get();
        			if(object instanceof BFastAccessList) {
        				BComponent falHome = (BComponent)((BFastAccessList) object).getParent();

        				falHome.rename(falHome.getProperty(((BFastAccessList) object).getName()), objectName, new UpdateFromHMIDeviceContext());
        			}
        			Subscriber falSubscriber = honBacnetHelperInterface.getFALSubscriber();
        			if(null != falSubscriber) {
        				falSubscriber.subscribe(srcComponent);
        			}
        		}catch (Exception e) {
        			logger.finest("[ObjectSubscriber] caught an exception: " + e.getMessage());
    			}
        	}

        }
        if(srcComponent instanceof BHMIBacnetMstpLinkLayer) {
          handleMstpLinkLayerChanged((BHMIBacnetMstpLinkLayer)srcComponent);
        }
      }

      /**
       * 
       * @param srcComponent
       * @param propertyInSrcComponent
       * @param actualValue
       */
      private void handlePropertyRemovedEvent(BComponent srcComponent, BValue propertyInSrcComponent, BValue actualValue) {
        HonPlantControllerHMILogger.checkAndLog(Level.FINE,
        "ObjectSubscriber: event: BComponentEvent.PROPERTY_REMOVED: srcComponent = " + srcComponent.toString() + " type " + srcComponent.getType());

        HonPlantControllerHMILogger.checkAndLog(Level.FINE,
        "ObjectSubscriber: event: BComponentEvent.PROPERTY_REMOVED: actualValue = " + actualValue.toString() + " type " + actualValue.getType());
        if(actualValue instanceof BDriverContainer) {
          BBacnetNetwork bacnetNetwork = BIHonPlantControllerHMIInterface.getBacnetNetwork();
          if (bacnetNetwork != null) {
            handleBacnetNetworkRemoved(bacnetNetwork);
          }
        }
        if(actualValue instanceof BBacnetNetwork) {
          handleBacnetNetworkRemoved((BBacnetNetwork)actualValue);
        }
        else if(actualValue instanceof BNetworkPort) {
          handleNetworkPortRemoved((BNetworkPort)actualValue, (BBacnetNetworkLayer)srcComponent);
        }
        else if (srcComponent instanceof BHonFastAccessLists) {
          if (actualValue instanceof BHonFastAccessList) {
            BHonFastAccessList element = (BHonFastAccessList) actualValue;
            handleRemoveFastAccessList(element);
          }
        } 
        else if (srcComponent instanceof BBacnetArray) {
          BBacnetArray comp = (BBacnetArray) srcComponent;

          if(actualValue instanceof BSvoSubordinate) {
            BSvoSubordinate falEntry = (BSvoSubordinate) actualValue;
            BComplex parent = comp.getParent();

            if(parent instanceof BHonFastAccessList) {
              BHonFastAccessList exportedFAl = (BHonFastAccessList)parent;
              String objectName = (!exportedFAl.getObjectName().isEmpty()) ? exportedFAl.getObjectName() : comp.getName();
        	  objectName = SlotPath.escape(objectName);

              BHonPlantControllerFALHome falHome = thisSubscriber.service.getHome().getFastAccessLists();
              BFastAccessList fastAccessList = (BFastAccessList) falHome.get(objectName);
              if (fastAccessList == null) {
                //Check if objectName is the displayName of a FastAccessList
                BFastAccessList[] fastAccessLists = falHome.getChildren(BFastAccessList.class);
                for (BFastAccessList fal : fastAccessLists) {
                  if (fal.getDisplayName(null).equals(objectName)) {
                    //Entry found
                    fastAccessList = fal;
                    break;
                  }
                }
              }

              if (fastAccessList == null) {
            	falHome.add(objectName, new BFastAccessList(), new UpdateFromHMIDeviceContext());
                fastAccessList = (BFastAccessList) falHome.get(objectName);
              }
              handleRemoveFastAccessListEntry(falEntry, fastAccessList);
            }
          }
        }
      }

      private void handleAddFastAccessList(BHonFastAccessList element) {
        synchronized (syncObject) {
          HonPlantControllerHMILogger.checkAndLog(Level.FINE,
              "ObjectSubscriber: event: BComponentEvent.PROPERTY_ADDED: handleAddFastAccessList");

          BHonPlantControllerFALHome falHome = thisSubscriber.service.getHome().getFastAccessLists();
          String objectName = (!element.getObjectName().isEmpty()) ? element.getObjectName() : element.getName();
          BFastAccessList fastAccessList = new BFastAccessList();
    	  objectName = SlotPath.escape(objectName);

    	  if (falHome.get(objectName) == null) {
            //Add new fastAccessList to falHome
        	falHome.add(objectName, fastAccessList, new UpdateFromHMIDeviceContext());
          }
          //TODO: Need to synchronize
          try
          {
            falHome.loadSlots();
            falHome.getComponentSpace().sync();
          }
          catch (Exception e) {
            HonPlantControllerHMILogger.checkAndLog(Level.WARNING, e.getMessage());
          }

          //get the newly added or existing fastAccessList
          fastAccessList = (BFastAccessList) falHome.get(objectName);
          element.setServiceFALOrd(fastAccessList.getHandleOrd());

          BBacnetArray subOrdinates = element.getSubordinates();
          BSvoSubordinate[] svoSubordinate = subOrdinates.getChildren(BSvoSubordinate.class);

          for (BSvoSubordinate subordinate : svoSubordinate) {
            HonPlantControllerHMILogger.checkAndLog(Level.FINE,
                    "ObjectSubscriber: event: BComponentEvent.PROPERTY_ADDED: handleAddFastAccessList subOrdinate = " + subordinate + "; type = " + subordinate.getType());
            handleAddFastAccessListEntry(subordinate, fastAccessList);
          }
          //subscribe to newly added object
          subscribe(element);
        }
      }

      private void handleRemoveFastAccessList(BHonFastAccessList element) {
        synchronized (syncObject) {
          HonPlantControllerHMILogger.checkAndLog(Level.FINE,
              "ObjectSubscriber: event: BComponentEvent.PROPERTY_REMOVED: handleRemoveFastAccessList element = " + element.getObjectName());
          BHonPlantControllerFALHome falHome = thisSubscriber.service.getHome().getFastAccessLists();
          String objectName = element.getObjectName();
          BFastAccessList fastAccessList = (BFastAccessList) falHome.get(objectName);
          if (fastAccessList == null) {
            //Check if objectName is the displayName of a FastAccessList
            BFastAccessList[] fastAccessLists = falHome.getChildren(BFastAccessList.class);
            for (BFastAccessList fal : fastAccessLists) {
              if (fal.getDisplayName(null).equals(objectName)) {
                //Entry found
                fastAccessList = fal;
                break;
              }
            }
          }

          if (fastAccessList != null) {
        	  falHome.remove(fastAccessList.getName(), new UpdateFromHMIDeviceContext());
          }

          //unsubscribe to newly removed object
          unsubscribe(element);
        }
      }

      private void handleAddFastAccessListEntry(BSvoSubordinate falEntry,  BFastAccessList fastAccessList) {
        BComponent point = retrieveActualPointFromFALEntry(falEntry);
        if(null != point) {
          try {
            fastAccessList.addObjectWithNoStatusUpdate(point);
            BIBacnetExportObject retrieveBacnetDescriptorFromFALEntry = retrieveBacnetDescriptorFromFALEntry(falEntry);
            if(retrieveBacnetDescriptorFromFALEntry instanceof BIHonBacnetCustomDescriptor) {
            	((BIHonBacnetCustomDescriptor) retrieveBacnetDescriptorFromFALEntry).setPointInFAL(true);
            }
          } catch (Exception e) {
            HonPlantControllerHMILogger.checkAndLog(Level.WARNING, e.getMessage());
          }
        }
      }

      private void handleRemoveFastAccessListEntry(BSvoSubordinate falEntry,  BFastAccessList fastAccessList) {
        BComponent point = retrieveActualPointFromFALEntry(falEntry);
        if(null != point) {
          try {
            fastAccessList.doRemoveObject(point);
          } catch (Exception e) {
            HonPlantControllerHMILogger.checkAndLog(Level.WARNING, e.getMessage());
          }
        }
      }

      private BComponent retrieveActualPointFromFALEntry(BSvoSubordinate falEntry) {
        BBacnetExportTable exportTable = (BBacnetExportTable) thisSubscriber.service.getBacnetNetwork().getLocalDevice().getExportTable();

        BIBacnetExportObject bacnetDescriptor = exportTable.byObjectId(falEntry.getReference().getObjectId());

        if(bacnetDescriptor instanceof BBacnetPointDescriptor) {
          BComponent object = ((BBacnetPointDescriptor) bacnetDescriptor).getPoint();
          HonPlantControllerHMILogger.checkAndLog(Level.FINE,
              "ObjectSubscriber: event: BComponentEvent.PROPERTY_ADDED: handleAddFastAccessList comp object = " + object.getSlotPath());

          return object;
        }

        return null;
      }

      private BIBacnetExportObject retrieveBacnetDescriptorFromFALEntry(BSvoSubordinate falEntry) {
          BBacnetExportTable exportTable = (BBacnetExportTable) thisSubscriber.service.getBacnetNetwork().getLocalDevice().getExportTable();

          return exportTable.byObjectId(falEntry.getReference().getObjectId());
        }
    };
  }

  /**
   * 
   * @param bacnetNetwork
   */
  private void handleBacnetNetworkRemoved(BBacnetNetwork bacnetNetwork) {
    HonPlantControllerHMILogger.checkAndLog(Level.FINE, "bacnetNetwork removed: "+bacnetNetwork);
    //Check if network was hmi network, read it back
    if(HonPlantControllerBacnetUtils.isHMIBacnetNetwork(bacnetNetwork, service.getBacnetProperties())) {
      BBacnetNetwork remainingHMIbacnetNetwork = BIHonPlantControllerHMIInterface.getBacnetNetwork();
      if(remainingHMIbacnetNetwork == null) {

      /// ------------ Re-Added Network is not working, until Station restart -------------- 
      //   //add deleted network back
      //   // BLink sendIamLink = (BLink)bacnetNetwork.getLocalDevice().get("sendIamLink");
      //   bacnetNetwork.stop();
      //   if(bacnetNetwork.getLocalDevice().get("sendIAmLink") instanceof BLink) {
      //     bacnetNetwork.getLocalDevice().remove("sendIAmLink");
      //   }

      //   HonPlantControllerBacnetUtils.addBacnetNetworkToStation(bacnetNetwork, Sys.getStation());
      //   bacnetNetwork.start();
      // }
      }
    }
  }

  /**
   * 
   * @param networkPort
   */
  private void handleNetworkPortRemoved(BNetworkPort networkPort, BBacnetNetworkLayer bacnetNetworkLayer) {
    HonPlantControllerHMILogger.checkAndLog(Level.FINE, "networkPort removed: "+networkPort);
    //check if networkport was hmi
    if(HonPlantControllerBacnetUtils.compareMstpPorts(networkPort, service.getBacnetProperties().createMstpPort())) {
      BNetworkPort[] networkPorts = bacnetNetworkLayer.getChildren(BNetworkPort.class);

      //check if there are other hmi ports present
      BNetworkPort otherHMIPort = Arrays.stream(networkPorts).filter(port -> HonPlantControllerBacnetUtils.compareMstpPorts(port, service.getBacnetProperties().createMstpPort()))
                                  .findAny().orElse(null);
      
      //no other hmiPorts found, add back again
      if(otherHMIPort == null) {
        bacnetNetworkLayer.add(BHonPlantControllerBacnetProperties.MSTP_PORT_NAME, networkPort);
      }
    }

  }

  /**
   * 
   * @param bacnetMstpLinkLayer
   */
  private void handleMstpLinkLayerChanged(BHMIBacnetMstpLinkLayer bacnetMstpLinkLayer) {
    HonPlantControllerHMILogger.checkAndLog(Level.FINE, "bacnetMstpLinkLayer changed: "+bacnetMstpLinkLayer);
  }

  


  /**
   * 
   */
  public void subscribe() {
    HonPlantControllerHMILogger.checkAndLog(Level.FINE, "ObjectSubscriber: subscribe");
    //TODO: uncomment once needed

    /*
    subscribeObject("driver:DriverContainer");
    subscribeObject("bacnet:BacnetNetwork");
    subscribeObject("bacnet:BacnetMstpLinkLayer");
    subscribeObject("bacnet:NetworkPort");
    subscribeObject("bacnet:BacnetNetworkLayer");
    */

    subscribeObject("honBacnetHelper:HonFastAccessLists");
    subscribeObject("honBacnetHelper:HonFastAccessList");
  }

  /**
   * Commands the subscriber to subscribe to given types in station.
   * @param bqlQueryType TypeSpec of type to be subscribed
   */
  void subscribeObject(final String bqlQueryType) {
    HonPlantControllerHMILogger.checkAndLog(Level.FINE, "ObjectSubscriber: subscribeObject: bqlQueryType = "
              + bqlQueryType, null);

      final BOrd ord = BOrd.make("slot:/|bql:select * from " + bqlQueryType);
      final BITable<BObject> table = (BITable<BObject>) ord.resolve((BHonPlantControllerHMI) service).get();
      final Cursor<BObject> c = table.cursor();
      while (c.next()) {
          final BComponent comp = (BComponent) c.get();
          subscriber.subscribe(comp);
          if (comp instanceof BHonFastAccessList) {
            BComponent subordinates = ((BHonFastAccessList) comp).getSubordinates();
            subscriber.subscribe(subordinates);
          }
      }
  }

  public void unsubscribe() {
	  if(null == subscriber)
		  return;
	  
      subscriber.unsubscribeAll();
  }

  public Subscriber getFALSubscriber() {
	  return subscriber;
  }

  private Subscriber subscriber;
  private BHonPlantControllerHMI service;

  public static final Logger logger = Logger.getLogger("HonPlantControllerHMI");
}
