/*
 *  Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 *
 *  This file contains trade secrets of Honeywell International, Inc.  No part 
 *  may be reproduced or transmitted in any form by any means or for any 
 *  purpose without the express written permission of Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.user.authenticator;

    import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
    import com.tridium.nre.auth.NiagaraStationAlgorithmBundle;
    import com.tridium.nre.auth.Pbkdf2;
    import com.tridium.nre.security.KeyDerivationAlgorithmBundle;
    import com.tridium.nre.security.SecretChars;
    import java.security.MessageDigest;
    import java.security.NoSuchAlgorithmException;
    import java.security.SecureRandom;
    import javax.baja.agent.BIAgent;
    import javax.baja.nre.annotations.AgentOn;
    import javax.baja.nre.annotations.NiagaraType;
    import javax.baja.nre.util.ByteArrayUtil;
    import javax.baja.nre.util.SecurityUtil;
    import javax.baja.nre.util.TextUtil;
    import javax.baja.security.BAbstractPasswordEncoder;
    import javax.baja.sys.Sys;
    import javax.baja.sys.Type;

@NiagaraType(
    agent = {@AgentOn(
        types = {"baja:Password"}
    )}
)
public final class BHonPasswordEncoder extends BAbstractPasswordEncoder implements BIAgent {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.user.authenticator.BHonPasswordEncoder(3604161203)1.0$ @*/
/* Generated Wed May 18 11:04:32 CEST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPasswordEncoder.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/
  public BHonPasswordEncoder() {
  }

  public void encode(SecretChars password) throws Exception {
    byte[] salt = new byte[SALT_LENGTH];
    (new SecureRandom()).nextBytes(salt);
    //HMI/CPO restrictions
    salt[0] = 0x00;

    this.salt = ByteArrayUtil.toHexString(salt);
    this.iterationCount = ITERATION_COUNT;
    //byte[] key = HonPbkdf2.deriveKey(salt, this.iterationCount, password.get(), ALGORITHM_BUNDLE);

    //generate dk
    byte[] dk = new byte[DK_LENGTH];
    byte[] pwd = new String(password.get()).getBytes();
    byte[] saltedPwd = new byte[pwd.length + salt.length];
    System.arraycopy(pwd, 0, saltedPwd, 0, pwd.length);
    System.arraycopy(salt, 0, saltedPwd, pwd.length, salt.length);
    MessageDigest digest = MessageDigest.getInstance("SHA-256");
    byte[] hash = digest.digest(saltedPwd);

    //2nd iteration
    saltedPwd = new byte[pwd.length + salt.length];
    System.arraycopy(pwd, 0, saltedPwd, 0, pwd.length);
    System.arraycopy(hash, 0, saltedPwd, pwd.length, hash.length);
    digest = MessageDigest.getInstance("SHA-256");
    hash = digest.digest(saltedPwd);

    this.key = TextUtil.bytesToHexString(hash);
  }

  public void parse(String key) throws IllegalArgumentException {
    String[] data = ALGORITHM_BUNDLE.decode(key);
    this.salt = data[0];
    this.iterationCount = Integer.parseInt(data[1]);
    this.key = data[2];
  }

  public String getValue() {
    String[] data = new String[ALGORITHM_BUNDLE.getDataElementCount()];
    data[0] = this.salt;
    data[1] = String.valueOf(this.iterationCount);
    data[2] = this.key;
    return ALGORITHM_BUNDLE.encode(data);
  }

  public String getEncodingType() {
    return ENCODING_TYPE;
  }

  public boolean isReversible() {
    return false;
  }

  public boolean validate(SecretChars password) {
    try {
      byte[] key = Pbkdf2.deriveKey(ByteArrayUtil.hexStringToBytes(this.salt), this.iterationCount, password.get(), ALGORITHM_BUNDLE);
      String tkey = TextUtil.bytesToHexString(key);
      return SecurityUtil.equals(tkey, this.key);
    } catch (Exception var4) {
    	HonPlantControllerHMILogger.finest(var4.getMessage(), var4);
      return false;
    }
  }

  public String getEncodedValue() {
    return this.getValue();
  }

  public void encode(String password, String salt, int iterationCount) throws Exception {
    this.salt = salt;
    this.iterationCount = iterationCount;
    byte[] hexSalt = ByteArrayUtil.hexStringToBytes(salt);
    byte[] key = Pbkdf2.deriveKey(hexSalt, iterationCount, password, ALGORITHM_BUNDLE);
    this.key = TextUtil.bytesToHexString(key);
  }

  public String getSalt() {
    return this.salt;
  }

  public int getIterationCount() {
    return this.iterationCount;
  }

  public byte[] getKey() {
    return ByteArrayUtil.hexStringToBytes(this.key);
  }

  public static BHonPasswordEncoder makeFake(String userName) {
    byte[] key = new byte[ALGORITHM_BUNDLE.getKeyLength()];
    (new SecureRandom()).nextBytes(key);
    BHonPasswordEncoder fake = new BHonPasswordEncoder();
    fake.key = TextUtil.bytesToHexString(key);

    try {
      MessageDigest md = MessageDigest.getInstance("Sha-256");
      byte[] salt = md.digest(userName.getBytes());
      fake.salt = TextUtil.bytesToHexString(salt);
      fake.iterationCount = ITERATION_COUNT;
      return fake;
    } catch (NoSuchAlgorithmException var5) {
      throw new UnsupportedOperationException(var5.getLocalizedMessage());
    }
  }

  private String salt;
  private String key;
  private int iterationCount;
  private static final Pbkdf2.EncodingCalculationCache CALC_CACHE = new Pbkdf2.EncodingCalculationCache(100);
  private static final KeyDerivationAlgorithmBundle ALGORITHM_BUNDLE = NiagaraStationAlgorithmBundle.getInstance();
  public static final String ENCODING_TYPE = ALGORITHM_BUNDLE.getAlgorithmName();
  private static final int ITERATION_COUNT = 2; //4096;//10000;
  private static final int SALT_LENGTH = 32;
  private static final int DK_LENGTH = 32;
}
