package com.honeywell.honplantcontroller.hmi.user.authenticator;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.text.MessageFormat;
import java.util.concurrent.atomic.AtomicReference;
import java.util.logging.Logger;

import javax.baja.authn.BPasswordAuthenticationScheme;
import javax.baja.nre.annotations.AgentOn;
import javax.baja.nre.annotations.Facet;
import javax.baja.nre.annotations.NiagaraAction;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.nre.util.TextUtil;
import javax.baja.security.BPassword;
import javax.baja.security.BPasswordHistory;
import javax.baja.security.BPbkdf2HmacSha256PasswordEncoder;
import javax.baja.status.BStatus;
import javax.baja.sys.Action;
import javax.baja.sys.BAbsTime;
import javax.baja.sys.BBoolean;
import javax.baja.sys.BComponent;
import javax.baja.sys.BFacets;
import javax.baja.sys.BIMixIn;
import javax.baja.sys.BIcon;
import javax.baja.sys.BInteger;
import javax.baja.sys.BString;
import javax.baja.sys.BValue;
import javax.baja.sys.BasicContext;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.IPropertyValidator;
import javax.baja.sys.Localizable;
import javax.baja.sys.LocalizableRuntimeException;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.user.BUser;
import javax.baja.user.BUserService;
import javax.baja.util.CannotValidateException;
import javax.baja.util.Lexicon;

import com.honeywell.honplantcontroller.hmi.limitations.HonHMILimitations;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerUserUtils;
import com.tridium.nre.security.SecretChars;
import com.tridium.user.BUserPasswordConfiguration;

@NiagaraProperty(
  name = "enabled",
  type = "boolean",
  defaultValue = "false",
  flags = Flags.DEFAULT_ON_CLONE
)

@NiagaraProperty(
    name = "lockOut",
    type = "boolean",
    defaultValue = "false",
    flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)

@NiagaraProperty(
  name = "status",
  type = "baja:Status",
  defaultValue = "BStatus.ok",
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)

@NiagaraProperty(
  name = "faultCause",
  type = "String",
  defaultValue = "",
  flags = Flags.READONLY | Flags.DEFAULT_ON_CLONE
)

@NiagaraProperty(
  name = "pin",
  type = "baja:Password",
  defaultValue = "BPassword.DEFAULT",
  flags = Flags.DEFAULT_ON_CLONE,
  facets = {
    @Facet(name = "BFacets.FIELD_WIDTH", value = "BInteger.make(6)")
  }
)

@NiagaraProperty(
  name = "pinConfiguration",
  type = "baja:UserPinConfiguration",
  defaultValue = "new BUserPinConfiguration()"
)

@NiagaraProperty(
    name = "dk",
    type = "baja:String",
    defaultValue = "BString.DEFAULT",
    flags = Flags.READONLY | Flags.HIDDEN
)

@NiagaraProperty(
    name = "pwdFailCnt",
    type = "int",
    defaultValue = "0",
    flags = Flags.READONLY | Flags.HIDDEN | Flags.DEFAULT_ON_CLONE
)

@NiagaraAction(
    name = "clearLockout"
)

@NiagaraAction(
  name = "removePin"
)

@SuppressWarnings({"squid:S1845"})
@NiagaraType(
  agent = @AgentOn(types = {"baja:User"})
)
public class BHonPlantControllerHMIAuthenticator
    extends BComponent
    implements BIMixIn
{
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.user.authenticator.BHonPlantControllerHMIAuthenticator(2907718530)1.0$ @*/
/* Generated Wed Jun 29 10:10:09 CEST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "enabled"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code enabled} property.
   * @see #getEnabled
   * @see #setEnabled
   */
  public static final Property enabled = newProperty(Flags.DEFAULT_ON_CLONE, false, null);
  
  /**
   * Get the {@code enabled} property.
   * @see #enabled
   */
  public boolean getEnabled() { return getBoolean(enabled); }
  
  /**
   * Set the {@code enabled} property.
   * @see #enabled
   */
  public void setEnabled(boolean v) { setBoolean(enabled, v, null); }

////////////////////////////////////////////////////////////////
// Property "lockOut"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code lockOut} property.
   * @see #getLockOut
   * @see #setLockOut
   */
  public static final Property lockOut = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, false, null);
  
  /**
   * Get the {@code lockOut} property.
   * @see #lockOut
   */
  public boolean getLockOut() { return getBoolean(lockOut); }
  
  /**
   * Set the {@code lockOut} property.
   * @see #lockOut
   */
  public void setLockOut(boolean v) { setBoolean(lockOut, v, null); }

////////////////////////////////////////////////////////////////
// Property "status"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code status} property.
   * @see #getStatus
   * @see #setStatus
   */
  public static final Property status = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, BStatus.ok, null);
  
  /**
   * Get the {@code status} property.
   * @see #status
   */
  public BStatus getStatus() { return (BStatus)get(status); }
  
  /**
   * Set the {@code status} property.
   * @see #status
   */
  public void setStatus(BStatus v) { set(status, v, null); }

////////////////////////////////////////////////////////////////
// Property "faultCause"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code faultCause} property.
   * @see #getFaultCause
   * @see #setFaultCause
   */
  public static final Property faultCause = newProperty(Flags.READONLY | Flags.DEFAULT_ON_CLONE, "", null);
  
  /**
   * Get the {@code faultCause} property.
   * @see #faultCause
   */
  public String getFaultCause() { return getString(faultCause); }
  
  /**
   * Set the {@code faultCause} property.
   * @see #faultCause
   */
  public void setFaultCause(String v) { setString(faultCause, v, null); }

////////////////////////////////////////////////////////////////
// Property "pin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code pin} property.
   * @see #getPin
   * @see #setPin
   */
  public static final Property pin = newProperty(Flags.DEFAULT_ON_CLONE, BPassword.DEFAULT, BFacets.make(BFacets.FIELD_WIDTH, BInteger.make(6)));
  
  /**
   * Get the {@code pin} property.
   * @see #pin
   */
  public BPassword getPin() { return (BPassword)get(pin); }
  
  /**
   * Set the {@code pin} property.
   * @see #pin
   */
  public void setPin(BPassword v) { set(pin, v, null); }

////////////////////////////////////////////////////////////////
// Property "pinConfiguration"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code pinConfiguration} property.
   * @see #getPinConfiguration
   * @see #setPinConfiguration
   */
  public static final Property pinConfiguration = newProperty(0, new BUserPinConfiguration(), null);
  
  /**
   * Get the {@code pinConfiguration} property.
   * @see #pinConfiguration
   */
  public BUserPinConfiguration getPinConfiguration() { return (BUserPinConfiguration)get(pinConfiguration); }
  
  /**
   * Set the {@code pinConfiguration} property.
   * @see #pinConfiguration
   */
  public void setPinConfiguration(BUserPinConfiguration v) { set(pinConfiguration, v, null); }

////////////////////////////////////////////////////////////////
// Property "dk"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code dk} property.
   * @see #getDk
   * @see #setDk
   */
  public static final Property dk = newProperty(Flags.READONLY | Flags.HIDDEN, BString.DEFAULT, null);
  
  /**
   * Get the {@code dk} property.
   * @see #dk
   */
  public String getDk() { return getString(dk); }
  
  /**
   * Set the {@code dk} property.
   * @see #dk
   */
  public void setDk(String v) { setString(dk, v, null); }

////////////////////////////////////////////////////////////////
// Property "pwdFailCnt"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code pwdFailCnt} property.
   * @see #getPwdFailCnt
   * @see #setPwdFailCnt
   */
  public static final Property pwdFailCnt = newProperty(Flags.READONLY | Flags.HIDDEN | Flags.DEFAULT_ON_CLONE, 0, null);
  
  /**
   * Get the {@code pwdFailCnt} property.
   * @see #pwdFailCnt
   */
  public int getPwdFailCnt() { return getInt(pwdFailCnt); }
  
  /**
   * Set the {@code pwdFailCnt} property.
   * @see #pwdFailCnt
   */
  public void setPwdFailCnt(int v) { setInt(pwdFailCnt, v, null); }

////////////////////////////////////////////////////////////////
// Action "clearLockout"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code clearLockout} action.
   * @see #clearLockout()
   */
  public static final Action clearLockout = newAction(0, null);
  
  /**
   * Invoke the {@code clearLockout} action.
   * @see #clearLockout
   */
  public void clearLockout() { invoke(clearLockout, null, null); }

////////////////////////////////////////////////////////////////
// Action "removePin"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code removePin} action.
   * @see #removePin()
   */
  public static final Action removePin = newAction(0, null);
  
  /**
   * Invoke the {@code removePin} action.
   * @see #removePin
   */
  public void removePin() { invoke(removePin, null, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPlantControllerHMIAuthenticator.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/




  public BHonPlantControllerHMIAuthenticator() {}

  public BHonPlantControllerHMIAuthenticator(BPassword value) {
    setPin(value);
  }

  ////////////////////////////////////////////////////////////////
  // Component
  ////////////////////////////////////////////////////////////////

  ///CLOVER:OFF
  @Override
  public void started() {
    if (isInUserService()) {
      setConvertedPin(getPin());
    }
  }
  ///CLOVER:ON



  ///CLOVER:OFF
  @Override
  public void changed(Property property, Context context) {
    super.changed(property, context);
    if(!isRunning()) {
      return;
    }

    synchronized(syncObject) {
      if (isInUserService()) {
        if (enabled.equals(property)) {
          if (context == null || (context != null && !context.equals(invalidEnable)))
            validateNumberOfUsers();
        } else if (pin.equals(property) && context != pwConverted) {
          //getPin will contain the pin validated and accepted by getPropertyValidator
          setConvertedPin(getPin());
          validateNumberOfUsers();
        }
      }
    }
  }
  ///CLOVER:ON

  @Override
  public boolean isParentLegal(BComponent parent) {
    return parent instanceof BUser;
  }

  private void validateNumberOfUsers() {
    HonPlantControllerHMILogger.finest("[validateNumberOfUsers] NiagaraUserCount: " + HonPlantControllerUserUtils.getNiagaraUserCount());
    
    // If user limit is exceeded the user cannot be configured as HMI User
    // Should check if this User is the (HonHMILimitations.getMaxNumberOfUsers() + 1) User, then disable!
    if (getEnabled()) {
      if (HonPlantControllerUserUtils.getAllNiagaraUsers().count() > HonHMILimitations.getMaxNumberOfUsers()) {
        set(enabled, BBoolean.FALSE, invalidEnable);
        setStatus(BStatus.fault);
        setFaultCause(MessageFormat.format(LEXICON.get("hmi.limitations.exceeded.user.max.number"), HonHMILimitations.getMaxNumberOfUsers()));

        //Additionally a LOG entry
        LOG.severe(() -> MessageFormat.format(LEXICON.get("hmi.limitations.exceeded.user.max.number"), HonHMILimitations.getMaxNumberOfUsers()));
      } else {
        setStatus(BStatus.ok);
        setFaultCause("");
      }
    }
  }

  /**
   * This method is used to validate the pin given by user.
   * If the pin meets the requirements it will be accepted, if not user will have to provide another pin.
   * @param property Proposed property to be validated
   * @param context
   */
  @Override
  public IPropertyValidator getPropertyValidator(Property property, Context context) {

    if (property == pin) {
      return (validatable, context1) -> {
        ///CLOVER:OFF
        if (context1 != pwConverted) {
          checkPassword((BPassword) validatable.getProposedValue(pin), context1);
        }
        ///CLOVER:ON
      };
    }
    return null;
  }

  /**
   * Check if the specified password meets password requirements. Checks will
   * only be performed on reversible passwords, calls with an irreversible password
   * will skip checks and return successfully.
   *
   * @param newPassword the new password to check, must be reversible
   * @param context the context
   * @throws LocalizableRuntimeException if validation fails
   * @throws CannotValidateException if validation fails
   * @since Niagara 4.10
   */
  public void checkPassword(BPassword newPassword, Context context) {

    try {

      BUser user = (BUser)getParent();

      checkPassword(user, (BPasswordAuthenticationScheme)user.getAuthenticationScheme(),
                    getPinConfiguration(), newPassword, context);

      //pin is valid
      getPinConfiguration().passwordModified(newPassword);
    }
    catch(Exception ex) {
      throw ex;
    }
  }

  /**
   * Check if the specified password meets password requirements. Checks will
   * only be performed on reversible passwords, calls with an irreversible password
   * will skip checks and return successfully.
   *
   * @param user the user the password is for, or null if a new user
   * @param scheme the authentication scheme for the user, or null if not available
   * @param config the users UserPasswordConfiguration, or null if not available
   * @param newPassword the new password to check, must be reversible
   * @param context the context
   * @throws LocalizableRuntimeException if validation fails
   * @throws CannotValidateException if validation fails
   * @since Niagara 4.10
   */
  public void checkPassword(BUser user,
                            BPasswordAuthenticationScheme scheme,
                            BUserPasswordConfiguration config,
                            BPassword newPassword,
                            Context context)
  {

    if (!newPassword.getPasswordEncoder().isReversible()) {
      return;
    }

    try {
      BHonPlantControllerHMIPinStrength strength = new BHonPlantControllerHMIPinStrength();
      try (SecretChars passChars = AccessController.doPrivileged((PrivilegedAction<SecretChars>) newPassword::getSecretChars)) {

        AtomicReference<Localizable> messageRef = new AtomicReference<>();
        if (!strength.isPasswordValid(passChars.get(), messageRef::set)) {
          throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.user.pin.error.malformed");
        }
      }

      if (user != null && isDuplicatePin(newPassword, user, getPinConfiguration().getPasswordHistory())) {
        throw new LocalizableRuntimeException(TYPE.getModule().getModuleName(), "hmi.user.pin.error.alreadyInUse");
      }

      if (config != null) {
        config.changeIntervalCheck(scheme.getGlobalPasswordConfiguration());
      }
    }
    catch (LocalizableRuntimeException e) {
      throw e;
    }
    catch (Exception e) {
      throw new CannotValidateException(e);
    }
  }

  ///CLOVER:OFF
  private boolean isInUserService() {

    if(getParent() != null) {

      BValue parent = getParent().getParent();
      return parent instanceof BUserService;
    }
    else {
      return false;
    }
  }
  ///CLOVER:ON

  /**
   * This method is used to check if given pin was already used by the user.
   * @param newPin Pin given by user
   * @param user
   * @param passwordHistory Users pin history
   * @return true: given pin is in pin history; false: given pin is not in the pin history
   */
  ///CLOVER:OFF
  private boolean isDuplicatePin(BPassword newPin, BUser user, BPasswordHistory passwordHistory) {
    try {

      if(passwordHistory.contains(user.getUsername(),  newPin)) {
        return true;
      }
    }
    catch (Exception e) {
      HonPlantControllerHMILogger.warning("error while checking pin history ", e);
    }
    return false;
  }
  ///CLOVER:ON

  public boolean canLogin() {
    boolean enabled = getEnabled();
    boolean lockOut = getLockOut();
    BAbsTime expiration = getPinConfiguration().getExpiration();

    boolean ok = enabled && !(lockOut) && (expiration.isNull() || (!expiration.isNull() && expiration.isAfter(BAbsTime.now())));
    //TODO: this is actually commented, for unit test the method getParent().getName() throws nullPointerException
    //HonPlantControllerHMILogger.finest("[BHonPlantControllerHMIAuthenticator.canLogin] user: " + getParent().getName() + "; canlogin : " + ok);
    return ok;
  }


  public boolean canLogin(BUser user) {
    BHonPlantControllerHMIAuthenticator[] authenticators = user.getChildren(BHonPlantControllerHMIAuthenticator.class);
    if (authenticators.length < 1) {
      return false;
    }

    BUserPinConfiguration userConfig = authenticators[0].getPinConfiguration();
    BAbsTime expiration = userConfig.getExpiration();
    
    return !(!expiration.isNull() && expiration.isBefore(BAbsTime.now()));
  }

  ///CLOVER:OFF
  public BPassword convertToPbkdf2Password(BPassword pin) throws Exception {

    if (pin.getPasswordEncoder().isReversible()
        && !pin.getEncodingType().equals(BPbkdf2HmacSha256PasswordEncoder.ENCODING_TYPE))
    {
      //store salt and key locally to Authenticator
      passwordEncoder.encode(SecretChars.fromString(pin.getValue()));

      StringBuilder sb = new StringBuilder(TextUtil.bytesToHexString(getSalt()))
          .append(":")
          .append(getIterationCount())
          .append(":")
          .append(TextUtil.bytesToHexString(getKey()));
      setDk(sb.toString());

      //convert pin to Pbkdf2Password
      return BPassword.make(AccessController.doPrivileged((PrivilegedAction<String>)pin::getValue),
                            BPbkdf2HmacSha256PasswordEncoder.ENCODING_TYPE);
    }

    return pin;
  }
  ///CLOVER:ON

  /**
   * This methods is to be used for validating a given pin against the current pin
   * @param usersPin Pin provided by user
   * @return true: provided pin matches the current pin; false: provided pin does not match the current pin
   */
  public boolean validate(BPassword usersPin) {
    boolean valid = false;

    try {
      BPassword encodedPin = convertToPbkdf2Password(usersPin);

      try (SecretChars passChars = AccessController.doPrivileged((PrivilegedAction<SecretChars>) encodedPin::getSecretChars)) {

        BPassword currentPint = getPin();

        valid = currentPint.getPasswordEncoder().validate(passChars);
      } catch (Exception e) {
        HonPlantControllerHMILogger.warning("[BHonPlantControllerHMIAuthenticator.validate] error validating the pin: " + e.getMessage());
      }
    } catch (Exception e) {
        HonPlantControllerHMILogger.finest(e.getMessage(), e);
    }

    return valid;
  }

  /**
   * This methods is to be used for validating a given pin against the current pin
   * @param data Hash Value Pin received from HMI
   * @return true: provided pin matches the current pin; false: provided pin does not match the current pin
   */
  public boolean validateDk(byte[] data) {

    boolean valid = false;

    try {
      String[] tokens = getDk().split(":");
      String key = tokens[2];

      byte[] dk = hexStringToBytes(key);
      if (data.length == dk.length) {
        boolean ok = true;
        for (int i = 0; i < data.length; i++) {
          if (data[i] != dk[i]) {
            ok = false;
            break;
          }
        }
        valid = ok;
      }
    } catch (Exception e) {
      HonPlantControllerHMILogger.warning("[BHonPlantControllerHMIAuthenticator.validateDk] error validating the dk: " + e.getMessage());
    }

    //reset or increment pwdFailCnt; //is the 10 MaxFailCnt
    setPwdFailCnt((valid) ? 0 : getPwdFailCnt() + 1);
    setLockOut(getPwdFailCnt() >= MAX_FAIL_CNT);
    return valid;
  }

  public boolean updateDk(byte[] decryptedSalt, int iterationCount, byte[] decryptedDk) {
    StringBuilder sb = new StringBuilder(TextUtil.bytesToHexString(decryptedSalt))
        .append(":")
        .append(iterationCount)
        .append(":")
        .append(TextUtil.bytesToHexString(decryptedDk));

    boolean ok = false;
    if (!getDk().equals(sb.toString())) {
      setDk(sb.toString());
      ok = true;
    }

    return ok;
  }

  ///CLOVER:OFF
  private void setConvertedPin(BPassword pinToEncode) {
    try {
      BPassword encodedPin = convertToPbkdf2Password(pinToEncode);
      set("pin", encodedPin, pwConverted);
    }
    catch (Exception e) {
      HonPlantControllerHMILogger.warning("error during encoding the pin", e);
    }
  }
  ///CLOVER:ON

  private byte[] getKey() {
    return passwordEncoder.getKey();
  }

  private byte[] getSalt() {
    String salt = passwordEncoder.getSalt();
    return hexStringToBytes(salt);
  }

  private int getIterationCount() {
    return passwordEncoder.getIterationCount();
  }

  public byte[] hexStringToBytes(String data) {
    byte[] val = new byte[data.length() / 2];
    for (int i = 0; i < val.length; i++) {
      int index = i * 2;
      int j = Integer.parseInt(data.substring(index, index + 2), 16);
      val[i] = (byte) j;
    }
    return val;
  }

  ///CLOVER:OFF
  public static final Context pwConverted = new BasicContext() {
    @Override public boolean equals(Object obj) { return this == obj; }
    @Override public int hashCode() { return toString().hashCode(); }
    @Override public String toString() { return "Context.pwConverted"; }
  };
  
  private static final Context invalidEnable = new BasicContext() {
    @Override public boolean equals(Object obj) { return this == obj; }
    @Override public int hashCode() { return toString().hashCode(); }
    @Override public String toString() { return "Context.invalidEnable"; }
  };
  ///CLOVER:ON

  public void doClearLockout() {
    setLockOut(false);
  }

  public void doRemovePin() {
    setPin(BPassword.DEFAULT);
    getPinConfiguration().setPasswordHistory(new BPasswordHistory());
  }


  ///CLOVER:OFF
  @Override
  public String getDisplayNameInParent(Context context) {
    return LEXICON.get("hmi.user.pin.authenticator.displayName", "HonPlantControllerHMIAuthenticator");
  }
  ///CLOVER:ON

  ///CLOVER:OFF
  @Override
  public BIcon getIcon() {
    return icon;
  }
  ///CLOVER:ON

  //Attributes
  private static final Object syncObject = new Object();
  private static final BHonPasswordEncoder passwordEncoder = new BHonPasswordEncoder();
  public static final BHonPlantControllerHMIAuthenticator DEFAULT = new BHonPlantControllerHMIAuthenticator();
  private static final BIcon icon = BIcon.make("module://icons/x16/lock.png");

  private static final Lexicon LEXICON = Lexicon.make(TYPE.getModule().getModuleName());
  private static final Logger LOG = Logger.getLogger(BHonPlantControllerHMIAuthenticator.class.getName());

  private static int MAX_FAIL_CNT = 10;
}
