package com.honeywell.honplantcontroller.hmi.user.authenticator;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import javax.baja.nre.annotations.Facet;
import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BFacets;
import javax.baja.sys.Localizable;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.baja.user.BPasswordStrength;
import javax.baja.util.LexiconText;

@NiagaraProperty(
  name = "maximumLength",
  type = "int",
  defaultValue = "6",
  facets = {
    @Facet(name = "BFacets.MAX", value = "6")
  }
)
@NiagaraType
public class BHonPlantControllerHMIPinStrength extends BPasswordStrength {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.user.authenticator.BHonPlantControllerHMIPinStrength(1486318115)1.0$ @*/
/* Generated Mon Mar 14 15:52:49 CET 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "maximumLength"
////////////////////////////////////////////////////////////////

  /**
   * Slot for the {@code maximumLength} property.
   * @see #getMaximumLength
   * @see #setMaximumLength
   */
  public static final Property maximumLength = newProperty(0, 6, BFacets.make(BFacets.MAX, 6));

  /**
   * Get the {@code maximumLength} property.
   * @see #maximumLength
   */
  public int getMaximumLength() { return getInt(maximumLength); }

  /**
   * Set the {@code maximumLength} property.
   * @see #maximumLength
   */
  public void setMaximumLength(int v) { setInt(maximumLength, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPlantControllerHMIPinStrength.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  public BHonPlantControllerHMIPinStrength() {
    this(6, 0, 0, 6, 0, 6);
  }

  public BHonPlantControllerHMIPinStrength(int len, int lower, int upper, int digits, int special, int lenMax) {
    setMinimumLength(len);
    setMinimumLowerCase(lower);
    setMinimumUpperCase(upper);
    setMinimumDigits(digits);
    setMinimumSpecial(special);
    setMaximumLength(lenMax);
  }

  /**
   * Returns true if the given password meets this object's strength requirements, otherwise false.
   *
   *
   * @param messageConsumer if the strength requirements are unmet, a localizable message will be provided
   *                        to {@link Consumer#accept(Object)}
   */
  @Override
  public boolean isPasswordValid(char[] password, Consumer<Localizable> messageConsumer) {
    if (messageConsumer == null) messageConsumer = context -> {};

    int lowerCase = 0;
    int upperCase = 0;
    int digits = 0;
    int special = 0;

    int len = password.length;
    for (char character : password) {

      if (Character.isLetter(character)) {

        if (Character.isUpperCase(character)) {
          upperCase++;
        }
        else {
          lowerCase++;
        }
      }
      else if (Character.isDigit(character)) {
        digits++;
      }
      else {
        special++;
      }
    }

    if (
      len > getMaximumLength() ||
      len < getMinimumLength() ||
      digits < getMinimumDigits() ||
      lowerCase < getMinimumLowerCase() ||
      upperCase < getMinimumUpperCase() ||
      special < getMinimumSpecial())
    {
      List<Localizable> errors = new ArrayList<>();
      errors.add(LexiconText.toLocalizable("baja", "user.password.notStrong"));
      errors.addAll(getLocalizableRequirements());

      messageConsumer.accept(Localizable.concatenate("\n- ", errors));
      return false;
    }

    //verify consecutive numbers
    boolean ok = false;
    for (int i = 0; i < len - 1; i++) {
      if (password[i] + 1 != password[i + 1]) {
        // Not sequential
        ok = true;
        break;
      }
    }
    if (!ok) {
      List<Localizable> errors = new ArrayList<>();
      errors.add(LexiconText.toLocalizable("baja", "user.strongPassword.invalid"));
      errors.addAll(getLocalizableRequirements());

      messageConsumer.accept(Localizable.concatenate("\n- ", errors));
      return false;
    }

    //verify same digits
    ok = false;
    for (int i = 0; i < len - 1; i++) {
      if (password[i] != password[i + 1]) {
        // not same digits
        ok = true;
        break;
      }
    }
    if (!ok) {
      List<Localizable> errors = new ArrayList<>();
      errors.add(LexiconText.toLocalizable("baja", "user.strongPassword.invalid"));
      errors.addAll(getLocalizableRequirements());

      messageConsumer.accept(Localizable.concatenate("\n- ", errors));
      return false;
    }

    //verify the usage of the same digit 3 times
    ok = true;
    for (int i = 0; i < len - 1; i++) {
      int cnt = 1;
      for (int j = 0; j<len; j++) {
        if ( (j != i) && (password[j] == password[i]) ){
          cnt++;
        }
      }
      if (cnt >= 3) {
        ok = false;
        break;
      }
    }
    if (!ok) {
      List<Localizable> errors = new ArrayList<>();
      errors.add(LexiconText.toLocalizable("baja", "user.strongPassword.invalid"));
      errors.addAll(getLocalizableRequirements());

      messageConsumer.accept(Localizable.concatenate("\n- ", errors));
      return false;
    }

    return true;
  }

}
