/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/
package com.honeywell.honplantcontroller.hmi.util;


import com.honeywell.honbacnethelper.export.BIHonBacnetCustomDescriptor;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetScheduleExportUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerBacnetUtils;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;
import com.tridium.bacnet.stack.server.BBacnetExportTable;
import com.tridium.bacnet.stack.server.BBacnetExportTable;

import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Optional;
import java.util.Set;

import javax.baja.bacnet.BBacnetNetwork;
import javax.baja.bacnet.datatypes.BBacnetObjectIdentifier;
import javax.baja.bacnet.export.BIBacnetExportObject;
import javax.baja.data.BIDataValue;
import javax.baja.sys.BComponent;
import javax.baja.sys.BString;
import javax.baja.sys.Sys;
import javax.baja.tag.Id;
import javax.baja.tag.Tag;
import javax.baja.tag.Tags;

public class HonPlantControllerFastAccessListUtils {

    public static boolean checkRead(BComponent target)
    {
        //TODO: check User of HMI Service
        /*
        BUser user = BHonPlantControllerHMITempService.getUser();
        if(user == null)
            return false;

        try
        {
            user.checkRead(target, target.getPropertyInParent());
        }
        catch(Exception ex)
        {
            return false;
        }*/

        return true;
    }

    public static void addTag(BComponent comp, Id tagId, String value)
    {
        HonPlantControllerHMILogger.finest(MessageFormat.format("addTag comp [{0}] tagId [{1}] value [{2}]", comp, tagId, value));

        if(comp == null)
            return;

        Tags tags = comp.tags();

        BString valueOld = BString.make("");
        if(tags.contains(tagId))
        {
            Optional<BIDataValue> opt = tags.get(tagId);
            if(opt.isPresent())
                valueOld = (BString)opt.get();
        }

        String valueNew = value;
        String[] tokens = valueOld.getString().split(";");
        for(int i=0; i<tokens.length; i++)
        {
            String token = tokens[i];
            token = token.trim();

            if(!token.isEmpty() && !token.equals(value))
                valueNew += ";" + token;
        }

        tags.set(new Tag(tagId, BString.make(valueNew)));
    }

    public static void removeTag(BComponent comp, Id tagId, String value)
    {
    	
        if(comp == null)
            return;

        HonPlantControllerHMILogger.finest("BFastAccessList.removeTag(" + comp.getAbsoluteOrd() + comp.getHandleOrd() + tagId + value);

        Tags tags = comp.tags();

        BString valueOld = BString.make("");
        if(tags.contains(tagId))
        {
            Optional<BIDataValue> opt = tags.get(tagId);
            if(opt.isPresent())
                valueOld = (BString)opt.get();
        }
        HonPlantControllerHMILogger.finest("BFastAccessList.removeTag: valueOld" + valueOld);
        String valueNew = "";
        String[] tokens = valueOld.getString().split(";");
        for(int i=0; i<tokens.length; i++)
        {
            String token = tokens[i];
            token = token.trim();

            if(!token.isEmpty() && !token.equals(value))
            {
                if(valueNew.length() > 0)
                    valueNew += ";";

                valueNew += token;
            }
        }
        HonPlantControllerHMILogger.finest("BFastAccessList.removeTag: valueNew" + valueNew);

        if(valueNew.isEmpty()) {
        	HonPlantControllerHMILogger.finest("BFastAccessList.removeTag: tagId" + tagId + valueOld);
            tags.remove(tagId, valueOld);
            updateBacnetExportObject(comp);
        }else {
        	HonPlantControllerHMILogger.finest("BFastAccessList.removeTag: tagId set" + tagId + valueNew);
            tags.set(tagId, BString.make(valueNew));
        }
    }

    public static void updateTag(BComponent comp, Id tagId, String valueOld, String valueNew)
    {
        if(comp == null)
            return;

        removeTag(comp, tagId, valueOld);
        addTag(comp, tagId, valueNew);
    }

    private static void updateBacnetExportObject(BComponent comp) {
        if (HonPlantControllerBacnetUtils.findBacnetNetworks(Sys.getStation()) != null) {
            Set<BBacnetNetwork> bacnetNetworks = HonPlantControllerBacnetUtils.findBacnetNetworks(Sys.getStation());
            Iterator<BBacnetNetwork> iterator = bacnetNetworks.iterator();
            while (iterator.hasNext()) {
                BBacnetNetwork bacnetNetwork = iterator.next();
                BBacnetExportTable bacnetExportTable = (BBacnetExportTable)bacnetNetwork.getLocalDevice().getExportTable();
                BBacnetObjectIdentifier objectIdentifier = bacnetExportTable.byObjectOrd(comp.getHandleOrd());
                BIBacnetExportObject exportObject = bacnetExportTable.byObjectId(objectIdentifier);
                BIHonBacnetCustomDescriptor exportedObject = (BIHonBacnetCustomDescriptor) exportObject;
                exportedObject.setPointInFAL(false);
            }
        }
    }
}
