/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 * This file contains trade secrets of Honeywell International, Inc. No part
 * may be reproduced or transmitted in any form by any means or for any
 * purpose without the express written permission of Honeywell.
 */
package com.honeywell.honplantcontroller.hmi.utils;

import java.security.DigestException;
import java.security.InvalidAlgorithmParameterException;
import java.security.Key;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.interfaces.ECPrivateKey;
import java.security.interfaces.ECPublicKey;
import java.util.Arrays;

import javax.baja.bacnet.BacnetException;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BComponent;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;
import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import com.honeywell.honplantcontroller.hmi.comm.IConfirmedPrivateTransferSender;
import com.honeywell.honplantcontroller.hmi.enums.KeyExchangeState;
import com.honeywell.honplantcontroller.hmi.messages.VerifyCertAckMessage;
import com.honeywell.honplantcontroller.hmi.messages.VerifyMstSecretMessage;

@NiagaraType
public class BHonPlantControllerHMIInitializationHandler extends BComponent {
  /*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
  /*@ $com.honeywell.honplantcontroller.hmi.utils.BHonPlantControllerHMIInitializationHandler(2979906276)1.0$ @*/
  /* Generated Mon Apr 04 15:43:40 CEST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////

  @Override
  public Type getType() {
    return TYPE;
  }

  public static final Type TYPE = Sys.loadType(BHonPlantControllerHMIInitializationHandler.class);

  /*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/


  public void keyExchange() throws BacnetException {
    requestHMICertificate();
  }

  public void requestHMICertificate() throws BacnetException {
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_REQUEST_CERT_EXCHANGE, new byte[]{(byte) 0x80});
  }

  public void setConfirmedPrivateTransferSender(IConfirmedPrivateTransferSender confirmedPrivateTransferSender) {
    this.confirmedPrivateTransferSender = confirmedPrivateTransferSender;
  }

  public void startECDH() throws BacnetException, NoSuchAlgorithmException, NoSuchProviderException, InvalidAlgorithmParameterException {
    final String MSG_PREFIX = BHonPlantControllerHMIInitializationHandler.class.getSimpleName() + ".startECDH()";
    HonPlantControllerHMILogger.finest(MSG_PREFIX + " starting ECDH");

    mstSecret = calculateMstSecret(pubKeyHMI);
    //mstSecretKey = generateMasterKey(mstSecret);

    //confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_START_ECDH, new byte[] {0x00});
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_START_ECDH, new byte[]{});
  }

  public void sendVerifyMstSecret() throws Exception {
    final String MSG_PREFIX = BHonPlantControllerHMIInitializationHandler.class.getSimpleName() + ".sendVerifyMstSecret()";
    //keyExchangeState = keyExchangeState.nextState();

    random1 = new byte[KEY_SIZE];
    new SecureRandom().nextBytes(random1);

        /*
        byte[] encryptedByte = encryptByteArray(random1, mstSecret);
        HonPlantControllerHMILogger.info(MSG_PREFIX + " encrypted rand1 mstSecret: " + TextUtil.bytesToHexString(encryptedByte));
        byte[] decryptByte = decryptByteArray(encryptedByte, mstSecret);
        HonPlantControllerHMILogger.info(MSG_PREFIX + " encrypted rand1 mstSecretKey: " + TextUtil.bytesToHexString(decryptByte));*/

    VerifyMstSecretMessage response = new VerifyMstSecretMessage(random1);
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_VERIFY_MST_SECRET, response.getBytes());
  }

  public void sendChangeCipherMstSecret() throws BacnetException {

    //keyExchangeState =  keyExchangeState.nextState();
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_CHANGE_CIPHER_MST_SECRET, new byte[]{0x00});
  }

  public void sendDeriveSharedKey() throws Exception {
    //keyExchangeState = keyExchangeState.nextState();
    random2 = new byte[KEY_SIZE];
    new SecureRandom().nextBytes(random2);

    //TODO: remove this in production!!
    sharedKeyBytes = generateSharedKey(random2);

    VerifyMstSecretMessage response = new VerifyMstSecretMessage(encryptByteArray(random2, mstSecret));
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_DERIVE_SHARED_KEY, response.getBytes());
  }

  public void sendVerifySharedKey() throws Exception {
    //keyExchangeState = keyExchangeState.nextState();
    random3 = new byte[KEY_SIZE];
    new SecureRandom().nextBytes(random3);

    VerifyMstSecretMessage response = new VerifyMstSecretMessage(encryptByteArray(random3, mstSecret));
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_VERIFY_SHARED_KEY, response.getBytes());
  }

  public void sendChangeCipherSharedKey() throws BacnetException {
    final String MSG_PREFIX = BHonPlantControllerHMIInitializationHandler.class.getSimpleName() + ".sendChangeCipherSharedKey";
    //keyExchangeState = keyExchangeState.nextState();
    confirmedPrivateTransferSender.sendConfirmedPrivateTransferMessage(SERVICE_ID_CHANGE_CIPHER_SHARED_KEY, new byte[]{0x00});
  }
  
  protected byte[] calculateMstSecret(ECPublicKey pub_hmi) {
    final String MSG_PREFIX = BHonPlantControllerHMIInitializationHandler.class.getSimpleName() + ".calculateMstSecret ";
    try {
      // Perform key agreement
      KeyAgreement ka = KeyAgreement.getInstance("ECDH", "BC");
      ka.init(this.privKeyController);
      ka.doPhase(pub_hmi, true);

      // Read shared secret
      byte[] mstSecret = ka.generateSecret();
      return mstSecret;
    } catch (Exception e) {
      HonPlantControllerHMILogger.finest(MSG_PREFIX + e.getLocalizedMessage());
      return null;
    }
  }

  /**
   * performs a hash of the provider bytes
   *
   * @param keyBytes bytes to use to calculate hash
   * @return hash value
   * @throws NoSuchAlgorithmException
   */
  private final byte[] generateSharedKey(byte[] keyBytes) throws NoSuchAlgorithmException, DigestException {
    MessageDigest sha256 = MessageDigest.getInstance("SHA-256");
    //sha256.update(mstSecret, 0, 32);
    //sha256.update(keyBytes, 0, 32);

    byte[] TempRandSalt = new byte[64];
    int i = 0, j = 0, k = 0;
    for (i = 0, j = 0, k = 0; i < 64; i++) {
      if (j < 32)
        TempRandSalt[i] = mstSecret[j++];
      else
        TempRandSalt[i] = keyBytes[k++];
    }
    sha256.update(TempRandSalt, 0, 64);
    byte[] hash = sha256.digest();
    return hash;
  }

  public boolean handleVerifyMstSecretAck(byte[] randToVerify, byte[] rawDataRespose) throws Exception {
    final String MSG_PREFIX = BHonPlantControllerHMIInitializationHandler.class.getSimpleName() + ".handleVerifyMstSecretAck()";
    byte[] payload = VerifyCertAckMessage.getPayload(rawDataRespose);

    randToVerify = VerifyCertAckMessage.getPayload(randToVerify);

    if (payload.length != KEY_SIZE) {
      HonPlantControllerHMILogger.warning(MSG_PREFIX + " incorrect size of payload: expected " + KEY_SIZE + ", received: " + payload.length);
      return false;
    }

    //Now decrypt rand1_encrypted from HMI with MstSecret

    byte[] decryptedMessageBytes = decryptByteArray(payload, mstSecret);
    return (Arrays.equals(random1, decryptedMessageBytes));
  }

  public boolean handleVerifySharedKeyAck(byte[] rawDataRespose) throws Exception {
    final String MSG_PREFIX = BHonPlantControllerHMIInitializationHandler.class.getSimpleName() + ".handleVerifySharedKeyAck()";

    byte[] payload = VerifyCertAckMessage.getPayload(rawDataRespose);

    if (payload.length != KEY_SIZE) {
      return false;
    }

    byte[] mstDecryptedData = decryptByteArray(payload, mstSecret);
    byte[] sharedDecrypted = decryptByteArray(mstDecryptedData, sharedKeyBytes);
    byte[] decryptData = decryptByteArray(sharedDecrypted, mstSecret);

    return (Arrays.equals(random3, decryptData));
  }

  public static byte[] encryptByteArray(byte[] arrayToEncrypt) throws Exception {
    return encryptByteArray(arrayToEncrypt, sharedKeyBytes);
  }

  public static byte[] encryptByteArray(byte[] arrayToEncrypt, byte[] key) throws Exception {
    SecretKeySpec skeySpec = new SecretKeySpec(key, "AES");
    Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
    cipher.init(Cipher.ENCRYPT_MODE, skeySpec, iv);
    return cipher.doFinal(arrayToEncrypt);
  }

  public static byte[] decryptByteArray(byte[] arrayToDecrypt, byte[] key) throws Exception {
    SecretKeySpec skeySpec = new SecretKeySpec(key, "AES");
    Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
    cipher.init(Cipher.DECRYPT_MODE, skeySpec, iv);
    return cipher.doFinal(arrayToDecrypt);
  }

  public static byte[] createIV() {
    byte[] rand = new byte[16];

    for (int i = 0; i < rand.length; i++) {
      rand[i] = 0x55;
    }

    return rand;
  }

  //Secure User Login
  /*
  public byte[] generateRequestRand() throws Exception {
    if (userRand == null) {
      userRand = new byte[USER_RAND_SIZE];
      new SecureRandom().nextBytes(userRand);

      byte[] encryptedRand = encryptByteArray(userRand, sharedKeyBytes);
    }
    return userRand;
  }
  */

  public byte[] GenerateEncryptedSalt(byte[] salt, byte[] rand) throws Exception {

    byte[] tempRandSalt = new byte[64];
    System.arraycopy(salt, 0, tempRandSalt, 0, 32);
    System.arraycopy(rand, 0, tempRandSalt, 32, 32);

    byte[] encryptedSalt = encryptByteArray(tempRandSalt, sharedKeyBytes);
    return  encryptedSalt;
  }

  public void UserLoginDecryptDkRand(byte[] encryptedSaltDkRand, byte[] decryptedDk, byte[] decryptedRand) throws Exception {
    byte[] decryptedDKSaltRand = decryptByteArray(encryptedSaltDkRand, sharedKeyBytes);

    System.arraycopy(decryptedDKSaltRand, 0, decryptedDk, 0, 32);
    System.arraycopy(decryptedDKSaltRand, 32, decryptedRand, 0, 32);
  }

  public void UserLoginDecryptSaltDkRand(byte[] encryptedSaltDkRand, byte[] decryptedSalt, byte[] decryptedDk, byte[] decryptedRand) throws Exception {
    byte[] decryptedDKSaltRand = decryptByteArray(encryptedSaltDkRand, sharedKeyBytes);

    System.arraycopy(decryptedDKSaltRand, 0, decryptedSalt, 0, 32);
    System.arraycopy(decryptedDKSaltRand, 32, decryptedDk, 0, 32);
    System.arraycopy(decryptedDKSaltRand, 64, decryptedRand, 0, 32);
  }

  public boolean isCertVerified() {
    return certVerified;
  }

  public void setCertVerified(boolean certVerified) {
    this.certVerified = certVerified;
  }

  public ECPublicKey getPubKeyHMI() {
    return pubKeyHMI;
  }

  public void setPubKeyHMI(ECPublicKey pubKeyHMI) {
    this.pubKeyHMI = pubKeyHMI;
  }

  public IConfirmedPrivateTransferSender getConfirmedPrivateTransferSender() {
    return confirmedPrivateTransferSender;
  }

  public void setKeyExchangeState(KeyExchangeState keyExchangeState) {
    this.keyExchangeState = keyExchangeState;
  }

  public KeyExchangeState getKeyExchangeState() {
    return keyExchangeState;
  }

  public void setPrivKeyController(ECPrivateKey privKeyController) {
    this.privKeyController = privKeyController;
  }

  public ECPrivateKey getPrivKeyController() {
    return privKeyController;
  }

  public void setPubKeyController(ECPublicKey pubKeyController) {
    this.pubKeyController = pubKeyController;
  }

  public ECPublicKey getPubKeyController() {
    return pubKeyController;
  }

  ECPrivateKey privKeyController;
  ECPublicKey pubKeyController;

  static SecureRandom rnd = new SecureRandom();
  static IvParameterSpec iv = new IvParameterSpec(createIV());

  private boolean certVerified = false;

  ECPublicKey pubKeyHMI;
  Key sharedKey;
  Key mstSecretKey;

  byte[] mstSecret;
  static byte[] sharedKeyBytes;
  byte[] random1;
  byte[] random2;
  byte[] random3;
  byte[] userSalt = null;
  byte[] userRand = null;
  private IConfirmedPrivateTransferSender confirmedPrivateTransferSender;
  private KeyExchangeState keyExchangeState;

  private static final int KEY_SIZE = 32;
  private static final int SHARED_KEY_SIZE = 32;
  private static final int USER_SALT_SIZE = 32;
  private static final int USER_RAND_SIZE = 32;

  public static final int SERVICE_ID_REQUEST_CERT_EXCHANGE = 80;
  public static final int SERVICE_ID_VERIFY_DEVICE_CERT = 81;
  public static final int SERVICE_ID_START_ECDH = 82;
  public static final int SERVICE_ID_VERIFY_MST_SECRET = 83;
  public static final int SERVICE_ID_CHANGE_CIPHER_MST_SECRET = 84;
  public static final int SERVICE_ID_DERIVE_SHARED_KEY = 85;
  public static final int SERVICE_ID_VERIFY_SHARED_KEY = 86;
  public static final int SERVICE_ID_CHANGE_CIPHER_SHARED_KEY = 87;
  
  public static final int SERVICE_ID_SET_DEVICE_INSTANCE	=	92;
  public static final int SERVICE_ID_DEVICE_INFO_NOTIFICATION	=	93;
}
