/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

import javax.baja.bacnet.datatypes.BBacnetObjectIdentifier;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BComponent;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.honeywell.honbacnethelper.export.enums.BHonPointMode;
import com.honeywell.honbacnethelper.util.BIHonBacnetPointModeListener;
import com.honeywell.honbacnethelper.util.HonPointModeDao;
import com.honeywell.honplantcontroller.hmi.comm.IUnconfirmedPrivateTransferSender;
import com.honeywell.honplantcontroller.hmi.messages.WritePointModeMessage;
import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;

@NiagaraType
public class BHonPlantControllerHMIPointModeHandler extends BComponent implements BIHonBacnetPointModeListener {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.utils.BHonPlantControllerHMIPointModeHandler(2979906276)1.0$ @*/
/* Generated Thu Mar 17 07:59:05 CET 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPlantControllerHMIPointModeHandler.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/
    

    /**
     * Starting cyclic collecting pointMode CoVs
     */
    public void startCollect() {
      collecting = true;

      new Thread(this::cyclicCollect).start();
    }
  
    /**
     * cyclic collecting pointMode CoVs and notifying hmi
     */
    protected void cyclicCollect() {
      while (collecting) {

        try {
          synchronized (waitObject) {
            waitObject.wait(waitTime);
          }
          Thread.sleep(waitTime);

          if (!collecting) {
            break;
          }

        } catch (InterruptedException e) {
          HonPlantControllerHMILogger.checkAndLog(Level.WARNING, "[BHonPlantControllerHMIPointModeHandler.cyclicCollect] failed waiting ", e);
          Thread.currentThread().interrupt();
          return;
        }
        synchronized (pointModeBuffer) {

          if (pointModeBuffer.size() > 0 && !((BHonPlantControllerHMI) getParent()).getIsHmiJobActive()) {
            HonPlantControllerHMILogger.checkAndLog(Level.FINE, "[BHonPlantControllerHMIPointModeHandler.cyclicCollect] start freeing buffer entries: " + pointModeBuffer.size());
            WritePointModeMessage pointModeMessage = new WritePointModeMessage();

            pointModeBuffer.forEach(pointModeMessage::addPointMode);

            pointModeBuffer.clear();

            notifyHMI(pointModeMessage);
            HonPlantControllerHMILogger.checkAndLog(Level.FINE,"[BHonPlantControllerHMIPointModeHandler.cyclicCollect] stop freeing buffer entries " + pointModeBuffer.size());

          }
        }
      }
    }
  
    /**
     * stop cyclic collecting
     */
    public void stopRunning() {
      collecting = false;
      synchronized(waitObject) {
        waitObject.notifyAll();
      }
    }
    
    /**
     * Callback function for bacnet descriptors to signal change of pointMode
     * @param  pointModeDao struct containing bacnet ObjectId and current point mode
     */
    @Override
    public void pointModeChanged(HonPointModeDao pointModeDao) {
      synchronized(pointModeBuffer) {
        HonPlantControllerHMILogger.checkAndLog(Level.FINE, "[BHonPlantControllerHMIPointModeHandler.pointModeChanged] objectID: "+pointModeDao.getObjectIdentifier() + " mode "+pointModeDao.getPointMode().getTag());
        pointModeBuffer.put((BBacnetObjectIdentifier) pointModeDao.getObjectIdentifier().newCopy(), (BHonPointMode)pointModeDao.getPointMode().newCopy());
        synchronized(waitObject) {
          waitObject.notifyAll();
        }
      }
    }

    /**
     * helper function to send private transfer message to hmi
     * @param writePointModeMsg
     */
    protected void notifyHMI(WritePointModeMessage writePointModeMsg) {
      unconfirmedPrivateTransferSender.sendUnconfirmedPrivateTransferMessage(WRITE_POINT_MODE_SERVICE_ID, writePointModeMsg.getBytes());
    }

    public void setUnconfirmedPrivateTransferSender(IUnconfirmedPrivateTransferSender unconfirmedPrivateTransferSender) {
      this.unconfirmedPrivateTransferSender = unconfirmedPrivateTransferSender;
    }

    //Attributes
  
    private final Object waitObject = new Object();
    private Map<BBacnetObjectIdentifier, BHonPointMode> pointModeBuffer = new HashMap<>();
    private boolean collecting = false;
    private long waitTime = 1000;
    private IUnconfirmedPrivateTransferSender unconfirmedPrivateTransferSender;
    private static final int  WRITE_POINT_MODE_SERVICE_ID = 76;

    

}
