/*
 * Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
 *
 * This file contains trade secrets of Honeywell International, Inc. No part may be reproduced or
 * transmitted in any form by any means or for any purpose without the express written permission of
 * Honeywell.
 */

package com.honeywell.honplantcontroller.hmi.utils;

import com.tridium.bacnet.asn.AsnInputStream;
import com.tridium.bacnet.asn.AsnOutputStream;
import com.tridium.bacnet.services.confirmed.ReinitializeDeviceRequest;

import javax.baja.bacnet.enums.BCharacterSetEncoding;
import javax.baja.bacnet.io.AsnException;


public class HMIReinitializeDeviceDownloadRequest
  extends ReinitializeDeviceRequest
{
  public static final int START_FIRMWARE_DOWNLOAD = 126;
  public static final int END_FIRMWARE_DOWNLOAD = 127;
  public static final int START_LANGUAGE_DOWNLOAD = 128;
  public static final int END_LANGUAGE_DOWNLOAD = 129;
  public static final int ABORT_DOWNLOAD = 6;
  private int stateDownload;
  private BCharacterSetEncoding encoding;


  public HMIReinitializeDeviceDownloadRequest()
  {
    super();
    this.stateDownload = START_FIRMWARE_DOWNLOAD;
    this.encoding = null;
  }

  public HMIReinitializeDeviceDownloadRequest(int stateDownload, String password, BCharacterSetEncoding encoding)
  {
    super(null, password, encoding);

    this.stateDownload = stateDownload;
    this.encoding = encoding;
  }

  public int getStateDownload()
  {
    return stateDownload;
  }

  protected BCharacterSetEncoding getEncoding()
  {
    return encoding;
  }

  @Override
  public void writeEncoded(AsnOutputStream out) {
    out.writeEnumerated(0, stateDownload);
    if (isPasswordUsed())
    {
      out.writeCharacterString(1, getPassword(), encoding);
    }
  }

  @Override
  public void readEncoded(AsnInputStream in) throws AsnException
  {
    stateDownload = in.readEnumerated(0);
    if (in.peekTag() == 1)
    {
      encoding = in.peekEncoding(1);
      setPassword(in.readCharacterString(1));
    }
  }

  @Override
  public String toString()
  {
    StringBuilder sb = new StringBuilder(getClass().getSimpleName() + ": ");
    sb.append("stateDownload: ").append(stateDownload);
    if (isPasswordUsed())
    {
      sb.append(" encoding: ").append(encoding.getTag());
    }
    return sb.toString();
  }
}
