/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

package com.honeywell.honplantcontroller.hmi.utils;

import java.util.logging.Level;
import java.util.logging.Logger;

public class HonPlantControllerHMILogger {

    private static final String LOG_NAME = "HonPlantControllerHMI";
    private static final Logger LOGGER = Logger.getLogger(LOG_NAME);
    
    private static final String TIME_PROFILE_LOG_NAME = "HonPlantControllerHMITimeProfile";
    private static final Logger TIME_PROFILE_LOGGER = Logger.getLogger(TIME_PROFILE_LOG_NAME);

    private HonPlantControllerHMILogger() {
    }

    public static void fine(String msg) {
        log(Level.FINE, msg);
    }

    public static void fine(String msg, Object param) {
        log(Level.FINE, msg, param);
    }

    public static void fine(String msg, Object[] params) {
        log(Level.FINE, msg, params);
    }

    public static void fine(String msg, Throwable thrown) {
        log(Level.FINE, msg, thrown);
    }

    public static void finer(String msg) {
        log(Level.FINER, msg);
    }

    public static void finer(String msg, Object param) {
        log(Level.FINER, msg, param);
    }

    public static void finer(String msg, Object[] params) {
        log(Level.FINER, msg, params);
    }

    public static void finer(String msg, Throwable thrown) {
        log(Level.FINER, msg, thrown);
    }

    public static void finest(String msg) {
        log(Level.FINEST, msg);
    }

    public static void finest(String msg, Object param) {
        log(Level.FINEST, msg, param);
    }

    public static void finest(String msg, Object[] params) {
        log(Level.FINEST, msg, params);
    }

    public static void finest(String msg, Throwable thrown) {
        log(Level.FINEST, msg, thrown);
    }

    public static void info(String msg) {
        log(Level.INFO, msg);
    }

    public static void info(String msg, Object param) {
        log(Level.INFO, msg, param);
    }

    public static void info(String msg, Object[] params) {
        log(Level.INFO, msg, params);
    }

    public static void info(String msg, Throwable thrown) {
        log(Level.INFO, msg, thrown);
    }

    public static void warning(String msg) {
        log(Level.WARNING, msg);
    }

    public static void warning(String msg, Object param) {
        log(Level.WARNING, msg, param);
    }

    public static void warning(String msg, Object[] params) {
        log(Level.WARNING, msg, params);
    }

    public static void warning(String msg, Throwable thrown) {
        log(Level.WARNING, msg, thrown);
    }

    public static void error(String msg) {
        log(Level.SEVERE, msg);
    }

    public static void error(String msg, Object param) {
        log(Level.SEVERE, msg, param);
    }

    public static void error(String msg, Object[] params) {
        log(Level.SEVERE, msg, params);
    }

    public static void error(String msg, Throwable thrown) {
        log(Level.SEVERE, msg, thrown);
    }

    public static void log(Level level, String msg) {
        LOGGER.log(level, msg);
    }
    
    public static void timeLog(Level level, String msg) {
        TIME_PROFILE_LOGGER.log(level, msg);
    }

    public static void log(Level level, String msg, Object param) {
        LOGGER.log(level, msg, param);
    }

    public static void log(Level level, String msg, Object[] params) {
        LOGGER.log(level, msg, params);
    }

    public static void log(Level level, String msg, Throwable thrown) {
        LOGGER.log(level, msg, thrown);
    }

    public static void checkAndLog(Level level, String msg) {
        if(LOGGER.isLoggable(level)) {
            LOGGER.log(level, msg);
        }
    }

    public static void checkAndLog(Level level, String msg, Throwable thrown) {
        if(LOGGER.isLoggable(level)) {
            LOGGER.log(level, msg, thrown);
        }
    }

    public static boolean isLoggable(Level level) {
        return LOGGER.isLoggable(level);
    }
}
