/*
Copyright (c) 2022 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/
package com.honeywell.honplantcontroller.hmi.view;

import java.util.logging.Level;

import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.sys.BComplex;
import javax.baja.sys.BComponent;
import javax.baja.sys.BFacets;
import javax.baja.sys.BValue;
import javax.baja.sys.Context;
import javax.baja.sys.Flags;
import javax.baja.sys.LocalizableRuntimeException;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.honeywell.honbacnethelper.IgnoreAllChangesContext;
import com.honeywell.honplantcontroller.hmi.comm.UpdateFromHMIDeviceContext;
import com.honeywell.honplantcontroller.hmi.enums.BHMISyncStatus;
import com.honeywell.honplantcontroller.hmi.limitations.HonHMILimitations;
import com.honeywell.honplantcontroller.hmi.service.BHonPlantControllerHMI;
import com.honeywell.honplantcontroller.hmi.utils.HonPlantControllerHMILogger;

@NiagaraProperty(
        name = "syncStatus",
        type = "BHMISyncStatus",
        defaultValue = "BHMISyncStatus.DEFAULT",
        flags = Flags.HIDDEN | Flags.READONLY
)

@NiagaraType
public class BHonPlantControllerFALHome extends BHMIView {
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.honplantcontroller.hmi.view.BHonPlantControllerFALHome(859604863)1.0$ @*/
/* Generated Tue Jun 21 11:27:07 CEST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "syncStatus"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code syncStatus} property.
   * @see #getSyncStatus
   * @see #setSyncStatus
   */
  public static final Property syncStatus = newProperty(Flags.HIDDEN | Flags.READONLY, BHMISyncStatus.DEFAULT, null);
  
  /**
   * Get the {@code syncStatus} property.
   * @see #syncStatus
   */
  public BHMISyncStatus getSyncStatus() { return (BHMISyncStatus)get(syncStatus); }
  
  /**
   * Set the {@code syncStatus} property.
   * @see #syncStatus
   */
  public void setSyncStatus(BHMISyncStatus v) { set(syncStatus, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BHonPlantControllerFALHome.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  @Override
  public void checkAdd(String name, BValue value, int flags, BFacets facets, Context context) {
    
	super.checkAdd(name, value, flags, facets, context);
	
	if(!Sys.atSteadyState())
		return;
    
    ///CLOVER:OFF
    if (hasFALLimitExceeded()) {
      // this cannot be tested because the exception is not populated to the test an therefore not testable.
      throw new LocalizableRuntimeException("honPlantControllerHMI", "BFastAccessList.fal.limit.exceeded");
    }
    ///CLOVER:ON

    
    ///CLOVER:OFF
    if (isFALNameToLong(name)) {
      // this cannot be tested because the exception is not populated to the test an therefore not testable.
      throw new LocalizableRuntimeException("honPlantControllerHMI", "BFastAccessList.fal.name.limit.exceeded");
    }
    ///CLOVER:ON
  }

  private void updateFALStatusInHMIDevice() {
	if(!Sys.isStation() || !Sys.atSteadyState())
		return;
	
	BComplex home= getParent();
	if(!(home instanceof BHonPlantControllerHome)) 
		return;
	

	BComplex hmiDevice= home.getParent();
	if(!(hmiDevice instanceof BHonPlantControllerHMI))
		return;
	
	((BHonPlantControllerHMI)hmiDevice).setFalSyncStatus(BHMISyncStatus.outOfSync);
  }


  @Override
	public void changed(Property property, Context context) {
		super.changed(property, context);
		if(property == syncStatus && getSyncStatus().getOrdinal() == BHMISyncStatus.OUT_OF_SYNC) {
			updateFALStatusInHMIDevice();
		}
	}
  @Override
	public void added(Property property, Context context) {
		super.added(property, context);
		if(!Sys.isStationStarted())
			return;
		
		if(context instanceof UpdateFromHMIDeviceContext || context instanceof IgnoreAllChangesContext)
			return;
		if(get(property) instanceof BFastAccessList) {
			setSyncStatus(BHMISyncStatus.outOfSync);
		}
	}
  

	@Override
	public void checkRemove(Property property, Context context) {
		HonPlantControllerHMILogger.checkAndLog(Level.FINEST,
				"BHonPlantControllerFALHome.checkRemove property = " + property.getName());

		BValue value = get(property.getName());
		if (value instanceof BFastAccessList) {
			BFastAccessList fal = (BFastAccessList) value;
			fal.removeTags();
		}

		super.checkRemove(property, context);
	}

	  @Override
	public void removed(Property property, BValue oldValue, Context context) {
		super.removed(property, oldValue, context);
		if(!Sys.isStationStarted())
			return;

		if(oldValue instanceof BFastAccessList) {
            BFastAccessList fal = (BFastAccessList)oldValue;
    		if(context instanceof UpdateFromHMIDeviceContext || context instanceof IgnoreAllChangesContext)
    			return;
            setSyncStatus(BHMISyncStatus.outOfSync);
		}
	}
  
  @Override
  public void checkRename(Property property, String newName, Context context) {
    // this cannot be tested because the exception is not populated to the test an therefore not testable.
    ///CLOVER:OFF
    if (isFALNameToLong(newName)) {
      throw new LocalizableRuntimeException("honPlantControllerHMI", "BFastAccessList.fal.name.limit.exceeded");
    }
    ///CLOVER:OFF
    super.checkRename(property, newName, context);
  }
  
  @Override
	public void renamed(Property property, String oldName, Context context) {
		super.renamed(property, oldName, context);
		if(!Sys.isStationStarted())
			return;

		if(get(property) instanceof BFastAccessList) {
            BFastAccessList fal = (BFastAccessList)get(property);
            fal.updateTags();
    		if(context instanceof UpdateFromHMIDeviceContext || context instanceof IgnoreAllChangesContext)
    			return;
            setSyncStatus(BHMISyncStatus.outOfSync);
		}
	}
  
  
  public boolean hasFALLimitExceeded() {
    return getChildren(BFastAccessList.class).length >= HonHMILimitations.getMaxNumberOfFastAccessLists();
  }

  public boolean isFALNameToLong(String name) {
    if (name.startsWith("$")) {
      return name.length() > (HonHMILimitations.getMaxLengthOfFastAccessListName() + 2); //if name starts with number, "$3" is added at the beginning
    } else {
      return name.length() > (HonHMILimitations.getMaxLengthOfFastAccessListName());
    }
  }

  public BHMISyncStatus getFALSyncStatus() {

	  if(getSyncStatus().getOrdinal() == BHMISyncStatus.OUT_OF_SYNC) {
		  return BHMISyncStatus.outOfSync;
    }
	
    for (BComponent child : getChildComponents()) {
      if (child instanceof BFastAccessList) {
        BFastAccessList fal = (BFastAccessList)child;
        if (fal.getSyncStatus().equals(BHMISyncStatus.outOfSync)) {
          return BHMISyncStatus.outOfSync;
        }
      }
    }

    return BHMISyncStatus.inSync;

  }

  public void resetFALSyncStatus() {
	setSyncStatus(BHMISyncStatus.inSync);
    for (BComponent child : getChildComponents()) {
      if (child instanceof BFastAccessList) {
        BFastAccessList fal = (BFastAccessList)child;
        fal.resetSyncStatus();
      }
    }
  }
}
