/**
 * Copyright 2014, QuickLink Solutions - All Rights Reserved.
 */
package com.honeywell.platPanelbus;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.text.MessageFormat;

import javax.baja.nre.annotations.NiagaraProperty;
import javax.baja.nre.annotations.NiagaraType;
import javax.baja.nre.util.IntHashMap;
import javax.baja.serial.BISerialPort;
import javax.baja.serial.BISerialService;
import javax.baja.serial.BSerialBaudRate;
import javax.baja.serial.BSerialDataBits;
import javax.baja.serial.BSerialParity;
import javax.baja.serial.BSerialStopBits;
import javax.baja.serial.PortDeniedException;
import javax.baja.sys.Flags;
import javax.baja.sys.Property;
import javax.baja.sys.Sys;
import javax.baja.sys.Type;

import com.tridium.platSerial.BSerialPort;
import com.tridium.platSerial.BSerialPortPlatformService;

/**
 * BPanelbusPlatformServiceNpsdk
 */


@NiagaraProperty(
  name = "isBEATS",
  type = "boolean",
  flags = Flags.READONLY | Flags.HIDDEN | Flags.TRANSIENT | Flags.DEFAULT_ON_CLONE,
  defaultValue = "false"
)

@NiagaraType
public class BPanelbusPlatformServiceNpsdk
  extends BPanelbusPlatformService
{
  
/*+ ------------ BEGIN BAJA AUTO GENERATED CODE ------------ +*/
/*@ $com.honeywell.platPanelbus.BPanelbusPlatformServiceNpsdk(3422901901)1.0$ @*/
/* Generated Tue Jan 04 16:59:30 IST 2022 by Slot-o-Matic (c) Tridium, Inc. 2012 */

////////////////////////////////////////////////////////////////
// Property "isBEATS"
////////////////////////////////////////////////////////////////
  
  /**
   * Slot for the {@code isBEATS} property.
   * @see #getIsBEATS
   * @see #setIsBEATS
   */
  public static final Property isBEATS = newProperty(Flags.READONLY | Flags.HIDDEN | Flags.TRANSIENT| Flags.DEFAULT_ON_CLONE, false, null);
  
  /**
   * Get the {@code isBEATS} property.
   * @see #isBEATS
   */
  public boolean getIsBEATS() { return getBoolean(isBEATS); }
  
  /**
   * Set the {@code isBEATS} property.
   * @see #isBEATS
   */
  public void setIsBEATS(boolean v) { setBoolean(isBEATS, v, null); }

////////////////////////////////////////////////////////////////
// Type
////////////////////////////////////////////////////////////////
  
  @Override
  public Type getType() { return TYPE; }
  public static final Type TYPE = Sys.loadType(BPanelbusPlatformServiceNpsdk.class);

/*+ ------------ END BAJA AUTO GENERATED CODE -------------- +*/

  /**
   * Constructor.
   */
  public BPanelbusPlatformServiceNpsdk()
  {

  }


  public boolean isValidPlatform()
  {
	//check for OS
	  if(isDevOfflineMode()) {
		  log.trace("Device in offline mode, not loading the dll panelbus-npsdk");
		  return false;
	  }

    return loadLibraries();
  }

  private boolean isOnController() {
    return (isLINUX()|| isQnxOS());
  }

  // return true if the operation system is QNX; false otherwise
  private boolean isQnxOS()
  {
    boolean ret = false;
    String os = getOsName();
    if (null == os)
      return false;

    // compare with known OS name
    if (("QNX").equals(os))
      ret = true;

    return ret;
  }

  private String getOsName()
  {
    String os = System.getProperty("os.name");
    if (null != os) {
      // look OS string (force to upper case and remove leading and trailing
      // whitespace
      os = os.toUpperCase().trim();
      log.message("OS=" + os);
    }
    return os;
  }
  private boolean isLINUX()
  {
    boolean ret = false;
    String os = getOsName();
    if (null == os)
      return false;

    // compare with known OS name
    if (("LINUX").equals(os))
      ret = true;

    return ret;
  }

  // return true if to be in offline mode; false to use normal production
  // communication
  private boolean isDevOfflineMode() {
    return !isOnController();
  }

  private static BSerialPortPlatformService getSerialService()
  {
    // get a reference to the serial port service
    if (serialPortPlatformService == null)
    {
      try
      {
        // get a reference to the serial port service
        serialPortPlatformService = (BSerialPortPlatformService)Sys.getService(BSerialPortPlatformService.TYPE);
      }
      catch (Exception e)
      {
        // can not load the serial platform service, locking will not be available
      }
    }

    return serialPortPlatformService;
  }

  protected synchronized boolean loadLibraries()
  {
    if (nativesLoaded)
      return true;

    AccessController.doPrivileged((PrivilegedAction<Void>) () ->
    {
      try
      {
        System.loadLibrary("panelbus-npsdk");
        nativesLoaded = true;
        log.trace("panelbus-npsdk loaded");
        
        _eagleHawk = true;
      }
      catch (Throwable t)
      {
        log.trace("Unable load panelbus-npsdk native, maybe not running on Honeywell controller ", t);
        
    nativesLoaded = true; //do this check only once
    _eagleHawk = false;
      }
      log.message(MessageFormat.format("{0}.loadLibraries: {1} : {2}", TYPE.getTypeName(), _eagleHawk,
                                       getIsBEATS()));

      return null;
    });
        
    return nativesLoaded;
  }

  public void serviceStarted() throws Exception
  {
    String os = getOsName();
    if (null != os)
    {
      if (("LINUX").equals(os))
        setIsBEATS(true);
    }

    if (!initPanelbusd())
      throw new RuntimeException("Cannot run panelbus daemon processes");
    else
    {
      log.trace("Panelbus initialization complete");
      super.serviceStarted();
      return;
    }
  }

  public boolean initPanelbusd()
  {
    if (!loadLibraries())
      return false;
    
    if(_eagleHawk)
      return init0();
    else
      return true;
  }

  public void serviceStopped() throws Exception
  {
    if(_eagleHawk)
      cleanup0();
    
    log.trace("Cleanup panelbus driver done");
    super.serviceStopped();
  }

  public static String getFirmwareFilename()
  {
    return PB_FIRMWARE_PATH + PB_FIRMWARE_FILE;
  }

  public static String getSnaponFirmwareFilename()
  {
    return PB_FIRMWARE_PATH + PB_SNAPON_FIRMWARE_FILE;
  }

  /* (non-Javadoc)
   * @see com.centraline.platPanelbus.BPanelbusPlatformService#startDriver(int)
   */
  public int openPort(String portName, PanelbusListener listener)
  {

//    final String LOG_MSG_PREFIX = TYPE.getTypeName() + ".openPort";

    log.trace("Starting panelbus driver: " + portName);
    if (!loadLibraries())
      throw new RuntimeException("Cannot load panelbus natives");

    int portIndex = -1;
    String serialName = "";
    try
    {
      if(_eagleHawk)
      {
        serialName = portName;
        
        String osPortName = lockSerial(portName, listener.getListenerName());
        if (osPortName == null)
          throw new RuntimeException("Cannot lock panelbus port");
  
        portIndex = openPort0(osPortName);
  
        if (portIndex == -1)
        {
          log.error("Could not open panelbus: " + portName);
  
          try
          {
            unlockSerial(portName);
          }
          catch (PortDeniedException e)
          {
            log.warning("Failed to unlock panelbus serial port " + portName, e);
          }
  
          throw new RuntimeException("Cannot open panelbus port");
        }
      }
      else
      {
        BISerialService platSvc = (BISerialService)Sys.getService(BISerialService.TYPE);
        platSvc.checkPropertiesLoaded();
        String [] names = platSvc.getPortNames();

//        log.message(MessageFormat.format("{0} portName[{1}]", LOG_MSG_PREFIX, portName));
//        log.message(MessageFormat.format("{0} availPorts[{1}] platSvc.type[{2}]", LOG_MSG_PREFIX, names.length,
//                                         platSvc.getType().getTypeName()));
//        for (int index = 0; index < names.length; index++)
//        {
//          log.message(MessageFormat.format("{0}   index[{1}] name[{2}]", LOG_MSG_PREFIX, index, names[index]));
//        }

        for (int index = 0; index < names.length; index++)
        {
          if (portName.equals(names[index]))
          {
            portIndex = index;
            break;
          }
        }
        if (portIndex == -1)
        {
          // fallback
          portIndex = Integer.parseInt(portName.substring(portName.lastIndexOf("_") + 1)) - 1;
//          log.message(MessageFormat.format("{0} using portIndex[{1}] (index from portName)", LOG_MSG_PREFIX, portIndex));
        }
//        else
//        {
//          log.message(MessageFormat.format("{0} using portIndex[{1}] (index from name compare)", LOG_MSG_PREFIX, portIndex));
//        }

        serialName = "none";
        if(names.length > portIndex)
          serialName = names[portIndex]; //that should work for all future controller types

//        String osPortName = lockSerial(serialName, listener.getListenerName());
//        if (osPortName == null)
//          throw new RuntimeException("Cannot lock panelbus port");
        
        BISerialPort serialPort = platSvc.openPort(serialName, "PanelBus");
        if(serialPort != null)
        { 
          serialPort.setSerialPortParams(BSerialBaudRate.baud115200, BSerialDataBits.dataBits8, BSerialStopBits.stopBit1, BSerialParity.none);
          serialPort.enableReceiveTimeout(platSvc.getMinTimeout());

          _serialPorts.put(portIndex, serialPort);
        }
        else
        {
          portIndex = -1;
          log.error("Error opening serial port!");

//          try
//          {
//            unlockSerial(portName);
//          }
//          catch (PortDeniedException e)
//          {
//            log.warning("Failed to unlock panelbus serial port " + serialName, e);
//          }
  
          throw new RuntimeException("Cannot open panelbus port");
        }
      }
      
      if(portIndex > -1)
      {
        // hash the handle so we can unlock later the serial port
        handleToName.put(portIndex, serialName);

        PanelbusTrunkListener trunkListener = new PanelbusTrunkListener(portIndex, listener);
        Thread thread = new Thread(trunkListener, "panelbus" + portIndex);
        thread.setPriority(Thread.MAX_PRIORITY);
        thread.start();
        trunkListener.myThread = thread;
        trunkListeners.put(portIndex, trunkListener);
      }
    }
    catch (PortDeniedException e)
    {
      // failed to obtain lock
      log.error("Could not open panelbus on serial port " + portName + ", failed to obtain lock");
      throw new RuntimeException("Cannot open panelbus port");
    }
    catch (Exception e)
    {
      // failed to obtain lock
      log.error("Could not open Serial port " + portName + "/" + serialName + ", error " + e.getMessage());
      throw new RuntimeException("Cannot open panelbus port");
    }

    return portIndex;
  }

  /* (non-Javadoc)
   * @see com.centraline.platPanelbus.BPanelbusPlatformService#stopDriver(int)
   */
  public void closePort(int portIndex)
  {
    log.trace("Stopping panelbus driver");
    if (!loadLibraries())
      throw new RuntimeException("Cannot load panelbus natives");

    PanelbusTrunkListener trunkListener = (PanelbusTrunkListener)trunkListeners.get(portIndex);
    if (trunkListener == null)
      throw new RuntimeException("Invalid handle " + portIndex);
    else
    {
      trunkListener.done = true;
      trunkListener.myThread.interrupt();
      
      if(_eagleHawk)
      {
        closePort0(portIndex);
  
        String portName = (String)handleToName.remove(portIndex);
        try
        {
          unlockSerial(portName);
        }
        catch (PortDeniedException e)
        {
          log.error("Failed to unlock panelbus serial port " + portName, e);
        }
      }
      else
      {
        BISerialPort serialPort = (BISerialPort)_serialPorts.remove(portIndex);
        
        try
        {
          serialPort.getInputStream().close();
        }
        catch (Exception e)
        {
          log.error("Unable to close serial input stream. ", e);
        }

          try
          {
          serialPort.getOutputStream().close();
          }
          catch (Exception e)
          {
          log.error("Unable to close serial output stream. ", e);
          }
    
        if(serialPort != null)
        {
          serialPort.disableReceiveTimeout();
          serialPort.close();
        }
        
        String serialName = (String)handleToName.remove(portIndex);
//        try
//        {
//          unlockSerial(serialName);
//        }
//        catch (PortDeniedException e)
//        {
//          log.error("Failed to unlock panelbus serial port " + serialName, e);
//        }
      }
      
      log.trace("Panelbus driver stopped");
    }
  }

  /* (non-Javadoc)
   * @see com.centraline.platPanelbus.BPanelbusPlatformService#sendFrame(int, byte, byte[])
   */
  public void sendFrame(int portIndex, byte devType, byte dstAddress, byte[] data)
  {
    if (!loadLibraries())
      throw new RuntimeException("Cannot load panelbus natives");

    if (data.length > 260)
      throw new IllegalArgumentException("Invalid frame length " + data.length);
    else
    {
      if(_eagleHawk)
        sendFrame0(portIndex, devType, dstAddress, data);
      else
      {
        try
        {
          BISerialPort serialPort = (BISerialPort)_serialPorts.get(portIndex);
          OutputStream out = serialPort.getOutputStream();
          out.write(data);
        }
        catch (Exception e)
        {
          log.error("Serial port write error " + e.getMessage());
        }
      }
    }
  }

////////////////////////////////////////////////////////////////
//Lock/Unlock serial port
////////////////////////////////////////////////////////////////

  private synchronized String lockSerial(String portName, String owner)
    throws PortDeniedException
  {
    log.trace("Attempting to lock " + portName + " for owner " + owner);

    PortLock lock = getLockFileName(portName);

    if (lock == null)
    {
      log.trace("Can not lock " + portName + ", can not map lock file");
      return null;
    }

    // determine who owns the lock
    String currentOwner = getLockFileOwner(lock.lockFileName);
    if (!"none".equalsIgnoreCase(currentOwner))
      throw new PortDeniedException("Unabled to lock ther serial port for owner panelbus, already owned by " + currentOwner);

    try
    {
      // lets now try to create the lock
      try(FileOutputStream fos = new FileOutputStream(lock.lockFileName)){
    	  fos.write((STATION_PREFIX + "panelbus").getBytes());
      }

      // set the current owner based on the listener name
      lock.serialPort.setOwner(owner);
    }
    catch(Exception e)
    {
      // could not write the lock file
      throw new PortDeniedException("Unable to lock the port " + portName + " for owner panelbus: " + e.getMessage());
    }

    // we created the lock file, serial service should now see it when it calls checkOwner()
    log.trace(portName + " locked for owner panelbus");

    return lock.serialPort.getOsPortName();
  }

  private synchronized BSerialPort unlockSerial(String portName)
    throws PortDeniedException
  {
    log.trace("Attempting to unlock " + portName + " for owner panelbus");

    PortLock lock = getLockFileName(portName);

    if (lock == null)
    {
      log.trace("Can not unlock " + portName + ", can not map lock file");
      return null;
    }

    // determine who owns the lock
    String currentOwner = getLockFileOwner(lock.lockFileName);
    if (!currentOwner.equals("none") && !currentOwner.equalsIgnoreCase(STATION_PREFIX + "panelbus"))
      throw new PortDeniedException("Unable to unlock the port" + portName + ", panelbus does not have the port lock");

    // delete the lock file if we can
    File file = new File(lock.lockFileName);
    if (file.exists() && !file.delete())
      throw new PortDeniedException("Unable to unlock the port " + portName + ", failed to delete " + lock.lockFileName);

    // success
    log.trace(getName() + " unlocked");

    // reset the current owner
    lock.serialPort.setOwner("none");

    return lock.serialPort;
  }

  private String getLockFileOwner(String lockFileName)
  {
    File lockFile = new File(lockFileName);

    if (!lockFile.exists())
      return "none";

    try
    {
      // file exists, determine it's contents
      try(FileInputStream fis = new FileInputStream(lockFile)){
	      StringBuffer buf = new StringBuffer();
	      int c = fis.read();
	      while (c != -1)
	      {
	        buf.append((char) c);
	        c = fis.read();
	      }
	      return buf.toString();
      }
    }
    catch (IOException e)
    {
      // we know the file exists but we can't determine who owns it
      log.trace("Problem reading lock file " + lockFileName + " for panelbus defaulting to no ownership", e);
      return "none";
    }
  }

  private PortLock getLockFileName(String serialPortName)
  {
    // Use the serial port service to determine the port index
    //
    // I.E /dev/tty* should be in the port identifiers known to the serial port service,
    // we need to figure out what the lock file name would be according to the serial port service.
    if (getSerialService() == null)
    {
      // we can't mimic the serial services locking ability
      log.trace("Coult not obtain serial platform service reference, can not mimic locking behavior in panelbus");
      return null;
    }

    // get the available serial ports
    String[] portNames = serialPortPlatformService.getPortNames();

    if (portNames == null)
    {
      // try to load the serial port identifiers
      serialPortPlatformService.checkPropertiesLoaded();

      portNames = serialPortPlatformService.getPortNames();
    }

    int index = -1;
    BSerialPort serialPort = null;

    // now has a list of slot names, use these to look up the os name,
    // if we find a match, we know we should lock this file
    for (int i=0; i<portNames.length; ++i)
    {
      serialPort = ((BSerialPort)serialPortPlatformService.get(portNames[i]));
      if (serialPort.getName().equalsIgnoreCase(serialPortName))
      {
        // we found what the slot we want, try to figure out what the index is,
        // we are assuming that indexes start
        index = serialPort.getPortIndex();
        break;
      }
    }

    if (index == -1)
    {
      // we could not find the port in the serial service, can't lock it
      log.trace("Coult not obtain serial port index from serial platform service reference, can not mimic locking behavior in panelbus");
      return null;
    }

    // return the lock file name the serial port service would use
    return new PortLock(serialPort, System.getProperty("java.io.tmpdir", "/tmp") + File.separatorChar + "niagara-ser" + index + ".owner");
  }

  class PortLock
  {
    public PortLock(BSerialPort serialPort, String lockFileName)
    {
      this.serialPort = serialPort;
      this.lockFileName = lockFileName;
    }

    private BSerialPort serialPort;
    private String lockFileName;
  }

////////////////////////////////////////////////////////////////
//Listener
////////////////////////////////////////////////////////////////

  class PanelbusTrunkListener
    implements Runnable
  {

    public PanelbusTrunkListener(int portIndex, PanelbusListener listener)
    {
      handle = portIndex;
      linkListener = listener;
    }

    public void run()
    {
      PanelbusFrame frame = new PanelbusFrame();
      BPanelbusPlatformServiceNpsdk.log.trace("Panelbus trunk listener starting");

      InputStream in = null;
      if(!_eagleHawk)
      {
        try
        {
          in = ((BISerialPort)_serialPorts.get(handle)).getInputStream();
        }
        catch (IOException e)
        {
          log.error(e.getMessage());
          in = null;
        }
        
        if(in == null)
        {
          String msg = "Error getting InputStream for serial port!"; 
          log.error(msg);
          throw new RuntimeException(msg);
        }
      }
      
      while (!done)
      {
        try
        {
          if(_eagleHawk)
          { 
            rcvFrame0(handle, frame);
            
            if (linkListener != null)
              linkListener.receiveFrame(frame.getDevType(), frame.getSrcAddress(), frame.getData(), (frame.getData()[3] & 0xFF) + 5);
          }
          else
          {
            byte [] msg = new byte[256];
            
            byte [] data = new byte [256];
            int count = 0;
            boolean receive = true;
             
            while (receive)
            {
              //Break if COM is unlocked and serial port is closed
              if(done)
                break;

              int n = in.read(data, count, data.length - count);
              if (n > 0)
              {
                int cut=0;
                count += n;
                // check header (4 bytes)
                for (int i=0; i<(count-5); ++i)
                {
                  if (data[i] == 0x20)
                  {
                    cut = i;
                    int length = data[i+3];
                    if (length < 0) length += 256;
                    length += 5;
                    if (i+length <= count)
                    {
                       for (int k=0; k<length; ++k)
                       {
                          msg[k] = data[i+k];
                       }
                       receive = false;                           
                    }                           
                    break;
                  }
                }
                if (!receive && cut > 0)
                {
                  for (int i=0; cut<count; ++i, ++cut)
                  {
                    data[i] = data[cut];
                  } 
                  count -= cut;
                }
              }                  
            }
            
            if (linkListener != null)
              linkListener.receiveFrame(msg[1], msg[2], msg, (msg[3] & 0xFF) + 5);
          }
        }
        catch (Exception e)
        {
          try{ Thread.sleep(10); } catch(Exception ex){;}
          
          if (!done)
            BPanelbusPlatformServiceNpsdk.log.warning("Exception in rcvFrame0!", e);
        }
      }

      BPanelbusPlatformServiceNpsdk.log.trace("Panelbus trunk listener done");
    }

    private int handle;
    private PanelbusListener linkListener;
    private Thread myThread;
    private boolean done = false;
  }

  public boolean isEagleHawk()
  {
    return _eagleHawk;
  }
  
////////////////////////////////////////////////////////////////
//Natives
////////////////////////////////////////////////////////////////

  private final native boolean init0();

  private final native void cleanup0();

  private final native int openPort0(String portName);

  private final native void closePort0(int portIndex);

  private final native void rcvFrame0(int portIndex, PanelbusFrame frame);

  private final native void sendFrame0(int portIndex, byte devType, byte dstAddress, byte data[]);

// Compile the JNI headers
// D:\Niagara\r3dev\platPanelBus\libJar\com\centraline\platPanelbus>"C:\Program Files\Java\jdk1.6.0_21\bin\javah" -classpath D:\Niagara\r3dev\platPanelBus\libJar com.centraline.platPanelbus.BPanelbusPlatformServiceNpsdk

////////////////////////////////////////////////////////////////
//Attributes
////////////////////////////////////////////////////////////////

  private int numTrunks = 6;
  private static boolean nativesLoaded = false;
  private static boolean _eagleHawk = true;
  private IntHashMap trunkListeners = new IntHashMap();

  // hash map for handle -> serial port name to be used to lock/unlock files
  private IntHashMap handleToName = new IntHashMap();

  private static String STATION_PREFIX = "station:";

  // store a reference to the serial port service so we can lock/unlock files the same way
  private static BSerialPortPlatformService serialPortPlatformService = null;
  
  private IntHashMap _serialPorts = new IntHashMap();

  // location of Panelbus firmware image
  private static final String PB_FIRMWARE_PATH = "module://honFirmwarePackage/res/panelbusFirmware/";
  private static final String PB_FIRMWARE_FILE = "Pb_fw.bin";
  private static final String PB_SNAPON_FIRMWARE_FILE = "Pb_fw_Snapon.bin";
}
